Package: pkgdown
Title: Make Static HTML Documentation for a Package
Version: 2.2.0
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Jay", "Hesselberth", role = "aut",
           comment = c(ORCID = "0000-0002-6299-179X")),
    person("Maëlle", "Salmon", role = "aut",
           comment = c(ORCID = "0000-0002-2815-0399")),
    person("Olivier", "Roy", role = "aut"),
    person("Salim", "Brüggemann", role = "aut",
           comment = c(ORCID = "0000-0002-5329-5987")),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Generate an attractive and useful website from a source
    package.  'pkgdown' converts your documentation, vignettes, 'README',
    and more to 'HTML' making it easy to share information about your
    package online.
License: MIT + file LICENSE
URL: https://pkgdown.r-lib.org/, https://github.com/r-lib/pkgdown
BugReports: https://github.com/r-lib/pkgdown/issues
Depends: R (>= 4.1)
Imports: bslib (>= 0.5.1), callr (>= 3.7.3), cli (>= 3.6.1), desc (>=
        1.4.0), downlit (>= 0.4.4), fontawesome, fs (>= 1.4.0), httr2
        (>= 1.0.2), jsonlite, lifecycle, openssl, purrr (>= 1.0.0),
        ragg (>= 1.4.0), rlang (>= 1.1.4), rmarkdown (>= 2.27), tibble,
        whisker, withr (>= 2.4.3), xml2 (>= 1.3.1), yaml (>= 2.3.9)
Suggests: covr, diffviewer, evaluate (>= 0.24.0), gert, gt, htmltools,
        htmlwidgets, knitr (>= 1.50), magick, methods, pkgload (>=
        1.0.2), quarto, rsconnect, rstudioapi, rticles, sass, testthat
        (>= 3.1.3), tools
VignetteBuilder: knitr, quarto
Config/Needs/website: usethis, servr
Config/potools/style: explicit
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/testthat/start-first: build-article, build-quarto-article,
        build-reference, build
Config/usethis/last-upkeep: 2025-09-07
Encoding: UTF-8
RoxygenNote: 7.3.3
SystemRequirements: pandoc
NeedsCompilation: no
Packaged: 2025-11-05 16:48:47 UTC; hadleywickham
Author: Hadley Wickham [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4757-117X>),
  Jay Hesselberth [aut] (ORCID: <https://orcid.org/0000-0002-6299-179X>),
  Maëlle Salmon [aut] (ORCID: <https://orcid.org/0000-0002-2815-0399>),
  Olivier Roy [aut],
  Salim Brüggemann [aut] (ORCID: <https://orcid.org/0000-0002-5329-5987>),
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2025-11-06 06:10:20 UTC
Built: R 4.6.0; ; 2026-04-25 11:52:52 UTC; unix

Package: testpackage
Version: 1.0.0
Title: A test package
Description: A longer statement about the package.
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@rstudio.com", role = c("aut", "cre")),
    person("RStudio", role = c("cph", "fnd"))
    )
RoxygenNote: 6.1.1
Encoding: UTF-8

Package: testpackage
Version: 1.0.0
Title: A test package
Description: A longer statement about the package.
Authors@R:
    person("Hadley", "Wickham", , "hadley@rstudio.com", role = c("aut", "cre"))
RoxygenNote: 6.0.1

# Generated by roxygen2: do not edit by hand

S3method("[",tag)
S3method(as_data,"NULL")
S3method(as_data,tag_arguments)
S3method(as_data,tag_author)
S3method(as_data,tag_description)
S3method(as_data,tag_details)
S3method(as_data,tag_format)
S3method(as_data,tag_note)
S3method(as_data,tag_references)
S3method(as_data,tag_section)
S3method(as_data,tag_seealso)
S3method(as_data,tag_source)
S3method(as_data,tag_usage)
S3method(as_data,tag_value)
S3method(as_example,COMMENT)
S3method(as_example,RCODE)
S3method(as_example,TEXT)
S3method(as_example,VERB)
S3method(as_example,tag)
S3method(as_example,tag_dontrun)
S3method(as_example,tag_dontshow)
S3method(as_example,tag_donttest)
S3method(as_example,tag_dots)
S3method(as_example,tag_if)
S3method(as_example,tag_ifelse)
S3method(as_example,tag_ldots)
S3method(as_example,tag_out)
S3method(as_example,tag_testonly)
S3method(as_html,"#ifdef")
S3method(as_html,"#ifndef")
S3method(as_html,COMMENT)
S3method(as_html,LIST)
S3method(as_html,RCODE)
S3method(as_html,Rd)
S3method(as_html,TEXT)
S3method(as_html,USERMACRO)
S3method(as_html,VERB)
S3method(as_html,character)
S3method(as_html,tag)
S3method(as_html,tag_R)
S3method(as_html,tag_S3method)
S3method(as_html,tag_S4method)
S3method(as_html,tag_Sexpr)
S3method(as_html,tag_acronym)
S3method(as_html,tag_bold)
S3method(as_html,tag_cite)
S3method(as_html,tag_code)
S3method(as_html,tag_command)
S3method(as_html,tag_cr)
S3method(as_html,tag_dQuote)
S3method(as_html,tag_deqn)
S3method(as_html,tag_describe)
S3method(as_html,tag_dfn)
S3method(as_html,tag_dots)
S3method(as_html,tag_email)
S3method(as_html,tag_emph)
S3method(as_html,tag_enc)
S3method(as_html,tag_enumerate)
S3method(as_html,tag_env)
S3method(as_html,tag_eqn)
S3method(as_html,tag_figure)
S3method(as_html,tag_file)
S3method(as_html,tag_href)
S3method(as_html,tag_if)
S3method(as_html,tag_ifelse)
S3method(as_html,tag_item)
S3method(as_html,tag_itemize)
S3method(as_html,tag_kbd)
S3method(as_html,tag_ldots)
S3method(as_html,tag_link)
S3method(as_html,tag_linkS4class)
S3method(as_html,tag_method)
S3method(as_html,tag_newcommand)
S3method(as_html,tag_option)
S3method(as_html,tag_out)
S3method(as_html,tag_pkg)
S3method(as_html,tag_preformatted)
S3method(as_html,tag_renewcommand)
S3method(as_html,tag_sQuote)
S3method(as_html,tag_samp)
S3method(as_html,tag_special)
S3method(as_html,tag_strong)
S3method(as_html,tag_subsection)
S3method(as_html,tag_tab)
S3method(as_html,tag_tabular)
S3method(as_html,tag_url)
S3method(as_html,tag_var)
S3method(as_html,tag_verb)
S3method(pkgdown_print,default)
S3method(pkgdown_print,gt_tbl)
S3method(pkgdown_print,htmlwidget)
S3method(print,pkgdown_xml)
S3method(print,print_yaml)
S3method(print,tag)
S3method(usage_code,"NULL")
S3method(usage_code,COMMENT)
S3method(usage_code,RCODE)
S3method(usage_code,Rd)
S3method(usage_code,TEXT)
S3method(usage_code,VERB)
S3method(usage_code,tag)
S3method(usage_code,tag_S3method)
S3method(usage_code,tag_S4method)
S3method(usage_code,tag_dots)
S3method(usage_code,tag_ldots)
S3method(usage_code,tag_method)
S3method(usage_code,tag_special)
S3method(usage_code,tag_usage)
export(as_pkgdown)
export(build_article)
export(build_articles)
export(build_articles_index)
export(build_favicons)
export(build_home)
export(build_home_index)
export(build_llm_docs)
export(build_news)
export(build_redirects)
export(build_reference)
export(build_reference_index)
export(build_search)
export(build_site)
export(build_site_github_pages)
export(build_tutorials)
export(check_pkgdown)
export(clean_cache)
export(clean_site)
export(data_template)
export(deploy_site_github)
export(deploy_to_branch)
export(fig_settings)
export(in_pkgdown)
export(init_site)
export(pkgdown_print)
export(pkgdown_sitrep)
export(preview_site)
export(rd2html)
export(render_page)
export(template_articles)
export(template_navbar)
export(template_reference)
import(fs)
import(rlang)
importFrom(lifecycle,deprecated)
importFrom(utils,installed.packages)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# pkgdown <a href="https://pkgdown.r-lib.org"><img src="man/figures/logo.png" align="right" height="138" alt="pkgdown website" /></a>

<!-- badges: start -->

<a href="https://cran.r-project.org/package=pkgdown"
class="pkgdown-release"><img
src="https://www.r-pkg.org/badges/version/pkgdown"
alt="CRAN Status" /></a> <a
href="https://github.com/r-lib/pkgdown/actions/workflows/R-CMD-check.yaml"
class="pkgdown-devel"><img
src="https://github.com/r-lib/pkgdown/actions/workflows/R-CMD-check.yaml/badge.svg"
alt="R-CMD-check" /></a> [![Codecov test
coverage](https://codecov.io/gh/r-lib/pkgdown/graph/badge.svg)](https://app.codecov.io/gh/r-lib/pkgdown)
<!-- badges: end -->

pkgdown is designed to make it quick and easy to build a website for
your package. You can see pkgdown in action at
<https://pkgdown.r-lib.org>: this is the output of pkgdown applied to
the latest version of pkgdown. Learn more in `vignette("pkgdown")` or
`?build_site`.

## Installation

<div class=".pkgdown-release">

``` r
# Install released version from CRAN
install.packages("pkgdown")
```

</div>

<div class=".pkgdown-devel">

``` r
# Install development version from GitHub
# install.packages("pak")
pak::pak("r-lib/pkgdown")
```

</div>

## Usage

Get started with [usethis](https://usethis.r-lib.org/):

``` r
# Run once to configure your package to use and deploy pkgdown
usethis::use_pkgdown_github_pages()
```

``` r
# Preview your site locally before publishing
pkgdown::build_site()
```

This adds the necessary components and sets up GitHub Actions[^1] for
automatic site building when deploying. Your `README.md` becomes the
homepage, documentation in `man/` generates a function reference, and
vignettes will be rendered into `articles/`.

### pkgdown 2.0.0 and Bootstrap 5

pkgdown 2.0.0 includes an upgrade from Bootstrap 3 to Bootstrap 5, which
is accompanied by a whole bunch of minor UI improvements. If you’ve
heavily customised your site, there’s a small chance that this will
break your site, so everyone needs to explicitly opt-in to the upgrade
by adding the following to `_pkgdown.yml`:

``` yaml
template:
  bootstrap: 5
```

Then learn about the many new ways to customise your site in
`vignette("customise")`.

## In the wild

At last count, pkgdown is used [by over 12,000
packages](https://github.com/search?q=path%3A_pkgdown.yml+language%3AYAML&type=code&l=YAML).
Here are a few examples:

- [bayesplot](http://mc-stan.org/bayesplot/index.html)
  ([source](https://github.com/stan-dev/bayesplot/tree/gh-pages)):
  plotting functions for posterior analysis, model checking, and MCMC
  diagnostics.

- [valr](https://rnabioco.github.io/valr/)
  ([source](https://github.com/rnabioco/valr)): read and manipulate
  genome intervals and signals.

- [mkin](https://pkgdown.jrwb.de/mkin/)
  ([source](https://github.com/jranke/mkin)): calculation routines based
  on the FOCUS Kinetics Report

- [NMF](http://renozao.github.io/NMF/master/index.html)
  ([source](https://github.com/renozao/NMF)): a framework to perform
  non-negative matrix factorization (NMF).

- [tidyverse and r-lib packages
  source](https://github.com/search?q=path%3A%22_pkgdown.yml%22+AND+%28org%3Atidyverse+OR+org%3Ar-lib%29&type=code)

Comparing the source and output of these sites is a great way to learn
new pkgdown techniques.

## Code of conduct

Please note that this project is released with a [Contributor Code of
Conduct](https://pkgdown.r-lib.org/CODE_OF_CONDUCT.html). By
participating in this project you agree to abide by its terms.

[^1]: If you don’t use GitHub, you can use `usethis::use_pkgdown()` +
    `pkgdown::build_site()` to create a website.

<!-- README.md is generated from README.Rmd. Please edit that file -->

# pkgdown

<!-- badges: start -->

[![Lifecycle:
maturing](https://img.shields.io/badge/lifecycle-maturing-blue.svg)](https://www.tidyverse.org/lifecycle/#maturing)
[![CRAN
Status](https://www.r-pkg.org/badges/version/pkgdown)](https://cran.r-project.org/package=pkgdown)
[![R build
status](https://github.com/r-lib/pkgdown/workflows/R-CMD-check/badge.svg)](https://github.com/r-lib/pkgdown/actions)
[![Codecov test
coverage](https://codecov.io/gh/r-lib/pkgdown/branch/master/graph/badge.svg)](https://codecov.io/gh/r-lib/pkgdown?branch=master)
<!-- badges: end -->

pkgdown is designed to make it quick and easy to build a website for
your package. 

## -----------------------------------------------------------------------------
# # Run this once to publish your site regularly
# usethis::use_pkgdown_github_pages()

## -----------------------------------------------------------------------------
# # Run once
# # Remove docs/ from gitignore to ensure it is checked into git.
# usethis::use_pkgdown()
# # Run everytime you want to update your site
# pkgdown::build_site()

#' Build articles section
#'
#' @description
#' `build_articles()` renders each R Markdown file underneath `vignettes/` and
#' saves it to `articles/`. There are two exceptions:
#'
#' * Files that start with `_` (e.g., `_index.Rmd`) are ignored,
#'   enabling the use of child documents.
#'
#' * Files in `vignettes/tutorials` are handled by [build_tutorials()]
#'
#' Vignettes are rendered using a special document format that reconciles
#' [rmarkdown::html_document()] with the pkgdown template. This means articles
#' behave slightly differently to vignettes, particularly with respect to
#' external files, and custom output formats. See below for more details.
#'
#' Note that when you run `build_articles()` directly (outside of
#' [build_site()]) vignettes will use the currently installed version of the
#' package, not the current source version. This makes iteration quicker when
#' you are primarily working on the text of an article.
#'
#' # Index and navbar
#' You can control the articles index and navbar with a `articles` field in
#' your `_pkgdown.yml`. If you use it, pkgdown will check that all articles
#' are included, and will error if you have missed any.
#'
#' The `articles` field defines a list of sections, each of which
#' can contain four fields:
#'
#' * `title` (required): title of section, which appears as a heading on the
#'   articles index.
#'
#' * `desc` (optional): An optional markdown description displayed underneath
#'   the section title.
#'
#' * `navbar` (optional): A couple of words used to label this section in
#'   the navbar. If omitted, this section of vignettes will not appear in the
#'   navbar.
#'
#' * `contents` (required): a list of article names to include in the
#'   section. This can either be names of individual vignettes or a
#'   call to `starts_with()`. The name of a vignette includes its
#'   path under `vignettes` without extension so that the name of the vignette
#'   found at `vignettes/pizza/slice.Rmd` is `pizza/slice`.
#'
#' The title and description of individual vignettes displayed on the index
#' comes from `title` and `description` fields of the YAML header in the Rmds.
#'
#' For example, this yaml might be used for some version of dplyr:
#'
#' ```yaml
#' articles:
#' - title: Main verbs
#'   navbar: ~
#'   contents:
#'   - one-table
#'   - two-table
#'   - rowwise
#'   - colwise
#'
#' - title: Developer
#'   desc: Vignettes aimed at package developers
#'   contents:
#'   - programming
#'   - packages
#' ```
#'
#' Note the use of the `navbar` fields. `navbar: ~` means that the "Main verbs"
#' will appear in the navbar without a heading; the absence of the `navbar`
#' field in the developer vignettes means that they will only be
#' accessible via the articles index.
#'
#' The navbar will include a link to the articles index if one or more
#' vignettes are not available through the navbar. If some vignettes appear
#' in the navbar drop-down list and others do not, the list will automatically
#' include a "More ..." link at the bottom; if no vignettes appear in the
#' the navbar, it will link directly to the articles index instead of
#' providing a drop-down.
#'
#' ## Get started
#' Note that a vignette with the same name as the package (e.g.,
#' `vignettes/pkgdown.Rmd` or `vignettes/articles/pkgdown.Rmd`) automatically
#' becomes a top-level "Get started" link, and will not appear in the articles
#' drop-down.
#'
#' (If your package name includes a `.`, e.g. `pack.down`, use a `-` in the
#' vignette name, e.g. `pack-down.Rmd`.)
#'
#' ## Missing articles
#'
#' pkgdown will warn if there are (non-internal) articles that aren't listed
#' in the articles index. You can suppress such warnings by listing the
#' affected articles in a section with `title: internal` (case sensitive);
#' this section will not be displayed on the index page.
#'
#' ## External articles
#'
#' You can link to arbitrary additional articles by adding an
#' `external-articles` entry to `_pkgdown.yml`. It should contain an array
#' of objects with fields `name`, `title`, `href`, and `description`.
#'
#' ```yaml
#' external-articles:
#' - name: subsampling
#'   title: Subsampling for Class Imbalances
#'   description: Improve model performance in imbalanced data sets through undersampling or oversampling.
#'   href: https://www.tidymodels.org/learn/models/sub-sampling/
#' ```
#'
#' If you've defined a custom articles index, you'll need to include the name
#' in one of the `contents` fields.
#'
#' # External files
#' pkgdown differs from base R in its handling of external files. When building
#' vignettes, R assumes that vignettes are self-contained (a reasonable
#' assumption when most vignettes were PDFs) and only copies files explicitly
#' listed in `.install_extras`. pkgdown takes a different approach based on
#' [rmarkdown::find_external_resources()], and it will also copy any images that
#' you link to. If for some reason the automatic detection doesn't work, you
#' will need to add a `resource_files` field to the yaml metadata, e.g.:
#'
#' ```yaml
#' ---
#' title: My Document
#' resource_files:
#'   - data/mydata.csv
#'   - images/figure.png
#' ---
#' ```
#'
#' Note that you can not use the `fig.path` to change the output directory of
#' generated figures as its default value is a strong assumption of rmarkdown.
#'
#' # Embedding Shiny apps
#' If you would like to embed a Shiny app into an article, the app will have
#' to be hosted independently, (e.g. <https://www.shinyapps.io>). Then, you
#' can embed the app into your article using an `<iframe>`, e.g.
#' `<iframe src = "https://gallery.shinyapps.io/083-front-page" class="shiny-app">`.
#'
#' See <https://github.com/r-lib/pkgdown/issues/838#issuecomment-430473856> for
#' some hints on how to customise the appearance with CSS.
#'
#' # Output formats
#' By default, pkgdown builds all articles using the
#' [rmarkdown::html_document()] `output` format, ignoring whatever is set in
#' your YAML metadata. This is necessary because pkgdown has to integrate the
#' HTML/CSS/JS from the vignette with the HTML/CSS/JS from rest of the site.
#' Because of the challenges of combining two sources of HTML/CSS/JS, there is
#' limited support for other output formats and you have to opt-in by setting
#' the `as_is` field in your `.Rmd` metadata:
#'
#' ```yaml
#' pkgdown:
#'   as_is: true
#' ```
#'
#' If the output format produces a PDF, you'll also need to specify the
#' `extension` field:
#'
#' ```yaml
#' pkgdown:
#'   as_is: true
#'   extension: pdf
#' ```
#'
#' To work with pkgdown, the output format must accept `template`, `theme`, and
#' `self_contained` arguments, and must work without any additional CSS or
#' JSS files. Note that if you use
#' [`_output.yml`](https://bookdown.org/yihui/rmarkdown/html-document.html#shared-options)
#' or [`_site.yml`](https://rmarkdown.rstudio.com/docs/reference/render_site.html)
#' you'll still need to add `as_is: true` to each individual vignette.
#'
#' Additionally, htmlwidgets do not work when `as_is: true`.
#'
#' # Suppressing vignettes
#' If you want [articles](https://r-pkgs.org/vignettes.html#sec-vignettes-article)
#' that are not vignettes, use `usethis::use_article()` to create it. An articles link will be
#' automatically added to the default navbar if the vignettes directory is
#' present: if you do not want this, you will need to customise the navbar. See
#' [build_site()] details.
#'
#' @inheritSection build_reference Figures
#' @family site components
#'
#' @inheritParams as_pkgdown
#' @param quiet Set to `FALSE` to display output of knitr and
#'   pandoc. This is useful when debugging.
#' @param lazy If `TRUE`, will only re-build article if input file has been
#'   modified more recently than the output file.
#' @param seed Seed used to initialize random number generation in order to
#'   make article output reproducible. An integer scalar or `NULL` for no seed.
#' @param preview If `TRUE`, or `is.na(preview) && interactive()`, will preview
#'   freshly generated section in browser.
#' @export
#' @order 1
build_articles <- function(
  pkg = ".",
  quiet = TRUE,
  lazy = TRUE,
  seed = 1014L,
  override = list(),
  preview = FALSE
) {
  pkg <- section_init(pkg, "articles", override = override)
  check_bool(quiet)
  check_bool(lazy)
  check_number_whole(seed, allow_null = TRUE)

  if (nrow(pkg$vignettes) == 0L) {
    return(invisible())
  }

  cli::cli_rule("Building articles")

  build_articles_index(pkg)
  unwrap_purrr_error(purrr::walk(
    pkg$vignettes$name[pkg$vignettes$type == "rmd"],
    build_article,
    pkg = pkg,
    lazy = lazy,
    seed = seed,
    quiet = quiet
  ))
  build_quarto_articles(pkg, quiet = quiet)

  preview_site(pkg, "articles", preview = preview)
}

# Articles index ----------------------------------------------------------

#' @export
#' @rdname build_articles
#' @order 3
build_articles_index <- function(pkg = ".", override = list()) {
  pkg <- section_init(pkg, "articles", override = override)
  render_page(
    pkg,
    "article-index",
    data = data_articles_index(pkg),
    path = path("articles", "index.html")
  )
  invisible()
}

data_articles_index <- function(pkg = ".", call = caller_env()) {
  pkg <- as_pkgdown(pkg)

  articles <- data_articles(pkg, is_index = TRUE, call = call)
  index <- config_pluck_list(pkg, "articles", call = call) %||%
    default_articles_index(pkg)
  sections <- unwrap_purrr_error(purrr::imap(
    index,
    data_articles_index_section,
    articles = articles,
    pkg = pkg,
    call = call
  ))

  # Check for unlisted vignettes
  all_names <- purrr::map(sections, function(section) {
    purrr::map_chr(section$contents, "name")
  })
  listed <- unique(purrr::list_c(all_names))

  missing <- setdiff(articles$name, listed)
  # Exclude get started vignette or article #2150
  missing <- missing[!article_is_intro(missing, package = pkg$package)]

  if (length(missing) > 0) {
    config_abort(
      pkg,
      "{length(missing)} vignette{?s} missing from index: {.val {missing}}.",
      call = caller_env()
    )
  }

  # Remove internal section after missing vignettes check
  sections <- Filter(function(x) x$title != "internal", sections)

  print_yaml(list(
    pagetitle = tr_("Articles"),
    sections = sections
  ))
}

data_articles <- function(pkg = ".", is_index = FALSE, call = caller_env()) {
  pkg <- as_pkgdown(pkg)

  internal <- tibble::tibble(
    name = pkg$vignettes$name,
    title = pkg$vignettes$title,
    href = pkg$vignettes$file_out,
    description = pkg$vignettes$description,
  )
  if (is_index) {
    internal$href <- path_rel(internal$href, "articles")
  }

  external <- config_pluck_external_articles(pkg, call = call)
  articles <- rbind(internal, external)

  articles$description <- lapply(
    articles$description,
    markdown_text_block,
    pkg = pkg
  )

  # Hack data structure so we can use select_topics()
  articles$alias <- as.list(articles$name)
  articles$internal <- FALSE

  articles
}

config_pluck_external_articles <- function(pkg, call = caller_env()) {
  external <- config_pluck_list(pkg, "external-articles", call = call)
  if (is.null(external)) {
    return(tibble::tibble(
      name = character(),
      title = character(),
      href = character(),
      description = character()
    ))
  }

  for (i in seq_along(external)) {
    config_check_list(
      external[[i]],
      has_names = c("name", "title", "href", "description"),
      error_path = paste0("external-articles[", i, "]"),
      error_pkg = pkg,
      error_call = call
    )
    config_check_string(
      external[[i]]$name,
      error_path = paste0("external-articles[", i, "].name"),
      error_pkg = pkg,
      error_call = call
    )
    config_check_string(
      external[[i]]$title,
      error_path = paste0("external-articles[", i, "].title"),
      error_pkg = pkg,
      error_call = call
    )
    config_check_string(
      external[[i]]$href,
      error_path = paste0("external-articles[", i, "].href"),
      error_pkg = pkg,
      error_call = call
    )
    config_check_string(
      external[[i]]$description,
      error_path = paste0("external-articles[", i, "].description"),
      error_pkg = pkg,
      error_call = call
    )
  }

  tibble::tibble(
    name = purrr::map_chr(external, "name"),
    title = purrr::map_chr(external, "title"),
    href = purrr::map_chr(external, "href"),
    description = purrr::map_chr(external, "description")
  )
}

data_articles_index_section <- function(
  section,
  index,
  articles,
  pkg,
  call = caller_env()
) {
  config_check_list(
    section,
    error_path = paste0("articles[", index, "]"),
    has_names = c("title", "contents"),
    error_pkg = pkg,
    error_call = call
  )
  config_check_string(
    section$title,
    error_path = paste0("articles[", index, "].title"),
    error_pkg = pkg,
    error_call = call
  )
  title <- markdown_text_inline(
    pkg,
    section$title,
    error_path = paste0("articles[", index, "].title"),
    error_call = call
  )

  config_check_string(
    section$desc,
    error_path = paste0("articles[", index, "].desc"),
    error_pkg = pkg,
    error_call = call
  )
  check_contents(
    section$contents,
    index,
    pkg,
    prefix = "articles",
    call = call
  )

  # Match topics against any aliases
  idx <- select_topics(
    section$contents,
    articles,
    error_path = paste0("articles[", index, "].contents"),
    error_pkg = pkg,
    error_call = call
  )
  contents <- articles[idx, , drop = FALSE]

  list(
    title = title,
    desc = markdown_text_block(pkg, section$desc),
    class = section$class,
    contents = purrr::transpose(contents)
  )
}

default_articles_index <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)

  if (nrow(pkg$vignettes) == 0L) {
    return(NULL)
  }

  print_yaml(list(
    list(
      title = tr_("All vignettes"),
      desc = NULL,
      contents = auto_quote(unname(pkg$vignettes$name))
    )
  ))
}

article_is_intro <- function(name, package) {
  package <- gsub(".", "-", package, fixed = TRUE)
  name %in% c(package, paste0("articles/", package))
}

test_that("can recognise intro variants", {
  expect_true(article_is_intro("package", "package"))
  expect_true(article_is_intro("articles/package", "package"))
  expect_true(article_is_intro("pack-age", "pack.age"))
  expect_true(article_is_intro("articles/pack-age", "pack.age"))
})

test_that("validates articles yaml", {
  data_articles_index_ <- function(x) {
    pkg <- local_pkgdown_site(meta = list(articles = x))
    data_articles_index(pkg)
  }

  expect_snapshot(error = TRUE, {
    data_articles_index_(1)
    data_articles_index_(list(1))
    data_articles_index_(list(list()))
    data_articles_index_(list(list(title = 1, contents = 1)))
    data_articles_index_(list(list(title = "a\n\nb", contents = 1)))
    data_articles_index_(list(list(title = "a", contents = 1)))
  })
})

test_that("validates external-articles", {
  data_articles_ <- function(x) {
    pkg <- local_pkgdown_site(meta = list(`external-articles` = x))
    data_articles(pkg)
  }
  expect_snapshot(error = TRUE, {
    data_articles_(1)
    data_articles_(list(1))
    data_articles_(list(list(name = "x")))
    data_articles_(list(list(name = 1, title = "x", href = "x", description = "x")))
    data_articles_(list(list(name = "x", title = 1, href = "x", description = "x")))
    data_articles_(list(list(name = "x", title = "x", href = 1, description = "x")))
    data_articles_(list(list(name = "x", title = "x", href = "x", description = 1)))
  })
})

test_that("data_articles includes external articles", {
  pkg <- local_pkgdown_site(meta = list(
    `external-articles` = list(
      list(name = "c", title = "c", href = "c", description = "*c*")
    )
  ))
  pkg <- pkg_add_file(pkg, "vignettes/a.Rmd")
  pkg <- pkg_add_file(pkg, "vignettes/b.Rmd")

  articles <- data_articles(pkg)
  expect_equal(articles$name, c("a", "b", "c"))
  expect_equal(articles$internal, rep(FALSE, 3))
  expect_equal(articles$description, list(NULL, NULL, "<p><em>c</em></p>"))
})

test_that("articles in vignettes/articles/ are unnested into articles/", {
  # weird path differences that I don't have the energy to dig into
  skip_on_cran()

  pkg <- local_pkgdown_site(meta = list(url = "https://example.com"))
  pkg <- pkg_add_file(pkg, "vignettes/articles/nested.Rmd")

  nested <- pkg$vignettes[pkg$vignettes$name == "articles/nested", ]
  expect_equal(nested$file_out, "articles/nested.html")

  # Check automatic redirect from articles/articles/foo.html -> articles/foo.html
  expect_snapshot(build_redirects(pkg))
})

test_that("warns about articles missing from index", {
  pkg <- local_pkgdown_site(meta = list(
    articles = list(
      list(title = "External", contents = c("a", "b"))
    )
  ))
  pkg <- pkg_add_file(pkg, "vignettes/a.Rmd")
  pkg <- pkg_add_file(pkg, "vignettes/b.Rmd")
  pkg <- pkg_add_file(pkg, "vignettes/c.Rmd")

  expect_snapshot(. <- data_articles_index(pkg), error = TRUE)
})

test_that("internal articles aren't included and don't trigger warning", {
  pkg <- local_pkgdown_site(meta = 
    list(articles = 
      list(
        list(title = "External", contents = c("a", "b")),
        list(title = "internal", contents = "c")
      )
    )
  )
  pkg <- pkg_add_file(pkg, "vignettes/a.Rmd")
  pkg <- pkg_add_file(pkg, "vignettes/b.Rmd")
  pkg <- pkg_add_file(pkg, "vignettes/c.Rmd")

  expect_no_error(index <- data_articles_index(pkg))
  expect_length(index$sections, 1)
  expect_length(index$sections[[1]]$contents, 2)
})

test_that("default template includes all articles", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/a.Rmd")

  expect_equal(default_articles_index(pkg)[[1]]$contents, "a")
})

test_that("check doesn't include getting started vignette", {
  pkg <- local_pkgdown_site(meta = list(
    articles = list(list(title = "Vignettes", contents = "a"))
  ))
  pkg <- pkg_add_file(pkg, "vignettes/a.Rmd")
  pkg <- pkg_add_file(pkg, "vignettes/testpackage.Rmd")

  expect_no_error(data_articles_index(pkg))
})

as_data <- function(x, ...) {
  UseMethod("as_data")
}

#' @export
as_data.NULL <- function(x, ...) {
  NULL
}

# Sections ----------------------------------------------------------------

parse_section <- function(x, title, ...) {
  text <- flatten_para(x, ...)

  list(
    title = title,
    contents = text
  )
}

#' @export
as_data.tag_details <- function(x, ...) {
  parse_section(x, tr_("Details"), ...)
}
#' @export
as_data.tag_description <- function(x, ...) {
  parse_section(x, tr_("Description"), ...)
}
#' @export
as_data.tag_references <- function(x, ...) {
  parse_section(x, tr_("References"), ...)
}
#' @export
as_data.tag_source <- function(x, ...) {
  parse_section(x, tr_("Source"), ...)
}
#' @export
as_data.tag_format <- function(x, ...) {
  parse_section(x, tr_("Format"), ...)
}
#' @export
as_data.tag_note <- function(x, ...) {
  parse_section(x, tr_("Note"), ...)
}
#' @export
as_data.tag_author <- function(x, ...) {
  parse_section(x, tr_("Author"), ...)
}
#' @export
as_data.tag_seealso <- function(x, ...) {
  section <- parse_section(x, tr_("See also"), ...)
  section$contents <- dont_index(section$contents)
  section
}
#' @export
as_data.tag_section <- function(x, ...) {
  parse_section(x[[2]], as_html(x[[1]], ...), ...)
}

# \arguments{} & \details{} -----------------------------------------------
# Both are like the contents of \description{} but can contain arbitrary
# text outside of \item{}

#' @export
as_data.tag_arguments <- function(x, ...) {
  list(
    title = tr_("Arguments"),
    contents = describe_contents(x, ..., id_prefix = "arg-")
  )
}

#' @export
as_data.tag_value <- function(x, ...) {
  list(
    title = tr_("Value"),
    contents = describe_contents(x, ...)
  )
}

describe_contents <- function(x, ..., id_prefix = NULL) {
  if (length(x) == 0) {
    return("")
  }

  # Group contiguous \items{}/whitespace into a <dl>; everything else
  # is handled as is
  block_id <- integer(length(x))
  block_id[[1]] <- 1
  cur_block_is_dl <- inherits(x[[1]], "tag_item")

  for (i in seq2(2, length(x))) {
    is_item <- inherits(x[[i]], "tag_item")
    if (cur_block_is_dl) {
      same_type <- is_item || is_whitespace(x[[i]])
    } else {
      same_type <- !is_item
    }

    if (same_type) {
      block_id[[i]] <- block_id[[i - 1]]
    } else {
      block_id[[i]] <- block_id[[i - 1]] + 1
      cur_block_is_dl <- !cur_block_is_dl
    }
  }

  parse_block <- function(x) {
    is_dl <- any(purrr::map_lgl(x, inherits, "tag_item"))
    if (is_dl) {
      paste0(
        "<dl>\n",
        parse_descriptions(x, ..., id_prefix = id_prefix),
        "</dl>"
      )
    } else {
      flatten_para(x, ...)
    }
  }
  blocks <- split(x, block_id)
  out <- purrr::map(blocks, parse_block)

  paste(unlist(out), collapse = "\n")
}

is_whitespace <- function(x) {
  inherits(x, "TEXT") && all(grepl("^\\s*$", x))
}


# For testing
value2html <- function(x) {
  rd <- rd_text(paste0("\\value{", x, "}"), fragment = FALSE)[[1]]
  html <- as_data(rd)$contents
  str_trim(strsplit(str_trim(html), "\n")[[1]])
}

# Value blocks ------------------------------------------------------------

test_that("leading text parsed as paragraph", {
  expected <- c(
    "<p>text</p>",
    "<dl>", "<dt>x</dt>", "<dd><p>y</p></dd>", "</dl>"
  )
  expect_equal(value2html("\ntext\n\\item{x}{y}"), expected)
  expect_equal(value2html("text\\item{x}{y}"), expected)
})

test_that("leading text is optional", {
  expect_equal(
    value2html("\\item{x}{y}"),
    c("<dl>", "<dt>x</dt>", "<dd><p>y</p></dd>", "</dl>")
  )
})

test_that("can process empty string", {
  expect_equal(value2html(""), character())
})

test_that("leading text is optional", {
  expect_equal( value2html("text"),"<p>text</p>")
})

test_that("items are optional", {
  value <- rd_text("\\value{text}", fragment = FALSE)
  expect_equal(as_data(value[[1]])$contents, "<p>text</p>")
})


test_that("whitespace between items doesn't affect grouping", {
  expect_equal(
    value2html("\\item{a}{b}\n\n\\item{c}{d}\n\n\\item{e}{f}"),
    c(
      "<dl>",
        "<dt>a</dt>", "<dd><p>b</p></dd>", "", "",
        "<dt>c</dt>", "<dd><p>d</p></dd>", "", "",
        "<dt>e</dt>", "<dd><p>f</p></dd>",
      "</dl>"
    )
  )
})

test_that("leading whitespace doesn't break items", {
  expect_equal(
    value2html("\n\\item{a}{b}\n\n\\item{c}{d}\n\n\\item{e}{f}"),
    c(
      "<dl>",
        "",
        "<dt>a</dt>", "<dd><p>b</p></dd>", "", "",
        "<dt>c</dt>", "<dd><p>d</p></dd>", "", "",
        "<dt>e</dt>", "<dd><p>f</p></dd>",
      "</dl>"
    )
  )
})

test_that("whitespace between text is preserved", {
  expect_equal(
    value2html("a\n\nb\n\nc"),
    c(
      "<p>a</p>", "", "",
      "<p>b</p>", "", "",
      "<p>c</p>"
    )
  )
})

test_that("can have multiple interleaved blocks", {
  expect_equal(
    value2html("text1\\item{a}{b}\\item{c}{d}text2\\item{e}{f}"),
    c(
      "<p>text1</p>",
      "<dl>",
        "<dt>a</dt>", "<dd><p>b</p></dd>",
        "<dt>c</dt>", "<dd><p>d</p></dd>",
      "</dl>",
      "<p>text2</p>",
      "<dl>", "<dt>e</dt>", "<dd><p>f</p></dd>", "</dl>"
    )
  )
})

#' Build reference section
#'
#' By default, pkgdown will generate an index that lists all functions in
#' alphabetical order. To override this, provide a `reference` section in your
#' `_pkgdown.yml` as described below.
#'
#' # Reference index
#' To tweak the index page, add a section called `reference` to `_pkgdown.yml`.
#' It can contain three different types of element:
#'
#' * A **title** (`title` + `desc`), which generates an row containing an `<h2>`
#'   with optional paragraph description.
#' * A **subtitle** (`subtitle` + `desc`), which generates an row containing an
#'   `<h3>` with optional paragraph description.
#' * A **list of topics** (`contents`), which generates one row for each topic,
#'   with a list of aliases for the topic on the left, and the topic title
#'   on the right.
#'
#' (For historical reasons you can include `contents` with a title or
#' subtitle, but this is no longer recommended).
#'
#' Most packages will only need to use `title` and `contents` components.
#' For example, here's a snippet from the YAML that pkgdown uses to generate
#' its own reference index:
#'
#' ```yaml
#' reference:
#' - title: Build
#'   desc:  Build a complete site or its individual section components.
#' - contents:
#'   - starts_with("build_")
#' - title: Templates
#' - contents:
#'   - template_navbar
#'   - render_page
#' ```
#'
#' Bigger packages, e.g. ggplot2, may need an additional layer of
#' structure in order to clearly organise large number of functions:
#'
#' ```yaml
#' reference:
#' - title: Layers
#' - subtitle: Geoms
#'   desc: Geom is short for geometric element
#' - contents:
#'   - starts_with("geom")
#' - subtitle: Stats
#'   desc: Statistical transformations transform data before display.
#'   contents:
#'   - starts_with("stat")
#' ```
#'
#' `desc` can use markdown, and if you have a long description it's a good
#' idea to take advantage of the YAML `>` notation:
#'
#' ```yaml
#' desc: >
#'   This is a very _long_ and **overly** flowery description of a
#'   single simple function. By using `>`, it's easy to write a description
#'   that runs over multiple lines.
#' ```
#'
#' ## Topic matching
#' `contents` can contain:
#'
#' * Individual function/topic names.
#' * Weirdly named functions with doubled quoting, once for YAML and once for
#'   R, e.g. `` "`+.gg`" ``.
#' * `starts_with("prefix")` to select all functions with common prefix.
#' * `ends_with("suffix")` to select all functions with common suffix.
#' * `matches("regexp")` for more complex regular expressions.
#' * `has_keyword("x")` to select all topics with keyword "x";
#'   `has_keyword("datasets")` selects all data documentation.
#' * `has_concept("blah")` to select all topics with concept "blah".
#'   If you are using roxygen2, `has_concept()` also matches family tags, because
#'   roxygen2 converts them to concept tags.
#' * `lacks_concepts(c("concept1", "concept2"))` to select all topics
#'    without those concepts. This is useful to capture topics not otherwise
#'    captured by `has_concepts()`.
#' * Topics from other installed packages, e.g. `rlang::is_installed()` (function name)
#'  or `sass::font_face` (topic name).
#' * `has_lifecycle("deprecated")` will select all topics with lifecycle 
#'   deprecated.
#'
#' All functions (except for `has_keyword()`) automatically exclude internal
#' topics (i.e. those with `\keyword{internal}`). You can choose to include
#' with (e.g.) `starts_with("build_", internal = TRUE)`.
#'
#' Use a leading `-` to remove topics from a section, e.g. `-topic_name`,
#' `-starts_with("foo")`.
#'
#' pkgdown will check that all non-internal topics are included on
#' the reference index page, and error if you have missed any.
#'
#' ## Missing topics
#'
#' pkgdown will warn if there are (non-internal) topics that not listed
#' in the reference index. You can suppress these warnings by listing the
#' topics in section with "title: internal" (case sensitive) which will not be
#' displayed on the reference index.
#'
#' ## Icons
#' You can optionally supply an icon for each help topic. To do so, you'll need
#' a top-level `icons` directory. This should contain `.png` files that are
#' either 30x30 (for regular display) or 60x60 (if you want retina display).
#' Icons are matched to topics by aliases.
#'
#' # Examples
#'
#' If you need to run extra code before or after all examples are run, you
#' can create `pkgdown/pre-reference.R` and `pkgdown/post-reference.R`.
#'
#' # Figures
#'
#' You can control the default rendering of figures by specifying the `figures`
#' field in `_pkgdown.yml`. The default settings are equivalent to:
#'
#' ```yaml
#' figures:
#'   dev: ragg::agg_png
#'   dpi: 96
#'   dev.args: []
#'   fig.ext: png
#'   fig.width: 7.2916667
#'   fig.height: ~
#'   fig.retina: 2
#'   fig.asp: 1.618
#'   bg: NA
#'   other.parameters: []
#' ```
#'
#' Most of these parameters are interpreted similarly to knitr chunk
#' options. `other.parameters` is a list of parameters
#' that will be available to custom graphics output devices such
#' as HTML widgets.
#'
#' @inheritParams build_articles
#' @family site components
#' @param lazy If `TRUE`, only rebuild pages where the `.Rd`
#'   is more recent than the `.html`. This makes it much easier to
#'   rapidly prototype. It is set to `FALSE` by [build_site()].
#' @param run_dont_run Run examples that are surrounded in \\dontrun?
#' @param examples Run examples?
#' @param devel Determines how code is loaded in order to run examples.
#'   If `TRUE` (the default), assumes you are in a live development
#'   environment, and loads source package with [pkgload::load_all()].
#'   If `FALSE`, uses the installed version of the package.
#' @param topics Build only specified topics. If supplied, sets `lazy`
#'   and `preview` to `FALSE`.
#' @export
build_reference <- function(pkg = ".",
                            lazy = TRUE,
                            examples = TRUE,
                            run_dont_run = FALSE,
                            seed = 1014L,
                            override = list(),
                            preview = FALSE,
                            devel = TRUE,
                            topics = NULL) {
  pkg <- section_init(pkg, "reference", override = override)
  check_bool(lazy)
  check_bool(examples)
  check_bool(run_dont_run)
  check_number_whole(seed, allow_null = TRUE)
  check_bool(devel)
  check_character(topics, allow_null = TRUE)

  cli::cli_rule("Building function reference")
  build_reference_index(pkg)

  copy_figures(pkg)

  if (examples) {
    examples_env <- examples_env(pkg, seed = seed, devel = devel)
  } else {
    examples_env <- NULL
  }

  if (!is.null(topics)) {
    topics <- purrr::transpose(pkg$topics[pkg$topics$name %in% topics, ])
    lazy <- FALSE
    preview <- FALSE
  } else {
    topics <- purrr::transpose(pkg$topics)
  }

  unwrap_purrr_error(purrr::map(
    topics,
    build_reference_topic,
    pkg = pkg,
    lazy = lazy,
    examples_env = examples_env,
    run_dont_run = run_dont_run
  ))

  preview_site(pkg, "reference", preview = preview)
}

copy_figures <- function(pkg) {
  # copy everything from man/figures to docs/reference/figures
  dir_copy_to(
    src_dir = path(pkg$src_path, "man", "figures"),
    src_root = pkg$src_path,
    dst_dir = path(pkg$dst_path, "reference", "figures"),
    dst_root = pkg$dst_path
  )
}

examples_env <- function(pkg, seed = 1014L, devel = TRUE, envir = parent.frame()) {
  # Re-loading pkgdown while it's running causes weird behaviour with
  # the context cache
  if (isTRUE(devel) && !(pkg$package %in% c("pkgdown", "rprojroot"))) {
    check_installed("pkgload", "to use `build_reference(devel = TRUE)`")
    pkgload::load_all(pkg$src_path, export_all = FALSE, helpers = FALSE, quiet = TRUE)
  } else {
    library(pkg$package, character.only = TRUE)
  }

  # Need to compute before changing working directory
  pre_path <- path_abs(path(pkg$src_path, "pkgdown", "pre-reference.R"))
  post_path <- path_abs(path(pkg$src_path, "pkgdown", "post-reference.R"))

  withr::local_dir(path(pkg$dst_path, "reference"), .local_envir = envir)
  width <- config_pluck_number_whole(pkg, "code.width", default = 80)
  withr::local_options(width = width, .local_envir = envir)
  withr::local_seed(seed, .local_envir = envir)
  if (requireNamespace("htmlwidgets", quietly = TRUE)) {
    htmlwidgets::setWidgetIdSeed(seed)
  }

  examples_env <- child_env(globalenv())
  if (file_exists(pre_path)) {
    sys.source(pre_path, envir = examples_env)
  }
  if (file_exists(post_path)) {
    withr::defer(sys.source(post_path, envir = examples_env), envir = envir)
  }

  examples_env
}

#' @export
#' @rdname build_reference
build_reference_index <- function(pkg = ".", override = list()) {
  pkg <- section_init(pkg, "reference", override = override)

  # Copy icons, if needed
  dir_copy_to(
    src_dir = path(pkg$src_path, "icons"),
    src_root = pkg$src_path,
    dst_dir = path(pkg$dst_path, "reference", "icons"),
    dst_root = pkg$dst_path
  )

  render_page(
    pkg, "reference-index",
    data = data_reference_index(pkg),
    path = "reference/index.html"
  )

  invisible()
}


build_reference_topic <- function(topic,
                                  pkg,
                                  lazy = TRUE,
                                  examples_env = globalenv(),
                                  run_dont_run = FALSE) {

  in_path <- path(pkg$src_path, "man", topic$file_in)
  out_path <- path(pkg$dst_path, "reference", topic$file_out)

  if (lazy && !out_of_date(in_path, out_path))
    return(invisible())

  cli::cli_inform("Reading {src_path(path('man', topic$file_in))}")

  data <- withCallingHandlers(
    data_reference_topic(
      topic,
      pkg,
      examples_env = examples_env,
      run_dont_run = run_dont_run
    ),
    error = function(err) {
      cli::cli_abort(
        "Failed to parse Rd in {.file {topic$file_in}}",
        parent = err,
        call = quote(build_reference())
      )
    }
  )

  deps <- data$dependencies
  data$has_deps <- !is.null(deps)
  if (data$has_deps) {
    deps <- bs_theme_deps_suppress(deps)
    deps <- htmltools::resolveDependencies(deps)
    deps <- purrr::map(
      deps,
      htmltools::copyDependencyToDir,
      outputDir = path(pkg$dst_path, "reference", "libs"),
      mustWork = FALSE
    )
    deps <- purrr::map(
      deps,
      htmltools::makeDependencyRelative,
      basepath = path(pkg$dst_path, "reference"),
      mustWork = FALSE
    )
    data$dependencies <- htmltools::renderDependencies(deps, c("file", "href"))
  }

  render_page(
    pkg, "reference-topic",
    data = data,
    path = path("reference", topic$file_out)
  )

  invisible()
}


# Convert Rd to list ------------------------------------------------------

data_reference_topic <- function(topic,
                                 pkg,
                                 examples_env = globalenv(),
                                 run_dont_run = FALSE) {

  local_context_eval(pkg$figures, pkg$src_path)
  withr::local_options(list(downlit.rdname = get_rdname(topic)))

  tag_names <- purrr::map_chr(topic$rd, ~ class(.)[[1]])
  tags <- split(topic$rd, tag_names)

  out <- list()

  # Single top-level converted to string
  out$name <- flatten_text(tags$tag_name[[1]][[1]])
  out$title <- extract_title(tags$tag_title)
  out$pagetitle <- paste0(strip_html_tags(out$title), " \u2014 ", out$name)

  # File source
  out$source <- repo_source(pkg, topic$source)
  out$filename <- topic$file_in

  # Multiple top-level converted to string
  out$author <- purrr::map_chr(tags$tag_author %||% list(), flatten_para)
  out$aliases <- purrr::map_chr(tags$tag_alias %||% list(), flatten_text)
  out$keywords <- purrr::map_chr(tags$tag_keyword %||% list(), flatten_text)

  # Sections that contain arbitrary text and need cross-referencing
  out$description <- as_data(tags$tag_description[[1]])
  out$opengraph <- list(description = strip_html_tags(out$description$contents))
  if (length(tags$tag_usage[[1]])) {
    out$usage <- list(
      title = tr_("Usage"),
      contents = as_data(tags$tag_usage[[1]])
    )
  }


  if (!is.null(tags$tag_examples)) {
    out$examples <- run_examples(
      tags$tag_examples[[1]],
      env = if (is.null(examples_env)) NULL else new.env(parent = examples_env),
      topic = tools::file_path_sans_ext(topic$file_in),
      run_dont_run = run_dont_run
    )
    deps <- attr(out$examples, "dependencies")
    if (!is.null(deps)) {
      attr(out$examples, "dependencies") <- NULL
      out$dependencies <- deps
    }
  }

  # Everything else stays in original order, and becomes a list of sections.
  section_tags <- paste0("tag_", c(
    "arguments", "value", "details", "references", "source", "format", "note",
    "seealso", "section", "author"
  ))
  sections <- topic$rd[tag_names %in% section_tags]
  out$sections <- purrr::map(sections, function(section) {
    data <- as_data(section)
    data$slug <- make_slug(data$title)
    data
  })
  out
}

make_slug <- function(x) {
  x <- strip_html_tags(x)
  x <- tolower(x)
  x <- gsub("[^a-z]+", "-", x)
  x
}

get_rdname <- function(topics) {
  gsub("\\.[Rr]d$", "", topics$file_in)
}

test_that("parse failures include file name", {
  pkg <- local_pkgdown_site(test_path("assets/reference-fail"))
  expect_snapshot(build_reference(pkg), error = TRUE)
})

test_that("examples_env runs pre and post code", {
  dst_path <- withr::local_tempdir()
  dir_create(path(dst_path, "reference"))

  pkg <- list(
    package = "test",
    src_path = test_path("assets/reference-pre-post"),
    dst_path = dst_path
  )

  env <- local(examples_env(pkg))
  expect_equal(env$a, 2)
})

test_that("examples_env sets width", {
  pkg <- local_pkgdown_site(
    test_path("assets/reference"),
    list(code = list(width = 50))
  )
  dir_create(path(pkg$dst_path, "reference"))

  examples_env(pkg)
  expect_equal(getOption("width"), 50)
})

test_that("test usage ok on rendered page", {
  pkg <- local_pkgdown_site(
    test_path("assets/reference"),
    list(template = list(bootstrap = 3))
  )
  suppressMessages(init_site(pkg))
  suppressMessages(build_reference(pkg, topics = "c"))

  html <- xml2::read_html(path(pkg$dst_path, "reference", "c.html"))
  expect_equal(xpath_text(html, "//div[@id='ref-usage']", trim = TRUE), "c()")
  clean_site(pkg, quiet = TRUE)

  pkg <- local_pkgdown_site(test_path("assets/reference"))
  suppressMessages(init_site(pkg))
  suppressMessages(build_reference(pkg, topics = "c"))
  html <- xml2::read_html(path(pkg$dst_path, "reference", "c.html"))
  # tweak_anchors() moves id into <h2>
  expect_equal(xpath_text(html, "//div[h2[@id='ref-usage']]/div", trim = TRUE), "c()")
})

test_that(".Rd without usage doesn't get Usage section", {
  pkg <- local_pkgdown_site(
    test_path("assets/reference"),
    list(template = list(bootstrap = 3))
  )
  suppressMessages(init_site(pkg))
  suppressMessages(build_reference(pkg, topics = "e"))

  html <- xml2::read_html(path(pkg$dst_path, "reference", "e.html"))
  expect_equal(xpath_length(html, "//div[@id='ref-usage']"), 0)
  clean_site(pkg, quiet = TRUE)

  pkg <- local_pkgdown_site(test_path("assets/reference"))
  suppressMessages(init_site(pkg))
  suppressMessages(build_reference(pkg, topics = "e"))
  html <- xml2::read_html(path(pkg$dst_path, "reference", "e.html"))
  # tweak_anchors() moves id into <h2>
  expect_equal(xpath_length(html, "//div[h2[@id='ref-usage']]"), 0)
})

test_that("pkgdown html dependencies are suppressed from examples in references", {
  pkg <- local_pkgdown_site(test_path("assets/reference-html-dep"))
  suppressMessages(init_site(pkg))
  suppressMessages(build_reference(pkg, topics = "a"))
  html <- xml2::read_html(path(pkg$dst_path, "reference", "a.html"))

  # jquery is only loaded once, even though it's included by an example
  expect_equal(xpath_length(html, ".//script[(@src and contains(@src, '/jquery'))]"), 1)

  # same for bootstrap js and css
  str_subset_bootstrap <- function(x) {
    bs_rgx <- "bootstrap-[\\d.]+" # ex: bootstrap-5.1.0 not bootstrap-toc,
    grep(bs_rgx, x, value = TRUE, perl = TRUE)
  }
  bs_js_src <- str_subset_bootstrap(
    xpath_attr(html, ".//script[(@src and contains(@src, '/bootstrap'))]", "src")
  )
  expect_length(bs_js_src, 1)

  bs_css_href <- str_subset_bootstrap(
    xpath_attr(html, ".//link[(@href and contains(@href, '/bootstrap'))]", "href")
  )
  expect_length(bs_css_href, 1)
})

test_that("examples are reproducible by default, i.e. 'seed' is respected", {
  pkg <- local_pkgdown_site(test_path("assets/reference"))
  suppressMessages(init_site(pkg))
  suppressMessages(build_reference(pkg, topics = "f"))

  html <- xml2::read_html(path(pkg$dst_path, "reference", "f.html"))
  examples <- xpath_text(html, ".//code//*[contains(@class, 'r-out')]")

  expect_snapshot(cat(examples))
})

test_that("arguments get individual ids", {
  pkg <- local_pkgdown_site(test_path("assets/reference"))
  suppressMessages(init_site(pkg))
  suppressMessages(build_reference(pkg, topics = "a"))

  html <- xml2::read_html(path(pkg$dst_path, "reference", "a.html"))
  expect_equal(xpath_attr(html, "//dt", "id"), c("arg-a", "arg-b", "arg-c"))

})

test_that("title and page title escapes html", {
  pkg <- local_pkgdown_site(test_path("assets/reference"))
  suppressMessages(init_site(pkg))
  suppressMessages(build_reference(pkg, topics = "g"))

  html <- xml2::read_html(path(pkg$dst_path, "reference", "g.html"))
  expect_equal(xpath_text(html, "//title", trim = TRUE), "g <-> h — g • testpackage")
  expect_equal(xpath_text(html, "//h1", trim = TRUE), "g <-> h")
})

test_that("get_rdname handles edge cases", {
  expect_equal(get_rdname(list(file_in = "foo..Rd")), "foo.")
  expect_equal(get_rdname(list(file_in = "foo.rd")), "foo")
})

#' Clean site
#'
#' Delete all files in `docs/` (except for `CNAME`).
#'
#' @param quiet If `TRUE`, suppresses a message.
#' @param force If `TRUE`, delete contents of `docs` even if it is not a pkgdown site.
#' @inheritParams build_site
#' @rdname clean
#' @export
clean_site <- function(pkg = ".", quiet = FALSE, force = FALSE) {
  pkg <- as_pkgdown(pkg)

  if (!quiet) {
    cli::cli_inform(
      "Cleaning {.pkg {pkg$package}} pkgdown docs from {.path {pkg$dst_path}}"
    )
  }

  if (!dir_exists(pkg$dst_path)) {
    return(invisible())
  }

  top_level <- dest_files(pkg)
  if (length(top_level) > 0 && !force) {
    check_dest_is_pkgdown(pkg)
  }

  is_dir <- is_dir(top_level)
  dir_delete(top_level[is_dir])
  file_delete(top_level[!is_dir])

  invisible(TRUE)
}

#' Clean cache
#'
#' Delete all files in the pkgdown cache directory.
#'
#' @rdname clean
#' @export
clean_cache <- function(pkg = ".", quiet = FALSE) {
  pkg <- as_pkgdown(pkg)
  cache_path <- tools::R_user_dir("pkgdown", "cache")

  if (dir_exists(cache_path)) {
    if (!quiet) {
      cli::cli_inform(
        "Cleaning {.pkg {pkg$package}} cache files from {.path {cache_path}}"
      )
    }

    dir_delete(cache_path)
  }

  invisible(TRUE)
}

check_dest_is_pkgdown <- function(pkg) {
  if (file_exists(path(pkg$dst_path, "pkgdown.yml"))) {
    return()
  }

  cli::cli_abort(c(
    "{.file {pkg$dst_path}} is non-empty and not built by pkgdown",
    "!" = "Make sure it contains no important information \\
            and use {.run pkgdown::clean_site(force = TRUE)} to delete its contents."
  ))
}

dest_files <- function(pkg) {
  if (!dir_exists(pkg$dst_path)) {
    character()
  } else {
    top_level <- dir_ls(pkg$dst_path)
    top_level[!path_file(top_level) %in% c("CNAME", "dev")]
  }
}

# Tabsets tweaking: find Markdown recommended in
# https://bookdown.org/yihui/rmarkdown-cookbook/html-tabs.html
# and https://bookdown.org/yihui/rmarkdown/html-document.html#tabbed-sections
# i.e. "## Heading {.tabset}" or "## Heading {.tabset .tabset-pills}"
#  no matter the heading level -- the headings one level down are the tabs
# and transform to tabsets HTML a la Bootstrap

tweak_tabsets <- function(html) {
  tabsets <- xml2::xml_find_all(html, ".//div[contains(@class, 'tabset')]")
  purrr::walk(tabsets, tweak_tabset)
  invisible()
}

tweak_tabset <- function(div) {
  # Get tabs and remove them from original HTML
  tabs <- xml2::xml_find_all(div, "div")
  xml2::xml_remove(tabs)

  # Add empty ul for nav and div for content
  nav_class <- if (has_class(div, "tabset-pills")) {
    "nav nav-pills"
  } else {
    "nav nav-tabs"
  }
  fade <- has_class(div, "tabset-fade")

  id <- section_id(div)
  nav <- xml2::xml_add_child(
    div,
    "ul",
    class = nav_class,
    id = id,
    role = "tablist"
  )
  content <- xml2::xml_add_child(div, "div", class = "tab-content")

  # Fill the ul for nav and div for content
  purrr::walk(tabs, tablist_item, nav = nav, parent_id = id)
  purrr::walk(
    tabs,
    tablist_content,
    content = content,
    parent_id = id,
    fade = fade
  )

  # if not tabs active, activate the first tab
  if (!any(has_class(xml2::xml_children(content), "active"))) {
    first_tab <- xml2::xml_find_first(nav, ".//li/button")
    tweak_class_prepend(first_tab, "active")
    xml2::xml_attr(first_tab, "aria-selected") <- "true"

    tab_class <- paste("active", if (has_class(div, "tabset-fade")) "show")
    tweak_class_prepend(xml2::xml_child(content), tab_class)
  }
}

# Add an item (tab) to the tablist
tablist_item <- function(tab, nav, parent_id) {
  id <- section_id(tab)
  title <- tablist_title(tab)

  # Activate (if there was "{.active}" in the source Rmd)
  active <- has_class(tab, "active")
  li_class <- paste0("nav-link", if (active) " active")
  li <- xml2::xml_add_child(
    nav,
    "li",
    role = "presentation",
    class = "nav-item"
  )
  button <- xml2::xml_add_child(
    li,
    "button",
    `data-bs-toggle` = "tab",
    `data-bs-target` = paste0("#", id),
    id = paste0(id, "-tab"),
    type = "button",
    role = "tab",
    `aria-controls` = id,
    `aria-selected` = tolower(active),
    class = li_class
  )

  # Preserve html in title by adding from xml_nodeset item by item
  for (title_item in title) {
    xml2::xml_add_child(button, title_item)
  }

  invisible()
}

tablist_title <- function(tab) {
  # remove anchor link from tab heading
  tab_heading_anchor <- xml2::xml_find_first(tab, ".//a[@class = 'anchor']")
  xml2::xml_remove(tab_heading_anchor)

  xml2::xml_contents(xml2::xml_child(tab))
}

# Add content of a tab to a tabset
tablist_content <- function(tab, content, parent_id, fade) {
  id <- section_id(tab)
  # remove the header, the first child
  xml2::xml_remove(xml2::xml_child(tab))

  xml2::xml_attr(tab, "id") <- id

  # Activate (if there was "{.active}" in the source Rmd)
  active <- has_class(tab, "active")
  tab_class <- c(
    if (fade && active) "show",
    if (active) "active",
    if (fade) "fade",
    "tab-pane"
  )
  xml2::xml_attr(tab, "class") <- paste(tab_class, collapse = " ")

  xml2::xml_attr(tab, "role") <- "tabpanel"
  xml2::xml_attr(tab, "aria-labelledby") <- paste0(id, "-tab")

  xml2::xml_add_child(content, tab)

  invisible()
}

dir_copy_to <- function(src_dir,
                        dst_dir,
                        src_root,
                        dst_root,
                        src_label = "",
                        dst_label = "") {
  check_string(src_dir)
  check_string(dst_dir)

  if (!dir_exists(src_dir)) {
    return()
  }

  src_paths <- dir_ls(src_dir, recurse = TRUE)
  is_dir <- is_dir(src_paths)

  dst_paths <- path(dst_dir, path_rel(src_paths, src_dir))

  # First create directories
  dir_create(dst_paths[is_dir])
  # Then copy files
  file_copy_to(
    src_paths = src_paths[!is_dir],
    dst_paths = dst_paths[!is_dir],
    src_root = src_root,
    dst_root = dst_root,
    src_label = src_label,
    dst_label = dst_label
  )
}

file_copy_to <- function(src_paths,
                         dst_paths,
                         src_root,
                         dst_root,
                         src_label = NULL,
                         dst_label = "") {
  # Ensure all the "to" directories exist
  dst_dirs <- unique(path_dir(dst_paths))
  dir_create(dst_dirs)

  eq <- purrr::map2_lgl(src_paths, dst_paths, file_equal)
  if (any(!eq)) {
    dst <- paste0(dst_label, path_rel(dst_paths[!eq], dst_root))
    if (is.null(src_label)) {
      purrr::walk(dst, function(dst) {
        cli::cli_inform("Copying {dst_path(dst)}")
      })
    } else {
      src <- paste0(src_label, path_rel(src_paths[!eq], src_root))
      purrr::walk2(src, dst, function(src, dst) {
        cli::cli_inform("Copying {src_path(src)} to {dst_path(dst)}")
      }) 
    }
  }

  file_copy(src_paths[!eq], dst_paths[!eq], overwrite = TRUE)
}

out_of_date <- function(source, target, call = caller_env()) {
  if (!file_exists(target)) {
    return(TRUE)
  }
  if (!file_exists(source)) {
    cli::cli_abort("{.path {source}} does not exist", call = call)
  }

  file_info(source)$modification_time > file_info(target)$modification_time
}

# Path helpers ------------------------------------------------------------

path_abs <- function(path, start = ".") {
  is_abs <- is_absolute_path(path)

  path[is_abs] <- path_norm(path[is_abs])
  path[!is_abs] <- fs::path_abs(path(start, path))

  path_tidy(path)
}

path_first_existing <- function(...) {
  paths <- path(...)
  for (path in paths) {
    if (file_exists(path))
      return(path)
  }

  NULL
}

path_package_pkgdown <- function(path,
                                 package,
                                 bs_version,
                                 error_call = caller_env()) {
  # package will usually be a github package, and check_installed()
  # tries to install from CRAN, which is highly likely to fail.
  if (!is_installed(package)) {
    cli::cli_abort(
      c(
        "Template package {.val {package}} is not installed.",
        i = "Please install before continuing."
      ),
      call = error_call
    )
  }
  base <- system_file("pkgdown", package = package)

  # If bs_version supplied, first try for versioned template
  if (!is.null(bs_version)) {
    ver_path <- path(base, paste0("BS", bs_version), path)
    if (file_exists(ver_path)) {
      return(ver_path)
    }
  }

  path(base, path)
}

path_pkgdown <- function(...) {
  system_file(..., package = "pkgdown")
}

test_that("missing template package yields custom error", {
  expect_snapshot(path_package_pkgdown("x", "missing", 3), error = TRUE)
})


test_that("out_of_date works as expected", {
  temp1 <- file_create(withr::local_tempfile())
  expect_true(out_of_date(temp1, "doesntexist"))
  expect_snapshot(out_of_date("doesntexist", temp1), error = TRUE)

  temp2 <- file_create(withr::local_tempfile())
  file_touch(temp2, Sys.time() + 10)

  expect_true(out_of_date(temp2, temp1))
  expect_false(out_of_date(temp1, temp2))
  expect_false(out_of_date(temp1, temp1))
})

# nocov start --- r-lib/rlang compat-friendly-type
#
# Changelog
# =========
#
# 2021-04-19:
# - Added support for matrices and arrays (#141).
# - Added documentation.
# - Added changelog.


#' Return English-friendly type
#' @param x Any R object.
#' @param length Whether to mention the length of vectors and lists.
#' @return A string describing the type. Starts with an indefinite
#'   article, e.g. "an integer vector".
#' @noRd
friendly_type_of <- function(x, length = FALSE) {
  if (is.object(x)) {
    if (inherits(x, "quosure")) {
      type <- "quosure"
    } else {
      type <- paste(class(x), collapse = "/")
    }
    return(sprintf("a <%s> object", type))
  }

  if (!rlang::is_vector(x)) {
    return(.rlang_as_friendly_type(typeof(x)))
  }

  n_dim <- length(dim(x))
  type <- .rlang_as_friendly_vector_type(typeof(x), n_dim)

  if (length && !n_dim) {
    type <- paste0(type, sprintf(" of length %s", length(x)))
  }

  type
}

.rlang_as_friendly_vector_type <- function(type, n_dim) {
  if (type == "list") {
    if (n_dim < 2) {
      return("a list")
    } else if (n_dim == 2) {
      return("a list matrix")
    } else {
      return("a list array")
    }
  }

  type <- switch(
    type,
    logical = "a logical %s",
    integer = "an integer %s",
    numeric = ,
    double = "a double %s",
    complex = "a complex %s",
    character = "a character %s",
    raw = "a raw %s",
    type = paste0("a ", type, " %s")
  )

  if (n_dim < 2) {
    kind <- "vector"
  } else if (n_dim == 2) {
    kind <- "matrix"
  } else {
    kind <- "array"
  }
  sprintf(type, kind)
}

.rlang_as_friendly_type <- function(type) {
  switch(
    type,

    list = "a list",

    NULL = "NULL",
    environment = "an environment",
    externalptr = "a pointer",
    weakref = "a weak reference",
    S4 = "an S4 object",

    name = ,
    symbol = "a symbol",
    language = "a call",
    pairlist = "a pairlist node",
    expression = "an expression vector",

    char = "an internal string",
    promise = "an internal promise",
    ... = "an internal dots object",
    any = "an internal `any` object",
    bytecode = "an internal bytecode object",

    primitive = ,
    builtin = ,
    special = "a primitive function",
    closure = "a function",

    type
  )
}

# nocov end

#' Render page with template
#'
#' Each page is composed of four templates: "head", "header", "content", and
#' "footer". Each of these templates is rendered using the `data`, and
#' then assembled into an overall page using the "layout" template.
#'
#' @param pkg Path to package to document.
#' @param name Name of the template (e.g. "home", "vignette", "news")
#' @param data Data for the template.
#'
#'   This is automatically supplemented with three lists:
#'
#'   * `site`: `title` and path to `root`.
#'   * `yaml`: the `template` key from `_pkgdown.yml`.
#'   * `package`: package metadata including `name` and`version`.
#'
#'   See the full contents by running [data_template()].
#' @param path Location to create file; relative to destination directory.
#' @param depth Depth of path relative to base directory.
#' @param quiet If `quiet`, will suppress output messages
#' @export
render_page <- function(pkg = ".", name, data, path, depth = NULL, quiet = FALSE) {
  pkg <- as_pkgdown(pkg)

  if (is.null(depth)) {
    depth <- length(strsplit(path, "/")[[1]]) - 1L
  }

  html <- render_page_html(pkg, name = name, data = data, depth = depth)

  tweak_page(html, name, pkg = pkg)
  if (pkg$bs_version > 3) {
    activate_navbar(html, data$output_file %||% path, pkg)
  }

  rendered <- as.character(html, options = character())
  write_if_different(pkg, rendered, path, quiet = quiet)
}

render_page_html <- function(pkg, name, data = list(), depth = 0L) {
  data <- utils::modifyList(data_template(pkg, depth = depth), data)

  # render template components
  pieces <- c(
    "head",
    "in-header",
    "before-body",
    "navbar",
    "content",
    "footer",
    "after-body",
    if (pkg$bs_version == 3) c("header", "docsearch")
  )

  templates <- purrr::map_chr(pieces, find_template, name = name, pkg = pkg)
  components <- purrr::map(templates, render_template, data = data)
  components <- purrr::set_names(components, pieces)
  components$template <- name
  components$lang <- pkg$lang
  components$translate <- data$translate

  # render complete layout
  template <- find_template("layout", name, pkg = pkg)
  rendered <- render_template(template, components)

  xml2::read_html(rendered, encoding = "UTF-8")
}

#' @export
#' @rdname render_page
data_template <- function(pkg = ".", depth = 0L) {
  pkg <- as_pkgdown(pkg)
  out <- list()

  # Basic metadata
  out$package <- list(
    name = pkg$package,
    version = as.character(pkg$version)
  )
  out$logo <- list(src = logo_path(pkg, depth = depth))
  out$site <- list(
    root = up_path(depth),
    title = pkg$meta$title %||% pkg$package
  )
  out$year <- strftime(Sys.time(), "%Y")

  # Language and translations
  out$lang <- pkg$lang
  out$translate <- list(
    skip = tr_("Skip to contents"),
    toggle_nav = tr_("Toggle navigation"),
    search_for = tr_("Search for"),
    on_this_page = tr_("On this page"),
    source = tr_("Source"),
    abstract = tr_("Abstract"),
    authors = tr_("Authors"),
    version = tr_("Version"),
    examples = tr_("Examples"),
    citation = tr_("Citation")
  )

  # Components that mostly end up in the <head>
  out$has_favicons <- has_favicons(pkg)
  out$opengraph <- data_open_graph(pkg)
  out$extra <- list(
    css = path_first_existing(pkg$src_path, "pkgdown", "extra.css"),
    js = path_first_existing(pkg$src_path, "pkgdown", "extra.js")
  )
  out$includes <- purrr::pluck(pkg, "meta", "template", "includes", .default = list())
  out$yaml <- purrr::pluck(pkg, "meta", "template", "params", .default = list())
  # Force inclusion so you can reliably refer to objects inside yaml
  # in the mustache templates
  out$yaml$.present <- TRUE
  if (pkg$bs_version > 3) {
    out$headdeps <- data_deps(pkg = pkg, depth = depth)
  }

  # Development settings; tooltip needs to be generated at render time
  out$development <- pkg$development
  out$development$version_tooltip <- version_tooltip(pkg$development$mode)

  out$navbar <- data_navbar(pkg, depth = depth)
  out$footer <- data_footer(pkg)

  print_yaml(out)
}

data_open_graph <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)
  og <- pkg$meta$template$opengraph %||% list()
  og <- check_open_graph(og)
  if (is.null(og$image) && !is.null(find_logo(pkg$src_path))) {
    og$image <- list(src = path_file(find_logo(pkg$src_path)))
  }
  if (!is.null(og$image) && !grepl("^http", og$image$src)) {
    site_url <- pkg$meta$url %||% "/"
    if (!grepl("/$", site_url)) {
      site_url <- paste0(site_url, "/")
    }
    og$image$src <- gsub("^man/figures/", "reference/figures/", og$image$src)
    og$image$src <- paste0(site_url, og$image$src)
  }

  og$twitter$creator <- og$twitter$creator %||% og$twitter$site
  og$twitter$site <- og$twitter$site %||% og$twitter$creator
  og
}

check_open_graph <- function(og) {
  if (!is.list(og)) {
    abort(paste("`opengraph` must be a list, not", friendly_type_of(og)))
  }
  supported_fields <- c("image", "twitter")
  unsupported_fields <- setdiff(names(og), supported_fields)
  if (length(unsupported_fields)) {
    warn(paste0(
      "Unsupported `opengraph` field(s): ",
      paste(unsupported_fields, collapse = ", ")
    ))
  }
  if ("twitter" %in% names(og)) {
    if (is.character(og$twitter) && length(og$twitter) == 1 && grepl("^@", og$twitter)) {
      abort(paste(
        "The `opengraph: twitter` option must be a list. Did you mean this?",
        "opengraph:",
        "  twitter:",
        paste("    creator:", og$twitter),
        sep = "\n"
      ))
    }
    if (!is.list(og$twitter)) {
      abort("The `opengraph: twitter` option must be a list.")
    }
    if (is.null(og$twitter$creator) && is.null(og$twitter$site)) {
      abort(
        "The `opengraph: twitter` option must include either 'creator' or 'site'."
      )
    }
  }
  if ("image" %in% names(og)) {
    if (is.character(og$image) && length(og$image) == 1) {
      abort(paste(
        "The `opengraph: image` option must be a list. Did you mean this?",
        "opengraph",
        "  image:",
        paste("    src:", og$image),
        sep = "\n"
      ))
    }
    if (!is.list(og$image)) {
      abort("The `opengraph: image` option must be a list.")
    }
  }
  og[intersect(supported_fields, names(og))]
}

render_template <- function(path, data) {
  template <- read_file(path)
  if (length(template) == 0)
    return("")

  whisker::whisker.render(template, data)
}

write_if_different <- function(pkg, contents, path, quiet = FALSE, check = TRUE) {
  # Almost all uses are relative to destination, except for rmarkdown templates
  full_path <- path_abs(path, start = pkg$dst_path)

  if (check && !made_by_pkgdown(full_path)) {
    if (!quiet) {
      message("Skipping '", path, "': not generated by pkgdown")
    }
    return(FALSE)
  }

  if (same_contents(full_path, contents)) {
    # touching the file to update its modification time
    # which is important for proper lazy behavior
    fs::file_touch(full_path)
    return(FALSE)
  }

  if (!quiet) {
    href <- paste0("ide:run:pkgdown::preview_page('", path_rel(full_path, pkg$dst_path), "')")
    cat_line("Writing ", cli::style_hyperlink(dst_path(path), href))
  }
  write_lines(contents, path = full_path)
  TRUE
}

same_contents <- function(path, contents) {
  if (!file_exists(path))
    return(FALSE)

  new_hash <- digest::digest(contents, serialize = FALSE)

  cur_contents <- paste0(read_lines(path), collapse = "\n")
  cur_hash <-  digest::digest(cur_contents, serialize = FALSE)

  identical(new_hash, cur_hash)
}

file_digest <- function(path) {
  if (file_exists(path)) {
    digest::digest(file = path, algo = "xxhash64")
  } else {
    "MISSING"
  }
}


made_by_pkgdown <- function(path) {
  if (!file_exists(path)) return(TRUE)

  first <- paste(read_lines(path, n = 2), collapse = "\n")
  check_made_by(first)
}

check_made_by <- function(first) {
  if (length(first) == 0L) return(FALSE)
  grepl("<!-- Generated by pkgdown", first, fixed = TRUE)
}

test_that("capture data_template()", {
  pkg <- local_pkgdown_site()
  data <- data_template(pkg)
  data$year <- "<year>"
  data$footer$right <- gsub(packageVersion("pkgdown"), "{version}", data$footer$right, fixed = TRUE)
  expect_snapshot_output(data)
})

test_that("can include text in header, before body, and after body", {
  pkg <- local_pkgdown_site(meta = list(
    template = list(
      includes = list(
        in_header = "<test>in header</test>",
        before_body = "<test>before body</test>",
        after_body = "<test>after body</test>"
      )
    )
  ))

  expect_named(
    data_template(pkg)$includes,
    c("in_header", "before_body", "after_body")
  )

  pkg$bs_version <- 3
  html <- render_page_html(pkg, "title-body")
  expect_equal(
    xpath_text(html, ".//test"),
    c("in header", "before body", "after body")
  )

  pkg$bs_version <- 5
  suppressMessages(init_site(pkg))
  html <- render_page_html(pkg, "title-body")
  expect_equal(
    xpath_text(html, ".//test"),
    c("in header", "before body", "after body")
  )
})

test_that("check_opengraph validates inputs", {
  data_open_graph_ <- function(x) {
    pkg <- local_pkgdown_site(meta = list(template = list(opengraph = x)))
    data_open_graph(pkg)
    invisible()
  }

  expect_snapshot(error = TRUE, {
    data_open_graph_(list(foo = list()))
    data_open_graph_(list(foo = list(), bar = list()))
    data_open_graph_(list(twitter = 1))
    data_open_graph_(list(twitter = list()))
    data_open_graph_(list(image = 1))
  })
})

#' Build redirects
#'
#' @description
#' If you change the structure of your documentation (by renaming vignettes or
#' help topics) you can setup redirects from the old content to the new content.
#' One or several now-absent pages can be redirected to a new page (or to a new
#' section of a new page). This works by creating a html page that performs a
#' "meta refresh", which isn't the best way of doing a redirect but works
#' everywhere that you might deploy your site.
#'
#' The syntax is the following, with old paths on the left, and new paths or
#' URLs on the right.
#'
#' ```yaml
#' redirects:
#'   - ["articles/old-vignette-name.html", "articles/new-vignette-name.html"]
#'   - ["articles/another-old-vignette-name.html", "articles/new-vignette-name.html"]
#'   - ["articles/yet-another-old-vignette-name.html", "https://pkgdown.r-lib.org/dev"]
#' ```
#'
#' If for some reason you choose to redirect an existing page make sure to
#' exclude it from the search index, see `?build_search`.
#'
#' @inheritParams as_pkgdown
#' @export
build_redirects <- function(pkg = ".", override = list()) {
  pkg <- section_init(pkg, override = override)
  has_url <- !is.null(config_pluck_string(pkg, "url"))

  redirects <- data_redirects(pkg, has_url)
  if (length(redirects) == 0) {
    return(invisible())
  }

  cli::cli_rule("Building redirects")
  if (!has_url) {
    config_abort(pkg, "{.field url} is required to generate redirects.")
  }

  # Ensure user redirects override automatic ones
  from <- purrr::map_chr(redirects, 1)
  redirects <- redirects[!duplicated(from)]

  purrr::iwalk(redirects, build_redirect, pkg = pkg)
}

build_redirect <- function(entry, index, pkg) {
  new <- entry[2]
  old <- path(pkg$dst_path, entry[1])

  path <- find_template("layout", "redirect", pkg = pkg)
  template <- read_file(path)

  url <- sprintf("%s/%s%s", config_pluck_string(pkg, "url"), pkg$prefix, new)
  lines <- whisker::whisker.render(template, list(url = url))
  dir_create(path_dir(old))

  if (!file_exists(old)) {
    cli::cli_inform("Adding redirect from {entry[1]} to {entry[2]}.")
  }
  write_lines(lines, old)
}

data_redirects <- function(pkg, has_url = FALSE, call = caller_env()) {
  c(
    if (has_url) reference_redirects(pkg),
    if (has_url) article_redirects(pkg),
    config_pluck_redirects(pkg, call = call)
  )
}

reference_redirects <- function(pkg) {
  aliases <- unname(pkg$topics$alias)
  aliases <- purrr::map2(aliases, pkg$topics$name, setdiff)
  names(aliases) <- pkg$topics$file_out

  redirects <- invert_index(aliases)
  if (length(redirects) == 0) {
    return(list())
  }

  names(redirects) <- paste0(names(redirects), ".html")

  # Ensure we don't create an invalid file name
  redirects <- redirects[valid_filename(names(redirects))]

  # Ensure we don't override an existing file
  redirects <- redirects[setdiff(names(redirects), pkg$topics$file_out)]

  unname(purrr::imap(
    redirects,
    function(to, from) paste0("reference/", c(from, to))
  ))
}

valid_filename <- function(x) {
  x == path_sanitize(x)
}

article_redirects <- function(pkg) {
  is_vig_in_articles <- path_has_parent(pkg$vignettes$name, "articles")
  if (!any(is_vig_in_articles)) {
    return(NULL)
  }

  articles <- pkg$vignettes$file_out[is_vig_in_articles]
  purrr::map(articles, ~ paste0(c("articles/", ""), .x))
}

config_pluck_redirects <- function(pkg, call = caller_env()) {
  redirects <- config_pluck_list(pkg, "redirects", call = call)
  for (i in seq_along(redirects)) {
    redirect <- redirects[[i]]
    if (!is.character(redirect) || length(redirect) != 2) {
      not <- obj_type_friendly(redirect)
      config_abort(
        pkg,
        "{.field redirects[{i}]} must be a character vector of length 2, not {not}.",
        call = call
      )
    }
  }
  redirects
}

test_that("build_redirect() works", {
  pkg <- list(
    src_path = withr::local_tempdir(),
    dst_path = withr::local_tempdir(),
    meta = list(url = "https://example.com"),
    prefix = "",
    bs_version = 5
  )
  pkg <- structure(pkg, class = "pkgdown")
  expect_snapshot(
    build_redirect(c("old.html", "new.html#section"), 1, pkg = pkg)
  )

  html <- xml2::read_html(path(pkg$dst_path, "old.html"))
  expect_equal(
    xpath_attr(html, "//link", "href"),
    "https://example.com/new.html#section"
  )
})

test_that("build_redirect() errors if one entry is not right.", {
  data_redirects_ <- function(...) {
    pkg <- local_pkgdown_site(meta = list(...))
    data_redirects(pkg)
  }

  expect_snapshot(error = TRUE, {
    data_redirects_(redirects = "old.html")
    data_redirects_(redirects = list("old.html"))
  })
})

test_that("article_redirects() creates redirects for vignettes in vignettes/articles", {
  dir <- withr::local_tempdir()
  dir_create(path(dir, "vignettes", "articles"))
  file_create(path(dir, "vignettes", "articles", "test.Rmd"))

  pkg <- list(
    meta = list(url = "http://foo.com"),
    vignettes = package_vignettes(dir)
  )

  expect_equal(
    article_redirects(pkg),
    list(c("articles/articles/test.html", "articles/test.html"))
  )
})

# reference_redirects ----------------------------------------------------------

test_that("generates redirects only for non-name aliases", {
  pkg <- list(
    meta = list(url = "http://foo.com"),
    topics = list(
      alias = list("foo", c("bar", "baz")),
      name = c("foo", "bar"),
      file_out = c("foo.html", "bar.html")
    )
  )
  expect_equal(
    reference_redirects(pkg),
    list(c("reference/baz.html", "reference/bar.html"))
  )
})

test_that("doesn't generates redirect for aliases that can't be file names", {
  pkg <- list(
    meta = list(url = "http://foo.com"),
    topics = list(
      name = "bar",
      alias = list(c("bar", "baz", "[<-.baz")),
      file_out = "bar.html"
    )
  )
  expect_equal(
    reference_redirects(pkg),
    list(c("reference/baz.html", "reference/bar.html"))
  )
})

test_that("never redirects away from existing topic", {
  pkg <- list(
    meta = list(url = "http://foo.com"),
    topics = list(
      alias = list("foo", c("bar", "foo")),
      name = c("foo", "bar"),
      file_out = c("foo.html", "bar.html")
    )
  )
  expect_equal(
    reference_redirects(pkg),
    list()
  )
})

test_that("no redirects if no aliases", {
  pkg <- list(
    meta = list(url = "http://foo.com"),
    topics = list(
      alias = list(c("foo", "bar")),
      name = c("foo", "bar"),
      file_out = c("foo.html", "bar.html")
    )
  )
  expect_equal(reference_redirects(pkg), list())
})

# @return An integer vector giving selected topics
select_topics <- function(
  match_strings,
  topics,
  check = FALSE,
  error_path,
  error_pkg,
  error_call = caller_env()
) {
  n <- nrow(topics)
  if (length(match_strings) == 0) {
    return(integer())
  }

  indexes <- unwrap_purrr_error(purrr::imap(
    match_strings,
    match_eval,
    env = match_env(topics),
    error_path = error_path,
    error_pkg = error_pkg,
    error_call = error_call
  ))

  # If none of the specified topics have a match, return no topics
  if (purrr::every(indexes, is_empty)) {
    if (check) {
      msg <- "{.field {error_path}} failed to match any topics."
      config_abort(error_pkg, msg, call = error_call)
    }
    return(integer())
  }

  no_match <- match_strings[purrr::map_lgl(indexes, rlang::is_empty)]
  if (check && length(no_match) > 0) {
    topic_must(
      "match a function or concept",
      toString(no_match),
      error_pkg = error_pkg,
      error_path = error_path,
      error_call = error_call
    )
  }

  indexes <- purrr::discard(indexes, is_empty)
  # Combine integer positions; adding if +ve, removing if -ve
  sign <- all_sign(
    indexes[[1]],
    match_strings[[1]],
    error_pkg = error_pkg,
    error_path = paste0(error_path, "[1]"),
    error_call = error_call
  )
  sel <- switch(sign, "+" = integer(), "-" = seq_len(n)[!topics$internal])

  for (i in seq2(1, length(indexes))) {
    index <- indexes[[i]]

    sign <- all_sign(
      indexes[[i]],
      match_strings[[i]],
      error_pkg = error_pkg,
      error_path = paste0(error_path, "[", i, "]"),
      error_call = error_call
    )
    sel <- switch(
      sign,
      "+" = union(sel, indexes[[i]]),
      "-" = setdiff(sel, -indexes[[i]])
    )
  }
  sel
}

all_sign <- function(
  x,
  text,
  error_pkg,
  error_path,
  error_call = caller_env()
) {
  if (is.numeric(x)) {
    if (all(x < 0)) {
      return("-")
    }

    if (all(x > 0)) {
      return("+")
    }
  }
  config_abort(
    error_pkg,
    "{.field {error_path}} ({text}) must be all negative or all positive.",
    call = error_call
  )
}

match_env <- function(topics) {
  fns <- env(empty_env(), "-" = function(x) -x, "c" = function(...) c(...))
  out <- env(fns)

  topic_index <- seq_along(topics$name)

  # Each \alias{} is matched to its position
  topics$alias <- lapply(topics$alias, unique)
  aliases <- set_names(
    rep(topic_index, lengths(topics$alias)),
    unlist(topics$alias)
  )
  env_bind(out, !!!aliases)

  # As is each \name{} - we bind these second so that if \name{x} and \alias{x}
  # are in different files, \name{x} wins. This doesn't usually matter, but
  # \name{} needs to win so that the default_reference_index() matches the
  # correct files
  env_bind(out, !!!set_names(topic_index, topics$name))

  # dplyr-like matching functions

  any_alias <- function(f, ..., .internal = FALSE) {
    f <- as_function(f)
    alias_match <- purrr::map_lgl(unname(topics$alias), function(x) {
      any(f(x, ...))
    })
    name_match <- purrr::map_lgl(topics$name, f, ...)

    which((alias_match | name_match) & is_public(.internal))
  }

  is_public <- function(internal) {
    if (!internal) !topics$internal else rep(TRUE, nrow(topics))
  }
  fns$starts_with <- function(x, internal = FALSE) {
    check_string(x)
    check_bool(internal)

    any_alias(~ grepl(paste0("^", x), .), .internal = internal)
  }
  fns$ends_with <- function(x, internal = FALSE) {
    check_string(x)
    check_bool(internal)

    any_alias(~ grepl(paste0(x, "$"), .), .internal = internal)
  }
  fns$matches <- function(x, internal = FALSE) {
    check_string(x)
    check_bool(internal)

    any_alias(~ grepl(x, .), .internal = internal)
  }
  fns$contains <- function(x, internal = FALSE) {
    check_string(x)
    check_bool(internal)

    any_alias(~ grepl(x, ., fixed = TRUE), .internal = internal)
  }
  fns$has_keyword <- function(x) {
    check_character(x)
    which(purrr::map_lgl(topics$keywords, ~ any(. %in% x)))
  }
  fns$has_lifecycle <- function(x) {
    check_string(x)
    which(purrr::map_lgl(topics$lifecycle, ~ any(. %in% x)))
  }
  fns$has_concept <- function(x, internal = FALSE) {
    check_string(x)
    check_bool(internal)

    match <- purrr::map_lgl(topics$concepts, ~ any(str_trim(.) == x))
    which(match & is_public(internal))
  }
  fns$lacks_concepts <- function(x, internal = FALSE) {
    check_character(x)
    check_bool(internal)

    match <- purrr::map_lgl(topics$concepts, ~ any(str_trim(.) == x))
    which(!match & is_public(internal))
  }
  fns$lacks_concept <- fns$lacks_concepts
  out
}

match_eval <- function(
  string,
  index,
  env,
  error_pkg,
  error_path,
  error_call = caller_env()
) {
  error_path <- paste0(error_path, "[", index, "]")

  # Early return in case string already matches symbol
  if (env_has(env, string)) {
    val <- env[[string]]
    if (is.integer(val)) {
      return(val)
    }
  }

  expr <- tryCatch(parse_expr(string), error = function(e) NULL)
  if (is.null(expr)) {
    topic_must(
      "be valid R code",
      string,
      error_pkg = error_pkg,
      error_path = error_path,
      error_call = error_call
    )
  }

  if (is_string(expr) || is_symbol(expr)) {
    expr <- as.character(expr)
    val <- env_get(env, expr, default = NULL)
    if (is.integer(val)) {
      val
    } else {
      topic_must(
        "be a known topic name or alias",
        string,
        error_pkg = error_pkg,
        error_path = error_path,
        error_call = error_call
      )
    }
  } else if (is_call(expr, "::")) {
    name <- paste0(expr[[2]], "::", expr[[3]])
    val <- env_get(env, name, default = NULL)
    if (is.integer(val)) {
      val
    } else {
      topic_must(
        "be a known topic name or alias",
        string,
        error_pkg = error_pkg,
        error_path = error_path,
        error_call = error_call
      )
    }
  } else if (is_call(expr)) {
    withCallingHandlers(
      eval(expr, env),
      error = function(e) {
        config_abort(
          error_pkg,
          "{.field {error_path}} ({string}) failed to evaluate.",
          parent = e,
          call = error_call
        )
      }
    )
  } else {
    topic_must(
      "be a string or function call",
      string,
      error_pkg = error_pkg,
      error_path = error_path,
      error_call = error_call
    )
  }
}

topic_must <- function(message, topic, error_pkg, error_path, error_call, ...) {
  msg <- "{.field {error_path}} ({topic}) must {message}."
  config_abort(error_pkg, msg, call = error_call, ...)
}

section_topics <- function(
  pkg,
  match_strings,
  error_path,
  error_call = error_call()
) {
  # Add rows for external docs
  ext_strings <- match_strings[grepl("::", match_strings, fixed = TRUE)]
  topics <- rbind(pkg$topics, ext_topics(ext_strings))

  idx <- select_topics(
    match_strings,
    topics,
    error_pkg = pkg,
    error_path = error_path,
    error_call = error_call
  )
  selected <- topics[idx, , drop = FALSE]

  tibble::tibble(
    name = selected$name,
    path = selected$file_out,
    title = selected$title,
    lifecycle = selected$lifecycle,
    aliases = purrr::map2(
      selected$funs,
      selected$alias,
      ~ if (length(.x) > 0) .x else .y
    ),
    icon = find_icons(selected$alias, path(pkg$src_path, "icons"))
  )
}

select_topics_ <- function(topic, topics, check = TRUE) {
  pkg <- local_pkgdown_site()
  select_topics(
    topic,
    topics,
    check = check,
    error_path = "reference[1].contents",
    error_pkg = pkg
  )
}

test_that("bad inputs give informative warnings", {
  topics <- tibble::tribble(
    ~name, ~alias,        ~internal,  ~concepts,
    "x",   c("x", "x1"), FALSE,      character(),
  )

  expect_snapshot(error = TRUE, {
    select_topics_("x + ", topics)
    select_topics_("y", topics)
    select_topics_("paste(1)", topics)
    select_topics_("starts_with", topics)
    select_topics_("1", topics)

    select_topics_("starts_with('y')", topics)
  })
})

test_that("selector functions validate their inputs", {
  topics <- tibble::tribble(
    ~name, ~alias,        ~internal,  ~concepts,
    "x",   c("x", "x1"), FALSE,      character(),
  )

  expect_snapshot(error = TRUE, {
    select_topics_("starts_with('x', 'y')", topics)
    select_topics_("starts_with(c('x', 'y'))", topics)
  })
})


test_that("empty input returns empty vector", {
  topics <- tibble::tribble(
    ~name, ~alias,        ~internal,  ~concepts,
    "x",   c("x", "x1"), FALSE,      character(),
  )
  expect_equal(select_topics(character(), topics), integer())
})

test_that("can select by name or alias", {
  topics <- tibble::tribble(
    ~name, ~alias,
    "x",   c("a1", "a2"),
    "a",   c("a3"),
    "a-b", "b-a",
    "c::d", "d",
  )

  expect_equal(select_topics_("x", topics), 1)
  expect_equal(select_topics_("'x'", topics), 1)
  expect_equal(select_topics_("a1", topics), 1)
  expect_equal(select_topics_("a2", topics), 1)
  expect_equal(select_topics_("c::d", topics), 4)

  # Even if name is non-syntactic
  expect_equal(select_topics_("a-b", topics), 3)
  expect_equal(select_topics_("b-a", topics), 3)

  # Or missing
  expect_snapshot(error = TRUE, {
    select_topics_("a4", topics)
    select_topics_("c::a", topics)
  })
})

test_that("selection preserves original order", {
  topics <- tibble::tribble(
    ~name, ~alias,
    "x",   c("a1", "a2"),
    "a",   c("a3"),
    "b",   "b1"
  )

  expect_equal(select_topics_(c("a", "b1", "x"), topics), c(2, 3, 1))
})

test_that("can select by name", {
  topics <- tibble::tribble(
    ~name, ~alias,   ~internal,
    "a",   "a",      FALSE,
    "b1",  "b1",     FALSE,
    "b2",  "b2",     FALSE,
    "b3",  "b3",     TRUE,
  )
  topics$alias <- as.list(topics$alias)

  expect_equal(select_topics_("starts_with('a')", topics), 1)
  expect_equal(select_topics_("ends_with('a')", topics), 1)
  expect_equal(select_topics_("contains('a')", topics), 1)
  expect_equal(select_topics_("matches('[a]')", topics), 1)

  # Match internal when requested
  expect_equal(select_topics_("starts_with('b')", topics), c(2, 3))
  expect_equal(select_topics_("starts_with('b', internal = TRUE)", topics), 2:4)
})

test_that("can select by presense or absence of concept", {
  topics <- tibble::tribble(
    ~name, ~alias,        ~internal,  ~concepts,
    "b1",  "b1",          FALSE,      "a",
    "b2",  "b2",          FALSE,      c("a", "b"),
    "b3",  "b3",          FALSE,      character()
  )
  topics$alias <- as.list(topics$alias)

  expect_equal(select_topics_("has_concept('a')", topics), c(1, 2))
  expect_equal(select_topics_("lacks_concept('b')", topics), c(1, 3))
  expect_equal(select_topics_("lacks_concepts(c('a', 'b'))", topics), 3)
})

test_that("can select by keyword", {
  topics <- tibble::tribble(
    ~name, ~alias,        ~internal,  ~keywords,
    "b1",  "b1",          FALSE,      "a",
    "b2",  "b2",          FALSE,      c("a", "b"),
  )
  topics$alias <- as.list(topics$alias)
  expect_equal(select_topics_("has_keyword('a')", topics), c(1, 2))
  expect_equal(select_topics_("has_keyword('b')", topics), c(2))
  expect_equal(select_topics_("has_keyword('c')", topics, check = FALSE), integer())
})

test_that("can select by lifecycle", {
  topics <- tibble::tribble(
    ~name, ~alias,        ~internal,  ~keywords, ~lifecycle,
    "b1",  "b1",          FALSE,      "a",         list("stable"),
    "b2",  "b2",          FALSE,      c("a", "b"), NULL
  )
  expect_equal(select_topics_("has_lifecycle('stable')", topics), 1)
  expect_equal(select_topics_("has_lifecycle('deprecated')", topics, check = FALSE), integer())
})

test_that("can combine positive and negative selections", {
  topics <- tibble::tribble(
    ~name, ~alias,        ~internal,
    "x",   c("a1", "a2"), FALSE,
    "a",   c("a3"),       FALSE,
    "b",   "b1",          FALSE,
    "d",   "d",           TRUE,
  )

  expect_equal(select_topics_("-x", topics), c(2, 3))
  expect_equal(select_topics_(c("-x", "-a"), topics), 3)
  expect_equal(select_topics_(c("-x", "x"), topics), c(2, 3, 1))
  expect_equal(select_topics_(c("a", "x", "-a"), topics), 1)

  expect_snapshot(select_topics_("c(a, -x)", topics), error = TRUE)
})

test_that("an unmatched selection generates a warning", {
  topics <- tibble::tribble(
    ~name, ~alias,        ~internal,
    "x",   c("a1", "a2"), FALSE,
    "a",   c("a3"),       FALSE,
    "b",   "b1",          FALSE,
    "d",   "d",           TRUE,
  )

  expect_snapshot(error = TRUE,
    select_topics_(c("a", "starts_with('unmatched')"), topics),
  )
})

test_that("uses funs or aliases", {
  pkg <- local_pkgdown_site()
  pkg$topics <- tibble::tribble(
    ~name, ~funs,         ~alias,        ~file_out, ~title, ~lifecycle,
    "x",   character(),   c("x1", "x2"), "x.html",  "X", NULL,
    "y",   c("y1", "y2"), "y3",          "y.html",  "Y", NULL
  )

  out <- section_topics(pkg, c("x", "y"), error_path = "reference[1].contents")
  expect_equal(out$aliases, list(c("x1", "x2"), c("y1", "y2")))
})

test_that("full topic selection process works", {
  pkg <- local_pkgdown_site(test_path("assets/reference"))

  # can mix local and remote
  out <- section_topics(
    pkg,
    c("a", "base::mean"),
    error_path = "reference[1].contents"
  )
  expect_equal(unname(out$name), c("a", "base::mean"))

  # concepts and keywords work
  out <- section_topics(
    pkg,
    c("has_concept('graphics')", "has_keyword('foo')"),
    error_path = "reference[1].contents"
  )
  expect_equal(unname(out$name), c("b", "a"))
})

test_that("an unmatched selection with a matched selection does not select everything", {
  topics <- tibble::tribble(
    ~name, ~alias,        ~internal,
    "x",   c("a1", "a2"), FALSE,
    "a",   c("a3"),       FALSE,
    "b",   "b1",          FALSE,
    "d",   "d",           TRUE,
  )

  expect_equal(
    select_topics_(c("a", "starts_with('unmatched')"), topics, check = FALSE),
    2
  )

  expect_equal(
    select_topics_(c("starts_with('unmatched')", "a"), topics, check = FALSE),
    2
  )
})

copy_logo <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)

  logo_path <- find_logo(pkg$src_path)
  if (!is.null(logo_path)) {
    file_copy_to(
      src_paths = logo_path,
      src_root = pkg$src_path,
      dst_paths = path(pkg$dst_path, path_file(logo_path)),
      dst_root = pkg$dst_path
    )
  }
}

find_logo <- function(path) {
  path_first_existing(
    c(
      path(path, "logo.svg"),
      path(path, "man", "figures", "logo.svg"),
      path(path, "logo.png"),
      path(path, "man", "figures", "logo.png")
    )
  )
}

has_logo <- function(pkg) {
  logo_path <- find_logo(pkg$src_path)
  !is.null(logo_path)
}

logo_path <- function(pkg, depth) {
  path <- find_logo(pkg$src_path)
  if (is.null(path)) {
    return()
  }

  paste0(up_path(depth), path_file(path))
}

test_that("can handle logo in subdir", {
  src <- withr::local_tempdir()
  dst <- withr::local_tempdir()

  dir_create(path(src, "man", "figures"))
  file_create(path(src, "man", "figures", "logo.svg"))
  pkg <- structure(list(src_path = src, dst_path = dst), class = "pkgdown")
  expect_true(has_logo(pkg))

  suppressMessages(copy_logo(pkg))
  expect_true(file_exists(path(dst, "logo.svg")))

  expect_equal(logo_path(pkg, 0), "logo.svg")
  expect_equal(logo_path(pkg, 1), "../logo.svg")
})

section_init <- function(pkg,
                         subdir = NULL,
                         override = list(),
                         .frame = parent.frame()) {
  rstudio_save_all()
  pkg <- as_pkgdown(pkg, override = override)
  
  if (length(dest_files(pkg)) > 0) {
    check_dest_is_pkgdown(pkg) 
  } else {
    init_site(pkg)
  }
 
  if (is.null(subdir)) {
    depth <- 0
  } else {
    depth <- 1
    dir_create(path(pkg$dst_path, subdir))
  }

  local_envvar_pkgdown(pkg, .frame)
  local_options_link(pkg, depth = depth, .frame = .frame)
  cache_cli_colours(.frame = .frame)

  pkg
}

cache_cli_colours <- function(.frame = parent.frame()) {
  # https://github.com/r-lib/cli/issues/607
  num_col <- getOption("cli.num_colors", default = cli::num_ansi_colors())
  withr::local_options(cli.num_colors = num_col, .local_envir = .frame)
}

local_options_link <- function(pkg, depth, .frame = parent.frame()) {
  article_index <- article_index(pkg)
  Rdname <- get_rdname(pkg$topics)
  topic_index <- unlist(invert_index(set_names(pkg$topics$alias, Rdname)))

  withr::local_options(
    list(
      downlit.package = pkg$package,
      downlit.article_index = article_index,
      downlit.topic_index = topic_index,
      downlit.article_path = paste0(up_path(depth), "articles/"),
      downlit.topic_path = paste0(up_path(depth), "reference/")
    ),
    .local_envir = .frame
  )
}

local_context_eval <- function(
                               figures = NULL,
                               src_path = getwd(),
                               sexpr_env = child_env(globalenv()),
                               .frame = parent.frame()) {
  context_set_scoped("figures", figures, scope = .frame)
  context_set_scoped("src_path", src_path, scope = .frame)
  context_set_scoped("sexpr_env", sexpr_env, scope = .frame)
}

# Manage current topic index ----------------------------------------------------

context <- new_environment()
context$packages <- character()

context_set <- function(name, value) {
  old <- if (env_has(context, name)) env_get(context, name)

  if (is.null(value)) {
    if (env_has(context, name)) {
      env_unbind(context, name)
    }
  } else {
    env_bind(context, !!name := value)
  }
  invisible(old)
}

context_get <- function(name) {
  if (env_has(context, name)) {
    env_get(context, name)
  } else {
    cli::cli_abort("Context {.str name} has not been initialised")
  }
}

context_set_scoped <- function(name, value, scope = parent.frame()) {
  old <- context_set(name, value)
  withr::defer(context_set(name, old), envir = scope)
}

article_index <- function(pkg) {
  set_names(
    path_rel(pkg$vignettes$file_out, "articles"),
    pkg$vignettes$name
  )
}

test_that("widgets and browseable html are kept as is", {
  widget <- htmlwidgets::createWidget("test", list())
  expect_s3_class(pkgdown_print(widget), "htmlwidget")

  html <- htmltools::browsable(htmltools::div("foo"))
  expect_s3_class(pkgdown_print(html), "shiny.tag")
})

test_that("htmlwidgets get sized", {
  local_context_eval(list(fig.width = 7, dpi = 100, fig.asp = 1))

  widget <- htmlwidgets::createWidget("test", list())
  value <- pkgdown_print(widget)

  expect_equal(value$width, 700)
  expect_equal(value$height, 700)
})

test_that("respect htmlwidgets width", {
  local_context_eval(list(fig.width = 7, dpi = 100, fig.asp = 1))

  widget <- htmlwidgets::createWidget("test", list(), width = "100px")
  value <- pkgdown_print(widget)

  expect_equal(value$width, "100px")
})

tweak_anchors <- function(html) {
  headings <- xml2::xml_find_all(html, ".//h1|.//h2|.//h3|.//h4|.//h5|.//h6")
  # Find all headings that are contained in a div with an id and
  # have class 'section'

  is_ok <- xml2::xml_find_lgl(
    headings,
    "boolean(
      (parent::div[contains(@class, 'section') and @id]) or
      (parent::section[@id])
    )"
  )
  headings <- headings[is_ok]
  if (length(headings) == 0) {
    return(invisible())
  }

  id <- xml2::xml_find_chr(
    headings,
    "string(parent::div/@id|parent::section/@id)"
  )

  # Update ids: dot in the anchor breaks scrollspy and rd translation
  # doesn't have enough information to generate unique ids
  new_id <- make.unique(gsub(".", "-", id, fixed = TRUE), "-")

  # Move ids to headings so that the js TOC doesn't add create new ids
  divs <- xml2::xml_parent(headings)
  xml2::xml_attr(divs, "id") <- NULL
  xml2::xml_attr(headings, "id") <- new_id

  # Insert anchors
  anchor <- anchor_html(new_id)
  for (i in seq_along(headings)) {
    heading <- headings[[i]]
    if (length(xml2::xml_contents(heading)) == 0) {
      # skip empty headings
      next
    }
    # Insert anchor in first element of header
    xml2::xml_add_child(heading, xml2::read_xml(anchor[[i]]))
  }
  invisible()
}

anchor_html <- function(id) {
  paste0("<a class='anchor' aria-label='anchor' href='#", id, "'></a>")
}

tweak_link_md <- function(html) {
  links <- xml2::xml_find_all(html, ".//a")
  if (length(links) == 0) {
    return()
  }

  hrefs <- xml2::xml_attr(links, "href")

  urls <- xml2::url_parse(hrefs)
  needs_tweak <- urls$scheme == "" & grepl("\\.md$", urls$path)

  fix_links <- function(x) {
    x <- gsub("\\.md\\b", ".html", x)
    x <- gsub("\\.github/", "", x)
    x
  }

  if (any(needs_tweak)) {
    purrr::walk2(
      links[needs_tweak],
      fix_links(hrefs[needs_tweak]),
      xml2::xml_set_attr,
      attr = "href"
    )
  }

  invisible()
}

tweak_link_external <- function(html, pkg = list()) {
  links <- xml2::xml_find_all(html, ".//a")
  if (length(links) == 0) {
    return()
  }

  links <- links[!has_class(links, "external-link")]

  hrefs <- xml2::xml_attr(links, "href")
  links <- links[!is_internal_link(hrefs, pkg = pkg)]

  # Users might have added absolute URLs to e.g. the Code of Conduct
  tweak_class_prepend(links, "external-link")

  invisible()
}

# Fix relative image links
tweak_img_src <- function(html) {
  fix_path <- function(x) {
    x <- gsub("(^|/)vignettes/", "\\1articles/", x, perl = TRUE)
    x <- gsub("(^|/)man/figures/", "\\1reference/figures/", x, perl = TRUE)
    x
  }

  imgs <- xml2::xml_find_all(html, ".//img[not(starts-with(@src, 'http'))]")
  urls <- fix_path(xml2::xml_attr(imgs, "src"))
  purrr::map2(imgs, urls, ~ xml2::xml_set_attr(.x, "src", .y))

  imgs <- xml2::xml_find_all(
    html,
    ".//source[not(starts-with(@srcset, 'http'))]"
  )
  urls <- fix_path(xml2::xml_attr(imgs, "srcset"))
  purrr::map2(imgs, urls, ~ xml2::xml_set_attr(.x, "srcset", .y))

  invisible()
}

tweak_link_absolute <- function(html, pkg = list()) {
  # If there's no URL links can't be made absolute
  if (is.null(pkg$meta$url)) {
    return()
  }

  url <- paste0(pkg$meta$url, "/")

  # <a> + <link> use href
  href <- xml2::xml_find_all(html, ".//a | .//link")
  xml2::xml_attr(href, "href") <- xml2::url_absolute(
    xml2::xml_attr(href, "href"),
    url
  )

  # <img> + <script> uses src
  src <- xml2::xml_find_all(html, ".//script | .//img")
  xml2::xml_attr(src, "src") <- xml2::url_absolute(
    xml2::xml_attr(src, "src"),
    url
  )

  invisible()
}

tweak_link_R6 <- function(html, cur_package) {
  r6_span <- xml2::xml_find_all(html, ".//span[@class=\"pkg-link\"]")
  if (length(r6_span) == 0) {
    return()
  }

  pkg <- xml2::xml_attr(r6_span, "data-pkg")
  topic <- xml2::xml_attr(r6_span, "data-topic")
  id <- xml2::xml_attr(r6_span, "data-id")

  url <- paste0(topic, ".html")
  external <- pkg != cur_package
  if (any(external)) {
    url[external] <- purrr::map2_chr(
      topic[external],
      pkg[external],
      downlit::href_topic
    )
  }
  url <- paste0(url, ifelse(is.na(id), "", "#method-"), id)

  r6_a <- xml2::xml_find_first(r6_span, "./a")
  xml2::xml_attr(r6_a, "href") <- url

  invisible()
}

# Fix seealso links
tweak_link_seealso <- function(html) {
  seealso_links <- xml2::xml_find_all(html, "//code[a and text() = '()']")

  # add () inside link
  seealso_text <- xml2::xml_children(seealso_links)
  xml2::xml_text(seealso_text) <- paste0(xml2::xml_text(seealso_text), "()")

  # remove () outside the link
  text_nodes <- xml2::xml_find_all(seealso_links, "./text()[. = '()']")
  xml2::xml_remove(text_nodes)

  invisible()
}

tweak_tables <- function(html) {
  # Ensure all tables have class="table" apart from arguments
  table <- xml2::xml_find_all(html, ".//table")
  table <- table[!has_class(table, "ref-arguments")]

  tweak_class_prepend(table, "table")

  invisible()
}

# from https://github.com/rstudio/bookdown/blob/ed31991df3bb826b453f9f50fb43c66508822a2d/R/bs4_book.R#L307
tweak_footnotes <- function(html) {
  container <- xml2::xml_find_all(
    html,
    ".//div[contains(@class, 'footnotes')]|.//section[contains(@class, 'footnotes')]"
  )
  if (length(container) != 1) {
    return()
  }
  # Find id and contents
  footnotes <- xml2::xml_find_all(container, ".//li")
  id <- xml2::xml_attr(footnotes, "id")
  xml2::xml_remove(xml2::xml_find_all(footnotes, "//a[@class='footnote-back']"))
  contents <- vapply(footnotes, FUN.VALUE = character(1), function(x) {
    paste(
      as.character(xml2::xml_children(x), options = character()),
      collapse = "\n"
    )
  })
  # Add popover attributes to links
  for (i in seq_along(id)) {
    links <- xml2::xml_find_all(html, paste0(".//a[@href='#", id[[i]], "']"))
    xml2::xml_attr(links, "href") <- NULL
    xml2::xml_attr(links, "id") <- NULL
    xml2::xml_attr(links, "tabindex") <- "0"
    xml2::xml_attr(links, "data-bs-toggle") <- "popover"
    xml2::xml_attr(links, "data-bs-content") <- contents[[i]]
  }
  # Delete container
  xml2::xml_remove(container)
}

tweak_strip <- function(html, in_dev = FALSE) {
  to_remove <- if (in_dev) "pkgdown-release" else "pkgdown-devel"
  xpath <- paste0(
    ".//*[contains(@class, '",
    to_remove,
    "')]|",
    ".//*[contains(@class, 'pkgdown-hide')]"
  )
  nodes <- xml2::xml_find_all(html, xpath)
  xml2::xml_remove(nodes)
}

# tables -------------------------------------------------------------

test_that("tables get additional table class", {
  html <- xml2::read_html("
    <body>
      <table></table>
      <table class='a'></table>
      <table class='b'></table>
      </body>
  ")
  tweak_tables(html)

  expect_equal(
    xpath_attr(html, ".//table", "class"),
    c("table", "table a", "table b")
  )
})

test_that("except in the argument list", {
  html <- xml2::read_html("<body>
    <div class='template-reference-index'>
      <table class='ref-arguments'></table>
    </div>
  </body>")
  tweak_tables(html)
  expect_equal(xpath_attr(html, ".//table", "class"), "ref-arguments")
})


# anchors -------------------------------------------------------------

test_that("ids move from div to headings", {
  html <- xml2::read_xml('<body>
    <div id="1" class="section"><h1>abc</h1></div>
    <div id="2" class="section"><h2>abc</h2></div>
    <div id="3" class="section"><h3>abc</h3></div>
    <div id="4" class="section"><h4>abc</h4></div>
    <div id="5" class="section"><h5>abc</h5></div>
    <div id="6" class="section"><h6>abc</h6></div>
  </body>')
  tweak_anchors(html)
  expect_equal(xpath_attr(html, ".//h1|//h2|//h3|//h4|//h5|//h6", "id"), as.character(1:6))
  expect_equal(xpath_attr(html, ".//div", "id"), rep(NA_character_, 6))
})

test_that("must be in div with section an class and id", {
  html <- xml2::read_xml('<body>
    <h1>abc</h1>
    <div id="1"><h1>abc</h1></div>
    <div class="section"><h1>abc</h1></div>
  </body>')
  tweak_anchors(html)
  expect_equal(xpath_attr(html, ".//h1", "id"), rep(NA_character_, 3))
})

test_that("anchor html added to headings", {
  html <- xml2::read_xml('<body>
    <div id="x" class="section"><h1>abc</h1></div>
  </body>')
  tweak_anchors(html)
  expect_snapshot_output(xpath_xml(html, ".//h1"))
})

test_that("deduplicates ids", {
  html <- xml2::read_xml('<body>
    <div id="x" class="section"><h1>abc</h1></div>
    <div id="x" class="section"><h1>abc</h1></div>
    <div id="x" class="section"><h1>abc</h1></div>
  </body>')
  tweak_anchors(html)
  expect_equal(xpath_attr(html, ".//h1", "id"), c("x", "x-1", "x-2"))
})

test_that("can process multiple header levels", {
  html <- xml2::read_xml('<body>
      <div id="1" class="section"><h1>abc</h1></div>
      <div id="2" class="section"><h2>abc</h2></div>
      <div id="3" class="section"><h3>abc</h3></div>
      <div id="4" class="section"><h4>abc</h4></div>
  </body>')
  tweak_anchors(html)
  expect_equal(xpath_attr(html, ".//a", "href"), c("#1", "#2", "#3", "#4"))
})

test_that("can handle multiple header", {
  html <- xml2::read_xml('<body>
    <div id="x" class="section"><h1>one</h1><h1>two</h1></div>
  </body>')
  tweak_anchors(html)
  expect_equal(xpath_attr(html, ".//div", "id"), NA_character_)
  expect_equal(xpath_attr(html, ".//h1", "id"), c("x", "x-1"))
  expect_equal(xpath_attr(html, ".//h1/a", "href"), c("#x", "#x-1"))
})

test_that("anchors don't get additional newline", {
  html <- xml2::read_xml('<div id="x" class="section">
    <h1>abc</h1>
  </div>')
  tweak_anchors(html)
  expect_equal(xpath_text(html, ".//h1"), "abc")
})

test_that("empty headings are skipped", {
  html <- xml2::read_xml('<div id="x" class="section">
    <h1></h1>
  </div>')
  tweak_anchors(html)
  expect_equal(xpath_length(html, ".//h1/a"), 0)
})

test_that("docs with no headings are left unchanged", {
  html <- xml2::read_xml('<div>Nothing</div>')
  tweak_anchors(html)
  expect_equal(as.character(xpath_xml(html, ".")), '<div>Nothing</div>')
})

# links -----------------------------------------------------------------

test_that("local md links are replaced with html", {
  html <- xml2::read_html('
    <a href="local.md"></a>
    <a href="local.md#fragment"></a>
    <a href="http://remote.com/remote.md"></a>
  ')
  tweak_link_md(html)

  expect_equal(
    xpath_attr(html, "//a", "href"),
    c("local.html", "local.html#fragment", "http://remote.com/remote.md")
  )
})


test_that("tweak_link_external() add the external-link class if needed", {
  html <- xml2::read_html('
    <a href="#anchor"></a>
    <a href="http://remote.com/remote.md"></a>
    <a href="http://remote.com/remote.md" class="external-link"></a>
    <a href="http://remote.com/remote.md" class="thumbnail" ></a>
    <a href="http://example.com/remote.md"></a>
  ')

  pkg <- list(meta = list(url = "http://example.com"))
  tweak_link_external(html, pkg = pkg)

  expect_equal(
    xpath_attr(html, "//a", "class"),
    c(NA, "external-link", "external-link", "external-link thumbnail", NA)
  )
})

test_that("tweak_link_absolute() fixes relative paths in common locations", {
  html <- xml2::read_html('
    <a href="a"></a>
    <link href="link"></link>
    <script src="script"></script>
    <img src="img">
  ')
  pkg <- list(meta = list(url = "https://example.com"))
  tweak_link_absolute(html, pkg)

  expect_equal(xpath_attr(html, "//a", "href"), "https://example.com/a")
  expect_equal(xpath_attr(html, "//link", "href"), "https://example.com/link")
  expect_equal(xpath_attr(html, "//img", "src"), "https://example.com/img")
})

test_that("tweak_link_absolute() leaves absolute paths alone", {
  html <- xml2::read_html('<a href="https://a.com"></a>')
  pkg <- list(list(url = "https://example.com"))
  tweak_link_absolute(html, pkg)

  expect_equal(xpath_attr(html, "//a", "href"), "https://a.com")
})


test_that("tweak_link_r6() correctly modifies link to inherited R6 classes", {
  skip_on_cran() # in case downlit url changes
  html <- xml2::read_html("
    <span class=\"pkg-link\" data-pkg=\"pkgdown\" data-topic=\"Animal\" data-id=\"x\">
      <a href='replace-me.html'>text</a>
    </span>
    <span>
      <a href='leave-me.html'>text</a>
    </span>
    <span class=\"pkg-link\" data-pkg=\"downlit\" data-topic=\"autolink_url\" data-id=\"x\">
      <a href='replace_me.html'>text</a>
    </span>
  ")

  tweak_link_R6(html, "pkgdown")
  expect_equal(
    xpath_attr(html, "//a", "href"),
    c(
      "Animal.html#method-x",
      "leave-me.html",
      "https://downlit.r-lib.org/reference/autolink.html#method-x"
    )
  )
})


test_that("tweak_img_src() updates img and source tags", {
  html <- xml2::read_html('<body>
    <source srcset="man/figures/foo.png" />
    <img src="man/figures/bar.png" />
  </body>')

  tweak_img_src(html)
  expect_equal(xpath_attr(html, ".//img", "src"), "reference/figures/bar.png")
  expect_equal(xpath_attr(html, ".//source", "srcset"), "reference/figures/foo.png")
})

test_that("tweak_img_src() doesn't modify absolute links", {
  html <- xml2::read_html('<body>
    <img src="https://raw.githubusercontent.com/OWNER/REPO/main/vignettes/foo" />
    <img src="https://github.com/OWNER/REPO/raw/main/man/figures/foo" />
  </body>')
  urls_before <- xpath_attr(html, ".//img", "src")

  tweak_img_src(html)
  expect_equal(
    xpath_attr(html, ".//img", "src"),
    urls_before
  )
})

# stripped divs etc -------------------------------------------------------

test_that("selectively remove hide- divs", {
  html <- xml2::read_xml("<body>
    <div class='pkgdown-devel'>devel</div>
    <div class='pkgdown-release'>release</div>
    <div class='pkgdown-hide'>all</div>
  </body>")
  tweak_strip(html, in_dev = TRUE)
  expect_equal(xpath_text(html, ".//div"), "devel")

  html <- xml2::read_xml("<body>
    <div class='pkgdown-devel'>devel</div>
    <div class='pkgdown-release'>release</div>
    <div class='pkgdown-hide'>all</div>
  </body>")
  tweak_strip(html, in_dev = FALSE)
  expect_equal(xpath_text(html, ".//div"), "release")
})


# footnotes ---------------------------------------------------------------

test_that("can process footnote with code", {
  skip_if_no_pandoc("2.17.1")
  pkg <- local_pkgdown_site()
  html <- markdown_to_html(pkg, "
Hooray[^1]

[^1]: Including code:
```
1 +
2 +
```
And more text
  ")
  tweak_footnotes(html)

  expect_equal(xpath_length(html, "//a[@class='footnote-back']"), 0)
  expect_equal(xpath_attr(html, ".//a", "class"), "footnote-ref")
  expect_equal(xpath_attr(html, ".//a", "tabindex"), "0")
  expect_snapshot(xpath_attr(html, ".//a", "data-bs-content"))
})

# Menu constructors -----------------------------------------------------------

# Helpers for use within pkgdown itself - these must stay the same as the
# yaml structure defined in vignette("customise")
menu_submenu <- function(text, menu, icon = NULL, label = NULL, id = NULL) {
  if (length(menu) == 0) {
    return()
  } else {
    purrr::compact(list(
      text = text,
      icon = icon,
      "aria-label" = label,
      id = id,
      menu = menu
    ))
  }
}
menu_link <- function(
  text,
  href,
  target = NULL,
  class = NULL,
  id = NULL,
  title = NULL
) {
  purrr::compact(list(
    text = text,
    href = href,
    target = target,
    class = class,
    id = id,
    title = title
  ))
}
menu_links <- function(text, href) {
  purrr::map2(text, href, menu_link)
}
menu_theme <- function(text, icon, theme) {
  purrr::compact(list(text = text, theme = theme, icon = icon))
}

menu_heading <- function(text, ...) list(text = text, ...)
menu_separator <- function() list(text = "---------")
menu_search <- function() list(search = list())
menu_icon <- function(icon, href, label) {
  list(icon = icon, href = href, "aria-label" = label)
}

menu_type <- function(x, menu_depth = 0L) {
  if (!is.list(x) || !is_named(x)) {
    not <- obj_type_friendly(x)
    cli::cli_abort("Navbar components must be named lists, not {not}.")
  } else if (!is.null(x$menu)) {
    # https://github.com/twbs/bootstrap/pull/6342

    if (menu_depth > 0) {
      cli::cli_abort("Nested menus are not supported.")
    }

    "menu"
  } else if (!is.null(x$text) && grepl("^\\s*-{3,}\\s*$", x$text)) {
    "separator"
  } else if (!is.null(x$theme)) {
    "theme"
  } else if (!is.null(x$text) && is.null(x$href)) {
    "heading"
  } else if ((!is.null(x$text) || !is.null(x$icon)) && !is.null(x$href)) {
    "link"
  } else if (!is.null(x$search)) {
    "search"
  } else if (!is_named(x)) {
    "list"
  } else {
    cli::cli_abort("Unknown navbar component with names {names(x)}.")
  }
}

# Menu renderers --------------------------------------------------------------

navbar_html <- function(
  x,
  path_depth = 0L,
  menu_depth = 0L,
  side = c("left", "right")
) {
  if (is.null(x)) {
    return("")
  }

  side <- arg_match(side)
  type <- menu_type(x, menu_depth = menu_depth)

  text <- switch(
    type,
    menu = navbar_html_menu(
      x,
      menu_depth = menu_depth,
      path_depth = path_depth,
      side = side
    ),
    heading = navbar_html_heading(x),
    link = navbar_html_link(x, menu_depth = menu_depth),
    separator = navbar_html_separator(),
    search = navbar_html_search(x, path_depth = path_depth),
    theme = navbar_html_theme(x)
  )

  class <- c(
    if (menu_depth == 0L) "nav-item",
    if (type == "menu") "dropdown"
  )
  html_tag("li", class = class, text)
}

navbar_html_list <- function(
  x,
  path_depth = 0L,
  menu_depth = 0L,
  side = "left"
) {
  tags <- unwrap_purrr_error(purrr::map_chr(
    x,
    navbar_html,
    path_depth = path_depth,
    menu_depth = menu_depth,
    side = side
  ))
  paste0(tags, collapse = "\n")
}

navbar_html_menu <- function(
  x,
  path_depth = 0L,
  menu_depth = 0L,
  side = "left"
) {
  id <- paste0("dropdown-", x$id %||% make_slug(x$text))

  button <- html_tag(
    "button",
    type = "button",
    class = c(if (menu_depth == 0L) "nav-link", "dropdown-toggle"),
    id = id,
    `data-bs-toggle` = "dropdown",
    "aria-expanded" = "false",
    "aria-haspopup" = "true",
    "aria-label" = x$`aria-label`,
    navbar_html_text(x),
  )

  li <- navbar_html_list(
    x$menu,
    path_depth = path_depth,
    menu_depth = menu_depth + 1,
    side = side
  )
  ul <- html_tag(
    "ul",
    class = c("dropdown-menu", if (side == "right") "dropdown-menu-end"),
    "aria-labelledby" = id,
    paste0("\n", indent(li, "  "), "\n")
  )

  paste0("\n", indent(paste0(button, "\n", ul), "  "), "\n")
}

navbar_html_link <- function(x, menu_depth = 0) {
  base_class <- if (menu_depth == 0) "nav-link" else "dropdown-item"
  html_tag(
    "a",
    class = paste0(c(base_class, x$class), collapse = " "),
    id = x$id,
    href = x$href,
    target = x$target,
    title = x$title,
    "aria-label" = x$`aria-label`,
    navbar_html_text(x)
  )
}

navbar_html_theme <- function(x) {
  html_tag(
    "button",
    class = "dropdown-item",
    "aria-label" = x$`aria-label`,
    "data-bs-theme-value" = x$theme,
    navbar_html_text(x)
  )
}

navbar_html_heading <- function(x) {
  html_tag(
    "h6",
    class = "dropdown-header",
    "data-toc-skip" = NA,
    navbar_html_text(x)
  )
}

navbar_html_separator <- function() {
  '<hr class="dropdown-divider">'
}

navbar_html_search <- function(x, path_depth = 0) {
  input <- html_tag(
    "input",
    type = "search",
    class = "form-control",
    name = "search-input",
    id = "search-input",
    autocomplete = "off",
    "aria-label" = tr_("Search site"),
    placeholder = tr_("Search for"),
    "data-search-index" = paste0(up_path(path_depth), "search.json")
  )

  html_tag("form", class = "form-inline", role = "search", "\n", input, "\n")
}

# Reused HTML components -----------------------------------------------------

html_tag <- function(tag, ..., class = NULL) {
  dots <- list2(...)
  dots_attr <- dots[have_name(dots)]
  dots_child <- dots[!have_name(dots)]

  if (!is.null(class)) {
    class <- paste0(class, collapse = " ")
  }
  attr <- purrr::compact(c(list(class = class), dots_attr))
  if (length(attr) > 0) {
    html_attr <- ifelse(
      is.na(attr),
      names(attr),
      paste0(names(attr), '="', attr, '"')
    )
    html_attr <- paste0(" ", paste0(html_attr, collapse = " "))
  } else {
    html_attr <- ""
  }

  html_child <- paste0(purrr::compact(dots_child), collapse = " ")
  needs_close <- !tag %in% "input"

  paste0(
    "<",
    tag,
    html_attr,
    ">",
    html_child,
    if (needs_close) paste0("</", tag, ">")
  )
}

navbar_html_text <- function(x) {
  if (is.null(x$icon)) {
    icon <- ""
  } else {
    # Extract icon set from class name
    classes <- strsplit(x$icon, " ")[[1]]
    icon_classes <- classes[grepl("-", classes)]
    iconset <- purrr::map_chr(strsplit(icon_classes, "-"), 1)

    icon <- html_tag("span", class = unique(c(iconset, classes)))

    if (is.null(x$`aria-label`) && is.null(x$text)) {
      cli::cli_inform(
        c(
          x = "Icon {.str {x$icon}} lacks an {.var aria-label}.",
          i = "Specify {.var aria-label} to make the icon accessible to screen readers.",
          i = "Learn more in {.vignette pkgdown::accessibility}."
        ),
        .frequency = "regularly",
        .frequency_id = "icon-aria-label"
      )
    }
  }

  paste0(
    icon,
    if (!is.null(x$icon) && !is.null(x$text)) " ",
    escape_html(x$text)
  )
}

indent <- function(x, indent) {
  paste0(indent, gsub("\n", paste0("\n", indent), x))
}

test_that("can construct menu with children", {
  menu <- menu_submenu(
    "Title",
    list(
      menu_heading("Heading"),
      menu_separator(),
      menu_link("Link", "https://example.com")
    )
  )
  expect_snapshot(cat(navbar_html(menu)))
})

test_that("bad inputs give clear error", {
  submenu <- menu_submenu(
    "Title",
    list(
      menu_submenu("Heading", list(menu_heading("Hi")))
    )
  )
  expect_snapshot(error = TRUE, {
    navbar_html(1)
    navbar_html(list(foo = 1))
    navbar_html(submenu)
  })
})

test_that("can construct bullets", {
  expect_snapshot({
    cat(navbar_html(menu_icon("fa-question", "https://example.com", "label")))
    cat(navbar_html(menu_heading("Hi")))
    cat(navbar_html(menu_link("Hi", "https://example.com")))
  })
})

test_that("bullet class varies based on depth", {
  expect_equal(
    navbar_html(menu_separator(), menu_depth = 0),
    '<li class="nav-item"><hr class="dropdown-divider"></li>'
  )

  expect_equal(
    navbar_html(menu_separator(), menu_depth = 1),
    '<li><hr class="dropdown-divider"></li>'
  )
})

test_that("icons warn if no aria-label", {
  reset_message_verbosity("icon-aria-label")

  expect_snapshot({
    . <- navbar_html(menu_icon("fa-question", "https://example.com", NULL))
  })
})

test_that("icons extract base iconset class automatically", {
  expect_match(
    navbar_html(menu_icon("fa-question", "https://example.com", "label")),
    'class="fa fa-question"',
    fixed = TRUE
  )

  expect_match(
    navbar_html(menu_icon("fab fab-github", "https://example.com", "label")),
    'class="fab fab-github"',
    fixed = TRUE
  )
})

test_that("can specify link target", {
  expect_match(
    navbar_html(menu_link("a", "b", target = "_blank")),
    'target="_blank"',
    fixed = TRUE
  )
})

test_that("can specify class, id, and title", {
  expect_equal(
    navbar_html(menu_link("a", "#", class = "class")),
    '<li class="nav-item"><a class="nav-link class" href="#">a</a></li>'
  )
  expect_equal(
    navbar_html(menu_link("a", "#", id = "id")),
    '<li class="nav-item"><a class="nav-link" id="id" href="#">a</a></li>'
  )
  expect_equal(
    navbar_html(menu_link("a", "#", title = "title")),
    '<li class="nav-item"><a class="nav-link" href="#" title="title">a</a></li>'
  )
})

test_that("can construct theme menu", {
  pkg <- local_pkgdown_site(
    meta = list(template = list(bootstrap = 5, `light-switch` = TRUE))
  )
  lightswitch <- navbar_components(pkg)$lightswitch
  expect_snapshot(cat(navbar_html(lightswitch)))
})

test_that("simple components don't change without warning", {
  expect_snapshot({
    cat(navbar_html(menu_heading("a")))
    cat(navbar_html(menu_link("a", "b")))
    cat(navbar_html(menu_separator()))
    cat(navbar_html(menu_search()))
  })
})

# Building blocks -----------------------------------------------------------

test_that("navbar_html_text() combines icons and text", {
  expect_equal(navbar_html_text(list(text = "a")), 'a')
  expect_equal(
    navbar_html_text(list(icon = "fas-github", `aria-label` = "github")),
    '<span class="fas fas-github"></span>'
  )
  expect_equal(
    navbar_html_text(list(
      text = "a",
      icon = "fas-github",
      `aria-label` = "github"
    )),
    '<span class="fas fas-github"></span> a'
  )
})

test_that("navbar_html_text() escapes text", {
  expect_equal(navbar_html_text(list(text = "<>")), '&lt;&gt;')
})

test_that("named arguments become attributes", {
  expect_equal(html_tag("a"), '<a></a>')
  expect_equal(html_tag("a", x = NULL), '<a></a>')
  expect_equal(html_tag("a", x = NA), '<a x></a>')
  expect_equal(html_tag("a", x = 1), '<a x="1"></a>')
})

test_that("unnamed arguments become children", {
  expect_equal(html_tag("a", "b"), '<a>b</a>')
  expect_equal(html_tag("a", "b", NULL), '<a>b</a>')
})

test_that("class components are pasted together", {
  expect_equal(html_tag("a", class = NULL), '<a></a>')
  expect_equal(html_tag("a", class = "a"), '<a class="a"></a>')
  expect_equal(html_tag("a", class = c("a", "b")), '<a class="a b"></a>')
})

data_reference_index <- function(pkg = ".", error_call = caller_env()) {
  pkg <- as_pkgdown(pkg)

  meta <- config_pluck_reference(pkg, error_call)
  if (length(meta) == 0) {
    return(list())
  }

  rows <- unwrap_purrr_error(purrr::imap(
    meta,
    data_reference_index_rows,
    pkg = pkg,
    call = error_call
  ))
  rows <- purrr::list_c(rows)

  has_icons <- purrr::some(rows, ~ .x$row_has_icons %||% FALSE)

  check_missing_topics(rows, pkg, error_call = error_call)
  rows <- Filter(function(x) !x$is_internal, rows)

  print_yaml(list(
    pagetitle = tr_("Package index"),
    rows = rows,
    has_icons = has_icons
  ))
}

config_pluck_reference <- function(pkg, call = caller_env()) {
  ref <- config_pluck_list(
    pkg,
    "reference",
    default = default_reference_index(pkg)
  )

  for (i in seq_along(ref)) {
    section <- ref[[i]]
    config_check_list(
      section,
      error_path = paste0("reference[", i, "]"),
      error_pkg = pkg,
      error_call = call
    )
    config_check_string(
      section$title,
      error_path = paste0("reference[", i, "].title"),
      error_pkg = pkg,
      error_call = call
    )
    config_check_string(
      section$subtitle,
      error_path = paste0("reference[", i, "].subtitle"),
      error_pkg = pkg,
      error_call = call
    )
    if (has_name(section, "contents")) {
      check_contents(
        section$contents,
        i,
        pkg,
        prefix = "reference",
        call = call
      )
    }
  }

  ref
}

check_contents <- function(contents, index, pkg, prefix, call = caller_env()) {
  if (length(contents) == 0) {
    config_abort(
      pkg,
      "{.field {prefix}[{index}].contents} is empty.",
      call = call
    )
  }

  is_null <- purrr::map_lgl(contents, is.null)
  if (any(is_null)) {
    j <- which(is_null)[1]
    config_abort(
      pkg,
      "{.field {prefix}[{index}].contents[{j}]} is empty.",
      call = call
    )
  }

  is_char <- purrr::map_lgl(contents, is.character)
  if (!all(is_char)) {
    j <- which(!is_char)[1]
    config_abort(
      pkg,
      c(
        "{.field {prefix}[{index}].contents[{j}]} must be a string.",
        i = "You might need to add '' around special YAML values like 'N' or 'off'"
      ),
      call = call
    )
  }
}


data_reference_index_rows <- function(
  section,
  index,
  pkg,
  call = caller_env()
) {
  is_internal <- identical(section$title, "internal")

  rows <- list()
  if (has_name(section, "title")) {
    rows[[1]] <- list(
      title = markdown_text_inline(
        pkg,
        section$title,
        error_path = paste0("reference[", index, "].title"),
        error_call = call
      ),
      slug = make_slug(section$title),
      desc = markdown_text_block(pkg, section$desc),
      is_internal = is_internal
    )
  }

  if (has_name(section, "subtitle")) {
    rows[[2]] <- list(
      subtitle = markdown_text_inline(
        pkg,
        section$subtitle,
        error_path = paste0("reference[", index, "].subtitle"),
        error_call = call
      ),
      slug = make_slug(section$subtitle),
      desc = markdown_text_block(pkg, section$desc),
      is_internal = is_internal
    )
  }

  if (has_name(section, "contents")) {
    topics <- section_topics(
      pkg,
      section$contents,
      error_path = paste0("reference[", index, "].contents"),
      error_call = call
    )

    names <- topics$name
    topics$name <- NULL

    rows[[3]] <- list(
      topics = purrr::transpose(topics),
      names = names,
      row_has_icons = !purrr::every(topics$icon, is.null),
      is_internal = is_internal
    )
  }

  purrr::compact(rows)
}

find_icons <- function(x, path) {
  purrr::map(x, find_icon, path = path)
}
find_icon <- function(aliases, path) {
  names <- paste0(aliases, ".png")
  exists <- file_exists(path(path, names))

  if (!any(exists)) {
    NULL
  } else {
    names[which(exists)[1]]
  }
}

default_reference_index <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)

  exported <- pkg$topics[!pkg$topics$internal, , drop = FALSE]
  if (nrow(exported) == 0) {
    return(list())
  }

  print_yaml(list(
    list(
      title = tr_("All functions"),
      contents = auto_quote(unname(exported$name))
    )
  ))
}

check_missing_topics <- function(rows, pkg, error_call = caller_env()) {
  # Cross-reference complete list of topics vs. topics found in index page
  all_topics <- purrr::list_c(purrr::map(rows, "names"))
  in_index <- pkg$topics$name %in% all_topics

  missing <- !in_index & !pkg$topics$internal

  if (any(missing)) {
    config_abort(
      pkg,
      c(
        "{sum(missing)} topic{?s} missing from index: {.val {pkg$topics$name[missing]}}.",
        i = paste(
          "Either add to the reference index,",
          "or use {.code @keywords internal} to drop from the index."
        )
      ),
      call = error_call
    )
  }
}

test_that("can generate three types of row", {
  ref <- list(
    list(title = "A"),
    list(subtitle = "B"),
    list(contents = c("a", "b", "c", "e", "?"))
  )
  meta <- list(reference = ref)
  pkg <- as_pkgdown(test_path("assets/reference"), override = meta)

  expect_snapshot(data_reference_index(pkg))
})

test_that("can use markdown in title and subtitle", {
  ref <- list(
    list(title = "*A*"),
    list(subtitle = "*B*"),
    list(contents = c("a", "b", "c", "e", "?"))
  )
  meta <- list(reference = ref)
  pkg <- as_pkgdown(test_path("assets/reference"), override = meta)

  data <- data_reference_index(pkg)
  expect_equal(data$rows[[1]]$title, "<em>A</em>")
  expect_equal(data$rows[[2]]$subtitle, "<em>B</em>")
})

test_that("rows with title internal are dropped", {
  ref <- list(
    list(title = "internal", contents = c("a", "b")),
    list(contents = c("c", "e", "?")),
    list(title = "internal")
  )
  meta <- list(reference = ref)
  pkg <- as_pkgdown(test_path("assets/reference"), override = meta)

  expect_warning(index <- data_reference_index(pkg), NA)
  expect_equal(length(index$rows), 1)
})

test_that("duplicate entries within a group is dropped", {
  ref <- list(
    list(contents = c("a", "b", "a", "a")),
    list(contents = c("b", "c", "?", "e"))
  )
  meta <- list(reference = ref)
  pkg <- as_pkgdown(test_path("assets/reference"), override = meta)

  index <- data_reference_index(pkg)
  expect_equal(length(index$rows[[1]]$topics), 2)
  expect_equal(length(index$rows[[2]]$topics), 4)
})

test_that("warns if missing topics", {
  ref <- list(
    list(contents = c("a", "b"))
  )
  meta <- list(reference = ref)
  pkg <- as_pkgdown(test_path("assets/reference"), override = meta)

  expect_snapshot(data_reference_index(pkg), error = TRUE)
})

test_that("default reference includes all functions, only escaping non-syntactic", {
  ref <- default_reference_index(test_path("assets/reference"))
  expect_equal(ref[[1]]$contents, c("a", "b", "c", "e", "`?`"))
})

test_that("gives informative errors", {
  data_reference_index_ <- function(x) {
    pkg <- local_pkgdown_site(meta = list(reference = x))
    data_reference_index(pkg)
  }

  expect_snapshot(error = TRUE, {
    data_reference_index_(1)
    data_reference_index_(list(1))
    data_reference_index_(list(list(title = 1)))
    data_reference_index_(list(list(title = "a\n\nb")))
    data_reference_index_(list(list(subtitle = 1)))
    data_reference_index_(list(list(subtitle = "a\n\nb")))
    data_reference_index_(list(list(title = "bla", contents = 1)))
    data_reference_index_(list(list(title = "bla", contents = NULL)) )
    data_reference_index_(list(list(title = "bla", contents = list("a", NULL))))
    data_reference_index_(list(list(title = "bla", contents = list())))
    data_reference_index_(list(list(title = "bla", contents = "notapackage::lala")))
    data_reference_index_(list(list(title = "bla", contents = "rlang::lala")))
  })
})

test_that("can exclude topics", {
  pkg <- local_pkgdown_site(
    test_path("assets/reference"),
    list(
      reference = list(
        list(title = "Exclude", contents = c("a", "b", "-a")),
        list(title = "Exclude multiple", contents = c("a", "b", "c", "-matches('a|b')")),
        list(title = "Everything else", contents = c("a", "c", "e", "?"))
      )
    )
  )

  ref <- data_reference_index(pkg)
  # row 1 is the title row
  expect_equal(length(ref$rows[[2]]$topics), 1)
  expect_equal(ref$rows[[2]]$topics[[1]]$aliases, "b()")
  expect_equal(length(ref$rows[[4]]$topics), 1)
  expect_equal(ref$rows[[4]]$topics[[1]]$aliases, "c()")
})

test_that("can use a topic from another package", {
  pkg <- local_pkgdown_site(meta = list(reference = list(
    list(title = "bla", contents = c("rlang::is_installed()", "bslib::bs_add_rules"))
  )))

  expect_snapshot(data_reference_index(pkg))
})

test_that("can use a selector name as a topic name", {
  pkg <- local_pkgdown_site(
    test_path("assets/reference-selector"),
    list(
      reference = list(
        list(title = "bla", contents = c("matches", "matches('A')"))
      )
    )
  )

  expect_snapshot(data_reference_index(pkg))
})

has_class <- function(html, class) {
  classes <- strsplit(xml2::xml_attr(html, "class"), " ")
  purrr::map_lgl(classes, ~ any(class %in% .x))
}

tweak_class_prepend <- function(x, class) {
  if (length(x) == 0) {
    return(invisible())
  }

  cur <- xml2::xml_attr(x, "class")
  xml2::xml_attr(x, "class") <- ifelse(is.na(cur), class, paste(class, cur))
  invisible()
}

rd2ex <- function(x, ...) {
  x <- rd_text(paste0("\\examples{", x, "}"), fragment = FALSE)[[1]]
  x <- process_conditional_examples(x)
  x <- flatten_ex(x, ...)

  if (grepl("\n", x)) {
    strsplit(x, "\n")[[1]]
  } else {
    x
  }
}

run_examples <- function(
  x,
  topic = "unknown",
  env = globalenv(),
  run_dont_run = FALSE
) {
  if (!inherits(x, "tag")) {
    x <- rd_text(x)
  }

  # Trim newline that usually occurs after \examples{
  if (is_newline(x[[1]], trim = TRUE)) {
    x <- x[-1]
  }

  x <- process_conditional_examples(x)
  code <- flatten_ex(x, run_dont_run = run_dont_run)

  if (!can_parse(code)) {
    cli::cli_warn("Failed to parse example for topic {.val {topic}}")
    return("")
  }

  if (!is.null(env)) {
    highlight_examples(code, topic, env = env)
  } else {
    highlight_text(code)
  }
}

process_conditional_examples <- function(rd) {
  if (is.list(rd)) {
    which_exif <- which(purrr::map_lgl(rd, function(x) {
      "tag_dontshow" %in%
        class(x) &&
        is.character(x[[1]]) &&
        grepl("# examplesIf$", x[[1]])
    }))
    if (length(which_exif) == 0) {
      return(rd)
    }
    if (length(which_exif) %% 2 != 0) {
      cli::cli_abort("@examplesIf error, not closed?", call = caller_env())
    }
    remove <- integer()
    modes <- c("begin", "end")
    for (idx in which_exif) {
      if (rd[[idx]] != "}) # examplesIf") {
        # Start of @examplesIf
        if (modes[1] == "end") {
          cli::cli_abort("@examplesIf error, not closed?", call = caller_env())
        }
        cond_expr <- parse(text = paste0(rd[[idx]], "\n})"))[[1]][[2]]
        cond <- eval(cond_expr)
        if (isTRUE(cond)) {
          remove <- c(remove, idx, idx + 1L)
        } else {
          cond_expr_str <- deparse1(cond_expr)
          is_false <- cond_expr_str == "FALSE"
          if (!is_false) {
            new_cond <- paste0("if (FALSE) { # ", cond_expr_str)
            cli::cli_warn(
              "@examplesIf condition {.val {cond_expr_str}} is {.val FALSE}"
            )
          } else {
            new_cond <- "if (FALSE) {"
          }
          rd[[idx]] <- structure(list(new_cond), class = c("RCODE", "tag"))
        }
      } else {
        # End of @examplesIf
        if (modes[1] == "begin") {
          cli::cli_abort(
            "@examplesIf error, closed twice?",
            call = caller_env()
          )
        }
        if (isTRUE(cond)) {
          remove <- c(remove, idx, idx + 1L)
        } else {
          rd[[idx]] <- structure(list("}"), class = c("RCODE", "tag"))
        }
      }
      modes <- rev(modes)
    }
    if (length(remove)) {
      rd <- rd[-remove]
    }
    rd
  } else {
    rd
  }
}


# as_example --------------------------------------------------------------

as_example <- function(x, run_dont_run = FALSE) {
  UseMethod("as_example")
}

#' @export
as_example.RCODE <- function(x, run_dont_run = FALSE) as.character(x)
#' @export
as_example.VERB <- as_example.RCODE
#' @export
as_example.TEXT <- as_example.RCODE
#' @export
as_example.COMMENT <- function(x, run_dont_run = FALSE) {
  if (grepl("^%[^ ]*%", x)) {
    meant <- gsub("%", "\\\\%", x)
    xun <- unclass(x)
    cli::cli_warn(c(
      "In the examples, {.val {xun}} is an Rd comment",
      "x" = "did you mean {.val {meant}}?"
    ))
  }
  ""
}
#' @export
as_example.tag_dontrun <- function(x, run_dont_run = FALSE) {
  if (run_dont_run) {
    block_tag_to_comment("\\dontrun", x, run_dont_run = run_dont_run)
  } else {
    ex <- flatten_ex(x, run_dont_run = run_dont_run)
    if (is_newline(x[[1]], trim = TRUE)) {
      paste0("if (FALSE) { # \\dontrun{", ex, "} # }")
    } else {
      paste0("if (FALSE) ", ex, " # \\dontrun{}")
    }
  }
}

#' @export
as_example.tag_donttest <- function(x, run_dont_run = FALSE) {
  block_tag_to_comment("\\donttest", x, run_dont_run = run_dont_run)
}
#' @export
as_example.tag_dontshow <- function(x, run_dont_run = FALSE) {
  ex <- flatten_ex(x, run_dont_run = run_dont_run)
  paste0("DONTSHOW({", ex, "})")
}
#' @export
as_example.tag_testonly <- function(x, run_dont_run = FALSE) {
  ex <- flatten_ex(x, run_dont_run = run_dont_run)
  paste0("TESTONLY({", ex, "})")
}

block_tag_to_comment <- function(tag, x, run_dont_run = FALSE) {
  ex <- flatten_ex(x, run_dont_run = run_dont_run)

  # Not easy to strip leading whitespace because it's attached to the previous
  # tag. So instead we add a comment to occupy that space
  if (is_newline(x[[1]], trim = TRUE)) {
    ex <- paste0("# ", tag, "{", ex, "# }")
  }
  ex
}

#' @export
as_example.tag <- function(x, run_dont_run = FALSE) {
  untag <- paste(class(x), collapse = "/")
  cli::cli_warn("Unknown tag: {.val {untag}}")
}

#' @export
as_example.tag_dots <- function(x, run_dont_run = FALSE) {
  "..."
}
#' @export
as_example.tag_ldots <- as_example.tag_dots

#' @export
as_example.tag_if <- function(x, run_dont_run = FALSE) {
  if (x[[1]] == "html") {
    flatten_ex(x[[2]], run_dont_run = run_dont_run)
  } else {
    ""
  }
}
#' @export
as_example.tag_ifelse <- function(x, run_dont_run = FALSE) {
  if (x[[1]] == "html") {
    flatten_ex(x[[2]], run_dont_run = run_dont_run)
  } else {
    flatten_ex(x[[3]], run_dont_run = run_dont_run)
  }
}
#' @export
as_example.tag_out <- function(x, run_dont_run = FALSE) {
  flatten_ex(x, run_dont_run = run_dont_run)
}

# Helpers -----------------------------------------------------------------

flatten_ex <- function(x, run_dont_run = FALSE) {
  out <- purrr::map_chr(x, as_example, run_dont_run = run_dont_run)
  paste(out, collapse = "")
}

can_parse <- function(x) {
  tryCatch(
    {
      parse(text = x)
      TRUE
    },
    error = function(e) FALSE
  )
}

# run_examples() -----------------------------------------------------------

test_that("warns if unparseable", {
  expect_warning(
    run_examples("1 + \\dontrun{2 + }"),
    "Failed to parse"
  )
})

# as_example() ------------------------------------------------------------

test_that("dontrun{} wrapped in if(FALSE)", {
  expect_equal(rd2ex("\\dontrun{1}"), "if (FALSE) 1 # \\dontrun{}")
  expect_equal(
    rd2ex("\\dontrun{\n  1\n}"),
    c("if (FALSE) { # \\dontrun{", "  1", "} # }")
  )

  # unless run_dont_run is true
  expect_equal(rd2ex("\\dontrun{1}", run_dont_run = TRUE), "1")
  expect_equal(
    rd2ex("\\dontrun{\n  1\n}", run_dont_run = TRUE),
    c("# \\dontrun{", "  1", "# }")
  )
})

test_that("block donttest{} gets a comment to preserve spacing", {
  expect_equal(rd2ex("\\donttest{1}"), "1")
  expect_equal(
    rd2ex("\\donttest{\n  1\n}"),
    c("# \\donttest{", "  1", "# }")
  )
})

test_that("dontshow{} becomes DONTSHOW", {
  expect_equal(rd2ex("\\dontshow{1}"), "DONTSHOW({1})")
  expect_equal(rd2ex("\\dontshow{\n  1\n}"), c("DONTSHOW({", "  1", "})"))
})

test_that("testonly{} becomes TESTONLY", {
  expect_equal(rd2ex("\\testonly{1}"), "TESTONLY({1})")
  expect_equal(rd2ex("\\testonly{\n  1\n}"), c("TESTONLY({", "  1", "})"))
})

test_that("handles nested tags", {
  expect_equal(
    rd2ex("if(TRUE {\n  \\dontrun{\n    1 + 2\n  }\n}"),
    c(
      "if(TRUE {",
      "  if (FALSE) { # \\dontrun{",
      "    1 + 2",
      "  } # }",
      "}"
    )
  )
})

test_that("translate dots and ldots to ...", {
  expect_equal(rd2ex("\\ldots"), "...")
  expect_equal(rd2ex("\\dots"), "...")
})

test_that("ignores out", {
  expect_equal(rd2ex("\\out{1 + 2}"), "1 + 2")
})

test_that("extracts conditions from if", {
  expect_equal(rd2ex("\\if{html}{1 + 2}"), "1 + 2")
  expect_equal(rd2ex("\\if{latex}{1 + 2}"), "")
  expect_equal(rd2ex("\\ifelse{html}{1 + 2}{3 + 4}"), "1 + 2")
  expect_equal(rd2ex("\\ifelse{latex}{1 + 2}{3 + 4}"), "3 + 4")
})

test_that("@examplesIf", {

  rd <- paste0(
    "\\dontshow{if (1 == 0) (if (getRversion() >= \"3.4\") withAutoprint else force)(\\{ # examplesIf}\n",
    "answer <- 43\n",
    "\\dontshow{\\}) # examplesIf}"
  )
  exp <- c(
    "if (FALSE) { # 1 == 0",
    "answer <- 43",
    "}"
  )
  expect_warning(
    expect_equal(rd2ex(rd), exp),
    "@examplesIf condition"
  )

  rd2 <- paste0(
    "\\dontshow{if (TRUE) (if (getRversion() >= \"3.4\") withAutoprint else force)(\\{ # examplesIf}\n",
    "answer <- 43\n",
    "\\dontshow{\\}) # examplesIf}"
  )
  exp2 <- c(
    "answer <- 43"
  )
  expect_equal(rd2ex(rd2), exp2)

  cnd <- paste0(strrep("TRUE && ", 100), "FALSE")
  rd3 <- paste0(
    "\\dontshow{if (", cnd, ") (if (getRversion() >= \"3.4\") withAutoprint else force)(\\{ # examplesIf}\n",
    "answer <- 43\n",
    "\\dontshow{\\}) # examplesIf}"
  )
  exp3 <- c(
    paste0("if (FALSE) { # ", cnd),
    "answer <- 43",
    "}"
  )
  expect_snapshot(
    expect_equal(strtrim(rd2ex(rd3), 40), strtrim(exp3, 40))
  )
})

# Set various env vars (copied from tools::texi2dvi) to ensure that 
# latex can find bib and style files in the right places
local_texi2dvi_envvars <- function(input_path, env = caller_env()) {
  withr::local_envvar(
    BSTINPUTS = bst_paths(input_path),
    TEXINPUTS = tex_paths(input_path),
    BIBINPUTS = bib_paths(input_path),  
    .local_envir = env
  )
}

bst_paths <- function(path) {
  paths <- c(
    Sys.getenv("BSTINPUTS"),
    path_dir(path),
    path(R.home("share"), "texmf", "bibtex", "bst")
  )
  paste(paths, collapse = .Platform$path.sep)
}
tex_paths <- function(path) {
  paths <- c(
    Sys.getenv("TEXINPUTS"),
    path_dir(path),
    path(R.home("share"), "texmf", "tex", "latex")
  )
  paste(paths, collapse = .Platform$path.sep)
}
bib_paths <- function(path) {
  paths <- c(
    Sys.getenv("BIBINPUTS"),
    tex_paths(path)
  )
  paste(paths, collapse = .Platform$path.sep)
}

#' Build site for GitHub pages
#'
#' @description
#' Designed to be run as part of automated workflows for deploying
#' to GitHub pages. It cleans out the old site, builds the site into `dest_dir`
#' adds a `.nojekyll` file to suppress rendering by Jekyll, and adds a `CNAME`
#' file if needed.
#'
#' It is designed to be run in CI, so by default it:
#'
#' * Clean out the old site.
#' * Does not install the package.
#' * Runs [build_site()] in process.
#'
#' @inheritParams build_site
#' @inheritParams deploy_to_branch
#' @param dest_dir Directory to build site in.
#' @export
build_site_github_pages <- function(pkg = ".",
                                    ...,
                                    dest_dir = "docs",
                                    clean = TRUE,
                                    install = FALSE,
                                    new_process = FALSE) {
 pkg <- as_pkgdown(pkg, override = list(destination = dest_dir))

  if (clean) {
    rule("Cleaning files from old site", line = 1)
    clean_site(pkg)
  }

  build_site(pkg, preview = FALSE, install = install, new_process = new_process, ...)
  build_github_pages(pkg)

  invisible()
}

build_github_pages <- function(pkg = ".") {
  rule("Extra files for GitHub pages")
  pkg <- as_pkgdown(pkg)

  # Add .nojekyll since site is static HTML
  write_if_different(pkg, "", ".nojekyll", check = FALSE)

  # Add CNAME if url present
  cname <- cname_url(pkg$meta$url)
  if (is.null(cname)) {
    return(invisible())
  }

  write_if_different(pkg, cname, "CNAME", check = FALSE)
  invisible()
}

cname_url <- function(url) {
  if (is.null(url))
    return(NULL)

  pieces <- xml2::url_parse(url)
  if (!pieces$path %in% c("", "/"))
    return(NULL)

  pieces$server
}

test_that("a CNAME record is built if a url exists in metadata", {
  pkg <- local_pkgdown_site(test_path("assets/cname"))

  dir_create(path(pkg$dst_path, "docs"))
  expect_output(build_github_pages(pkg))
  expect_equal(read_lines(path(pkg$dst_path, "CNAME")), "testpackage.r-lib.org")
})

test_that("CNAME URLs are valid", {
  expect_equal(cname_url("http://google.com/"), "google.com")
  expect_equal(cname_url("https://google.com/"), "google.com")

  # this is not a valid URL because it has a trailing path
  expect_null(cname_url("http://google.com/path/"))
})

#' Test case: lists
#'
#' @noMd
#' @description
#'
#' \subsection{Bulleted list}{
#' \itemize{
#'   \item a
#'   \item This is an item...
#'
#'     That spans multiple paragraphs.
#' }
#' }
#'
#' \subsection{Bulleted list (single item)}{
#' \itemize{\item a}
#' }
#'
#' \subsection{Numbered list}{
#' \enumerate{
#'   \item a
#'   \item b
#' }
#' }
#'
#' \subsection{Definition list}{
#' \describe{
#'   \item{short}{short}
#'   \item{short}{Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do
#'   eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad
#'   minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip
#'   ex ea commodo consequat. Duis aute irure dolor in reprehenderit in
#'   voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur
#'   sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt
#'   mollit anim id est laborum.}
#'   \item{Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do
#'   eiusmod tempor incididunt ut labore et dolore magna aliqua.}{short}
#'   \item{Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do
#'   eiusmod tempor incididunt ut labore et dolore magna aliqua.}{Lorem ipsum
#'   adipiscing elit, sed do  eiusmod tempor incididunt ut labore et dolore ad
#'   minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip
#'   ex ea commodo consequat. Duis aute irure dolor in reprehenderit in
#'   voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur
#'   sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt
#'   mollit anim id est laborum.}
#' }
#' }
#' @keywords internal
#' @family tests
#' @name test-lists
NULL

#' Test case: links
#'
#' ```{r}
#' jsonlite::minify("{}")
#' ```
#'
#' @name test-links
#' @keywords internal
#' @family tests
#' @examples
#' jsonlite::minify("{}")
#'
#' library(jsonlite, warn.conflicts = FALSE)
#' minify("{}")
NULL

#' Test case: figures
#'
#' \figure{bacon.jpg}
#'
#' @name test-figures
#' @keywords internal
#' @family tests
#' @examples
#' x <- seq(0, 2 * pi, length.out = 25)
#' plot(x, sin(x))
#'
#' plot(1:10)
#' lines(1:10)
#' text(2, 5, "Hello", srt = 30, cex = 2)
#'
#' plot(1:3, col = c("red", "green", "blue"), pch = 16, cex = 10)
NULL

#' Test case: tables
#'
#' @name test-tables
#' @keywords internal
#' @family tests
#' @examples
#' gt::gt(head(mtcars))
NULL

#' Test case: don't
#'
#' @name test-dont
#' @keywords internal
#' @family tests
#' @examples
#' # \dontrun{} --------------------------------------------------------
#' # always shown; never run
#'
#' x <- 1
#' \dontrun{x <- 2}
#' \dontrun{
#'   x <- 3
#'   x <- 4
#' }
#' x # should be 1
#'
#' # \donttest{} -------------------------------------------------------
#' # only multiline are shown; always run
#'
#' x <- 1
#' \donttest{x <- 2}
#' \donttest{
#'   x <- 3
#'   x <- 4
#' }
#' x # should be 4
#'
#' # \testonly{} -----------------------------------------------------
#' # never shown, never run
#'
#' x <- 1
#' \testonly{x <- 2}
#' \testonly{
#'   x <- 3
#'   x <- 4
#' }
#' x # should be 1
#'
#' # \dontshow{} -------------------------------------------------------
#' # never shown, always run
#'
#' x <- 1
#' \dontshow{x <- 2}
#' \dontshow{
#'   x <- 3
#'   x <- 4
#' }
#' x # should be 4
#'
#' # @examplesIf ------------------------------------------------------
#' # If FALSE, wrapped in if; if TRUE, not seen
#'
#' x <- 1
#'
#' @examplesIf FALSE
#' x <- 2
#' @examplesIf TRUE
#' x <- 3
#' @examples
#' x # should be 3
NULL

#' Test case: params
#'
#' @name test-params
#' @param ... ellipsis
#' @keywords internal
#' @family tests
NULL

#' Test case: output styles
#'
#' @name test-output-styles
#' @keywords internal
#' @family tests
#'
#' @examples
#' # This example illustrates some important output types
#' # The following output should be wrapped over multiple lines
#' a <- 1:100
#' a
#'
#' cat("This some text!\n")
#' message("This is a message!")
#' warning("This is a warning!")
#'
#' # This is a multi-line block
#' {
#'   1 + 2
#'   2 + 2
#' }
NULL

#' Test case: long-lines
#'
#' The example results should have the copy button correctly placed when
#' scrollings
#'
#' @name test-long-lines
#' @keywords internal
#' @family tests
#' @examples
#' pkgdown:::ruler()
#'
#' cat(rep("x ", 100), sep = "")
#' cat(rep("xy", 100), sep = "")
#' cat(rep("x ", 100), sep = "")
#' cat(rep("xy", 100), sep = "")
NULL

#' Test case: crayon
#'
#' @name test-crayon
#' @keywords internal
#' @family tests
#'
#' @examples
#' cat(cli::col_red("This is red"), "\n")
#' cat(cli::col_blue("This is blue"), "\n")
#'
#' message(cli::col_green("This is green"))
#'
#' warning(cli::style_bold("This is bold"))
NULL

#' Test case: preformatted blocks & syntax highlighting
#'
#' Manual test cases for various ways of embedding code in sections.
#' All code blocks should have copy and paste button.
#'
#' # Should be highlighted
#'
#' Valid R code in `\preformatted{}`:
#'
#' ```
#' mean(a + 1)
#' ```
#'
#' R code in `R` block:
#'
#' ```R
#' mean(a + 1)
#' ```
#'
#' R code in `r` block:
#'
#' ```r
#' mean(a + 1)
#' ```
#'
#' Yaml
#'
#' ```yaml
#' yaml: [a, 1]
#' ```
#'
#' # Shouldn't be highlighted
#'
#' Non-R code in `\preformatted{}`
#'
#' ```
#' yaml: [a, b, c]
#' ```
#'
#' @name test-verbatim
#' @keywords internal
#' @family tests
NULL

#' Index
#'
#' @aliases test-index
#' @name index
#' @keywords internal
#' @family tests
NULL

#' Test case: \Sexpr[stage=render,results=rd]{"sexpr"}
#'
#' @name test-sexpr-title
#' @keywords internal
#' @family tests
NULL

#' Test case: math rendering in examples
#'
#' @param x \eqn{f(x) > 0}: inline equation
#'
#' @details
#' Display equation:
#'
#' \deqn{y = \alpha + \beta X + \varepsilon}
#'
#' Multi-line equation (correctly rendered by katex only):
#'
#' \deqn{\text{Minimize } l \\
#' \text{ subject to } \\
#' \sum_{i = 1}^{I} x_i r_{ij} + y_j \geq t_j \forall j \in J \\
#' l \geq \frac{y_j}{t_j} \forall j \in J \\
#' \sum_{i = 1}^{I} x_i c_i \leq B}{
#' Minimize l subject to
#' sum_i^I (xi * rij) + yj >= tj for all j in J &
#' l >= (yj / tj) for all j in J &
#' sum_i^I (xi * ci) <= B}
#'
#' @name test-math-examples
#' @keywords internal
#' @family tests
NULL

#' Report package pkgdown situation
#'
#' @description
#'
#' `pkgdown_sitrep()` reports
#'
#' * If there is an `url` field in the pkgdown configuration;
#'
#' * If that pkgdown website URL is stored in the DESCRIPTION file.
#'
#' @inheritParams as_pkgdown
#'
#' @export
#'
pkgdown_sitrep <- function(pkg = ".") {
  pkg_dir <- pkg
  pkg <- as_pkgdown(pkg_dir)
  if (is.null(pkg$meta[["url"]])) {
    cat(sprintf("* %s not configured.", pkgdown_field(pkg, "url")))
  } else {
    urls <- desc::desc(pkg_dir)$get_urls()
    urls <- sub("/$", "", urls)
    if (!pkg$meta[["url"]] %in% urls) {
      cat("* URL missing from the DESCRIPTION URL field.")
    } else {
      cat("All good :-)")
    }
  }

  invisible()
}

test_that("pkgdown_sitrep works", {
  # URL not in the pkgdown config
  pkg <- test_path("assets/figure")
  expect_snapshot_output(pkgdown_sitrep(pkg))
  # URL only in the pkgdown config
  pkg <- test_path("assets/cname")
  expect_snapshot_output(pkgdown_sitrep(pkg))
  # URL everywhere
  pkg <- test_path("assets/open-graph")
  expect_snapshot_output(pkgdown_sitrep(pkg))
})

build_home_md <- function(pkg) {
  mds <- package_mds(pkg$src_path, in_dev = pkg$development$in_dev)

  lapply(mds, render_md, pkg = pkg)
  invisible()
}

package_mds <- function(path, in_dev = FALSE) {
  mds <- dir_ls(path, glob = "*.md")

  # Also looks in .github, if it exists
  github_path <- path(path, ".github")
  if (dir_exists(github_path)) {
    mds <- c(mds, dir_ls(github_path, glob = "*.md"))
  }

  # Remove files handled elsewhere
  handled <- c("README.md", "LICENSE.md", "LICENCE.md", "NEWS.md")
  mds <- mds[!path_file(mds) %in% handled]

  # Do not build 404 page if in-dev
  if (in_dev) {
    mds <- mds[path_file(mds) != "404.md"]
  }

  # Remove files that don't need to be rendered
  no_render <- c(
    "issue_template.md",
    "pull_request_template.md",
    "cran-comments.md"
  )
  mds <- mds[!path_file(mds) %in% no_render]

  unname(mds)
}

render_md <- function(pkg, filename) {
  cli::cli_inform("Reading {src_path(path_rel(filename, pkg$src_path))}")

  body <- markdown_body(pkg, filename, strip_header = TRUE)
  path <- path_ext_set(path_file(filename), "html")

  render_page(pkg, "title-body",
    data = list(
      pagetitle = attr(body, "title"),
      body = body,
      filename = filename,
      source = repo_source(pkg, path_rel(filename, pkg$src_path))
    ),
    path = path
  )

  if (path == "404.html") {
    update_html(path(pkg$dst_path, path), tweak_link_absolute, pkg = pkg)
  }
  check_missing_images(pkg, filename, path)

  invisible()
}

test_that("can find files in root and .github", {
  dir <- withr::local_tempdir()
  dir_create(path(dir, ".github"))
  file_create(path(dir, "a.md"))
  file_create(path(dir, ".github", "b.md"))

  mds <- withr::with_dir(dir, package_mds("."))
  expect_equal(mds, c("a.md", "./.github/b.md"))
})

test_that("drops files handled elsewhere", {
  dir <- withr::local_tempdir()
  dir_create(path(dir, ".github"))
  file_create(path(dir, c("README.md", "LICENSE.md", "NEWS.md")))

  expect_equal(withr::with_dir(dir, package_mds(".")), character())
})

test_that("drops files that don't need to be rendered", {
  dir <- withr::local_tempdir()
  dir_create(path(dir, ".github"))
  file_create(path(dir, c("cran-comments.md", "issue_template.md")))

  expect_equal(withr::with_dir(dir, package_mds(".")), character())
})

repo_type <- function(pkg) {
  home <- repo_home(pkg) %||% ""

  if (grepl("^https?://github\\..+/", home)) {
    "github"
  } else if (grepl("^https?://gitlab\\..+/", home)) {
    "gitlab"
  } else {
    "other"
  }
}

repo_home <- function(pkg, paths) {
  pkg$repo$url$home
}

repo_source <- function(pkg, paths) {
  url <- pkg$repo$url
  if (is.null(url$source) || length(paths) == 0) {
    return()
  }

  needs_slash <- !grepl("/$", url$source) & !grepl("^/", paths)

  links <- a(
    paste0("<code>", escape_html(paths), "</code>"),
    paste0(url$source, ifelse(needs_slash, "/", ""), paths)
  )

  n <- length(links)
  if (n >= 4) {
    links <- c(links[1:3], paste0("and ", n - 3, " more"))
  }

  paste0(tr_("Source:"), " ", paste(links, collapse = ", "))
}

repo_auto_link <- function(pkg, text) {
  url <- pkg$repo$url

  if (!is.null(url$user)) {
    user_link <- paste0("\\1<a href='", url$user, "\\2'>@\\2</a>")
    text <- gsub("(p>|\\s|^|\\()@([-\\w]+)", user_link, text, perl = TRUE)
  }

  if (!is.null(url$issue)) {
    issue_link <- paste0("<a href='", url$issue, "\\2'>#\\2</a>")
    text <- gsub("(p>|\\(|\\s)#(\\d+)", paste0("\\1", issue_link), text, perl = TRUE)

    if (!is.null(pkg$repo$jira_projects)) {
      issue_link <- paste0("<a href='", url$issue, "\\1\\2'>\\1\\2</a>")
      issue_regex <- paste0("(", paste0(pkg$repo$jira_projects, collapse = "|"),")(-\\d+)")
      text <- gsub(issue_regex, issue_link, text, perl = TRUE)
    }
  }

  text
}

# Package data -------------------------------------------------------------

package_repo <- function(pkg) {
  # Use metadata if available
  repo <- config_pluck_list(pkg, "repo")
  url <- config_pluck_list(pkg, "repo.url")
  

  if (!is.null(url)) {
    return(repo)
  }

  # Otherwise try and guess from `BugReports` (1st priority) and `URL`s (2nd priority)
  urls <- c(
    sub("/issues/?", "/", pkg$desc$get_field("BugReports", default = character())),
    pkg$desc$get_urls()
  )

  gh_links <- grep("^https?://git(hub|lab)\\..+/", urls, value = TRUE)
  if (length(gh_links) > 0) {
    branch <- config_pluck_string(pkg, "repo.branch")
    return(repo_meta_gh_like(gh_links[[1]], branch))
  }

  NULL
}

repo_meta <- function(home = NULL, source = NULL, issue = NULL, user = NULL) {
  list(
    url = list(
      home = home,
      source = source,
      issue = issue,
      user = user
    )
  )
}

repo_meta_gh_like <- function(link, branch = NULL) {
  gh <- parse_github_like_url(link)
  branch <- branch %||% gha_current_branch()

  repo_meta(
    paste0(gh$host, "/", gh$owner, "/", gh$repo, "/"),
    paste0(gh$host, "/", gh$owner, "/", gh$repo, "/blob/", branch, "/"),
    paste0(gh$host, "/", gh$owner, "/", gh$repo, "/issues/"),
    paste0(gh$host, "/")
  )
}

gha_current_branch <- function() {
  # Only set in pull requests
  ref <- Sys.getenv("GITHUB_HEAD_REF")
  if (ref != "") {
    return(ref)
  }
  
  # Set everywhere but might not be a branch
  ref <- Sys.getenv("GITHUB_REF_NAME")
  if (ref != "") {
    return(ref)
  }
  
  "HEAD"
}

parse_github_like_url <- function(link) {
  supports_subgroups <- grepl("^https?://gitlab\\.", link)
  rx <- paste0(
    "^",
    "(?<host>https?://[^/]+)/",
    "(?<owner>[^/]+)/",
    "(?<repo>[^#", "/"[!supports_subgroups], "]+)/"
  )
  re_match(sub("([^/]$)", "\\1/", link), rx)
}

#  repo_auto_link ---------------------------------------------------------

test_that("authors are automatically linked", {
  pkg <- list(repo = repo_meta(user = "TEST/"))

  # email addresses shouldn't get linked
  expect_equal(repo_auto_link(pkg, "x@y.com"), "x@y.com")

  # must have leading whitespace or open parens
  expect_equal(repo_auto_link(pkg, "@y"), "<a href='TEST/y'>@y</a>")
  expect_equal(repo_auto_link(pkg, " @y"), " <a href='TEST/y'>@y</a>")
  expect_equal(repo_auto_link(pkg, "(@y)"), "(<a href='TEST/y'>@y</a>)")

  expect_equal(
    repo_auto_link(pkg, "<p>@y some other text.</p>"),
    "<p><a href='TEST/y'>@y</a> some other text.</p>"
  )
})

test_that("issues are automatically linked", {
  pkg <- list(repo = repo_meta(issue = "TEST/"))
  expect_equal(repo_auto_link(pkg, "(#123"), "(<a href='TEST/123'>#123</a>")
  expect_equal(repo_auto_link(pkg, "in #123"), "in <a href='TEST/123'>#123</a>")
  expect_equal(
    repo_auto_link(pkg, "<p>#123 some other text.</p>"),
    "<p><a href='TEST/123'>#123</a> some other text.</p>"
  )
  expect_equal(
    repo_auto_link(pkg, "<p><a href='TEST/123/'>#123</a></p>"),
    "<p><a href='TEST/123/'>#123</a></p>"
  )
})

test_that("already linked issues aren't re-linked", {
  pkg <- list(repo = repo_meta(issue = "TEST/"))
  expect_equal(
    repo_auto_link(pkg, "<p><a href='NOT/ABC/'>#123</a></p>"),
    "<p><a href='NOT/ABC/'>#123</a></p>"
  )
})

test_that("URLs with hash (#) are preserved", {
  pkg <- list(repo = repo_meta(issue = "TEST/"))
  expect_equal(
    repo_auto_link(pkg, "[example 5.4](https:/my.site#5-4-ad)"),
    "[example 5.4](https:/my.site#5-4-ad)"
  )
})

test_that("Jira issues are automatically linked", {
  pkg <- list(repo = repo_meta(issue = "TEST/"))
  pkg$repo$jira_projects <- c("JIRA", "OTHER")
  expect_equal(
    repo_auto_link(pkg, "JIRA-123"),
    "<a href='TEST/JIRA-123'>JIRA-123</a>"
  )
  expect_equal(
    repo_auto_link(pkg, "OTHER-4321"),
    "<a href='TEST/OTHER-4321'>OTHER-4321</a>"
  )
  # but only the jira projects specified are caught
  expect_equal(repo_auto_link(pkg, "NOPE-123"), "NOPE-123")
})

# repo_source -------------------------------------------------------------

test_that("repo_source() truncates automatically", {
  withr::local_envvar(GITHUB_HEAD_REF = "HEAD")
  pkg <- list(repo = repo_meta_gh_like("https://github.com/r-lib/pkgdown"))

  expect_snapshot({
    cat(repo_source(pkg, character()))
    cat(repo_source(pkg, "a"))
    cat(repo_source(pkg, letters[1:10]))
  })
})

test_that("repo_source() is robust to trailing slash", {
  pkg <- list(repo = repo_meta_gh_like("https://github.com/r-lib/pkgdown"))

  meta <- function(x) {
    list(repo = list(url = list(source = x)))
  }
  source <- "Source: <a href='http://example.com/a'><code>a</code></a>"
  expect_equal(repo_source(meta("http://example.com"), "a"), source)
  expect_equal(repo_source(meta("http://example.com/"), "a"), source)
})

test_that("repo_source() uses the branch setting in meta", {
  pkg <- local_pkgdown_site(
    meta = list(repo = list(branch = "main")),
    desc = list(URL = "https://github.com/r-lib/pkgdown")
  )
  expect_match(
    repo_source(pkg, "a"),
    "https://github.com/r-lib/pkgdown/blob/main/a"
  )
})

# package_repo ------------------------------------------------------------

test_that("can find github from BugReports or URL", {
  withr::local_envvar(GITHUB_HEAD_REF = "HEAD")
  expected <- repo_meta_gh_like("https://github.com/r-lib/pkgdown")

  pkg <- local_pkgdown_site(
    desc = list(
      URL = "https://github.com/r-lib/pkgdown"
    )
  )
  expect_equal(package_repo(pkg), expected)

  # BugReports beats URL
  pkg <- local_pkgdown_site(
    desc = list(
      URL = "https://github.com/r-lib/BLAHBLAH",
      BugReports = "https://github.com/r-lib/pkgdown/issues"
    )
  )
  expect_equal(package_repo(pkg), expected)

  # URL can be in any position
  pkg <- local_pkgdown_site(
    desc = list(
      URL = "https://pkgdown.r-lib.org, https://github.com/r-lib/pkgdown"
    )
  )
  expect_equal(package_repo(pkg), expected)
})

test_that("can find gitlab url", {
  withr::local_envvar(GITHUB_HEAD_REF = "HEAD")
  url <- "https://gitlab.com/msberends/AMR"
  pkg <- local_pkgdown_site(desc = list(URL = url))
  expect_equal(package_repo(pkg), repo_meta_gh_like(url))
})

test_that("can find codeberg url", {
  withr::local_envvar(GITHUB_HEAD_REF = "HEAD")
  url <- "https://codeberg.org/msberends/AMR"
  pkg <- local_pkgdown_site(desc = list(URL = url))
  expect_equal(package_repo(pkg), repo_meta_gh_like(url))
})

test_that("uses GITHUB env vars if set", {
  withr::local_envvar(GITHUB_HEAD_REF = NA, GITHUB_REF_NAME = "abc")
  expect_equal(
    repo_meta_gh_like("https://github.com/r-lib/pkgdown")$url$source,
    "https://github.com/r-lib/pkgdown/blob/abc/"
  )

  withr::local_envvar(GITHUB_HEAD_REF = "xyz")
  expect_equal(
    repo_meta_gh_like("https://github.com/r-lib/pkgdown")$url$source,
    "https://github.com/r-lib/pkgdown/blob/xyz/"
  )
})

test_that("GitLab subgroups are properly parsed", {
  issue_url <- function(...) {
    pkg <- local_pkgdown_site(desc = list(...))
    pkg$repo$url$issue
  }

  base <- "https://gitlab.com/salim_b/r/pkgs/pal"
  expected <- paste0(base, "/-/issues/")

  expect_identical(issue_url(URL = base), expected)
  expect_identical(issue_url(URL = paste0(base, "/")), expected)
  expect_identical(issue_url(BugReports = paste0(base, "/issues")), expected)
  expect_identical(issue_url(BugReports = paste0(base, "/issues/")), expected)
  expect_identical(issue_url(BugReports = paste0(base, "/-/issues")), expected)
  expect_identical(issue_url(BugReports = paste0(base, "/-/issues/")), expected)
})

test_that("can find github enterprise url", {
  withr::local_envvar(GITHUB_HEAD_REF = "HEAD")

  url <- "https://github.acme.com/roadrunner/speed"
  pkg <- local_pkgdown_site(desc = list(BugReports = url))
  expect_equal(package_repo(pkg), repo_meta_gh_like(url))
})

test_that("meta overrides autodetection", {
  pkg <- local_pkgdown_site(
    meta = list(repo = list(url = list(home = "http://one.com"))),
    desc = list(URL = "http://two.com")
  )

  expect_equal(package_repo(pkg)$url$home, "http://one.com")
})

test_that("returns NULL if no urls found", {
  pkg <- local_pkgdown_site(desc = list(URL = "https://pkgdown.r-lib.org"))
  expect_null(package_repo(pkg))
})

test_that("repo_type detects repo type", {
  repo_type2 <- function(url) {
    repo_type(list(repo = list(url = list(home = url))))
  }

  expect_equal(repo_type2("https://github.com/r-lib/pkgdown"), "github")
  expect_equal(repo_type2("https://github.r-lib.com/pkgdown"), "github")
  expect_equal(repo_type2("https://gitlab.com/r-lib/pkgdown"), "gitlab")
  expect_equal(repo_type2("https://gitlab.r-lib.com/pkgdown"), "gitlab")
  expect_equal(repo_type2("https://codeberg.org/r-lib/pkgdown"), "codeberg")
  expect_equal(repo_type2(NULL), "other")
})

build_404 <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)

  # if this file exists, it will be handled by build_home_md()
  page_md <- path(pkg$src_path, ".github", "404.md")

  if (!file_exists(page_md)) {
    render_page(
      pkg, "title-body",
      data = list(
        pagetitle = tr_("Page not found (404)"),
        body = tr_("Content not found. Please use links in the navbar.")
      ),
      path = "404.html"
    )
    update_html(path_abs("404.html", start = pkg$dst_path), tweak_link_absolute, pkg = pkg)
  }

  invisible()
}

build_citation_authors <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)

  source <- if (has_citation(pkg$src_path)) {
    repo_source(pkg, "inst/CITATION")
  } else {
    repo_source(pkg, "DESCRIPTION")
  }

  authors <- data_authors(pkg)
  data <- list(
    pagetitle = tr_("Authors and Citation"),
    citations = data_citations(pkg),
    authors = unname(authors$all),
    inst = authors$inst,
    before = authors$before,
    after = authors$after,
    source = source
  )

  render_page(pkg, "citation-authors", data, "authors.html")
}

data_authors <- function(pkg = ".", roles = default_roles(), call = caller_env()) {
  pkg <- as_pkgdown(pkg)
  author_info <- config_pluck_list(pkg, "authors", default = list(), call = call)

  inst_path <- path(pkg$src_path, "inst", "AUTHORS")
  if (file_exists(inst_path)) {
    inst <- read_lines(inst_path)
  } else {
    inst <- NULL
  }

  authors_all <- pkg_authors(pkg)
  authors_main <- pkg_authors(pkg, roles)

  all <- purrr::map(authors_all, author_list, author_info, pkg = pkg)
  main <- purrr::map(authors_main, author_list, author_info, pkg = pkg)
  more_authors <- length(main) != length(all)

  comments <- purrr::compact(purrr::map(all, "comment"))

  print_yaml(list(
    all = all,
    main = main,
    inst = inst,
    needs_page = more_authors || length(comments) > 0 || !is.null(inst),
    before = config_pluck_markdown_block(pkg, "template.authors.before", call = call),
    after = config_pluck_markdown_block(pkg, "template.authors.after", call = call)
  ))
}

default_roles <- function() {
  c("aut", "cre", "fnd")
}

pkg_authors <- function(pkg, role = NULL) {
  if (pkg$desc$has_fields("Authors@R")) {
    authors <- unclass(pkg$desc$get_authors())
  } else {
    # Just show maintainer
    authors <- unclass(utils::as.person(pkg$desc$get_maintainer()))
    authors[[1]]$role <- "cre"
  }

  if (is.null(role)) {
    authors
  } else {
    purrr::keep(authors, ~ any(.$role %in% role))
  }
}

data_home_sidebar_authors <- function(pkg = ".", call = caller_env()) {
  pkg <- as_pkgdown(pkg)

  config_pluck_list(pkg, "authors.sidebar")

  roles <- config_pluck_character(
    pkg,
    "authors.sidebar.roles",
    default = default_roles(),
    call = call
  )
  data <- data_authors(pkg, roles)
  authors <- purrr::map_chr(data$main, author_desc, comment = FALSE)
  
  before <- config_pluck_markdown_inline(pkg, "authors.sidebar.before", call = call)
  after <- config_pluck_markdown_inline(pkg, "authors.sidebar.after", call = call)
  
  bullets <- c(before, authors, after)
  if (data$needs_page) {
    bullets <- c(bullets, a(tr_("More about authors..."), "authors.html"))
  }

  sidebar_section(tr_("Developers"), bullets)
}

author_name <- function(x, authors, pkg) {
  name <- format_author_name(x$given, x$family)

  if (!(name %in% names(authors))) {
    return(name)
  }

  author <- authors[[name]]

  if (!is.null(author$html)) {
    error_path <- paste0("authors.", name, ".html")
    name <- markdown_text_inline(pkg, author$html, error_path = error_path)
  }

  if (is.null(author$href)) {
    name
  } else {
    a(name, author$href)
  }
}

format_author_name <- function(given, family) {
  given <- paste(given, collapse = " ")

  if (is.null(family)) {
    given
  } else {
    paste0(given, " ", family)
  }
}

author_list <- function(x, authors_info = NULL, comment = FALSE, pkg = ".") {
  name <- author_name(x, authors_info, pkg = pkg)

  roles <- paste0(role_lookup(x$role), collapse = ", ")
  substr(roles, 1, 1) <- toupper(substr(roles, 1, 1))

  orcid <- purrr::pluck(x$comment, "ORCID")
  x$comment <- remove_orcid(x$comment)

  list(
    name = name,
    roles = roles,
    comment = linkify(x$comment),
    orcid = orcid_link(orcid)
  )
}

author_desc <- function(x, comment = TRUE) {
  paste(
    x$name,
    "<br />\n<small class = 'roles'>", x$roles, "</small>",
    if (!is.null(x$orcid)) {
      x$orcid
    },
    if (comment && !is.null(x$comment) && length(x$comment) != 0) {
      paste0("<br/>\n<small>(", linkify(x$comment), ")</small>")
    }
  )
}

orcid_link <- function(orcid) {
  if (is.null(orcid)) {
    return(NULL)
  }

  paste0(
    "<a href='https://orcid.org/", orcid, "' target='orcid.widget' aria-label='ORCID'>",
    "<span class='fab fa-orcid orcid' aria-hidden='true'></span></a>"
  )
}

# Derived from:
# db <- utils:::MARC_relator_db
# db <- db[db$usage != "",]
# dput(setNames(tolower(db$term), db$code))
# # and replace creater with maintainer
role_lookup <- function(abbr) {
  # CRAN roles are translated
  roles <- c(
    aut = tr_("author"),
    com = tr_("compiler"),
    ctr = tr_("contractor"),
    ctb = tr_("contributor"),
    cph = tr_("copyright holder"),
    cre = tr_("maintainer"),
    dtc = tr_("data contributor"),
    fnd = tr_("funder"),
    rev = tr_("reviewer"),
    ths = tr_("thesis advisor"),
    trl = tr_("translator")
  )

  # Other roles are left as is
  marc_db <- getNamespace("utils")$MARC_relator_db
  extra <- setdiff(marc_db$code, names(roles))
  roles[extra] <- tolower(marc_db$term[match(extra, marc_db$code)])

  out <- unname(roles[abbr])
  if (any(is.na(out))) {
    missing <- abbr[is.na(out)]
    cli::cli_warn("Unknown MARC role abbreviation{?s}: {.field {missing}}")
    out[is.na(out)] <- abbr[is.na(out)]
  }
  out
}

# citation ---------------------------------------------------------------------

has_citation <- function(path = ".") {
  file_exists(path(path, 'inst/CITATION'))
}

create_citation_meta <- function(path) {
  path <- path(path, "DESCRIPTION")

  dcf <- read.dcf(path)
  desc <- as.list(dcf[1, ])

  if (!is.null(desc$Encoding)) {
    desc <- lapply(desc, iconv, from = desc$Encoding, to = "UTF-8")
  } else {
    desc$Encoding <- "UTF-8"
  }

  if (!is.null(desc$Title)) desc$Title <- str_squish(desc$Title)

  desc
}

read_citation <- function(path = ".") {
  if (!has_citation(path)) {
    return(character())
  }
  meta <- create_citation_meta(path)
  cit_path <- path(path, 'inst/CITATION')

  utils::readCitationFile(cit_path, meta = meta)
}

data_home_sidebar_citation <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)

  sidebar_section(
    heading = tr_("Citation"),
    bullets = a(sprintf(tr_("Citing %s"), pkg$package), "authors.html#citation")
  )
}

data_citations <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)

  if (has_citation(pkg$src_path)) {
    return(citation_provided(pkg$src_path))
  }

  citation_auto(pkg)
}

citation_provided <- function(src_path) {
  provided_citation <- read_citation(src_path)

  text_version <- format(provided_citation, style = "textVersion")
  cit <- list(
    html = ifelse(
      text_version == "",
      format(provided_citation, style = "html"),
      paste0("<p>", escape_html(text_version), "</p>")
    ),
    bibtex = format(provided_citation, style = "bibtex")
  )

  purrr::transpose(cit)
}

citation_auto <- function(pkg) {
  desc <- read_desc(pkg$src_path)
  cit_info <- as.list(desc$get(desc$fields()))
  #  utils::packageDescription(
  #   pkg$package,
  #   lib.loc = path_dir(pkg$src_path)
  # )
  # browser()
# C
  cit_info$`Date/Publication` <- cit_info$`Date/Publication` %||% Sys.time()
  if (!is.null(cit_info$Title)) cit_info$Title <- str_squish(cit_info$Title)

  cit <- utils::citation(auto = cit_info)
  list(
    html = format(cit, style = "html"),
    bibtex = format(cit, style = "bibtex")
  )
}

# helpers -------------------------------------------------------------------------

# Not strictly necessary, but produces a simpler data structure testing
remove_orcid <- function(x) {
  out <- x[names2(x) != "ORCID"]
  if (all(names(out) == "")) {
    names(out) <- NULL
  }
  out
}

# A CITATION file anywhere except in `inst/CITATION` is an R CMD check note
# so 'site-citation' is build-ignored, and so the tests must be skipped
# during R CMD check

path <- test_path("assets/site-citation/encoding-UTF-8")
skip_if_not(dir_exists(path)[[1]])

test_that("UTF-8 encoding and `citation(auto = meta) can be read` (#416, #493)", {
  cit <- read_citation(test_path("assets/site-citation/encoding-UTF-8"))
  expect_s3_class(cit, "citation")
})

test_that("latin1 encoding and `citation(auto = meta) can be read` (#689)", {
  cit <- read_citation(test_path("assets/site-citation/encoding-latin1"))
  expect_s3_class(cit, "citation")
})

test_that("create_meta can read DESCRIPTION with an Encoding", {
  meta <- create_citation_meta(test_path("assets/site-citation/encoding-UTF-8"))
  expect_type(meta, "list")
  expect_equal(meta$`Authors@R`, 'person(\"Florian\", \"Privé\")')
})

test_that("source link is added to citation page", {
  pkg <- local_pkgdown_site(test_path("assets/site-citation/encoding-UTF-8"))
  expect_output(build_home(pkg))

  lines <- read_lines(path(pkg$dst_path, "authors.html"))
  expect_true(any(grepl("<code>inst/CITATION</code></a></small>", lines)))
})

test_that("multiple citations all have HTML and BibTeX formats", {
  citations <- data_citations(test_path("assets/site-citation/multi"))
  expect_snapshot_output(citations)
})

test_that("links in curly braces in authors comments are escaped", {
  pkg_dir <- withr::local_tempdir()
  desc <- desc::description$new(cmd = "!new")
  desc$add_author("Jane", "Doe", comment = "reviewed see <https://github.com/r-lib/pkgdown/pulls>")
  desc$write(file.path(pkg_dir, "DESCRIPTION"))
  authors_data <- data_authors(pkg_dir)
  expect_equal(
    authors_data$all[[2]]$comment,
    "reviewed see &lt;<a href='https://github.com/r-lib/pkgdown/pulls'>https://github.com/r-lib/pkgdown/pulls</a>&gt;"
  )
})

test_that("authors page includes inst/AUTHORS", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "inst/AUTHORS", "Hello")
  suppressMessages(build_citation_authors(pkg))

  lines <- read_lines(path(pkg$dst_path, "authors.html"))
  expect_match(lines, "<pre>Hello</pre>", all = FALSE)
})

test_that("data_authors validates yaml inputs", {
  data_authors_ <- function(...) {
    pkg <- local_pkgdown_site(meta = list(...))
    data_authors(pkg)
  }

  expect_snapshot(error = TRUE, {
    data_authors_(authors = 1)
    data_authors_(template = list(authors = list(before = 1)))
    data_authors_(template = list(authors = list(after = 1)))
  })
})

test_that("data_home_sidebar_authors validates yaml inputs", {
  data_home_sidebar_authors_ <- function(...) {
    pkg <- local_pkgdown_site(meta = list(...))
    data_home_sidebar_authors(pkg)
  }

  expect_snapshot(error = TRUE, {
    data_home_sidebar_authors_(authors = list(sidebar = list(roles = 1)))
    data_home_sidebar_authors_(authors = list(sidebar = list(before = 1)))
    data_home_sidebar_authors_(authors = list(sidebar = list(before = "x\n\ny")))
  })
})

# authors --------------------------------------------------------------------

test_that("ORCID can be identified & removed from all comment styles", {
  desc <- desc::desc(text = c(
    'Authors@R: c(',
    '    person("no comment"),',
    '    person("bare comment", comment = "comment"),',
    '    person("orcid only",   comment = c(ORCID = "1")),',
    '    person("both",         comment = c("comment", ORCID = "2"))',
    '  )'
  ))
  authors <- purrr::map(desc$get_authors(), author_list, list())
  expect_equal(
    purrr::map(authors, "orcid"),
    list(NULL, NULL, orcid_link("1"), orcid_link("2"))
  )

  expect_equal(
    purrr::map(authors, "comment"),
    list(character(), "comment", character(), "comment")
  )
})

test_that("author comments linkified with escaped angle brackets (#2127)", {
  p <- list(name = "Jane Doe", roles = "rev", comment = "<https://x.org/>")
  expect_match(
    author_desc(p),
    "&lt;<a href='https://x.org/'>https://x.org/</a>&gt;",
    fixed = TRUE
  )
})

test_that("authors data can be filtered with different roles", {
  pkg <- local_pkgdown_site(desc = list(`Authors@R` = '
    c(
    person("Hadley", "Wickham", , "hadley@rstudio.com", role = c("aut", "cre")),
    person("RStudio", role = c("cph", "fnd"))
    )'
  ))
  expect_length(data_authors(pkg)$main, 2)
  expect_length(data_authors(pkg, roles = "cre")$main, 1)
})

test_that("authors data includes inst/AUTHORS", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "inst/AUTHORS", "Hello")

  expect_equal(data_authors(pkg)$inst, "Hello")
})

test_that("sidebar can accept additional before and after text", {
  pkg <- local_pkgdown_site()
  pkg$meta$authors$sidebar$before <- "BEFORE"
  pkg$meta$authors$sidebar$after <- "AFTER"
  expect_snapshot(cat(data_home_sidebar_authors(pkg)))
})

test_that("role has multiple fallbacks", {
  expect_equal(role_lookup("cre"), "maintainer")
  expect_equal(role_lookup("res"), "researcher")
  expect_snapshot(role_lookup("unknown"))
})

# citations -------------------------------------------------------------------

test_that("can handle UTF-8 encoding (#416, #493)", {
  # Work around bug in utils::citation()
  local_options(warnPartialMatchDollar = FALSE)

  pkg <- local_pkgdown_site(desc = list(
    Title = "A søphîstiçated påckagé",
    Date = "2018-02-02"
  ))

  meta <- create_citation_meta(pkg$src_path)
  expect_type(meta, "list")
  expect_equal(meta$Title, "A søphîstiçated påckagé")

  pkg <- pkg_add_file(pkg, "inst/CITATION", c(
    'citEntry(',
    '  entry = "Article",',
    '  title="Title: é",',
    '  author="Author: é",',
    '  journal="Journal é",',
    '  year="2017",',
    '  textVersion = "é"',
    ')'
  ))
  cit <- read_citation(pkg$src_path)
  expect_s3_class(cit, "citation")

  pkg <- pkg_add_file(pkg, "inst/CITATION", "citation(auto = meta)")
  cit <- read_citation(pkg$src_path)
  expect_s3_class(cit, "citation")
})

test_that("can handle latin1 encoding (#689)", {
  pkg <- local_pkgdown_site(desc = list(
    Title = "A søphîstiçated påckagé",
    Date = "2018-02-02",
    Encoding = "latin1"
  ))
  meta <- create_citation_meta(pkg$src_path)
  expect_equal(meta$Title, "A søphîstiçated påckagé")
  expect_equal(Encoding(meta$Title), "UTF-8")

  pkg <- pkg_add_file(pkg, "inst/CITATION", c(
    'citEntry(',
    '  entry = "Article",',
    '  title="Title: é",',
    '  author="Author: é",',
    '  journal="Journal é",',
    '  year="2017",',
    '  textVersion = "é"',
    ')'
  ))
  cit_path <- path(pkg$src_path, "inst/CITATION")
  citation <- readLines(cit_path) # nolint
  con <- file(cit_path, open = "w+", encoding = "native.enc")
  withr::defer(close(con))
  base::writeLines(iconv(citation, to = "latin1"), con, useBytes = TRUE) # nolint

  cit <- read_citation(pkg$src_path)
  expect_s3_class(cit, "citation")

  pkg <- pkg_add_file(pkg, "inst/CITATION", "citation(auto = meta)")
  cit <- read_citation(pkg$src_path)
  expect_s3_class(cit, "citation")
})

test_that("source link is added to citation page", {
  # Work around bug in utils::citation()
  local_options(warnPartialMatchDollar = FALSE)

  pkg <- local_pkgdown_site(meta = list(
    repo = list(url = list(source = "http://github.com/test/test"))
  ))
  pkg <- pkg_add_file(pkg, "inst/CITATION", c(
    'citEntry(',
    '  entry = "Article",',
    '  title="Title",',
    '  author="Author",',
    '  journal="Journal",',
    '  year="2020",',
    '  textVersion = ""',
    ')'
  ))
  suppressMessages(build_citation_authors(pkg))

  lines <- read_lines(path(pkg$dst_path, "authors.html"))
  expect_match(lines, "<code>inst/CITATION</code></a>", all = FALSE, fixed = TRUE)
})

test_that("multiple citations all have HTML and BibTeX formats", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "inst/CITATION", c(
    'bibentry("misc", title="Proof of b < a > c", author=c("A", "B"), year="2021",
         textVersion="A & B (2021): Proof of b < a > c.")',
    'bibentry("misc", title="Title Two", author="Author Two", year="2022")'
  ))

  citations <- data_citations(pkg$src_path)
  expect_snapshot_output(citations)
})

test_that("bibtex is escaped", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "inst/CITATION", c(
    'citEntry(',
    '  entry = "Article",',
    '  title="test special HTML characters: <&>",',
    '  author="x",',
    '  journal="x",',
    '  year="2017",',
    '  textVersion = ""',
    ')'
  ))

  suppressMessages(build_citation_authors(pkg))
  html <- xml2::read_html(path(pkg$dst_path, "authors.html"))

  expect_match(xpath_text(html, "//pre"), "<&>", fixed = TRUE)
})

#' Check `_pkgdown.yml`
#'
#' @description
#' This pair of functions checks that your `_pkgdown.yml` is valid without
#' building the whole site. `check_pkgdown()` errors at the first problem;
#' `pkgdown_sitrep()` reports the status of all checks.
#'
#' Currently they check that:
#'
#' * There's a `url` in the pkgdown configuration, which is also recorded
#'   in the `URL` field of the `DESCRIPTION`.
#'
#' * All opengraph metadata is valid.
#'
#' * All reference topics are included in the index.
#'
#' * All articles/vignettes are included in the index.
#
#' @export
#' @inheritParams as_pkgdown
check_pkgdown <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)

  check_urls(pkg)
  data_open_graph(pkg)
  data_articles_index(pkg)
  data_reference_index(pkg)

  cli::cli_inform(c("v" = "No problems found."))
}

#' @export
#' @rdname check_pkgdown
pkgdown_sitrep <- function(pkg = ".") {
  cli::cli_rule("Sitrep")

  pkg <- as_pkgdown(pkg)
  if (pkg$bs_version == 3) {
    cli::cli_inform(c(
      x = "Bootstrap 3 is deprecated; please switch to Bootstrap 5.",
      i = "Learn more at {.url https://www.tidyverse.org/blog/2021/12/pkgdown-2-0-0/#bootstrap-5}."
    ))
  }

  error_to_sitrep("URLs", check_urls(pkg))
  error_to_sitrep("Favicons", check_favicons(pkg))
  error_to_sitrep("Open graph metadata", data_open_graph(pkg))
  error_to_sitrep("Articles metadata", data_articles_index(pkg))
  error_to_sitrep("Reference metadata", data_reference_index(pkg))
}

error_to_sitrep <- function(title, code) {
  tryCatch(
    {
      code
      cli::cli_inform(c("v" = "{title} ok."))
    },
    rlang_error = function(e) {
      bullets <- c(cnd_header(e), cnd_body(e))
      cli::cli_inform(c(x = "{title} not ok.", set_names(bullets, " ")))
    }
  )
  invisible()
}

check_urls <- function(pkg = ".", call = caller_env()) {
  pkg <- as_pkgdown(pkg)
  details <- c(i = "See details in {.vignette pkgdown::metadata}.")

  if (identical(pkg$meta, list())) {
    cli::cli_abort(
      c("No {.path _pkgdown.yml} found.", details),
      call = call
    )
  }

  url <- pkg$meta[["url"]]

  if (is.null(url)) {
    config_abort(pkg, c("{.field url} is missing.", details), call = call)
  } else {
    desc_urls <- pkg$desc$get_urls()
    desc_urls <- sub("/$", "", desc_urls)
    if (!pkg$meta[["url"]] %in% desc_urls) {
      msg <- "{.field URL} is missing package url ({url})."
      config_abort(pkg, c(msg, details), path = "DESCRIPTION", call = call)
    }
  }
}

check_favicons <- function(pkg) {
  if (!has_logo(pkg)) {
    return()
  }

  if (has_favicons(pkg)) {
    logo <- find_logo(pkg$src_path)
    favicon <- path(path_favicons(pkg), "favicon.ico")

    if (out_of_date(logo, favicon)) {
      cli::cli_abort(c(
        "Package logo is newer than favicons.",
        i = "Do you need to rerun {.run [build_favicons()](pkgdown::build_favicons())}?"
      ))
    }
  } else {
    cli::cli_abort(c(
      "Found package logo but not favicons.",
      i = "Do you need to run {.run [build_favicons()](pkgdown::build_favicons())}?"
    ))
  }
}

test_that("sitrep complains about BS3", {
  pkg <- local_pkgdown_site(
    meta = list(
      template = list(bootstrap = 3),
      url = "https://example.com"
    ),
    desc = list(URL = "https://example.com")
  )
  expect_snapshot(pkgdown_sitrep(pkg))
})

test_that("sitrep reports all problems", {
  pkg <- local_pkgdown_site(
    test_path("assets/reference"),
    list(
      reference = list(
        list(title = "Title", contents = c("a", "b", "c", "e"))
      )
    )
  )

  expect_snapshot(pkgdown_sitrep(pkg))
})

test_that("checks fails on first problem", {
  pkg <- local_pkgdown_site(
    test_path("assets/reference"),
    list(
      reference = list(
        list(title = "Title", contents = c("a", "b", "c", "e"))
      )
    )
  )

  expect_snapshot(check_pkgdown(pkg), error = TRUE)
})

test_that("both inform if everything is ok", {
  pkg <- local_pkgdown_site(
    meta = list(url = "https://example.com"),
    desc = list(URL = "https://example.com")
  )

  expect_snapshot({
    pkgdown_sitrep(pkg)
    check_pkgdown(pkg)
  })
})

# check urls ------------------------------------------------------------------

test_that("check_urls reports problems", {
  # URL not in the pkgdown config
  pkg <- local_pkgdown_site()
  expect_snapshot(check_urls(pkg), error = TRUE)

  # URL only in the pkgdown config
  pkg <- local_pkgdown_site(meta = list(url = "https://testpackage.r-lib.org"))
  expect_snapshot(check_urls(pkg), error = TRUE)
})

# check favicons --------------------------------------------------------------

test_that("check_favicons reports problems", {
  pkg <- local_pkgdown_site()

  # no logo no problems
  expect_no_error(check_favicons(pkg))

  # logo but no favicons
  file_touch(path(pkg$src_path, "logo.svg"))
  expect_snapshot(check_favicons(pkg), error = TRUE)

  # logo and old favicons
  dir_create(path_favicons(pkg))
  file_touch(path(path_favicons(pkg), "favicon.ico"), Sys.time() - 86400)
  expect_snapshot(check_favicons(pkg), error = TRUE)

  # logo and new favicons
  file_touch(path(path_favicons(pkg), "favicon.ico"), Sys.time() + 86400)
  expect_no_error(check_favicons(pkg))
})

up_path <- function(depth) {
  paste(rep.int("../", depth), collapse = "")
}

dir_depth <- function(x) {
  # length(strsplit(path, "/")[[1]]) - 1L
  purrr::map_int(strsplit(x, ""), function(x) sum(x == "/"))
}

invert_index <- function(x) {
  stopifnot(is.list(x))

  if (length(x) == 0)
    return(list())

  key <- rep(names(x), purrr::map_int(x, length))
  val <- unlist(x, use.names = FALSE)

  split(key, val)
}

rstudio_save_all <- function() {
  if (is_installed("rstudioapi") && rstudioapi::hasFun("documentSaveAll")) {
    rstudioapi::documentSaveAll()
  }
}

is_syntactic <- function(x) x == make.names(x)

auto_quote <- function(x) {
  ifelse(is_syntactic(x), x, paste0("`", x, "`"))
}

str_trim <- function(x) gsub("^\\s+|\\s+$", "", x)

str_squish <- function(x) str_trim(gsub("\\s+", " ", x))

unwrap_purrr_error <- function(code) {
  withCallingHandlers(
    code,
    purrr_error_indexed = function(err) {
      cnd_signal(err$parent)
    }
  )
}

tr_ <- function(...) {
  enc2utf8(gettext(..., domain = "R-pkgdown"))
}

# devtools metadata -------------------------------------------------------

system_file <- function(..., package) {
  if (is.null(devtools_meta(package))) {
    path(system.file(package = package), ...)
  } else {
    path(getNamespaceInfo(package, "path"), "inst", ...)
  }
}

devtools_meta <- function(x) {
  ns <- .getNamespace(x)
  ns[[".__DEVTOOLS__"]]
}

# CLI ---------------------------------------------------------------------

dst_path <- cli::combine_ansi_styles(
  cli::style_bold, cli::col_cyan
)

src_path <- cli::combine_ansi_styles(
  cli::style_bold, cli::col_green
)

writing_file <- function(path, show) {
  path <- as.character(path)
  text <- dst_path(as.character(show))
  cli::cli_inform("Writing {.run [{text}](pkgdown::preview_site(path='{path}'))}")
}

has_internet <- function() {
  getOption("pkgdown.internet", default = TRUE)
}

modify_list <- function(x, y) {
  if (is.null(x)) {
    return(y)
  } else if (is.null(y)) {
    return(x)
  }

  utils::modifyList(x, y)
}

# from https://github.com/r-lib/rematch2/blob/8098bd06f251bfe0f20c0598d90fc20b741d13f8/R/package.R#L47
re_match <- function(text, pattern, perl = TRUE, ...) {

  stopifnot(is.character(pattern), length(pattern) == 1, !is.na(pattern))
  text <- as.character(text)

  match <- regexpr(pattern, text, perl = perl, ...)

  start  <- as.vector(match)
  length <- attr(match, "match.length")
  end    <- start + length - 1L

  matchstr <- substring(text, start, end)
  matchstr[ start == -1 ] <- NA_character_

  res <- data.frame(
    .text = text,
    .match = matchstr
  )

  if (!is.null(attr(match, "capture.start"))) {

    gstart  <- attr(match, "capture.start")
    glength <- attr(match, "capture.length")
    gend    <- gstart + glength - 1L

    groupstr <- substring(text, gstart, gend)
    groupstr[ gstart == -1 ] <- NA_character_
    dim(groupstr) <- dim(gstart)

    res <- cbind(groupstr, res)
  }

  names(res) <- c(attr(match, "capture.names"), ".text", ".match")
  class(res) <- c("tbl_df", "tbl", class(res))
  res
}

# external links can't be an active item
# external links start with http(s)
# but are NOT an absolute URL to the pkgdown site at hand
is_internal_link <- function(links, pkg) {
  if (is.null(pkg$meta$url)) {
    !grepl("https?://", links)
  } else {
    !grepl("https?://", links) | grepl(pkg$meta$url, links)
  }
}

ruler <- function(width = getOption("width")) {
  x <- seq_len(width)
  y <- rep("-", length(x))
  y[x %% 5 == 0] <- "+"
  y[x %% 10 == 0] <- (x[x%%10 == 0] %/% 10) %% 10
  cat(y, "\n", sep = "")
  cat(x %% 10, "\n", sep = "")
}

get_section_level <- function(section) {
  class <- xml2::xml_attr(section, "class")

level <- as.numeric(re_match(class, "level(\\d+)")[[1]])
  level[is.na(level)] <- 0
  level
}

section_id <- function(section) {
  h <- xml2::xml_find_first(section, ".//h1|.//h2|.//h3|.//h4|.//h5|.//h6")
  xml2::xml_attr(h, "id")
}

on_ci <- function() {
  isTRUE(as.logical(Sys.getenv("CI", "false")))
}

# yaml ------------------------------------------------------------

print_yaml <- function(x) {
  structure(x, class = "print_yaml")
}
#' @export
print.print_yaml <- function(x, ...) {
  cat(yaml::as.yaml(x), "\n", sep = "")
}

write_yaml <- function(x, path) {
  yaml::write_yaml(
    x,
    path,
    handlers = list(logical = yaml::verbatim_logical)
  )
}

# Helpers for testing -----------------------------------------------------

xpath_xml <- function(x, xpath = NULL) {
  if (!is.null(xpath)) {
    x <- xml2::xml_find_all(x, xpath)
  }
  structure(x, class = c("pkgdown_xml", class(x)))
}
xpath_contents <- function(x, xpath) {
  x <- xml2::xml_find_all(x, xpath)

  contents <- xml2::xml_contents(x)
  if (length(contents) == 0) {
    NULL
  } else {
    xml2str(contents)
  }
}
xml2str <- function(x) {
  strings <- as.character(x, options = c("format", "no_declaration"))
  paste0(strings, collapse = "")
}

xpath_attr <- function(x, xpath, attr) {
  gsub("\r", "", xml2::xml_attr(xml2::xml_find_all(x, xpath), attr), fixed = TRUE)
}
xpath_text <- function(x, xpath, trim = FALSE) {
  xml2::xml_text(xml2::xml_find_all(x, xpath), trim = trim)
}
xpath_length <- function(x, xpath) {
  length(xml2::xml_find_all(x, xpath))
}
#' @export
print.pkgdown_xml <- function(x, ...) {
  cat(as.character(x, options = c("format", "no_declaration")), sep = "\n")
  invisible(x)
}

test_that("render_rmarkdown copies image files in subdirectories", {
  skip_if_no_pandoc()
  tmp <- dir_create(file_temp())
  pkg <- list(src_path = test_path("."), dst_path = tmp, bs_version = 3)

  expect_output(
    render_rmarkdown(pkg, "assets/vignette-with-img.Rmd", "test.html")
  )
  expect_equal(
    as.character(path_rel(dir_ls(tmp, type = "file", recurse = TRUE), tmp)),
    c("open-graph/logo.png", "test.html")
  )
})

test_that("render_rmarkdown yields useful error", {
  skip_on_cran() # fragile due to pandoc dependency
  skip_if_no_pandoc("2.18")

  tmp <- dir_create(file_temp())
  pkg <- list(src_path = test_path("."), dst_path = tmp, bs_version = 3)

  expect_snapshot(error = TRUE, {
    render_rmarkdown(pkg, "assets/pandoc-fail.Rmd", "test.html",
        output_format = rmarkdown::html_document(pandoc_args = "--fail-if-warnings"))
  })
})

test_that("render_rmarkdown styles ANSI escapes", {
  skip_if_no_pandoc()
  tmp <- dir_create(file_temp())
  pkg <- list(src_path = test_path("."), dst_path = tmp, bs_version = 5)

  expect_output({
    path <- render_rmarkdown(pkg,
      input = "assets/vignette-with-crayon.Rmd",
      output = "test.html"
    )
  })
  html <- xml2::read_html(path)
  expect_snapshot_output(xpath_xml(html, ".//code//span[@class='co']"))
})

test_that("is_internal_link() works", {
  pkg = list(meta = list(url = "https://pkgdown.r-lib.org"))
  expect_false(is_internal_link("https://github.com", pkg = pkg))
  expect_false(is_internal_link("http://github.com", pkg = pkg))
  expect_true(is_internal_link("https://pkgdown.r-lib.org/articles", pkg = pkg))
  expect_true(is_internal_link("reference/index.html", pkg = pkg))
  expect_true(
    all.equal(
      is_internal_link(
        c("reference/index.html", "https://github.com"),
        pkg = pkg
      ),
      c(TRUE, FALSE)
    )
  )
})

test_that("single dt/dd pair converts to simple li", {
  html <- xml2::read_html("<dl></dl>")
  simplify_dls(html)

  expect_equal(xpath_length(html, ".//dl"), 0)
  expect_equal(xpath_length(html, ".//ul"), 1)
})

test_that("single dt/dd pair converts to simple li", {
  html <- xml2::read_html(
    "<dl>
        <dt>a</dt>
        <dd>b</dd>
      </dl>"
  )
  simplify_dls(html)

  expect_equal(xpath_length(html, ".//dl"), 0)
  expect_equal(xpath_text(html, ".//li"), "a: b")
})

test_that("dd with block elements simplifies correctly", {
  html <- xml2::read_html(
    "<dl>
        <dt>a</dt>
        <dd>
          <p>b</p>
          <p>c</p>
        </dd>
      </dl>"
  )
  simplify_dls(html)

  expect_equal(xpath_length(html, ".//dl"), 0)
  expect_equal(xpath_length(html, ".//ul"), 1)
  expect_snapshot(xpath_xml(html, ".//li"))
})

test_that("warns if not applied", {
  html <- xml2::read_html(
    "
      <dl>
        <dt>a</dt>
      </dl>
  "
  )
  expect_snapshot(. <- simplify_dls(html))
})

test_that("correctly detects simple dls", {
  expect_false(is_simple_dl("dt"))
  expect_false(is_simple_dl(c("dd", "dt")))
  expect_false(is_simple_dl(c("dt", "dd", "dt")))
  expect_false(is_simple_dl(c("dd", "dt", "dd", "dt")))

  expect_true(is_simple_dl(c()))
  expect_true(is_simple_dl(c("dt", "dd")))
  expect_true(is_simple_dl(c("dt", "dd", "dt", "dd")))
})

test_that("first header is wrapped in page-header div", {
  html <- xml2::read_html(
    '
    <h1>First</h1>
    <h1>Second</h1>
  '
  )

  tweak_homepage_html(html)
  expect_equal(xpath_attr(html, ".//div", "class"), "page-header")
})

test_that("removes dummy page-header", {
  html <- xml2::read_html(
    '
    <div class="page-header"><h1>Page header</h1></div>
    <h1>Header</h1>
  '
  )

  tweak_homepage_html(html)
  expect_equal(xpath_text(html, ".//h1"), "Header")
})


test_that("can remove first header", {
  html <- xml2::read_html(
    '
    <h1>First</h1>
    <h1>Second</h1>
  '
  )

  tweak_homepage_html(html, strip_header = TRUE)
  expect_equal(xpath_length(html, ".//div"), 0)
})

test_that("can remove logo", {
  # Without link
  html <- xml2::read_html(
    '
    <h1>First <img src="logo.png" /></h1>
  '
  )
  tweak_homepage_html(html, bs_version = 5, logo = "mylogo.png")
  expect_snapshot(xpath_xml(html, ".//div"))

  # With link
  html <- xml2::read_html(
    '
    <h1>First <a><img src="logo.png" /></a></h1>
  '
  )
  tweak_homepage_html(html, bs_version = 5, logo = "mylogo.png")
  expect_snapshot(xpath_xml(html, ".//div"))
})

# badges -------------------------------------------------------------------

test_that("can move badges to sidebar", {
  html <- xml2::read_html(
    '
    <h1>Title</h1>
    <div id="badges">
      <p><a href="x"><img src="y"></a></p>
    </div>
    <div class="dev-status"></div>
  '
  )
  tweak_sidebar_html(html)
  expect_snapshot(xpath_xml(html, ".//div"))
})


test_that("remove badges even if no dev-status div", {
  html <- xml2::read_html(
    '
    <h1>Title</h1>
    <div id="badges">
      <p><a href="x"><img src="y"></a></p>
    </div>
  '
  )
  tweak_sidebar_html(html)
  expect_snapshot(html)
})

test_that("remove dev-status & badges if badges suppress", {
  html <- xml2::read_html(
    '
    <h1>Title</h1>
    <div id="badges">
      <p><a href="x"><img src="y"></a></p>
    </div>
    <div class="dev-status"></div>
  '
  )
  tweak_sidebar_html(html, show_badges = FALSE)
  expect_equal(xpath_length(html, "//div"), 0)
})

test_that("doesn't find badges when they don't exist", {
  expect_equal(badges_extract_text("<h1></h1>"), character())
  expect_equal(badges_extract_text("<p></p>"), character())

  # first paragraph contains non-image components
  expect_equal(
    badges_extract_text(
      '<p><a href="url"><img src="img" alt="alt" /></a>Hi!</p>'
    ),
    character()
  )
})

test_that("finds single badge", {
  expect_equal(
    badges_extract_text('<main><p><a href="x"><img src="y"></a></p></main>'),
    '<a href="x"><img src="y"></a>'
  )
})

test_that("finds badges in #badges div", {
  expect_equal(
    badges_extract_text(
      '<p></p><div id="badges"><a href="x"><img src="y"></a></div>'
    ),
    '<a href="x"><img src="y"></a>'
  )

  # even if there's extra text
  expect_equal(
    badges_extract_text(
      '<p></p><p><div id="badges"><a href="x"><img src="y"></a>Hi!</div></p>'
    ),
    '<a href="x"><img src="y"></a>'
  )
})

test_that("can find badges in comments", {
  html <- '
    <h1>blop</h1>
    <p>I am the first paragraph!</p>
    <!-- badges: start -->
    <p><a href="x"><img src="y"></a></p>
    <!-- badges: end -->
  '
  expect_equal(badges_extract_text(html), '<a href="x"><img src="y"></a>')

  # produced by usethis
  html <- '
    <h1>blop</h1>
    <p>I am the first paragraph!</p>
    <!-- badges: start -->
    <p><a href="x"><img src="y"></a>
    <!-- badges: end -->
    </p>
  '
  expect_equal(badges_extract_text(html), '<a href="x"><img src="y"></a>')
})

test_that("ignores extraneous content", {
  html <- '
    <h1>blop</h1>
    <p>I am the first paragraph!</p>
    <!-- badges: start -->
    <p><a href="x"><img src="y"></a></p>
    <p>a</p>
    <p><a href="b.html">B</a></p>
    <!-- badges: end -->
  '
  expect_equal(badges_extract_text(html), '<a href="x"><img src="y"></a>')
})

test_that("links to vignettes & figures tweaked", {
  html <- xml2::read_html(
    '<body>
    <img src="vignettes/x.png" />
    <img src="../vignettes/x.png" />
    <img src="man/figures/x.png" />
    <img src="../man/figures/x.png" />
  </body>'
  )

  tweak_page(
    html,
    "article",
    list(bs_version = 3, desc = desc::desc(text = ""))
  )
  expect_equal(
    xpath_attr(html, ".//img", "src"),
    c(
      "articles/x.png",
      "../articles/x.png",
      "reference/figures/x.png",
      "../reference/figures/x.png"
    )
  )
})


test_that("reference index table is not altered", {
  html <- xml2::read_html(
    "<body>
    <div class='template-reference-index'>
      <table></table>
    </div>
  </body>"
  )
  pkg <- list(bs_version = 3, desc = desc::desc(text = ""))
  tweak_page(html, "reference-index", pkg)
  expect_equal(xpath_attr(html, ".//table", "class"), NA_character_)
})


test_that("articles get rescue highlighting for non-collapsed output", {
  html <- xml2::read_xml(
    "<body>
    <pre><code>1</code></pre>
    <pre class='downlit'><code>1</code></pre>
    <div class='sourceCode'><pre><code>1</code></pre></div>
  </body>"
  )
  pkg <- list(bs_version = 3, desc = desc::desc(text = ""))
  tweak_page(html, "article", pkg)

  pre <- xml2::xml_find_all(html, ".//pre")
  expect_equal(xml2::xml_find_num(pre, "count(.//span[@class])"), c(1, 0, 0))
})

test_that("toc removed if one or fewer headings", {
  html <- xml2::read_html(
    "<body>
    <main><h2></h2><h2></h2></main>
    <nav id='toc'></nav>
  </body>"
  )
  tweak_useless_toc(html)
  expect_equal(xpath_length(html, ".//nav"), 1)

  html <- xml2::read_html(
    "<body>
    <main><h2></h2></main>
    <nav id='toc'></nav>
  </body>"
  )
  tweak_useless_toc(html)
  expect_equal(xpath_length(html, ".//nav"), 0)

  html <- xml2::read_html(
    "<body>
    <main></main>
    <nav id='toc'></nav>
  </body>"
  )
  tweak_useless_toc(html)
  expect_equal(xpath_length(html, ".//nav"), 0)
})

test_that("toc removed if one or fewer headings", {
  html <- xml2::read_html(
    "<body>
    <main><h2></h2><h2></h2></main>
    <aside><nav id='toc'></nav></aside>
  </body>"
  )
  tweak_useless_toc(html)
  expect_equal(xpath_length(html, ".//nav"), 1)

  html <- xml2::read_html(
    "<body>
    <main><h2></h2></main>
    <aside><nav id='toc'></nav></aside>
  </body>"
  )
  tweak_useless_toc(html)
  expect_equal(xpath_length(html, ".//nav"), 0)

  html <- xml2::read_html(
    "<body>
    <main></main>
    <aside><nav id='toc'></nav></aside>
  </body>"
  )
  tweak_useless_toc(html)
  expect_equal(xpath_length(html, ".//nav"), 0)
})


test_that("sidebar removed if empty", {
  html <- xml2::read_html(
    "<body>
    <main></main>
    <aside><nav id='toc'></nav></aside>
  </body>"
  )
  tweak_useless_toc(html)
  expect_equal(xpath_length(html, ".//div"), 0)
})


test_that("sidebar removed if empty", {
  html <- xml2::read_html(
    "<body>
    <main></main>
    <aside><nav id='toc'></nav></aside>
  </body>"
  )
  tweak_useless_toc(html)
  expect_equal(xpath_length(html, ".//aside"), 0)
})


# rmarkdown ---------------------------------------------------------------

test_that("h1 section headings adjusted to h2 (and so on)", {
  html <- xml2::read_html(
    "
    <div class='page-header'>
      <h1>Title</h1>
      <h4>Author</h4>
    </div>
    <div class='section level1'>
      <h1>1</h1>
      <div class='section level2'>
      <h2>1.1</h2>
      </div>
    </div>
    <div class='section level1'>
      <h1>2</h1>
    </div>
  "
  )
  tweak_rmarkdown_html(html)
  expect_equal(xpath_text(html, ".//h1"), "Title")
  expect_equal(xpath_text(html, ".//h2"), c("1", "2"))
  expect_equal(xpath_text(html, ".//h3"), "1.1")
  expect_equal(xpath_text(html, ".//h4"), "Author")
  expect_equal(
    xpath_attr(html, ".//div", "class"),
    c("page-header", "section level2", "section level3", "section level2")
  )
})

test_that("slashes not URL encoded during relative path conversion", {
  # Since the URL conversion process in tweak_markdown_html() makes calls to
  # fs::path_real() (which requires paths to exist), we create a
  # temporary pkgdown site directory and populate it with an image file, which
  # we can then reference in our test HTML file.

  # Create a site
  pkg <- local_pkgdown_site()

  # Add the referenced image in a subdirectory of vignettes.
  pkg <- pkg_add_kitten(pkg, "vignettes/img/")

  # Get the full path to the image.
  sim_path <- path_real(path(pkg$src_path, "vignettes/img/kitten.jpg"))

  # Simulate an output HTML file referencing the absolute path.
  html <- xml2::read_html(
    sprintf(
      '
    <body>
      <img src="%s" />
    </body>
    ',
      sim_path
    )
  )

  # Function should update the absolute path to a relative path.
  tweak_rmarkdown_html(html, path(pkg$src_path, "vignettes"))

  # Check that the relative path has a non-encoded slash.
  expect_equal(xpath_attr(html, ".//img", "src"), "img/kitten.jpg")
})

test_that("sections with class .tabset are converted to tabsets", {
  skip_on_os("windows") # some line ending problem
  html <- markdown_to_html("
    # Tabset {.tabset .tabset-pills}

    ## Tab 1

    Contents 1

    ## Tab 2

    Contents 2
  ")

  tweak_tabsets(html)

  # strip class for older pandoc compat on GHA
  headings <- xml2::xml_find_all(html, ".//h1")
  xml2::xml_set_attr(headings, "class", NULL)

  expect_snapshot_output(xpath_xml(html, ".//body/div"))
})

test_that("can adjust active tab", {
  skip_on_os("windows") # some line ending problem
  html <- markdown_to_html("
    ## Tabset {.tabset .tabset-pills}

    ### Tab 1

    Contents 1

    ### Tab 2 {.active}

    Contents 2
  ")
  tweak_tabsets(html)

  expect_equal(
    xpath_attr(html, "//div/div/div", "class"),
    c("tab-pane", "active tab-pane")
  )
})

test_that("can fade", {
  html <- markdown_to_html("
    ## Tabset {.tabset .tabset-fade}

    ### Tab 1

    Contents 1

    ### Tab 2 {.active}

    Contents 2
  ")

  tweak_tabsets(html)
  expect_equal(
    xpath_attr(html, "//div/div/div", "class"),
    c("fade tab-pane", "show active fade tab-pane")
  )
})

test_that("can accept html", {
  html <- markdown_to_html("
    ## Tabset {.tabset}

    ### Tab 1 `with_code` {#toc-1}

    Contents 1

    ### <i class=\"fa fab-github\"></i> Tab 2 {#toc-2}

    Contents 2

    ### Normal Tab {#toc-normal}

    Contents of normal tab
  ")

  tweak_tabsets(html)

  expect_match(
    as.character(xpath_xml(html, ".//*[@id = 'toc-1-tab']")),
    "Tab 1 <code>with_code</code>",
    fixed = TRUE
  )

  expect_match(
    as.character(xpath_xml(html, ".//*[@id = 'toc-2-tab']")),
    "<i class=\"fa fab-github\"></i> Tab 2",
    fixed = TRUE
  )

  expect_match(
    as.character(xpath_xml(html, ".//*[@id = 'toc-normal-tab']")),
    ">Normal Tab</button>",
    fixed = TRUE
  )
})

#' Determine if code is executed by pkgdown
#'
#' This is occasionally useful when you need different behaviour by
#' pkgdown and regular documentation.
#'
#' @export
#' @examples
#' in_pkgdown()
in_pkgdown <- function() {
  identical(Sys.getenv("IN_PKGDOWN"), "true")
}

local_envvar_pkgdown <- function(pkg, scope = parent.frame()) {
  withr::local_envvar(
    IN_PKGDOWN = "true",
    LANGUAGE = pkg$lang,
    .local_envir = scope
  )
}

#' Build news section
#'
#' @description
#' A `NEWS.md` will be broken up into versions using level one (`#`) or
#' level two headings (`##`) that (partially) match one of the following forms
#' (ignoring case):
#'
#' * `{package name} 1.3.0`
#' * `{package name} v1.3.0`
#' * `Version 1.3.0`
#' * `Changes in 1.3.0`
#' * `Changes in v1.3.0`
#'
#' @details
#' A [common structure](https://style.tidyverse.org/news.html) for news files
#' is to use a top level heading for each release, and use a second level
#' heading to break up individual bullets into sections.
#'
#' ```yaml
#' # foofy 1.0.0
#'
#' ## Major changes
#'
#' * Can now work with all grooveable grobbles!
#'
#' ## Minor improvements and bug fixes
#'
#' * Printing scrobbles no longer errors (@githubusername, #100)
#'
#' * Wibbles are now 55% less jibbly (#200)
#' ```
#'
#' Issues and contributors will be automatically linked to the corresponding
#' pages on GitHub if the GitHub repo can be discovered from the `DESCRIPTION`
#' (typically from a `URL` entry containing `github.com`)
#'
#' If a version is available on CRAN, the release date will automatically
#' be added to the heading (see below for how to suppress); if not
#' available on CRAN, "Unreleased" will be added.
#'
#' @section YAML config:
#'
#' To automatically link to release announcements, include a `releases`
#' section.
#'
#' ```yaml
#' news:
#'  releases:
#'  - text: "usethis 1.3.0"
#'    href: https://www.tidyverse.org/articles/2018/02/usethis-1-3-0/
#'  - text: "usethis 1.0.0 (and 1.1.0)"
#'    href: https://www.tidyverse.org/articles/2017/11/usethis-1.0.0/
#' ```
#'
#' Control whether news is present on one page or multiple pages with the
#' `one_page` field. The default is `true`.
#'
#' ```yaml
#' news:
#'   one_page: false
#' ```
#'
#' Suppress the default addition of CRAN release dates with:
#'
#' ```yaml
#' news:
#'   cran_dates: false
#' ```
#'
#' @seealso [Tidyverse style for News](https://style.tidyverse.org/news.html)
#'
#' @inheritParams build_articles
#' @export
build_news <- function(pkg = ".",
                       override = list(),
                       preview = NA) {
  pkg <- section_init(pkg, depth = 1L, override = override)
  if (!has_news(pkg$src_path))
    return()

  rule("Building news")
  dir_create(path(pkg$dst_path, "news"))

  switch(news_style(pkg$meta),
    single = build_news_single(pkg),
    multi = build_news_multi(pkg)
  )

  preview_site(pkg, "news", preview = preview)
}

build_news_single <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)
  news <- data_news(pkg)

  render_page(
    pkg,
    "news",
    list(
      contents = purrr::transpose(news),
      pagetitle = tr_("Changelog"),
      source = repo_source(pkg, "NEWS.md")
    ),
    path("news", "index.html")
  )
}

build_news_multi <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)
  news <- data_news(pkg)
  page <- factor(news$page, levels = unique(news$page))

  news_paged <- tibble::tibble(
    version = levels(page),
    file_out = paste0("news-", version, ".html"),
    contents = news[c("html", "version", "anchor")] %>% split(page)
  )

  render_news <- function(version, file_out, contents) {
    render_page(
      pkg,
      "news",
      list(
        version = version,
        contents = rev(purrr::transpose(contents)),
        pagetitle = sprintf(tr_("Version %s"), version)
      ),
      path("news", file_out),
    )
  }
  news_paged %>% purrr::pmap(render_news)

  render_page(
    pkg,
    "news-index",
    list(
      versions = news_paged %>% purrr::transpose(),
      pagetitle = tr_("News")
    ),
    path("news", "index.html")
  )
}

globalVariables(".")

data_news <- function(pkg = list()) {
  html <- markdown_body(path(pkg$src_path, "NEWS.md"))
  xml <- xml2::read_html(html)
  downlit::downlit_html_node(xml)

  sections <- xml2::xml_find_all(xml, "./body/div")
  footnotes <- has_class(sections, "footnotes")
  if (any(footnotes)) {
    warn("Footnotes in NEWS.md are not currently suppoted")
  }
  sections <- sections[!footnotes]

  levels <- sections %>%
    xml2::xml_find_first(".//h1|.//h2|.//h3|.//h4|.//h5") %>%
    xml2::xml_name()
  ulevels <- unique(levels)
  if (!identical(ulevels, "h1") && !identical(ulevels, "h2")) {
    abort(c(
      "Invalid NEWS.md: inconsistent use of section headings.",
      i = "Top-level headings must be either all <h1> or all <h2>.",
      i = "See ?build_news for more details."
    ))
  }
  if (ulevels == "h1") {
    # Bump every heading down a level so to get a single <h1> for the page title
    tweak_section_levels(xml)
  }

  titles <- xml2::xml_text(xml2::xml_find_first(sections, ".//h2"), trim = TRUE)

  versions <- news_version(titles, pkg$package)
  sections <- sections[!is.na(versions)]
  versions <- versions[!is.na(versions)]

  show_dates <- purrr::pluck(pkg, "meta", "news", "cran_dates", .default = !is_testing())
  if (show_dates) {
    timeline <- pkg_timeline(pkg$package)
  } else {
    timeline <- NULL
  }

  html <- sections %>%
    purrr::walk2(
      versions,
      tweak_news_heading,
      timeline = timeline,
      bs_version = pkg$bs_version
    ) %>%
    purrr::map_chr(as.character, options = character()) %>%
    purrr::map_chr(repo_auto_link, pkg = pkg)

  anchors <- xml2::xml_attr(sections, "id")
  news <- tibble::tibble(
    version = versions,
    page = purrr::map_chr(versions, version_page),
    anchor = anchors,
    html = html
  )

  news
}

news_version <- function(x, pkgname) {
  pattern <- paste0("(?x)
    (?:", pkgname, "|version|changes\\ in)
    \\s+   # whitespace
    v?     # optional v followed by
    (?<version>
      (?:\\d+[.-]\\d+)(?:[.-]\\d+)*     # digits, dots, and dashes
      |                             # OR
      \\(development\\ version\\)   # literal used by usethis
    )
  ")
  pieces <- re_match(x, pattern, ignore.case = TRUE)
  gsub("^[(]|[)]$", "", pieces$version)
}

version_page <- function(x) {
  if (x == "development version") {
    return("dev")
  }

  ver <- unclass(package_version(x))[[1]]

  if (length(ver) == 4 && ver[[4]] > 0) {
    "dev"
  } else {
    paste0(ver[[1]], ".", ver[[2]])
  }
}

navbar_news <- function(pkg) {
  releases_meta <- pkg$meta$news$releases
  if (!is.null(releases_meta)) {
    menu(tr_("News"),
      c(
        list(menu_text(tr_("Releases"))),
        releases_meta,
        list(
          menu_spacer(),
          menu_link(tr_("Changelog"), "news/index.html")
        )
      )
    )
  } else if (has_news(pkg$src_path)) {
    menu_link(tr_("Changelog"), "news/index.html")
  }
}

has_news <- function(path = ".") {
  file_exists(path(path, "NEWS.md"))
}

pkg_timeline <- function(package) {
  if (!has_internet()) {
    return(NULL)
  }

  url <- paste0("https://crandb.r-pkg.org/", package, "/all")

  resp <- httr::RETRY("GET", url, quiet = TRUE)
  if (httr::http_error(resp)) {
    return(NULL)
  }

  content <- httr::content(resp)
  timeline <- content$timeline

  data.frame(
    version = names(timeline),
    date = as.Date(unlist(timeline)),
    stringsAsFactors = FALSE,
    row.names = NULL
  )
}

tweak_news_heading <- function(html, version, timeline, bs_version) {
  class <- if (bs_version == 3) "page-header" else "pkg-version"

  h2 <- xml2::xml_find_all(html, ".//h2")
  xml2::xml_set_attr(h2, "class", class)
  xml2::xml_set_attr(h2, "data-toc-text", version)

  # Add release date, if known
  if (!is.null(timeline)) {
    date <- timeline$date[match(version, timeline$version)]
    if (!is.na(date)) {
      if (bs_version == 3) {
        release_str <- paste0(" <small>", date, "</small>")
        release_html <- xml2::xml_find_first(xml2::read_html(release_str), ".//small")
        xml2::xml_add_child(h2, release_html, .where = 1)
      } else {
        release_date <- sprintf(tr_("CRAN release: %s"), date)
        release_str <- paste0("<p class='text-muted'>", release_date, "</p>")
        release_html <- xml2::xml_find_first(xml2::read_html(release_str), ".//p")
        xml2::xml_add_sibling(h2, release_html, .where = "after")
      }
    }
  }

  tweak_news_anchor(html, version)

  invisible()
}

# Manually de-duplicate repeated section anchors using version
tweak_news_anchor <- function(html, version) {
  div <- xml2::xml_find_all(html, ".//div")
  div <- div[has_class(div, "section")]

  id <- xml2::xml_attr(div, "id")
  id <- gsub("-[0-9]+", "", id) # remove pandoc de-duplication suffixes
  id <- paste0(id, "-", gsub("[^a-z0-9]+", "-", version)) # . breaks scrollspy
  xml2::xml_attr(div, "id") <- id

  invisible()
}

tweak_section_levels <- function(html) {
  sections <- xml2::xml_find_all(html, ".//div[contains(@class, 'section level')]")

  # Update headings
  xml2::xml_set_name(xml2::xml_find_all(sections, ".//h5"), "h6")
  xml2::xml_set_name(xml2::xml_find_all(sections, ".//h4"), "h5")
  xml2::xml_set_name(xml2::xml_find_all(sections, ".//h3"), "h4")
  xml2::xml_set_name(xml2::xml_find_all(sections, ".//h2"), "h3")
  xml2::xml_set_name(xml2::xml_find_all(sections, ".//h1"), "h2")

  # Update section
  xml2::xml_attr(sections, "class") <- paste0("section level", get_section_level(sections) + 1)

  invisible()
}

news_style <- function(meta) {
  one_page <- purrr::pluck(meta, "news", "one_page") %||%
    purrr::pluck(meta, "news", 1, "one_page") %||%
    TRUE

  if (one_page) "single" else "multi"
}

test_that("data_news works as expected for h1 & h2", {
  skip_if_no_pandoc()

  lines_h1 <- c(
    "# testpackage 1.0.0.9000", "",
    "* bullet (#222 @someone)", "",
    "# testpackage 1.0.0", "",
    "## sub-heading", "",
    "* first thing (#111 @githubuser)", "",
    "* second thing", ""
  )
  lines_h2 <- gsub("^#", "##", lines_h1)

  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "NEWS.md", lines_h1)
  expect_snapshot_output(data_news(pkg)[c("version", "page", "anchor")])

  pkg <- pkg_add_file(pkg, "NEWS.md", lines_h2)
  expect_snapshot_output(data_news(pkg)[c("version", "page", "anchor")])
})

test_that("news is syntax highlighted once", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "NEWS.md", c(
    "# testpackage 1.0.0.9000",
    "```r",
    "x <- 1",
    "```"
  ))
  suppressMessages(build_news(pkg, preview = FALSE))
  html <- xml2::read_html(path(pkg$dst_path, "news", "index.html"))
  expect_equal(xpath_text(html, "//code"), "x <- 1")
})

test_that("multi-page news are rendered", {
  skip_if_no_pandoc()

  pkg <- local_pkgdown_site(meta = list(news = list(one_page = FALSE)))
  pkg <- pkg_add_file(pkg, "NEWS.md", c(
    "# testpackage 2.0", "",
    "* bullet (#222 @someone)", "",
    "# testpackage 1.1", "",
    "* bullet (#222 @someone)", "",
    "# testpackage 1.0.1", "",
    "* bullet (#222 @someone)", "",
    "# testpackage 1.0.0", "",
    "## sub-heading", "",
    "* first thing (#111 @githubuser)", "",
    "* second thing"
  ))

  expect_snapshot(data_news(pkg)[c("version", "page", "anchor")])
  expect_snapshot(build_news(pkg))

  # test that index links are correct
  lines <- xml2::read_html(path(pkg$dst_path, "news", "index.html"))
  expect_equal(
    xpath_attr(lines, "//main//a", "href"),
    c("news-2.0.html", "news-1.1.html", "news-1.0.html")
  )
  expect_equal(
    xpath_text(lines, "//main//a"),
    c("Version 2.0", "Version 1.1", "Version 1.0")
  )

  # test single page structure
  lines <- xml2::read_html(path(pkg$dst_path, "news", "news-1.0.html"))
  expect_equal(xpath_text(lines, ".//h1"), "Version 1.0")
  expect_equal(xpath_text(lines, ".//main//h2"), c("testpackage 1.0.0", "testpackage 1.0.1"))
})

test_that("github links are added to news items", {
  skip_if_no_pandoc()

  pkg <- local_pkgdown_site(meta = list(
    repo = list(
      url = list(
        home = "https://github.com/r-lib/pkgdown",
        user = "https://github.com/",
        issue = "https://github.com/r-lib/pkgdown/issues/"
      )
    )
  ))

  pkg <- pkg_add_file(pkg, "NEWS.md", c(
    "# testpackage 0.1.0", "",
    "## Major changes", "",
    "- Bug fixes (@hadley, #100)", "",
    "- Merges (@josue-rodriguez)"
  ))
  news_tbl <- data_news(pkg)
  html <- xml2::read_xml(news_tbl$html)

  expect_equal(
    xpath_attr(html, ".//a", "href"),
    c(
      "https://github.com/hadley",
      "https://github.com/r-lib/pkgdown/issues/100",
      "https://github.com/josue-rodriguez"
    )
  )
})

test_that("pkg_timeline fails cleanly for unknown package", {
  skip_on_cran()
  expect_null(pkg_timeline("__XYZ__"))
})

test_that("pkg_timeline works for package that have been archived", {
  skip_on_cran()
  expect_no_error(pkg_timeline("PF"))
})

test_that("correct timeline for first ggplot2 releases", {
  skip_on_cran()

  timeline <- pkg_timeline("ggplot2")[1:3, ]
  expected <- data.frame(
    version = c("0.5", "0.5.1", "0.5.2"),
    date = as.Date(c("2007-06-01", "2007-06-09", "2007-06-18"))
  )

  expect_equal(timeline, expected)
})

# news_title and version_page -----------------------------------------------

test_that("can recognise common forms of title", {
  # Variants in beginning
  version <- news_version(c(
    "foo 1.3.0",
    "foo v1.3.0",
    "foo 1.3.0",
    "VERSION 1.3.0",
    "changes in 1.3.0",
    "changes in foo version 1.3.0"
  ), "foo")
  expect_equal(version, rep("1.3.0", length(version)))

  # Variants in version spec
  expect_equal(
    news_version(c("foo 1-2", "foo 1-2-3", "foo 1-2-3-4"), "foo"),
    c("1-2", "1-2-3", "1-2-3-4")
  )

  expect_equal(
    news_version("foo (development version)", "foo"),
    "development version"
  )
})

test_that("correctly collapses version to page for common cases", {
  versions <- c("1.0.0", "1.0.0.0", "1.0.0.9000", "development version")
  pages <- purrr::map_chr(versions, version_page)
  expect_equal(pages, c("1.0", "1.0", "dev", "dev"))
})

# Tweaks ------------------------------------------------------------------

test_that("sections tweaked down a level", {
  html <- xml2::read_xml("<body>
    <div class='section level1'><h1></h1></div>
    <div class='section level2'><h2></h2></div>
    <div class='section level3'><h3></h3></div>
    <div class='section level4'><h4></h4></div>
    <div class='section level5'><h5></h5></div>
  </body>")
  tweak_section_levels(html)
  expect_equal(xml2::xml_name(xpath_xml(html, "//div/*")), paste0("h", 2:6))
  expect_equal(xpath_attr(html, "//div", "class"), paste0("section level", 2:6))
})

test_that("anchors de-duplicated with version", {
  html <- xml2::read_xml("<body>
    <div class='section' id='x-1'>Heading</div>
  </body>")
  tweak_news_anchor(html, "1.0")

  expect_equal(xpath_attr(html, ".//div", "id"), "x-1-0")
})

test_that("news headings get class and release date", {
  timeline <- tibble::tibble(version = "1.0", date = "2020-01-01")

  html <- xml2::read_xml("<div><h2></h2></div>")
  tweak_news_heading(html, version = "1.0", timeline = timeline, bs_version = 3)
  expect_snapshot_output(xpath_xml(html, "//div"))

  html <- xml2::read_xml("<div><h2></h2></div>")
  tweak_news_heading(html, version = "1.0", timeline = timeline, bs_version = 4)
  expect_snapshot_output(xpath_xml(html, "//div"))
})

# Header checks ----------------------------------------------------------
test_that("clear error for bad hierarchy - bad nesting", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "NEWS.md", c(
    "### testpackage 1.0.0.9000", "",
    "* bullet (#222 @someone)", "",
    "# testpackage 1.0.0", "",
    "## sub-heading", "",
    "* first thing (#111 @githubuser)", "",
    "* second thing", ""
  ))

  expect_snapshot(data_news(pkg), error = TRUE)
})

test_that("clear error for bad hierarchy - h3", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "NEWS.md", c(
    "### testpackage 1.0.0.9000", "",
    "* bullet (#222 @someone)", "",
    "### testpackage 1.0.0", "",
    "#### sub-heading", "",
    "* first thing (#111 @githubuser)", "",
    "* second thing", ""
  ))

  expect_snapshot(data_news(pkg), error = TRUE)
})

test_that("news can contain footnotes", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "NEWS.md", c(
    "## testpackage 1.0.0.9000",
    "",
    "* bullet",
    "",
    "* inline footnote^[used to fail] ",
    ""
  ))

  expect_snapshot(x <- data_news(pkg))
})

test_that("data_news warns if no headings found", {
  skip_if_no_pandoc()

  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "NEWS.md", c(
    "# mypackage", "",
    "## mypackage foo", "",
    "## mypackage bar", ""
  ))
  expect_snapshot(. <- data_news(pkg))
})

data_footer <- function(pkg = ".", call = caller_env()) {
  pkg <- as_pkgdown(pkg)

  config_pluck_list(pkg, "footer", call = call)
  meta_components <- config_pluck_list(pkg, "footer.components", call = call)
  components <- modify_list(footnote_components(pkg, call = call), meta_components)

  meta_structure <- config_pluck_list(pkg, "footer.structure", call = call)
  structure <- modify_list(footnote_structure(), meta_structure)

  left <- markdown_text_block(pkg, paste0(components[structure$left], collapse = " "))
  right <- markdown_text_block(pkg, paste0(components[structure$right], collapse = " "))

  list(left = left, right = right)
}

footnote_components <- function(pkg = ".", call = caller_env()) {
  pkg <- as_pkgdown(pkg)

  # Authors
  roles <- config_pluck_character(
    pkg,
    "authors.footer.roles",
    default = default_roles(),
    call = call
  )
  authors <- data_authors(pkg, roles = roles)$main
  authors_str <- paste(purrr::map_chr(authors, "name"), collapse = ", ")

  prefix <- config_pluck_string(
    pkg,
    "authors.footer.text",
    default = tr_("Developed by"),
    call = call
  )
  developed_by <- paste0(trimws(prefix), " ", authors_str, ".")

  # pkgdown
  built_with <- sprintf(
    tr_('Site built with <a href="%s">pkgdown</a> %s.'),
    "https://pkgdown.r-lib.org/",
    utils::packageVersion("pkgdown")
  )

  print_yaml(list(
    developed_by = developed_by,
    built_with = built_with,
    package = pkg[["package"]]
  ))
}

footnote_structure <- function() {
  print_yaml(list(
    left = "developed_by",
    right = "built_with"
  ))
}

test_that("works by default", {
  pkg <- structure(
    list(
      desc = desc::desc(text = "Authors@R: person('First', 'Last', role = 'cre')")
    ),
    class = "pkgdown"
  )
  footer <- data_footer(pkg)
  footer$right <- gsub(packageVersion("pkgdown"), "{version}", footer$right, fixed = TRUE)

  expect_snapshot_output(footer)
})

test_that("includes package component", {
  pkg <- structure(
    list(
      package = "noodlr",
      desc = desc::desc(text = "Authors@R: person('First', 'Last', role = 'cre')"),
      meta = list(
        footer = list(
          structure = list(left = "package")
        )
      )
    ),
    class = "pkgdown"
  )
  expect_equal(data_footer(pkg)$left, "<p>noodlr</p>")
})

test_that("can use custom components", {
  pkg <- structure(list(
    desc = desc::desc(text = "Authors@R: person('a', 'b', roles = 'cre')"),
    meta = list(
      footer = list(
        structure = list(left = "test"),
        components = list(test = "_test_")
      )
    )),
    class = "pkgdown"
  )
  expect_equal(data_footer(pkg)$left, "<p><em>test</em></p>")
})

test_that("multiple components are pasted together", {
  pkg <- structure(list(
    desc = desc::desc(text = "Authors@R: person('a', 'b', roles = 'cre')"),
    meta = list(
      footer = list(
        structure = list(left = c("a", "b")),
        components = list(a = "a", b = "b")
      )
    )),
    class = "pkgdown"
  )
  expect_equal(data_footer(pkg)$left, "<p>a b</p>")
})

config_pluck <- function(pkg, path, default = NULL) {
  check_string(path, allow_empty = FALSE, .internal = TRUE)

  where <- strsplit(path, ".", fixed = TRUE)[[1]]
  purrr::pluck(pkg$meta, !!!where, .default = default)
}

config_pluck_list <- function(
  pkg,
  path,
  has_names = NULL,
  default = NULL,
  call = caller_env()
) {
  check_string(path, allow_empty = FALSE, .internal = TRUE)

  x <- config_pluck(pkg, path, default)
  config_check_list(
    x,
    has_names = has_names,
    error_path = path,
    error_pkg = pkg,
    error_call = call
  )
}

config_pluck_character <- function(
  pkg,
  path,
  default = character(),
  call = caller_env()
) {
  x <- config_pluck(pkg, path, default)
  config_check_character(
    x,
    error_path = path,
    error_pkg = pkg,
    error_call = call
  )
}

config_pluck_string <- function(
  pkg,
  path,
  default = NULL,
  call = caller_env()
) {
  x <- config_pluck(pkg, path, default)
  config_check_string(
    x,
    error_path = path,
    error_pkg = pkg,
    error_call = call
  )
}

config_pluck_markdown_inline <- function(
  pkg,
  path,
  default = NULL,
  call = caller_env()
) {
  text <- config_pluck_string(pkg, path, default, call = call)
  markdown_text_inline(pkg, text, error_path = path, error_call = call)
}

config_pluck_markdown_block <- function(
  pkg,
  path,
  default = NULL,
  call = caller_env()
) {
  text <- config_pluck_string(pkg, path, default, call = call)
  markdown_text_block(pkg, text)
}

config_pluck_bool <- function(pkg, path, default = NULL, call = caller_env()) {
  x <- config_pluck(pkg, path, default)
  config_check_bool(
    x,
    error_path = path,
    error_pkg = pkg,
    error_call = call
  )
}

config_pluck_number_whole <- function(
  pkg,
  path,
  default = NULL,
  call = caller_env()
) {
  x <- config_pluck(pkg, path, default)
  config_check_number_whole(
    x,
    error_path = path,
    error_pkg = pkg,
    error_call = call
  )
}

# checks ---------------------------------------------------------------------

config_check_character <- function(
  x,
  error_pkg,
  error_path,
  error_call = caller_env()
) {
  if (is.character(x) || is.null(x)) {
    x
  } else if (identical(x, list())) {
    character()
  } else {
    config_abort_type(
      must_be = "a character vector",
      not = x,
      error_pkg = error_pkg,
      error_path = error_path,
      error_call = error_call
    )
  }
}

config_check_string <- function(
  x,
  error_pkg,
  error_path,
  error_call = caller_env()
) {
  if (is_string(x) || is.null(x)) {
    x
  } else {
    config_abort_type(
      must_be = "a string",
      not = x,
      error_pkg = error_pkg,
      error_path = error_path,
      error_call = error_call
    )
  }
}

config_check_bool <- function(
  x,
  error_pkg,
  error_path,
  error_call = caller_env()
) {
  if (is_bool(x) || is.null(x)) {
    x
  } else {
    config_abort_type(
      must_be = "true or false",
      not = x,
      error_pkg = error_pkg,
      error_path = error_path,
      error_call = error_call
    )
  }
}

config_check_number_whole <- function(
  x,
  error_pkg,
  error_path,
  error_call = caller_env()
) {
  is_whole <- (0 ==
    (exit_code <- .standalone_types_check_dot_call(
      ffi_standalone_check_number_1.0.7,
      x,
      allow_decimal = FALSE,
      min = NULL,
      max = NULL,
      allow_infinite = FALSE,
      allow_na = FALSE,
      allow_null = TRUE
    )))

  if (is_whole) {
    x
  } else {
    config_abort_type(
      must_be = "a whole number",
      not = x,
      error_pkg = error_pkg,
      error_path = error_path,
      error_call = error_call
    )
  }
}

config_check_list <- function(
  x,
  has_names = NULL,
  error_pkg,
  error_path,
  error_call = caller_env()
) {
  if (is_list(x) || is.null(x)) {
    if (!is.null(has_names) && !all(has_name(x, has_names))) {
      missing <- setdiff(has_names, names(x))
      config_abort(
        error_pkg,
        c(
          "{.field {error_path}} must have {cli::qty(has_names)} component{?s} {.str {has_names}}.",
          "{length(missing)} missing component{?s}: {.str {missing}}."
        ),
        call = error_call
      )
    } else {
      x
    }
  } else {
    not <- obj_type_friendly(x)
    config_abort(
      error_pkg,
      "{.field {error_path}} must be a list, not {not}.",
      call = error_call
    )
  }
}

config_abort_type <- function(must_be, not, error_pkg, error_path, error_call) {
  not_str <- obj_type_friendly(not)
  config_abort(
    error_pkg,
    "{.field {error_path}} must be {must_be}, not {not_str}.",
    call = error_call
  )
}

# generic error ---------------------------------------------------------------

config_abort <- function(
  pkg,
  message,
  path = NULL,
  ...,
  call = caller_env(),
  .envir = caller_env()
) {
  message <- config_message(pkg, message, path)
  cli::cli_abort(message, ..., call = call, .envir = .envir)
}

config_warn <- function(
  pkg,
  message,
  path = NULL,
  ...,
  call = caller_env(),
  .envir = caller_env()
) {
  message <- config_message(pkg, message, path)
  cli::cli_warn(message, ..., call = call, .envir = .envir)
}

config_message <- function(pkg, message, path = NULL) {
  # Not all projects necessary have a _pkgdown.yml (#2542)
  path <- path %||% pkgdown_config_path(pkg) %||% "_pkgdown.yml"
  if (is_absolute_path(path)) {
    path_label <- path_rel(path, pkg$src_path)
  } else {
    path_label <- path
  }

  link <- cli::style_hyperlink(path_label, paste0("file://", path))
  message[[1]] <- paste0("In ", link, ", ", message[[1]])
  message
}

test_that("config_pluck_character coerces empty values to character", {
  pkg <- local_pkgdown_site(meta = list(x = NULL, y = list()))
  expect_equal(config_pluck_character(pkg, "x"), character())
  expect_equal(config_pluck_character(pkg, "y"), character())
  expect_equal(config_pluck_character(pkg, "z"), character())
})

test_that("config_pluck_character generates informative error", {
  pkg <- local_pkgdown_site(meta = list(x = 1))
  expect_snapshot(config_pluck_character(pkg, "x"), error = TRUE)
})

test_that("config_pluck_string generates informative error", {
  pkg <- local_pkgdown_site(meta = list(x = 1))
  expect_snapshot(config_pluck_string(pkg, "x"), error = TRUE)
})

# checkers --------------------------------------------------------------------

test_that("config_check_list() returns list if ok", {
  x <- list(x = 1, y = 2)
  expect_equal(config_check_list(x), x)
  expect_equal(config_check_list(x, has_names = "x"), x)
  expect_equal(config_check_list(x, has_names = c("x", "y")), x)
})

test_that("config_check_list gives informative errors", {
  # Avoid showing unneeded call details in snapshot
  pkg <- local_pkgdown_site()
  config_check_list_ <- function(...) {
    config_check_list(..., error_pkg = pkg, error_path = "path")
  }

  expect_snapshot(error = TRUE, {
    config_check_list_(1, has_names = "x")
    config_check_list_(list(x = 1, y = 1), has_names = c("y", "z"))
  })
})

check_built_site <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)

  cli::cli_rule("Checking for problems")
  index_path <- path_index(pkg)
  if (!is.null(index_path)) {
    check_missing_images(pkg, index_path, "index.html")
  }
}

check_missing_images <- function(pkg, src_path, dst_path) {
  html <- xml2::read_html(path(pkg$dst_path, dst_path), encoding = "UTF-8")
  img <- xml2::xml_find_all(html, ".//img")
  src <- xml2::xml_attr(img, "src")

  rel_src <- xml2::url_unescape(src[xml2::url_parse(src)$scheme == ""])
  rel_path <- path_norm(path(path_dir(dst_path), rel_src))
  exists <- file_exists(path(pkg$dst_path, rel_path))

  if (any(!exists)) {
    paths <- rel_src[!exists]
    cli::cli_inform(c(
      "Missing images in {.file {path_rel(src_path, pkg$src_path)}}: {.file {paths}}",
      i = "pkgdown can only use images in {.file man/figures} and {.file vignettes}"
    ))
  }

  alt <- xml2::xml_attr(img, "alt")
  if (anyNA(alt)) {
    problems <- src[is.na(alt)]
    problems[grepl("^data:image", problems)] <- "<base64 encoded image>"
    cli::cli_inform(c(
      x = "Missing alt-text in {.file {path_rel(src_path, pkg$src_path)}}",
      set_names(problems, "*"),
      i = "Learn more in {.vignette pkgdown::accessibility}."
    ))
  }
}

build_quarto_articles <- function(pkg = ".", article = NULL, quiet = TRUE) {
  pkg <- as_pkgdown(pkg)

  qmds <- pkg$vignettes[pkg$vignettes$type == "qmd", ]
  if (!is.null(article)) {
    qmds <- qmds[qmds$name == article, ]
  }
  if (nrow(qmds) == 0) {
    return()
  }
  check_installed("quarto")
  if (quarto::quarto_version() < "1.5") {
    cli::cli_abort("Quarto articles require version 1.5 and above.", call = NULL)
  }
  # Let user know what's happening
  old_digest <- purrr::map_chr(path(pkg$dst_path, qmds$file_out), file_digest)
  for (file in qmds$file_in) {
    cli::cli_inform("Reading {src_path(file)}")
  }
  cli::cli_inform("Running {.code quarto render}")

  # If needed, temporarily make a quarto project so we can build entire dir
  if (is.null(article)) {
    project_path <- path(pkg$src_path, "vignettes", "_quarto.yaml")
    if (!file_exists(project_path)) {
      yaml::write_yaml(list(project = list(render = list("*.qmd"))), project_path)
      withr::defer(file_delete(project_path))
    }
  }

  if (is.null(article)) {
    src_path <- path(pkg$src_path, "vignettes")
  } else {
    src_path <- path(pkg$src_path, qmds$file_in)
  }
  output_dir <- quarto_render(pkg, src_path, quiet = quiet)

  # Read generated data from quarto template and render into pkgdown template
  unwrap_purrr_error(purrr::walk2(qmds$file_in, qmds$file_out, function(input_file, output_file) {
    built_path <- path(output_dir, path_rel(output_file, "articles"))
    if (!file_exists(built_path)) {
      cli::cli_abort("No built file found for {.file {input_file}}")
    }
    if (path_ext(output_file) == "html") {
      data <- data_quarto_article(pkg, built_path, input_file)
      render_page(pkg, "quarto", data, output_file, quiet = TRUE)

      update_html(path(pkg$dst_path, output_file), tweak_quarto_html)
    } else {
      file_copy(built_path, path(pkg$dst_path, output_file), overwrite = TRUE)
    }
  }))

  # Report on which files have changed
  new_digest <- purrr::map_chr(path(pkg$dst_path, qmds$file_out), file_digest)
  changed <- new_digest != old_digest
  for (file in qmds$file_out[changed]) {
    writing_file(path(pkg$dst_path, file), file)
  }

  # Copy resources
  resources <- setdiff(
    dir_ls(output_dir, recurse = TRUE, type = "file"),
    path(output_dir, path_rel(qmds$file_out, "articles"))
  )
  file_copy_to(
    src_paths = resources,
    dst_paths = path(pkg$dst_path, "articles", path_rel(resources, output_dir)),
    src_root = output_dir,
    dst_root = pkg$dst_path,
    src_label = NULL
  )

  invisible()
}

quarto_render <- function(pkg, path, quiet = TRUE, frame = caller_env()) {
  # Override default quarto format
  metadata_path <- withr::local_tempfile(
    fileext = ".yml",
    pattern = "pkgdown-quarto-metadata-",
  )
  write_yaml(quarto_format(pkg), metadata_path)

  output_dir <- withr::local_tempdir("pkgdown-quarto-", .local_envir = frame)
  quarto::quarto_render(
    path,
    metadata_file = metadata_path,
    execute_dir = output_dir,
    quarto_args = c("--output-dir", output_dir),
    quiet = quiet,
    as_job = FALSE
  )

  output_dir
}

quarto_format <- function(pkg) {
  list(
    lang = pkg$lang,
    format = list(
      html = list(
        template = system_file("quarto", "template.html", package = "pkgdown"),
        minimal = TRUE,
        theme = "none",
        `html-math-method` = config_math_rendering(pkg),
        `embed-resources` = FALSE,
        `citations-hover` = TRUE,
        `link-citations` = TRUE,
        `section-divs` = TRUE,
        toc = FALSE # pkgdown generates with js
      )
    )
  )
}

data_quarto_article <- function(pkg, path, input_path) { 
  html <- xml2::read_html(path, encoding = "UTF-8")
  meta_div <- xml2::xml_find_first(html, "//body/div[@class='meta']")

  # Manually drop any jquery deps
  head <- xpath_xml(html, "//head/script|//head/link")
  head <- head[!grepl("jquery", xml2::xml_attr(head, "src"))]

  list(
    pagetitle = escape_html(xpath_text(html, "//head/title")),
    toc = TRUE, 
    source = repo_source(pkg, input_path),
    includes = list(
      head = xml2str(head),
      before = xpath_contents(html, "//body/div[@class='includes-before']"),
      after = xpath_contents(html, "//body/div[@class='includes-after']"),
      style = xpath_text(html, "//head/style")
    ),
    meta = list(
      title = xpath_contents(meta_div, "./h1"),
      subtitle = xpath_contents(meta_div, "./p[@class='subtitle']"),
      author = xpath_contents(meta_div, "./p[@class='author']"),
      date = xpath_contents(meta_div, "./p[@class='date']"),
      abstract = xpath_contents(meta_div, "./div[@class='abstract']")
    ),
    body = xpath_contents(html, "//main")
  )
}

tweak_quarto_html <- function(html) { 
  # If top-level headings use h1, move everything down one level
  h1 <- xml2::xml_find_all(html, "//h1")
  if (length(h1) > 1) {
    tweak_section_levels(html)
  }
}

check_yaml_has <- function(missing, where, pkg) {
  if (length(missing) == 0) {
    return()
  }

  missing_components <- lapply(missing, function(x) c(where, x))
  missing_fields <- pkgdown_fields(pkg, missing_components)

  abort(paste0(
    "Can't find component", if (length(missing) > 1) "s", " ",
    missing_fields, "."
  ))
}

yaml_character <- function(pkg, where) {
  x <- purrr::pluck(pkg$meta, !!!where)

  if (identical(x, list()) || is.null(x)) {
    character()
  } else if (is.character(x)) {
    x
  } else {
    abort(paste0(pkgdown_field(pkg, where), " must be a character vector"))
  }
}

pkgdown_field <- function(pkg, field) {
  pkgdown_fields(pkg, list(field))
}

pkgdown_fields <- function(pkg, fields, join = ", ") {
  fields <- purrr::map_chr(fields, ~ paste0(cli::style_bold(.x), collapse = "."))
  fields_str <- paste0(fields, collapse = join)

  config_path <- pkgdown_config_path(pkg$src_path)

  if (is.null(config_path)) {
    fields_str
  } else {
    config <- src_path(fs::path_rel(config_path, pkg$src_path))
    paste0(fields_str, " in ", config)
  }
}

# print helper ------------------------------------------------------------

print_yaml <- function(x) {
  structure(x, class = "print_yaml")
}
#' @export
print.print_yaml <- function(x, ...) {
  cat(yaml::as.yaml(x), "\n", sep = "")
}

# IO ----------------------------------------------------------------------

write_yaml <- function(x, path) {
  write_lines(yaml::as.yaml(x), path = path)
}

test_that("pkgdown_field(s) produces useful description", {
  pkg <- local_pkgdown_site()
  file_touch(file.path(pkg$src_path, "_pkgdown.yml"))

  expect_snapshot({
    pkgdown_field(pkg, c("a", "b"))
    pkgdown_fields(pkg, list(c("a", "b"), "c"))
  })
})

test_that("pkgdown_field(s) produces useful description", {
  pkg <- local_pkgdown_site()
  file_touch(file.path(pkg$src_path, "_pkgdown.yml"))

  expect_snapshot(error = TRUE, {
    check_yaml_has("x", where = "a", pkg = pkg)
    check_yaml_has(c("x", "y"), where = "a", pkg = pkg)
  })
})

build_docsearch_json <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)

  index_name <- config_pluck_string(pkg, "template.params.docsearch.index_name")
  if (is.null(index_name)) {
    return()
  }

  data <- list(
    index = index_name,
    package = pkg$package,
    url = config_pluck_string(pkg, "url")
  )

  template <- find_template("config", "docsearch", ext = ".json", pkg = pkg)

  json <- render_template(template, data)

  json_path <- path(pkg$dst_path, "docsearch.json")

  write_if_different(pkg, json, json_path, check = FALSE)
}

build_sitemap <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)

  url <- paste0(config_pluck_string(pkg, "url"), "/")
  if (is.null(url)) {
    return()
  }

  cli::cli_rule("Building sitemap")
  if (pkg$development$in_dev && pkg$bs_version > 3) {
    url <- paste0(url, pkg$prefix)
  }

  urls <- paste0(url, get_site_paths(pkg))
  doc <- paste0(
    "<urlset xmlns = 'http://www.sitemaps.org/schemas/sitemap/0.9'>\n",
    paste0("<url><loc>", escape_html(urls), "</loc></url>\n", collapse = ""),
    "</urlset>\n"
  )

  write_if_different(pkg, doc, "sitemap.xml", check = FALSE)
  invisible()
}

#' Build search index
#'
#' @description
#' Generate a JSON search index from the built site. This is used by
#' [fuse.js](https://www.fusejs.io/) to provide a javascript powered search for
#' BS5 powered pkgdown sites.
#'
#' NB: `build_search()` is called automatically by [build_site()]; you don't
#' need call it yourself. This page documents how it works and its customisation
#' options.
#'
#' # YAML config
#' You can exclude some paths from the search index using `search.exclude`.
#' Below we exclude the changelog from the search index:
#'
#' ```yaml
#' search:
#'   exclude: ['news/index.html']
#' ```
#' # Debugging and local testing
#'
#' Locally (as opposed to on GitHub Pages or Netlify for instance),
#' search won't work if you simply use pkgdown preview of the static files.
#' You can use `servr::httw("docs")` instead.
#'
#' If search is not working, run `pkgdown::pkgdown_sitrep()` to eliminate
#' common issues such as the absence of URL in the pkgdown configuration file
#' of your package.
#'
#' @inheritParams build_articles
#' @export
#'
build_search <- function(pkg = ".", override = list()) {
  pkg <- section_init(pkg, override = override)
  cli::cli_rule("Building search index")

  search_index <- build_search_index(pkg)
  jsonlite::write_json(
    search_index,
    path(pkg$dst_path, "search.json"),
    auto_unbox = TRUE
  )
}

build_search_index <- function(pkg) {
  paths <- get_site_paths(pkg)
  paths <- paths[
    !paths %in% c("404.html", "articles/index.html", "reference/index.html")
  ]

  # user-defined exclusions
  paths <- paths[!paths %in% config_pluck_character(pkg, "search.exclude")]

  if ("news/index.html" %in% paths) {
    index <- lapply(
      paths[paths != "news/index.html"],
      file_search_index,
      pkg = pkg
    )
    index <- unlist(index, recursive = FALSE)
    index <- c(index, news_search_index("news/index.html", pkg = pkg))
  } else {
    index <- lapply(paths, file_search_index, pkg = pkg)
    index <- unlist(index, recursive = FALSE)
  }

  # Make URLs absolute if possible
  url <- config_pluck_string(pkg, "url", default = "")
  fix_path <- function(x) {
    x$path <- sprintf("%s%s", url, x$path)
    x
  }
  purrr::map(index, fix_path)
}

news_search_index <- function(path, pkg) {
  html <- xml2::read_html(path(pkg$dst_path, path), encoding = "UTF-8")

  # Get contents minus logo
  node <- xml2::xml_find_all(html, ".//main")
  xml2::xml_remove(xml2::xml_find_first(
    node,
    ".//img[contains(@class, 'pkg-logo')]"
  ))
  sections <- xml2::xml_find_all(node, ".//*[contains(@class, 'section')]")

  purrr::pmap(
    list(
      sections,
      purrr::map_chr(sections, get_headings, depth = 4),
      title = purrr::map_chr(sections, get_version)
    ),
    bs4_index_data,
    dir = "Changelog",
    path = paste0("/", pkg$prefix, path)
  )
}

file_search_index <- function(path, pkg) {
  html <- xml2::read_html(path(pkg$dst_path, path), encoding = "UTF-8")
  # Get page title
  title_element <- xml2::xml_find_first(html, ".//meta[@property='og:title']")
  title <- xml2::xml_attr(title_element, "content")

  # Get contents minus logo
  node <- xml2::xml_find_all(html, ".//main")
  xml2::xml_remove(xml2::xml_find_first(
    node,
    ".//img[contains(@class, 'pkg-logo')]"
  ))
  sections <- xml2::xml_find_all(
    node,
    ".//div[contains(@class, 'section')]|.//section"
  )

  purrr::pmap(
    list(
      sections,
      purrr::map_chr(sections, get_headings, depth = 3),
      title = title
    ),
    bs4_index_data,
    dir = get_dir(path),
    path = paste0("/", pkg$prefix, path)
  )
}
# Directory parts (where in the site)
get_dir <- function(path) {
  dir <- path_dir(path)
  if (dir == ".") {
    return("")
  }
  paste(capitalise(unlist(path_split(dir))), collapse = " > ")
}
# Headings (where in the page)
get_headings <- function(section, depth) {
  level <- get_section_level(section)
  if (level < depth) {
    return("")
  }

  headings <- purrr::map_chr(
    seq(depth - 1, level - 1),
    get_h,
    section = section
  )
  paste0(headings[headings != ""], collapse = " > ")
}
# Function for extracting all headers
get_h <- function(level, section) {
  parents <- xml2::xml_parents(section)
  if (length(parents) == 0) {
    return("")
  }
  parents <- parents[!is.na(xml2::xml_attr(parents, "class"))]
  h_section <- parents[grepl(
    paste0("section level", level),
    xml2::xml_attr(parents, "class")
  )]
  h <- xml2::xml_contents(h_section)[is_heading(xml2::xml_contents(h_section))]
  sub("^\\n", "", xml_text1(h))
}
get_version <- function(section) {
  parents <- xml2::xml_parents(section)
  parents <- parents[!is.na(xml2::xml_attr(parents, "class"))]
  h_section <- parents[grepl(
    "section level2",
    xml2::xml_attr(parents, "class")
  )]
  if (length(h_section) == 0) {
    h <- xml2::xml_contents(section)[is_heading(xml2::xml_contents(section))]
  } else {
    h <- xml2::xml_contents(h_section)[is_heading(xml2::xml_contents(
      h_section
    ))]
  }

  sub("^\\n", "", xml_text1(h))
}
# edited from https://github.com/rstudio/bookdown/blob/abd461593033294d82427139040a0a03cfa0390a/R/bs4_book.R#L518
# index -------------------------------------------------------------------

bs4_index_data <- function(node, previous_headings, title, dir, path) {
  # Make a copy of the node because we will remove contents from it for getting the data
  node_copy <- node
  # remove sections nested inside the current section to prevent duplicating content
  xml2::xml_remove(xml2::xml_find_all(
    node_copy,
    ".//*[contains(@class, 'section')]"
  ))
  # remove dont-index sections
  xml2::xml_remove(xml2::xml_find_all(
    node_copy,
    ".//*[contains(@class, 'dont-index')]"
  ))

  # Helpers for XPath queries
  # We want to find all nodes corresponding to ... but whose descendants
  # do not correspond to any ... otherwise we would treat some text/code twice,
  # e.g. the text of p nested within li.
  all <- function(...) {
    not <- sprintf(
      "[not(%s)]",
      paste0(paste0("descendant::", c(...)), collapse = "|")
    )
    paste0(".//", c(...), not, collapse = "|")
  }
  text_xpath <- all(
    "p",
    "li",
    "caption",
    "figcaption",
    "dt",
    "dd",
    "blockquote",
    "div[contains(@class, 'line-block')]"
  )
  code_xpath <- all("pre")
  code <- xml2::xml_find_all(node_copy, code_xpath)

  # Special case for definitions (mostly encountered in Rd files)
  if (xml2::xml_name(node_copy) == "dt") {
    # both argument name and definition
    text <- paste(
      xml_text1(node_copy),
      xml_text1(xml2::xml_find_first(node_copy, "following-sibling::*")),
      collapse = " "
    )
    heading <- paste(xml_text1(node_copy), "(argument)")
  } else {
    # Other cases
    text <- xml_text1(xml2::xml_find_all(node_copy, text_xpath))
    children <- xml2::xml_children(node_copy)
    heading_node <- children[purrr::map_lgl(children, is_heading)][1]
    heading <- xml_text1(heading_node)

    # Add heading for Usage section of Rd
    if (xml2::xml_attr(node_copy, "id", default = "") == "ref-usage") {
      heading <- "Usage"
    }
  }

  # If no specific heading, use the title
  if (nchar(heading) == 0) {
    heading <- title
    previous_headings <- ""
  }

  index_data <- list(
    path = path,
    id = section_id(node_copy),
    dir = dir,
    previous_headings = previous_headings,
    what = heading,
    title = title,
    text = strip_stop_words(text),
    code = xml_text1(code)
  )

  if (index_data$text == "" && index_data$code == "") {
    return(NULL)
  }

  index_data
}

xml_text1 <- function(x) {
  trimws(
    gsub("(\r\n|\r|\n)", " ", paste0(trimws(xml2::xml_text(x)), collapse = " "))
  )
}

strip_stop_words <- function(x) {
  # paste(tidytext::get_stopwords()$word, collapse = "|")
  pattern <- "\\b(i|me|my|myself|we|our|ours|ourselves|you|your|yours|yourself|yourselves|he|him|his|himself|she|her|hers|herself|it|its|itself|they|them|their|theirs|themselves|what|which|who|whom|this|that|these|those|am|is|are|was|were|be|been|being|have|has|had|having|do|does|did|doing|would|should|could|ought|i'm|you're|he's|she's|it's|we're|they're|i've|you've|we've|they've|i'd|you'd|he'd|she'd|we'd|they'd|i'll|you'll|he'll|she'll|we'll|they'll|isn't|aren't|wasn't|weren't|hasn't|haven't|hadn't|doesn't|don't|didn't|won't|wouldn't|shan't|shouldn't|can't|cannot|couldn't|mustn't|let's|that's|who's|what's|here's|there's|when's|where's|why's|how's|a|an|the|and|but|if|or|because|as|until|while|of|at|by|for|with|about|against|between|into|through|during|before|after|above|below|to|from|up|down|in|out|on|off|over|under|again|further|then|once|here|there|when|where|why|how|all|any|both|each|few|more|most|other|some|such|no|nor|not|only|own|same|so|than|too|very|will)\\b ?"
  gsub(pattern, "", x, ignore.case = TRUE)
}

is_heading <- function(node) {
  xml2::xml_name(node) %in% c("h1", "h2", "h3", "h4", "h5")
}

capitalise <- function(string) {
  paste0(toupper(substring(string, 1, 1)), substring(string, 2))
}

get_site_paths <- function(pkg) {
  paths <- dir_ls(pkg$dst_path, glob = "*.html", recurse = TRUE)
  paths_rel <- path_rel(paths, pkg$dst_path)

  # do not include dev package website in search index / sitemap
  paths_rel <- paths_rel[
    !path_has_parent(paths_rel, pkg$development$destination)
  ]

  # do not include redirects
  redirects <- purrr::map_chr(data_redirects(pkg, has_url = TRUE), 1)
  setdiff(paths_rel, redirects)
}

test_that("docsearch.json and sitemap.xml are valid for BS 3 site", {
  pkg <- local_pkgdown_site(
    meta = list(
      url = "https://example.com",
      template = list(
        bootstrap = 3,
        params = list(
          docsearch = list(
            api_key = "test-api-key",
            index_name = "test-index-name"
          )
        )
      )
    )
  )
  suppressMessages(build_docsearch_json(pkg))
  json <- path(pkg$dst_path, "docsearch.json")
  expect_true(jsonlite::validate(read_lines(json)))
  
  suppressMessages(build_sitemap(pkg))
  xml <- path(pkg$dst_path, "sitemap.xml")
  schema <- xml2::read_xml(test_path("assets/sitemaps-schema-0.9.xsd"))
  expect_true(xml2::xml_validate(xml2::read_xml(xml), schema))
})

test_that("build_search_index() has expected structure", {
  pkg <- local_pkgdown_site(
    desc = list(Version = "1.0.0"),
    meta = list(url = "https://example.com")
  )
  pkg <- pkg_add_file(pkg, "README.md", c(
    "# My Package",
    "What the pakage does"
  ))

  suppressMessages(init_site(pkg))
  suppressMessages(build_home_index(pkg))

  expect_snapshot(str(build_search_index(pkg)))
})

test_that("build sitemap only messages when it updates", {
  pkg <- local_pkgdown_site(meta = list(url = "https://example.com"))

  suppressMessages(init_site(pkg))
  suppressMessages(build_home(pkg))
  expect_snapshot({
    build_sitemap(pkg)
    build_sitemap(pkg)
  })
})

test_that("build_search() builds the expected search.json with no URL", {
  pkg <- local_pkgdown_site(desc = list(Version = "1.0.0"))
  pkg <- pkg_add_file(pkg, "README.md", c(
    "# My Package",
    "What the pakage does"
  ))

  suppressMessages(build_home(pkg))

  index <- build_search_index(pkg)
  paths <- purrr::map_chr(index, "path")
  expect_equal(paths, c("/authors.html", "/authors.html", "/index.html"))
})

test_that("sitemap excludes redirects", {
  pkg <- local_pkgdown_site(meta = list(
    url = "https://example.com",
    redirects = list(c("a.html", "b.html"))
  ))
  suppressMessages(build_redirects(pkg))
  expect_equal(get_site_paths(pkg), character())
})

#' Render RMarkdown document in a fresh session
#'
#' @noRd
render_rmarkdown <- function(pkg, input, output, ..., copy_images = TRUE, quiet = TRUE) {

  input_path <- path_abs(input, pkg$src_path)
  output_path <- path_abs(output, pkg$dst_path)

  if (!file_exists(input_path)) {
    stop("Can't find ", src_path(input), call. = FALSE)
  }

  cat_line("Reading ", src_path(input))
  digest <- file_digest(output_path)

  args <- list(
    input = input_path,
    output_file = path_file(output_path),
    output_dir = path_dir(output_path),
    intermediates_dir = tempdir(),
    encoding = "UTF-8",
    envir = globalenv(),
    ...,
    quiet = quiet
  )

  path <- tryCatch(
    callr::r_safe(
      function(...) rmarkdown::render(...),
      args = args,
      show = !quiet,
      env = c(
        callr::rcmd_safe_env(),
        BSTINPUTS = bst_paths(input_path),
        TEXINPUTS = tex_paths(input_path),
        BIBINPUTS = bib_paths(input_path),
        R_CLI_NUM_COLORS = 256
      )
    ),
    error = function(cnd) {
      rule("RMarkdown error")
      cat(gsub("\r", "", cnd$stderr, fixed = TRUE))
      rule()
      abort("Failed to render RMarkdown", parent = cnd)
    }
  )

  if (identical(path_ext(path)[[1]], "html")) {
    update_html(
      path,
      tweak_rmarkdown_html,
      input_path = path_dir(input_path),
      pkg = pkg
    )
  }
  if (digest != file_digest(output_path)) {
    href <- paste0("ide:run:pkgdown::preview_page('", path_rel(output_path, pkg$dst_path), "')")
    cat_line("Writing ", cli::style_hyperlink(dst_path(output), href))
  }

  # Copy over images needed by the document
  if (copy_images) {
    ext_src <- rmarkdown::find_external_resources(input_path)

    # temporarily copy the rendered html into the input path directory and scan
    # again for additional external resources that may be been included by R code
    tempfile_in_input_dir <- file_temp(ext = "html", tmp_dir = path_dir(input_path))
    file_copy(path, tempfile_in_input_dir)
    withr::defer(unlink(tempfile_in_input_dir))
    ext_post <- rmarkdown::find_external_resources(tempfile_in_input_dir)

    ext <- rbind(ext_src, ext_post)
    ext <- ext[!duplicated(ext$path), ]

    # copy web + explicit files beneath vignettes/
    is_child <- path_has_parent(ext$path, ".")
    ext_path <- ext$path[(ext$web | ext$explicit) & is_child]

    src <- path(path_dir(input_path), ext_path)
    dst <- path(path_dir(output_path), ext_path)
    # Make sure destination paths exist before copying files there
    dir_create(unique(path_dir(dst)))
    file_copy(src, dst, overwrite = TRUE)
  }
  check_missing_images(pkg, input, output)

  invisible(path)
}

# adapted from tools::texi2dvi
bst_paths <- function(path) {
  paths <- c(
    Sys.getenv("BSTINPUTS"),
    path_dir(path),
    path(R.home("share"), "texmf", "bibtex", "bst")
  )
  paste(paths, collapse = .Platform$path.sep)
}
tex_paths <- function(path) {
  paths <- c(
    Sys.getenv("TEXINPUTS"),
    path_dir(path),
    path(R.home("share"), "texmf", "tex", "latex")
  )
  paste(paths, collapse = .Platform$path.sep)
}
bib_paths <- function(path) {
  paths <- c(
    Sys.getenv("BIBINPUTS"),
    tex_paths(path)
  )
  paste(paths, collapse = .Platform$path.sep)

}

#' Print object in pkgdown output
#'
#' This lets package authors control how objects are printed just for
#' pkgdown examples. The default is to call [print()] apart from htmlwidgets
#' where the object is returned as is (with sizes tweaked).
#'
#' @param x Object to display
#' @param visible Whether it is visible or not
#' @return Either a character vector representing printed output (which
#'   will be escaped for HTML as necessary) or literal HTML produced
#'   by the htmltools or htmlwidgets packages.
#' @keywords internal
#' @export
pkgdown_print <- function(x, visible = TRUE) {
  UseMethod("pkgdown_print")
}

#' @export
pkgdown_print.default <- function(x, visible = TRUE) {
  if (!visible) {
    return(invisible())
  }

  # inlined from htmltools::is.browsable()
  if (isTRUE(attr(x, "browsable_html", exact = TRUE))) {
    x
  } else {
    print(x)
  }
}

#' @export
pkgdown_print.htmlwidget <- function(x, visible = TRUE) {
  if (!visible) {
    return(invisible())
  }

  settings <- fig_settings()
  x$width <- x$width %||% (settings$fig.width * settings$dpi)
  x$height <- x$height %||% (settings$fig.height * settings$dpi)
  x
}

external_dependencies <- function(pkg, call = caller_env()) {
  purrr::compact(list(
    fontawesome::fa_html_dependency(),
    cached_dependency(
      name = "headroom",
      version = "0.11.0",
      files = list(
        list(
          url = "https://cdnjs.cloudflare.com/ajax/libs/headroom/0.11.0/headroom.min.js",
          integrity = "sha256-AsUX4SJE1+yuDu5+mAVzJbuYNPHj/WroHuZ8Ir/CkE0="
        ),
        list(
          url = "https://cdnjs.cloudflare.com/ajax/libs/headroom/0.11.0/jQuery.headroom.min.js",
          integrity = "sha256-ZX/yNShbjqsohH1k95liqY9Gd8uOiE1S4vZc+9KQ1K4="
        )
      )
    ),
    cached_dependency(
      name = "bootstrap-toc",
      version = "1.0.1",
      files = list(
        list(
          url = "https://cdn.jsdelivr.net/gh/afeld/bootstrap-toc@v1.0.1/dist/bootstrap-toc.min.js",
          integrity = "sha256-4veVQbu7//Lk5TSmc7YV48MxtMy98e26cf5MrgZYnwo="
        )
      )
    ),
    cached_dependency(
      name = "clipboard.js",
      version = "2.0.11",
      files = list(
        list(
          url = "https://cdnjs.cloudflare.com/ajax/libs/clipboard.js/2.0.11/clipboard.min.js",
          integrity = "sha512-7O5pXpc0oCRrxk8RUfDYFgn0nO1t+jLuIOQdOMRp4APB7uZ4vSjspzp5y6YDtDs4VzUSTbWzBFZ/LKJhnyFOKw=="
        )
      )
    ),
    cached_dependency(
      name = "search",
      version = "1.0.0",
      files = list(
        list(
          url = "https://cdnjs.cloudflare.com/ajax/libs/fuse.js/6.4.6/fuse.min.js",
          integrity = "sha512-KnvCNMwWBGCfxdOtUpEtYgoM59HHgjHnsVGSxxgz7QH1DYeURk+am9p3J+gsOevfE29DV0V+/Dd52ykTKxN5fA=="
        ),
        list(
          url = "https://cdnjs.cloudflare.com/ajax/libs/autocomplete.js/0.38.0/autocomplete.jquery.min.js",
          integrity = "sha512-GU9ayf+66Xx2TmpxqJpliWbT5PiGYxpaG8rfnBEk1LL8l1KGkRShhngwdXK1UgqhAzWpZHSiYPc09/NwDQIGyg=="
        ),
        list(
          url = "https://cdnjs.cloudflare.com/ajax/libs/mark.js/8.11.1/mark.min.js",
          integrity = "sha512-5CYOlHXGh6QpOFA/TeTylKLWfB3ftPsde7AnmhuitiTX4K5SqCLBeKro6sPS8ilsz1Q4NRx3v8Ko2IBiszzdww=="
        )
      )
    ),
    math_dependency(pkg, call = call)
  ))
}

math_dependency <- function(pkg, call = caller_env()) {
  math <- config_math_rendering(pkg)
  if (math == "mathjax") {
    cached_dependency(
      name = "MathJax",
      version = "3.2.2",
      files = list(
        list(
          url = "https://cdnjs.cloudflare.com/ajax/libs/mathjax/3.2.2/es5/tex-chtml.min.js",
          integrity = "sha512-T8xxpazDtODy3WOP/c6hvQI2O9UPdARlDWE0CvH1Cfqc0TXZF6GZcEKL7tIR8VbfS/7s/J6C+VOqrD6hIo++vQ=="
        )
      )
    )
  } else if (math == "katex") {
    cached_dependency(
      name = "KaTex",
      version = "0.16.10",
      files = list(
        list(
          url = "https://cdn.jsdelivr.net/npm/katex@0.16.10/dist/katex.min.js",
          integrity = "sha384-hIoBPJpTUs74ddyc4bFZSM1TVlQDA60VBbJS0oA934VSz82sBx1X7kSx2ATBDIyd"
        ),
        list(
          url = "https://cdn.jsdelivr.net/npm/katex@0.16.10/dist/katex.min.css",
          integrity = "sha384-wcIxkf4k558AjM3Yz3BBFQUbk/zgIYC2R0QpeeYb+TwlBVMrlgLqwRjRtGZiK7ww"
        )
      )
    )
  } else {
    NULL
  }
}

cached_dependency <- function(name, version, files) {
  cache_dir <- path(tools::R_user_dir("pkgdown", "cache"), name, version)
  dir_create(cache_dir)

  for (file in files) {
    cache_path <- path(cache_dir, path_file(file$url))
    if (!file_exists(cache_path)) {
      utils::download.file(file$url, cache_path, quiet = TRUE, mode = "wb")
      check_integrity(cache_path, file$integrity)
    }
  }
  dep_files <- path_rel(dir_ls(cache_dir), cache_dir)

  htmltools::htmlDependency(
    name = name,
    version = version,
    src = cache_dir,
    script = dep_files[path_ext(dep_files) == "js"],
    stylesheet = dep_files[path_ext(dep_files) == "css"]
  )
}

check_integrity <- function(path, integrity) {
  parsed <- parse_integrity(integrity)
  if (!parsed$size %in% c(256L, 384L, 512L)) {
    cli::cli_abort(
      "{.field integrity} must use SHA-256, SHA-384, or SHA-512",
      .internal = TRUE
    )
  }

  hash <- compute_hash(path, parsed$size)
  if (hash != parsed$hash) {
    cli::cli_abort(
      "Downloaded asset does not match known integrity",
      .internal = TRUE
    )
  }

  invisible()
}

compute_hash <- function(path, size) {
  con <- file(path, encoding = "UTF-8")
  openssl::base64_encode(openssl::sha2(con, size))
}
 
parse_integrity <- function(x) {
  size <- as.integer(regmatches(x, regexpr("(?<=^sha)\\d{3}", x, perl = TRUE)))
  hash <- regmatches(x, regexpr("(?<=^sha\\d{3}-).+", x, perl = TRUE))

  list(size = size, hash = hash)
}

test_that("check integrity validates integrity", {
  temp <- withr::local_tempfile(lines = letters)

  expect_snapshot(error = TRUE, {
    check_integrity(temp, "sha123-abc")
    check_integrity(temp, "sha256-abc")
  })

  integrity <- paste0("sha256-", compute_hash(temp, 256L))
  expect_no_error(check_integrity(temp, integrity))
})

test_that("can parse integrity", {
  expect_equal(parse_integrity("sha256-abc"), list(size = 256L, hash = "abc"))
})

# Syntax highlighting for `\preformatted{}` blocks in reference topics
tweak_reference_highlighting <- function(html) {
  # There are three cases:
  # 1) <div> with class sourceCode + r/R, as created by ```R
  div <- xml2::xml_find_all(html, ".//div")
  # must have sourceCode and not be in examples or usage
  is_source <- has_class(div, "sourceCode") & !is_handled_section(div)

  div_sourceCode <- div[is_source]
  is_r <- has_class(div_sourceCode, c("r", "R"))
  div_sourceCode_r <- div_sourceCode[is_r]
  purrr::walk(div_sourceCode_r, tweak_highlight_r)

  # 2) <div> with class sourceCode + another language, e.g. ```yaml
  # or no language e.g. ```
  div_sourceCode_other <- div_sourceCode[!is_r]
  purrr::walk(div_sourceCode_other, tweak_highlight_other)

  # 3) <pre> with no wrapper <div>, as created by ```
  pre <- xml2::xml_find_all(html, ".//pre")
  handled <- is_wrapped_pre(pre) | is_handled_section(pre)
  purrr::walk(pre[!handled], tweak_highlight_r)
  # Add div.sourceCode for copy button
  xml2::xml_add_parent(pre[!handled], "div", class = "sourceCode")

  invisible()
}

is_wrapped_pre <- function(html) {
  xml2::xml_find_lgl(html, "boolean(parent::div[contains(@class, 'sourceCode')])")
}

is_handled_section <- function(html) {
  xml2::xml_find_lgl(html, "boolean(ancestor::div[@id='ref-examples' or @id='ref-usage'])")
}

tweak_highlight_r <- function(block) {
  code <- xml2::xml_find_first(block, ".//code")
  if (is.na(code)) {
    return(FALSE)
  }

  text <- xml2::xml_text(code)
  out <- downlit::highlight(text, classes = downlit::classes_pandoc())
  if (is.na(out) || identical(out, "")) {
    return(FALSE)
  }

  html <- xml2::read_html(out)
  xml_replace_contents(code, xml2::xml_find_first(html, "body"))

  TRUE
}

tweak_highlight_other <- function(div) {
  code <- xml2::xml_find_first(div, ".//code")
  if (is.na(code)) {
    return(FALSE)
  }

  lang <- sub("sourceCode ", "", xml2::xml_attr(div, "class"))
  # since roxygen 7.2.0 generic code blocks have sourceCode with no lang
  if (!is.na(lang) && lang == "sourceCode") lang <- "r"
  # Pandoc does not recognize rmd as a language :-)
  if (tolower(lang) %in% c("rmd", "qmd")) lang <- "markdown"
  # many backticks to account for possible nested code blocks
  # like a Markdown code block with code chunks inside
  md <- paste0("``````", lang, "\n", xml2::xml_text(code), "\n``````")
  html <- markdown_text(NULL, md)

  xml_replace_contents(code, xml2::xml_find_first(html, "body/div/pre/code"))
  TRUE
}

xml_replace_contents <- function(node, new) {
  xml2::xml_remove(xml2::xml_contents(node))

  contents <- xml2::xml_contents(new)
  for (child in contents) {
    xml2::xml_add_child(node, child)
  }
}


tweak_extra_logo <- function(html) {
  img <- xml2::xml_find_all(html, ".//div[contains(@class,'ref-description')]//img[contains(@src,'logo')]")
  xml2::xml_remove(img)

  invisible()
}

test_that("highlights <pre> wrapped in <div> with language info", {
  skip_if_no_pandoc("2.16")

  withr::local_options(downlit.topic_index = c(foo = "foo"))
  html <- xml2::read_html('
    <div id="ref-section">
      <div class="sourceCode r">
      <pre><code>foo(x)</code></pre>
      </div>
    </div>
  ')
  tweak_reference_highlighting(html)
  expect_equal(xpath_attr(html, ".//code//a", "href"), "foo.html")

  # Or upper case R
  html <- xml2::read_html('
    <div id="ref-section">
      <div class="sourceCode R">
      <pre><code>foo(x)</code></pre>
      </div>
    </div>
  ')
  tweak_reference_highlighting(html)
  expect_equal(xpath_attr(html, ".//code//a", "href"), "foo.html")

  html <- xml2::read_html('
    <div id="ref-section">
      <div class="sourceCode yaml">
        <pre><code>field: value</code></pre>
      </div>
    </div>
  ')
  tweak_reference_highlighting(html)
  # Select all leaf <span> to work around variations in pandoc styling
  expect_equal(xpath_attr(html, "//code//span[not(span)]", "class")[[1]], "fu")
  expect_equal(xpath_text(html, "//code//span[not(span)]")[[1]], "field")

  # But don't touch examples or usage
  html <- xml2::read_html('
    <div id="ref-examples">
      <div class="sourceCode R">
        <pre><code>foo(x)</code></pre>
      <div>
    </div>
  ')
  tweak_reference_highlighting(html)
  expect_equal(xpath_length(html, "//code//span"), 0)

  html <- xml2::read_html('
    <div id="ref-usage">
      <div class="sourceCode R">
        <pre><code>foo(x)</code></pre>
      <div>
    </div>
  ')
  tweak_reference_highlighting(html)
  expect_equal(xpath_length(html, "//code//span"), 0)
})

test_that("highlight unwrapped <pre>", {
  withr::local_options(downlit.topic_index = c(foo = "foo"))

  # If parseable, assume R
  html <- xml2::read_html('
    <div id="ref-sections">
      <pre><code>foo(x)</code></pre>
    </div>
  ')
  tweak_reference_highlighting(html)
  expect_equal(xpath_attr(html, ".//code//a", "href"), "foo.html")
  expect_equal(xpath_attr(html, ".//div/div", "class"), "sourceCode")

  # If not parseable, leave as is
  html <- xml2::read_html('
    <div id="ref-sections">
      <pre><code>foo(</code></pre>
    </div>
  ')
  tweak_reference_highlighting(html)
  expect_equal(xpath_length(html, "//code//span"), 0)
  expect_equal(xpath_attr(html, ".//div/div", "class"), "sourceCode")
})

# highlighting ------------------------------------------------------------

test_that("can highlight R code", {
  html <- xml2::read_xml('<div><pre><code>1 + 2</code></pre></div>')
  tweak_highlight_r(html)

  expect_equal(xpath_attr(html, "//code//span[@class]", "class"), c("fl", "op", "fl"))
  expect_equal(xpath_text(html, "//code//span[@class]"), c("1", "+", "2"))
})

test_that("fails cleanly", {
  html <- xml2::read_xml('<div><pre><code>1 + </code></pre></div>')
  expect_equal(tweak_highlight_r(html), FALSE)

  html <- xml2::read_xml('<div><pre><code></code></pre></div>')
  expect_equal(tweak_highlight_r(html), FALSE)

  html <- xml2::read_xml('<div><pre></pre></div>')
  expect_equal(tweak_highlight_r(html), FALSE)
})

test_that("can highlight other languages", {
  skip_if_no_pandoc("2.16")
  html <- xml2::read_xml('<div class="yaml"><pre><code>field: value</code></pre></div>')
  tweak_highlight_other(html)

  # Select all leaf <span> to work around variations in pandoc styling
  expect_equal(xpath_attr(html, "//code//span[not(span)]", "class")[[1]], "fu")
  expect_equal(xpath_text(html, "//code//span[not(span)]")[[1]], "field")
})

test_that("fails cleanly", {
  html <- xml2::read_xml('<div><pre><code></code></pre></div>')
  tweak_highlight_other(html)
  expect_equal(xpath_text(html, "//code"), "")

  html <- xml2::read_xml('<div><pre></pre></div>')
  expect_equal(tweak_highlight_other(html), FALSE)
})


# logo --------------------------------------------------------------------

test_that("can strip extra logo from description", {
  html <- xml2::read_html('
    <div class="ref-description">
    <p>Hi</p>
    <img src="logo.png" />
    <p>Bye</p>
    </div>
  ')
  tweak_extra_logo(html)
  expect_equal(xpath_length(html, "//img"), 0)
})

markdown_text <- function(pkg, text, ...) {
  if (identical(text, NA_character_) || is.null(text)) {
    return(NULL)
  }

  md_path <- withr::local_tempfile()
  write_lines(text, md_path)
  markdown_path_html(pkg, md_path, ...)
}

markdown_text_inline <- function(pkg, 
                                 text,
                                 error_path,
                                 error_call = caller_env()) {
  html <- markdown_text(pkg, text)
  if (is.null(html)) {
    return()
  }

  children <- xml2::xml_children(xml2::xml_find_first(html, ".//body"))
  if (length(children) > 1) {
    msg <- "{.field {error_path}} must be inline markdown."
    config_abort(pkg, msg, call = error_call)
  }

  paste0(xml2::xml_contents(children), collapse = "")
}

markdown_text_block <- function(pkg, text, ...) {
  html <- markdown_text(pkg, text, ...)
  if (is.null(html)) {
    return()
  }

  children <- xml2::xml_children(xml2::xml_find_first(html, ".//body"))
  paste0(as.character(children, options = character()), collapse = "")
}

markdown_body <- function(pkg, path, strip_header = FALSE) {
  xml <- markdown_path_html(pkg, path, strip_header = strip_header)

  if (is.null(xml)) {
    return(NULL)
  }

  # Extract body of html - as.character renders as xml which adds
  # significant whitespace in tags like pre
  transformed_path <- withr::local_tempfile()
  body <- xml2::xml_find_first(xml, ".//body")
  xml2::write_html(body, transformed_path, format = FALSE)

  lines <- read_lines(transformed_path)
  lines <- sub("<body>", "", lines, fixed = TRUE)
  lines <- sub("</body>", "", lines, fixed = TRUE)

  structure(
    paste(lines, collapse = "\n"),
    title = attr(xml, "title")
  )
}

markdown_path_html <- function(pkg, path, strip_header = FALSE) {
  html_path <- withr::local_tempfile()
  convert_markdown_to_html(pkg, path, html_path)
  xml <- xml2::read_html(html_path, encoding = "UTF-8")
  if (!inherits(xml, "xml_node")) {
    return(NULL)
  }

  # Capture heading, and optionally remove
  h1 <- xml2::xml_find_first(xml, ".//h1")
  title <- xml2::xml_text(h1)
  if (strip_header) {
    xml2::xml_remove(h1)
  }

  structure(xml, title = title)
}

markdown_to_html <- function(pkg, text, dedent = 4, bs_version = 3) {
  if (dedent) {
    text <- dedent(text, dedent)
  }

  md_path <- withr::local_tempfile()
  html_path <- withr::local_tempfile()

  write_lines(text, md_path)
  convert_markdown_to_html(pkg, md_path, html_path)

  html <- xml2::read_html(html_path, encoding = "UTF-8")
  tweak_page(html, "markdown", list(bs_version = bs_version))
  html
}

dedent <- function(x, n = 4) {
  gsub(paste0("($|\n)", strrep(" ", n)), "\\1", x, perl = TRUE)
}

convert_markdown_to_html <- function(pkg, in_path, out_path, ...) {
  if (rmarkdown::pandoc_available("2.0")) {
    from <- "markdown+gfm_auto_identifiers-citations+emoji+autolink_bare_uris"
  } else if (rmarkdown::pandoc_available("1.12.3")) {
    from <- "markdown_github-hard_line_breaks+tex_math_dollars+tex_math_single_backslash+header_attributes"
  } else {
    if (is_testing()) {
      testthat::skip("Pandoc not available")
    } else {
      cli::cli_abort("Pandoc not available")
    }
  }
  rmarkdown::pandoc_convert(
    input = in_path,
    output = out_path,
    from = from,
    to = "html",
    options = purrr::compact(c(
      if (!rmarkdown::pandoc_available("2.0")) "--smart",
      if (rmarkdown::pandoc_available("2.0")) c("-t", "html4"),
      "--indented-code-classes=R",
      "--section-divs",
      "--wrap=none",
      paste0("--", config_math_rendering(pkg)),
      ...
    ))
  )

  invisible()
}

config_math_rendering <- function(pkg, call = caller_env()) {
  if (is.null(pkg)) {
    # Special case for tweak_highlight_other() where it's too annoying to
    # pass down the package, and it doesn't matter much anyway.
    return("mathml")
  }

  math <- config_pluck_string(
    pkg,
    "template.math-rendering",
    default = "mathml",
    call = call
  )
  allowed <- c("mathml", "mathjax", "katex")

  if (!math %in% allowed) {
    msg <- "{.field template.math-rendering} must be one of {allowed}, not {math}."
    config_abort(pkg, msg, call = call)
  }

  math
}

test_that("handles empty inputs (returns NULL)", {
  pkg <- local_pkgdown_site()
  expect_null(markdown_text_inline(pkg, ""))
  expect_null(markdown_text_inline(pkg, NULL))
  expect_null(markdown_text_block(pkg, NULL))
  expect_null(markdown_text_block(pkg, ""))

  path <- withr::local_tempfile()
  file_create(path)
  expect_null(markdown_body(pkg, path))
})

test_that("header attributes are parsed", {
  pkg <- local_pkgdown_site()
  text <- markdown_text_block(pkg, "# Header {.class #id}")
  expect_match(text, "id=\"id\"")
  expect_match(text, "class=\".*? class\"")
})

test_that("markdown_text_inline() works with inline markdown", {
  pkg <- local_pkgdown_site()
  expect_equal(markdown_text_inline(pkg, "**lala**"), "<strong>lala</strong>")

  expect_snapshot(error = TRUE, {
    markdown_text_inline(pkg, "x\n\ny", error_path = "title")
  })
})

test_that("markdown_text_block() works with inline and block markdown", {
  skip_if_no_pandoc("2.17.1")

  pkg <- local_pkgdown_site()
  expect_equal(markdown_text_block(pkg, "**x**"), "<p><strong>x</strong></p>")
  expect_equal(markdown_text_block(pkg, "x\n\ny"), "<p>x</p><p>y</p>")
})

test_that("markdown_body() captures title", {
  pkg <- local_pkgdown_site()
  temp <- withr::local_tempfile(lines = "# Title\n\nSome text")

  html <- markdown_body(pkg, temp)
  expect_equal(attr(html, "title"), "Title")

  # And can optionally strip it
  html <- markdown_body(pkg, temp, strip_header = TRUE)
  expect_equal(attr(html, "title"), "Title")
  expect_no_match(html, "Title")
})

test_that("markdown_text_*() handles UTF-8 correctly", {
  pkg <- local_pkgdown_site()
  expect_equal(markdown_text_block(pkg, "\u00f8"), "<p>\u00f8</p>")
  expect_equal(markdown_text_inline(pkg, "\u00f8"), "\u00f8")
})

test_that("validates math yaml", {
  config_math_rendering_ <- function(...) {
    pkg <- local_pkgdown_site(meta = list(template = list(...)))
    config_math_rendering(pkg)
  }
  expect_snapshot(error = TRUE, {
    config_math_rendering_(`math-rendering` = 1)
    config_math_rendering_(`math-rendering` = "math")
  })
})

has_citation <- function(path = ".") {
  file_exists(path(path, 'inst/CITATION'))
}

create_citation_meta <- function(path) {
  path <- path(path, "DESCRIPTION")

  dcf <- read.dcf(path)
  meta <- as.list(dcf[1, ])

  if (!is.null(meta$Encoding)) {
    meta <- lapply(meta, iconv, from = meta$Encoding, to = "UTF-8")
  } else {
    meta$Encoding <- "UTF-8"
  }

  meta
}

read_citation <- function(path = ".") {
  if (!has_citation(path)) {
    return(character())
  }
  meta <- create_citation_meta(path)
  cit_path <- path(path, 'inst/CITATION')

  utils::readCitationFile(cit_path, meta = meta)
}

data_home_sidebar_citation <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)

  sidebar_section(
    heading = "Citation",
    bullets = a(sprintf(tr_("Citing %s"), pkg$package), "authors.html#citation")
  )
}

data_citations <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)

  if (has_citation(pkg$src_path)) {
    return(citation_provided(pkg$src_path))
  }

  citation_auto(pkg)

}

citation_provided <- function(src_path) {
  provided_citation <- read_citation(src_path)

  text_version <- format(provided_citation, style = "textVersion")
  cit <- list(
    html = ifelse(
      text_version == "",
      format(provided_citation, style = "html"),
      paste0("<p>", escape_html(text_version), "</p>")
    ),
    bibtex = format(provided_citation, style = "bibtex")
  )

  purrr::transpose(cit)
}

citation_auto <- function(pkg) {
  cit_info <- utils::packageDescription(
    path_file(pkg$src_path),
    lib.loc = path_dir(pkg$src_path)
  )
  cit_info$`Date/Publication` <- cit_info$`Date/Publication` %||% Sys.time()

  cit <- utils::citation(auto = cit_info)
  list(
    html = format(cit, style = "html"),
    bibtex = format(cit, style = "bibtex")
  )
}

build_citation_authors <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)

  source <- if (has_citation(pkg$src_path)) {
    repo_source(pkg, "inst/CITATION")
  } else {
    repo_source(pkg, "DESCRIPTION")
  }

  data <- list(
    pagetitle = tr_("Authors and Citation"),
    citations = data_citations(pkg),
    authors = unname(data_authors(pkg)$all),
    source = source
  )

  data$before <- markdown_text_block(pkg$meta$authors$before)
  data$after <- markdown_text_block(pkg$meta$authors$after)

  render_page(pkg, "citation-authors", data, "authors.html")
}

#' Build docs for LLMs
#'
#' @description
#' `build_llm_docs()` creates an `LLMs.txt` at the root of your site
#' that contains the contents of your `README.md`, your reference index,
#' and your articles index. It also creates a `.md` file for every existing
#' `.html` file in your site. Together, this gives an LLM an overview of your
#' package and the ability to find out more by following links.
#'
#' If you don't want these files generated for your site, you can opt-out by
#' adding the following to your `pkgdown.yml`:
#'
#' ```yaml
#' llm-docs: false
#' ```
#'
#' @family site components
#' @inheritParams build_site
#' @export
build_llm_docs <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)
  if (isFALSE(pkg$meta$`llm-docs`)) {
    return(invisible())
  }

  cli::cli_rule("Building docs for llms")

  paths <- get_site_paths(pkg)
  purrr::walk(paths, \(path) {
    src_path <- path(pkg[["dst_path"]], path)
    dst_path <- path_ext_set(src_path, "md")
    convert_md(src_path, dst_path, full_url(pkg, path))
  })

  index <- c(
    read_lines(path(pkg$dst_path, "index.md")),
    "",
    read_file_if_exists(path(pkg$dst_path, "reference", "index.md")),
    "",
    read_file_if_exists(path(pkg$dst_path, "articles", "index.md"))
  )
  write_lines(index, path(pkg$dst_path, "llms.txt"))

  invisible()
}

full_url <- function(pkg, path) {
  if (is.null(pkg$meta$url)) {
    return()
  }

  url <- paste0(pkg$meta$url, "/")
  if (pkg$development$in_dev) {
    url <- paste0(url, pkg$prefix)
  }

  xml2::url_absolute(paste0(path_dir(path), "/"), url)
}

convert_md <- function(src_path, dst_path, url = NULL) {
  html <- xml2::read_html(src_path)
  main_html <- xml2::xml_find_first(html, ".//main")
  if (length(main_html) == 0) {
    return()
  }

  simplify_page_header(main_html)
  simplify_anchors(main_html)
  simplify_code(main_html)
  simplify_popovers_to_footnotes(main_html)
  simplify_lifecycle_badges(main_html)
  simplify_dls(main_html)
  create_absolute_links(main_html, url)

  path <- file_temp()
  xml2::write_html(main_html, path, format = FALSE)
  on.exit(file_delete(path), add = TRUE)

  rmarkdown::pandoc_convert(
    input = path,
    output = dst_path,
    from = "html",
    to = "gfm+definition_lists-raw_html",
  )
}

# Helpers ---------------------------------------------------------------------

# simplify page header (which includes logo + source link)
simplify_page_header <- function(html) {
  title <- xml2::xml_find_first(html, ".//h1")
  # website for a package without README/index.md
  if (length(title) > 0) {
    xml2::xml_remove(xml2::xml_find_first(html, ".//div[@class='page-header']"))
    xml2::xml_add_child(html, title, .where = 0)
  }
  invisible()
}

# drop internal anchors
simplify_anchors <- function(html) {
  xml2::xml_remove(xml2::xml_find_all(html, ".//a[@class='anchor']"))
  invisible()
}

# strip extraneoous classes
simplify_code <- function(html) {
  extract_lang <- function(class) {
    trimws(gsub("sourceCode|downlit", "", class))
  }
  code <- xml2::xml_find_all(html, ".//pre[contains(@class, 'sourceCode')]")

  purrr::walk(code, \(x) {
    xml2::xml_attr(x, "class") <- extract_lang(xml2::xml_attr(x, "class"))
  })
  invisible()
}

simplify_popovers_to_footnotes <- function(main_html) {
  popover_refs <- xml2::xml_find_all(main_html, ".//a[@class='footnote-ref']")
  if (length(popover_refs) == 0) {
    return()
  }

  # Create footnotes section
  footnotes_section <- xml2::xml_find_first(
    main_html,
    ".//section[@class='footnotes']"
  )
  if (length(footnotes_section) == 0) {
    footnotes_section <- xml2::xml_add_child(
      main_html,
      "section",
      id = "footnotes",
      class = "footnotes footnotes-end-of-document",
      role = "doc-endnotes"
    )
    xml2::xml_add_child(footnotes_section, "hr")
    footnotes_ol <- xml2::xml_add_child(footnotes_section, "ol")
  } else {
    footnotes_ol <- xml2::xml_find_first(footnotes_section, ".//ol")
  }

  purrr::iwalk(popover_refs, function(ref, i) {
    text_content <- xml2::xml_attr(ref, "data-bs-content")
    fn_id <- paste0("fn", i)
    fnref_id <- paste0("fnref", i)
    xml2::xml_attrs(ref) <- list(
      href = paste0("#", fn_id),
      id = fnref_id,
      role = "doc-noteref",
      class = "footnote-ref"
    )

    fn_li <- xml2::xml_add_child(footnotes_ol, "li", id = fn_id)
    parsed_content <- xml2::read_html(text_content) |>
      xml2::xml_find_first(".//body") |>
      xml2::xml_children()
    purrr::walk(parsed_content, \(x) xml2::xml_add_child(fn_li, x))
  })
}

simplify_lifecycle_badges <- function(html) {
  # on reference index
  badges <- xml2::xml_find_all(html, "//span[contains(@class, 'lifecycle')]")
  xml2::xml_replace(badges, "strong", paste0("[", xml2::xml_text(badges), "]"))

  # on individual pages
  badges <- xml2::xml_find_all(
    html,
    "//a[.//img[starts-with(@src, 'figures/lifecycle-')]]"
  )
  imgs <- xml2::xml_find_first(badges, ".//img")
  xml2::xml_replace(badges, "strong", tolower(xml2::xml_attr(imgs, "alt")))

  invisible()
}

create_absolute_links <- function(main_html, url = NULL) {
  a <- xml2::xml_find_all(main_html, ".//a")
  xml2::xml_attr(a, "class") <- NULL

  href <- xml2::xml_attr(a, "href")
  is_internal <- !is.na(href) &
    !startsWith(href, "https") &
    !startsWith(href, "#")
  if (!is.null(url)) {
    href[is_internal] <- xml2::url_absolute(href[is_internal], url)
  }
  href[is_internal] <- sub("html$", "md", href[is_internal])

  xml2::xml_attr(a[is_internal], "href") <- href[is_internal]

  invisible()
}

read_file_if_exists <- function(path) {
  if (file_exists(path)) {
    read_lines(path)
  }
}

test_that("integration test for convert_md()", {
  skip_if_no_pandoc()

  path <- withr::local_tempfile(pattern = "pkgdown-llm")
  convert_md(test_path("assets", "llm.html"), path)
  expect_snapshot(write_lines(read_lines(path), stdout()))
})

test_that("simplifies page header", {
  html <- xml2::read_html(
    r"(
      <main><div class="page-header">
        <img src="../logo.png" class="logo" alt=""><h1>Package index</h1>
      </div></main>)"
  )
  simplify_page_header(xml2::xml_find_first(html, ".//main"))
  expect_equal(xpath_contents(html, ".//main"), "<h1>Package index</h1>")
})

test_that("replaces lifecycle badges with strong text", {
  html <- xml2::read_html(
    r"(
      <span class="badge lifecycle lifecycle-deprecated">deprecated</span>
      <a href="https://lifecycle.r-lib.org/articles/stages.html#experimental" class="external-link"><img src="figures/lifecycle-experimental.svg" alt="[Experimental]"></a>  
      )"
  )
  simplify_lifecycle_badges(html)
  expect_equal(
    xpath_text(html, ".//strong"),
    c("[deprecated]", "[experimental]")
  )
})

test_that("converts internal urls to absolute with .md ending", {
  html <- xml2::read_html(
    r"(
      <a href="llm.html">link</a>
      <a href="#fragment">link</a>
      <a href="https://example.org">link</a>
    )"
  )
  create_absolute_links(html, "https://pkgdown.r-lib.org")
  expect_equal(
    xpath_attr(html, ".//a", "href"),
    c(
      "https://pkgdown.r-lib.org/llm.md",
      "#fragment",
      "https://example.org"
    )
  )
})

test_that("adjusts extension even without url", {
  html <- xml2::read_html(r"(<a href="llm.html">link</a>)")
  create_absolute_links(html)
  expect_equal(xpath_attr(html, ".//a", "href"), "llm.md")
})

test_that("strip extra classes from pre", {
  html <- xml2::read_html(r"(<pre class="downlit sourceCode r">1+1</pre>)")
  simplify_code(html)
  expect_equal(xpath_attr(html, ".//pre", "class"), "r")
})

#' @export
#' @rdname build_home
build_home_index <- function(pkg = ".", override = list(), quiet = TRUE) {
  pkg <- section_init(pkg, override = override)

  src_path <- path_index(pkg)
  dst_path <- path(pkg$dst_path, "index.html")
  data <- data_home(pkg)

  if (is.null(src_path)) {
    cli::cli_inform("Reading {.file DESCRIPTION}")
    data$index <- linkify(pkg$desc$get_field("Description", ""))
  } else {
    cli::cli_inform("Reading {src_path(path_rel(src_path, pkg$src_path))}")
    local_options_link(pkg, depth = 0L)
    data$index <- markdown_body(pkg, src_path)
  }

  cur_digest <- file_digest(dst_path)
  render_page(pkg, "home", data, "index.html", quiet = quiet)

  strip_header <- config_pluck_bool(pkg, "home.strip_header", default = FALSE)
  hide_badges <- pkg$development$mode == "release" && !pkg$development$in_dev

  update_html(
    dst_path,
    tweak_homepage_html,
    strip_header = strip_header,
    sidebar = !isFALSE(pkg$meta$home$sidebar),
    show_badges = !hide_badges,
    bs_version = pkg$bs_version,
    logo = logo_path(pkg, depth = 0)
  )

  new_digest <- file_digest(dst_path)
  if (cur_digest != new_digest) {
    writing_file(path_rel(dst_path, pkg$dst_path), "index.html")
  }

  invisible()
}

path_index <- function(pkg) {
  path_first_existing(
    pkg$src_path,
    c("pkgdown/index.md", "index.md", "README.md")
  )
}

data_home <- function(pkg = ".", call = caller_env()) {
  pkg <- as_pkgdown(pkg)

  config_pluck_list(pkg, "home", call = call)

  title <- config_pluck_string(
    pkg,
    "home.title",
    default = cran_unquote(pkg$desc$get_field("Title", "")),
    call = call
  )
  description <- config_pluck_string(
    pkg,
    "home.description",
    default = cran_unquote(pkg$desc$get_field("Description", "")),
    call = call
  )
  trailing_slash <- config_pluck_bool(
    pkg,
    "template.trailing_slash_redirect",
    default = FALSE,
    call = call
  )

  print_yaml(list(
    pagetitle = title,
    sidebar = data_home_sidebar(pkg, call = call),
    opengraph = list(description = description),
    has_trailingslash = trailing_slash
  ))
}


data_home_sidebar <- function(pkg = ".", call = caller_env()) {
  pkg <- as_pkgdown(pkg)

  sidebar <- config_pluck(pkg, "home.sidebar")
  if (isFALSE(sidebar)) {
    return(FALSE)
  }

  config_pluck_list(pkg, "home", call = call)
  html_path <- config_pluck_string(pkg, "home.sidebar.html", call = call)
  if (!is.null(html_path)) {
    html_path_abs <- path(pkg$src_path, html_path)

    if (!file_exists(html_path_abs)) {
      msg <- "{.field home.sidebar.html} specifies a file that doesn't exist ({.file {html_path}})."
      config_abort(pkg, msg, call = call)
    }
    return(read_file(html_path_abs))
  }

  structure <- config_pluck_character(
    pkg,
    "home.sidebar.structure",
    default = default_sidebar_structure(),
    call = call
  )

  # compute all default sections
  default_components <- list(
    links = data_home_sidebar_links(pkg),
    license = data_home_sidebar_license(pkg),
    community = data_home_sidebar_community(pkg),
    citation = data_home_sidebar_citation(pkg),
    authors = data_home_sidebar_authors(pkg),
    dev = sidebar_section(
      tr_("Dev Status"),
      "placeholder",
      class = "dev-status"
    ),
    toc = data_home_toc(pkg)
  )

  needs_components <- setdiff(structure, names(default_components))
  custom_yaml <- config_pluck_sidebar_components(
    pkg,
    needs_components,
    call = call
  )
  custom_components <- purrr::map(custom_yaml, function(x) {
    sidebar_section(x$title, markdown_text_block(pkg, x$text))
  })
  components <- modify_list(default_components, custom_components)

  sidebar <- purrr::compact(components[structure])
  paste0(sidebar, collapse = "\n")
}

# Update sidebar-configuration.Rmd if this changes
default_sidebar_structure <- function() {
  c("links", "license", "community", "citation", "authors", "dev")
}

config_pluck_sidebar_components <- function(
  pkg,
  new_components,
  call = caller_env()
) {
  base_path <- "home.sidebar.components"
  components <- config_pluck_list(
    pkg,
    base_path,
    has_names = new_components,
    call = call
  )

  for (name in names(components)) {
    component <- components[[name]]
    component_path <- paste0(base_path, ".", name)

    config_pluck_list(
      pkg,
      component_path,
      has_names = c("title", "text"),
      call = call
    )
    config_pluck_string(pkg, paste0(component_path, ".title"), call = call)
    config_pluck_string(pkg, paste0(component_path, ".text"), call = call)
  }
  components
}

data_home_sidebar_links <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)

  repo <- cran_link(pkg$package)
  links <- config_pluck(pkg, "home.links")

  bug_reports <- pkg$desc$get_field("BugReports", default = NULL)
  if (
    !is.null(bug_reports) &&
      grepl("@", bug_reports) &&
      !startsWith(bug_reports, "http")
  ) {
    bug_reports <- paste0("mailto:", bug_reports)
  }

  links <- c(
    link_url(sprintf(tr_("View on %s"), repo$repo), repo$url),
    link_url(tr_("Browse source code"), repo_home(pkg)),
    link_url(tr_("Report a bug"), bug_reports),
    purrr::map_chr(links, ~ link_url(.$text, .$href))
  )

  sidebar_section(tr_("Links"), links)
}

data_home_toc <- function(pkg) {
  sidebar_section(
    tr_("Table of contents"),
    '<nav id="toc"></nav>'
  )
}

sidebar_section <- function(heading, bullets, class = make_slug(heading)) {
  if (length(bullets) == 0) {
    return(character())
  }

  paste0(
    "<div class='",
    class,
    "'>\n",
    "<h2 data-toc-skip>",
    heading,
    "</h2>\n",
    "<ul class='list-unstyled'>\n",
    paste0("<li>", bullets, "</li>\n", collapse = ""),
    "</ul>\n",
    "</div>\n"
  )
}

cran_link <- function(pkg) {
  cran_url <- paste0("https://cloud.r-project.org/package=", pkg)
  req <- httr2::request(cran_url)
  req <- req_pkgdown_cache(req)
  req <- httr2::req_error(req, function(resp) FALSE)
  resp <- httr2::req_perform(req)
  if (!httr2::resp_is_error(resp)) {
    return(list(repo = "CRAN", url = cran_url))
  }

  # bioconductor always returns a 200 status, redirecting to /removed-packages/
  bioc_url <- paste0("https://www.bioconductor.org/packages/", pkg)
  req <- httr2::request(bioc_url)
  req <- req_pkgdown_cache(req)
  req <- httr2::req_error(req, function(resp) FALSE)
  req <- httr2::req_retry(req, max_tries = 3)
  resp <- httr2::req_perform(req)

  if (
    !httr2::resp_is_error(resp) &&
      !grepl("removed-packages", httr2::resp_url(resp))
  ) {
    return(list(repo = "Bioconductor", url = bioc_url))
  }

  NULL
}

req_pkgdown_cache <- function(req) {
  cache_path <- dir_create(path(tools::R_user_dir("pkgdown", "cache"), "http"))
  httr2::req_cache(
    req,
    path = cache_path,
    max_age = 86400 # 1 day
  )
}

# authors forced to wrap words in '' to prevent spelling errors
cran_unquote <- function(string) {
  gsub("\\'(.*?)\\'", "\\1", string)
}

test_that("messages about reading and writing", {
  pkg <- local_pkgdown_site()

  expect_snapshot({
    build_home_index(pkg)
    build_home_index(pkg)
  })
})

test_that("title and description come from DESCRIPTION by default", {
  pkg <- local_pkgdown_site(desc = list(
    Title = "A test title",
    Description = "A test description."
  ))
  expect_equal(data_home(pkg)$pagetitle, "A test title")
  expect_equal(data_home(pkg)$opengraph$description, "A test description.")

  # but overridden by home
  pkg$meta <- list(home = list(title = "X", description = "Y"))
  expect_equal(data_home(pkg)$pagetitle, "X")
  expect_equal(data_home(pkg)$opengraph$description, "Y")
})

test_that("math is handled", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "README.md", "$1 + 1$")
  suppressMessages(build_home_index(pkg))

  html <- xml2::read_html(path(pkg$dst_path, "index.html"))
  expect_equal(xpath_length(html, ".//math"), 1)
})

test_that("data_home() validates yaml metadata", {
  data_home_ <- function(...) {
    pkg <- local_pkgdown_site(meta = list(...))
    data_home(pkg)
  }

  expect_snapshot(error = TRUE, {
    data_home_(home = 1)
    data_home_(home = list(title = 1))
    data_home_(home = list(description = 1))
    data_home_(template = list(trailing_slash_redirect = 1))
  })
})

test_that("version formatting in preserved", {
  pkg <- local_pkgdown_site(desc = list(Version = "1.0.0-9000"))
  expect_equal(pkg$version, "1.0.0-9000")

  suppressMessages(build_home_index(pkg))
  index <- read_lines(path(pkg$dst_path, "index.html"))
  expect_match(index, "1.0.0-9000", fixed = TRUE, all = FALSE)
})

test_that("data_home_sidebar() works by default", {
  pkg <- local_pkgdown_site()
  expect_snapshot(cat(data_home_sidebar(pkg)))

  # comments are not included
  pkg <- local_pkgdown_site(desc = list(
    `Authors@R` = 'c(
    person("Hadley", "Wickham", , "hadley@rstudio.com", role = c("aut", "cre")),
    person("RStudio", role = c("cph", "fnd"), comment = c("Thank you!"))
    )'
  ))
  html <- xml2::read_html(data_home_sidebar(pkg))
  expect_snapshot_output(xpath_xml(html, ".//div[@class='developers']"))
})

test_that("data_home_sidebar() can be removed", {
  pkg <- local_pkgdown_site(meta = list(home = list(sidebar = FALSE)))
  # not built by data_home_sidebar()
  expect_false(data_home_sidebar(pkg))

  # nor later -- so probably not to be tested here?!
  suppressMessages(build_home_index(pkg))
  html <- xml2::read_html(path(pkg$dst_path, "index.html"))
  expect_equal(xpath_length(html, ".//aside/*"), 0)
})

test_that("data_home_sidebar() can be defined by a HTML file", {
  pkg <- local_pkgdown_site(
    meta = list(home = list(sidebar = list(html = "sidebar.html")))
  )
  expect_snapshot(data_home_sidebar(pkg), error = TRUE)

  pkg <- pkg_add_file(pkg, "sidebar.html", "Hello, world!")
  expect_equal(data_home_sidebar(pkg), "Hello, world!\n")
})

test_that("data_home_sidebar() can get a custom markdown formatted component", {
  pkg <- local_pkgdown_site(meta = list(
    home = list(
      sidebar = list(
        structure = "fancy",
        components = list(
          fancy = list(
            title = "Fancy section",
            text = "How *cool* is pkgdown?!"
          )
        )
      )
    )
  ))
  html <- xml2::read_html(data_home_sidebar(pkg))
  expect_snapshot_output(xpath_xml(html, ".//div[@class='fancy-section']"))
})

test_that("data_home_sidebar() can add a TOC", {
  pkg <- local_pkgdown_site(meta = list(
    home = list(sidebar = list(structure = "toc"))
  ))

  html <- xml2::read_html(data_home_sidebar(pkg))
  expect_snapshot_output(xpath_xml(html, ".//div[@class='table-of-contents']"))
})

test_that("data_home_sidebar() outputs informative error messages", {
  data_home_sidebar_ <- function(...) {
    pkg <- local_pkgdown_site(meta = list(home = list(sidebar = list(...))))
    data_home_sidebar(pkg)
  }

  expect_snapshot(error = TRUE, {
    data_home_sidebar_(html = 1)
    data_home_sidebar_(structure = 1)
    data_home_sidebar_(structure = "fancy")
    data_home_sidebar_(structure = c("fancy", "cool"))
    data_home_sidebar_(structure = "fancy", components = list(fancy = list(text = "bla")))
    data_home_sidebar_(structure = "fancy", components = list(fancy = list()))
  })
})

test_that("package repo verification", {
  skip_on_cran() # requires internet connection

  expect_null(cran_link("notarealpkg"))
  expect_equal(
    cran_link("dplyr"),
    list(repo = "CRAN", url = "https://cloud.r-project.org/package=dplyr")
  )
  expect_equal(
    cran_link("Biobase"),
    list(repo = "Bioconductor", url = "https://www.bioconductor.org/packages/Biobase")
  )
})


test_that("cran_unquote works", {
  expect_equal(
    cran_unquote("'Quoting' is CRAN's thing."),
    "Quoting is CRAN's thing."
  )
})

test_that("allow email in BugReports", {
  # currently desc throws a warning if BugReports is an email
  pkg <- local_pkgdown_site(desc = list(BugReports = "me@tidyverse.com"))
  html <- xml2::read_html(data_home_sidebar(pkg))
  expect_snapshot(xpath_xml(html, ".//li/a"))
})

simplify_dls <- function(html) {
  dls <- xml2::xml_find_all(html, ".//dl")
  for (dl in dls) {
    simplify_dl(dl)
  }
  invisible()
}

simplify_dl <- function(dl) {
  children <- xml2::xml_children(dl)

  names <- xml2::xml_name(children)
  if (!is_simple_dl(names)) {
    cli::cli_warn("Skipping this <dl>: not a simple term-definition list")
    return()
  }

  groups <- split(children, (seq_along(children) - 1) %/% 2)

  bullets <- lapply(groups, create_li_from_group)
  ul <- xml2::read_xml("<ul></ul>")
  xml_insert(ul, bullets)

  xml2::xml_replace(dl, ul)
}

# Must have an even number of children that alternate between dt and dd
is_simple_dl <- function(names) {
  if (length(names) %% 2 != 0) {
    return(FALSE)
  }
  odd <- names[seq_along(names) %% 2 == 1]
  even <- names[seq_along(names) %% 2 == 0]

  all(odd == "dt") && all(even == "dd")
}

create_li_from_group <- function(group) {
  dt <- group[[1]]
  dd <- group[[2]]

  if (has_children(dd)) {
    # params case
    para <- xml2::read_xml("<p></p>")
    xml_insert(para, xml2::xml_contents(dt))
    xml2::xml_add_child(para, xml_text_node(": "))

    bullet <- xml2::read_xml("<li></li>")
    xml2::xml_add_child(bullet, para)
  } else {
    # reference index
    bullet <- xml2::read_xml("<li></li>")
    xml_insert(bullet, xml2::xml_contents(dt))
    xml2::xml_add_child(bullet, xml_text_node(": "))
  }
  xml_insert(bullet, xml2::xml_contents(dd))

  bullet
}

has_children <- function(x) length(xml2::xml_children(x)) > 0

xml_text_node <- function(x) {
  span <- xml2::read_xml(paste0("<span>", x, "</span>"))
  xml2::xml_find_first(span, ".//text()")
}

#' Generate pkgdown data structure
#'
#' You will generally not need to use this unless you need a custom site
#' design and you're writing your own equivalent of [build_site()].
#'
#' @param pkg Path to package.
#' @param override An optional named list used to temporarily override
#'   values in `_pkgdown.yml`
#' @export
as_pkgdown <- function(pkg = ".", override = list()) {
  if (!is.list(override)) {
    cli::cli_abort(
      "{.arg override} must be a list, not {obj_type_friendly(override)}."
    )
  }

  if (is_pkgdown(pkg)) {
    pkg$meta <- modify_list(pkg$meta, override)
    return(pkg)
  }

  check_string(pkg)
  if (!dir_exists(pkg)) {
    if (dir.exists(pkg)) {
      #nolint
      # path expansion with fs and base R is different on Windows.
      # By default "~/", is typically C:/Users/username/Documents, while fs see "~/" as C:/Users/username, to be more platform portable.
      # Read more in ?fs::path_expand
      cli::cli_abort(
        "pkgdown accepts {.href [fs paths](https://fs.r-lib.org/reference/path_expand.html#details)}."
      )
    }
    cli::cli_abort("{.file {pkg}} is not an existing directory")
  }

  if (!dir.exists(pkg)) {
    #nolint
    # Use complete path if fs path doesn't exist according to base R #2639
    pkg <- path_expand(pkg)
  }

  src_path <- pkg

  desc <- read_desc(src_path)
  meta <- read_meta(src_path)
  meta <- modify_list(meta, override)

  # Create a partial pkgdown object so we can use config_pluck_* functions
  pkg <- list(
    package = desc$get_field("Package"),
    version = desc$get_field("Version"),
    src_path = path_abs(src_path),

    meta = meta,
    desc = desc
  )
  class(pkg) <- "pkgdown"

  # If boostrap version set locally, it drives the template choice
  # But if it's not set locally, the template may have a default
  template <- config_pluck_list(pkg, "template")
  template_package <- config_pluck_string(pkg, "template.package")

  bs_version_local <- get_bootstrap_version(pkg, pkg$meta$template)
  template_meta <- find_template_config(template_package, bs_version_local)

  if (is.null(bs_version_local)) {
    bs_version_remote <- get_bootstrap_version(
      pkg,
      template_meta$template,
      template_package
    )
  } else {
    bs_version_remote <- NULL
  }
  bs_version <- bs_version_local %||% bs_version_remote %||% 3
  check_bootstrap_version(bs_version, error_pkg = pkg)
  pkg$bs_version <- bs_version

  pkg$meta <- modify_list(template_meta, pkg$meta)

  # Ensure the URL has no trailing slash
  if (!is.null(pkg$meta$url)) {
    pkg$meta$url <- sub("/$", "", pkg$meta$url)
  }

  pkg$development <- meta_development(pkg)
  pkg$prefix <- pkg$development$prefix

  dest <- config_pluck_string(pkg, "destination")
  pkg$dst_path <- path_abs(dest %||% "docs", start = src_path)
  if (pkg$development$in_dev) {
    pkg$dst_path <- path(pkg$dst_path, pkg$development$destination)
  }

  pkg$lang <- get_pkg_lang(pkg)
  pkg$install_metadata <- config_pluck_bool(
    pkg,
    "deploy.install_metadata",
    FALSE
  )
  pkg$figures <- meta_figures(pkg)
  pkg$repo <- package_repo(pkg)
  pkg$topics <- package_topics(src_path)
  pkg$tutorials <- package_tutorials(src_path, meta)
  pkg$vignettes <- package_vignettes(src_path)

  pkg
}

is_pkgdown <- function(x) inherits(x, "pkgdown")

read_desc <- function(path = ".") {
  path <- path(path, "DESCRIPTION")
  if (!file_exists(path)) {
    cli::cli_abort("Can't find {.file DESCRIPTION}", call = caller_env())
  }
  desc::description$new(path)
}

get_pkg_lang <- function(pkg) {
  if (!is.null(pkg$meta$lang)) {
    return(pkg$meta$lang)
  }

  if (pkg$desc$has_fields("Language")) {
    field <- pkg$desc$get_field("Language")
    if (length(field) && nchar(field) > 0) {
      return(regmatches(field, regexpr("[^,]+", field)))
    }
  }

  return("en")
}

get_bootstrap_version <- function(
  pkg,
  template,
  template_package = NULL,
  call = caller_env()
) {
  if (is.null(template)) {
    return(NULL)
  }

  template_bootstrap <- template[["bootstrap"]]
  template_bslib <- template[["bslib"]][["version"]]

  if (!is.null(template_bootstrap) && !is.null(template_bslib)) {
    if (!is.null(template_package)) {
      hint <- "Specified locally and in template package {.pkg {template_package}}."
    } else {
      hint <- NULL
    }

    msg <- "must set only one of {.field template.bootstrap} and {.field template.bslib.version}."
    config_abort(pkg, c(msg, i = hint), call = call)
  }

  template_bootstrap %||% template_bslib
}

check_bootstrap_version <- function(
  version,
  error_pkg,
  error_call = caller_env()
) {
  if (version %in% c(3, 5)) {
    version
  } else if (version == 4) {
    msg <- c(
      "{.var template.bootstrap: 4} no longer supported",
      i = "Using {.var template.bootstrap: 5} instead"
    )
    config_warn(error_pkg, msg, call = error_call)
    5
  } else {
    msg <- "{.field template.bootstrap} must be 3 or 5, not {.val {version}}."
    config_abort(error_pkg, msg, error_call = caller_env())
  }
}

# Metadata ----------------------------------------------------------------

pkgdown_config_path <- function(path) {
  if (is_pkgdown(path)) {
    path <- path$src_path
  }

  path_first_existing(
    path,
    c(
      "_pkgdown.yml",
      "_pkgdown.yaml",
      "pkgdown/_pkgdown.yml",
      "pkgdown/_pkgdown.yaml",
      "inst/_pkgdown.yml",
      "inst/_pkgdown.yaml"
    )
  )
}

read_meta <- function(path, call = caller_env()) {
  path <- pkgdown_config_path(path)

  if (is.null(path)) {
    yaml <- list()
  } else {
    yaml <- withCallingHandlers(
      yaml::yaml.load_file(path, error.label = NULL) %||% list(),
      error = function(e) {
        cli::cli_abort(
          "Could not parse config file at {.path {path}}.",
          call = call,
          parent = e
        )
      }
    )
  }
  yaml
}

# Topics ------------------------------------------------------------------

package_topics <- function(path = ".") {
  # Needed if title contains sexpr
  local_context_eval()

  rd <- package_rd(path)

  aliases <- purrr::map(rd, extract_tag, "tag_alias")
  names <- purrr::map_chr(rd, extract_tag, "tag_name")
  titles <- purrr::map_chr(rd, extract_title)
  concepts <- unname(purrr::map(rd, extract_tag, "tag_concept"))
  keywords <- unname(purrr::map(rd, extract_tag, "tag_keyword"))
  internal <- purrr::map_lgl(keywords, ~ "internal" %in% .)
  source <- purrr::map(rd, extract_source)
  lifecycle <- unname(purrr::map(rd, extract_lifecycle))

  file_in <- names(rd)
  file_out <- rd_output_path(file_in)

  funs <- purrr::map(rd, topic_funs)

  tibble::tibble(
    name = names,
    file_in = file_in,
    file_out = file_out,
    alias = aliases,
    funs = funs,
    title = titles,
    rd = rd,
    source = source,
    keywords = keywords,
    concepts = concepts,
    internal = internal,
    lifecycle = lifecycle
  )
}

rd_output_path <- function(x) {
  x <- gsub("\\.Rd$", ".html", x)
  x[x == "index.html"] <- "index-topic.html"
  x
}

package_rd <- function(path = ".") {
  man_path <- path(path, "man")

  if (!dir_exists(man_path)) {
    return(set_names(list(), character()))
  }

  rd <- dir_ls(man_path, regexp = "\\.[Rr]d$", type = "file")
  names(rd) <- path_file(rd)
  lapply(rd, rd_file, pkg_path = path)
}

extract_tag <- function(x, tag) {
  purrr::map_chr(purrr::keep(x, inherits, tag), c(1, 1))
}

extract_title <- function(x) {
  title <- purrr::detect(x, inherits, "tag_title")
  str_squish(flatten_text(title, auto_link = FALSE))
}

extract_source <- function(x) {
  nl <- purrr::map_lgl(x, inherits, "TEXT") & x == "\n"
  comment <- purrr::map_lgl(x, inherits, "COMMENT")

  first_comment <- cumsum(!(nl | comment)) == 0
  lines <- as.character(x[first_comment])
  text <- paste0(lines, collapse = "")

  if (!grepl("roxygen2", text)) {
    return(character())
  }

  m <- gregexpr("R/[^ ]+\\.[rR]", text)
  regmatches(text, m)[[1]]
}

extract_lifecycle <- function(x) {
  desc <- purrr::keep(x, inherits, "tag_description")
  fig <- extract_figure(desc)

  if (!is.null(fig) && length(fig) > 0 && length(fig[[1]]) > 0) {
    path <- as.character(fig[[1]][[1]])
    if (grepl("lifecycle", path)) {
      name <- gsub("lifecycle-", "", path)
      name <- path_ext_remove(name)

      # Translate the most common lifecylce names
      name <- switch(
        name,
        deprecated = tr_("deprecated"),
        superseded = tr_("superseded"),
        experimental = tr_("experimental"),
        stable = tr_("stable"),
        name
      )

      return(name)
    }
  }
  NULL
}

extract_figure <- function(elements) {
  for (element in elements) {
    if (inherits(element, "tag_figure")) {
      return(element)
    } else if (inherits(element, "tag")) {
      child <- extract_figure(element)
      if (!is.null(child)) {
        return(child)
      }
    }
  }
  NULL
}

# Vignettes ---------------------------------------------------------------

package_vignettes <- function(path = ".") {
  base <- path(path, "vignettes")

  if (!dir_exists(base)) {
    vig_path <- character()
  } else {
    vig_path <- dir_ls(
      base,
      regexp = "\\.[Rrq]md$",
      type = "file",
      recurse = TRUE
    )
  }

  vig_path <- path_rel(vig_path, base)
  vig_path <- vig_path[!grepl("^_", path_file(vig_path))]
  vig_path <- vig_path[!grepl("^tutorials", path_dir(vig_path))]

  type <- tolower(path_ext(vig_path))

  meta <- purrr::map(path(base, vig_path), article_metadata)
  title <- purrr::map_chr(meta, "title")
  desc <- purrr::map_chr(meta, "desc")
  ext <- purrr::map_chr(meta, "ext")

  # Vignettes will be written to /articles/ with path relative to vignettes/
  # *except* for vignettes in vignettes/articles, which are moved up a level
  file_in <- path("vignettes", vig_path)
  file_out <- path_ext_set(vig_path, ext)
  file_out[!path_has_parent(file_out, "articles")] <- path(
    "articles",
    file_out[!path_has_parent(file_out, "articles")]
  )
  check_unique_article_paths(file_in, file_out)

  out <- tibble::tibble(
    name = as.character(path_ext_remove(vig_path)),
    type = type,
    file_in = as.character(file_in),
    file_out = as.character(file_out),
    title = title,
    description = desc,
    depth = dir_depth(file_out)
  )
  out[order(path_file(out$file_out)), ]
}

article_metadata <- function(path) {
  if (path_ext(path) == "qmd") {
    inspect <- quarto::quarto_inspect(path)
    meta <- inspect$formats[[1]]$metadata

    out <- list(
      title = meta$title %||% "UNKNOWN TITLE",
      desc = meta$description %||% NA_character_,
      ext = path_ext(inspect$formats[[1]]$pandoc$`output-file`) %||% "html"
    )
  } else {
    yaml <- rmarkdown::yaml_front_matter(path)
    out <- list(
      title = yaml$title[[1]] %||% "UNKNOWN TITLE",
      desc = yaml$description[[1]] %||% NA_character_,
      ext = yaml$pkgdown$extension %||% "html"
    )
  }

  if (out$ext == "pdf") {
    out$title <- paste0(out$title, " (PDF)")
  }

  out
}

find_template_config <- function(
  package,
  bs_version = NULL,
  error_call = caller_env()
) {
  if (is.null(package)) {
    return(list())
  }

  config <- path_package_pkgdown(
    "_pkgdown.yml",
    package,
    bs_version,
    error_call = error_call
  )
  if (!file_exists(config)) {
    return(list())
  }

  yaml::yaml.load_file(config) %||% list()
}

check_unique_article_paths <- function(file_in, file_out) {
  if (!any(duplicated(file_out))) {
    return()
  }
  # Since we move vignettes/articles/* up one level, we may end up with two
  # vignettes destined for the same final location. We also know that if there
  # are conflicting final paths, they are the result of exactly two source files

  file_out_dup <- file_out[duplicated(file_out)]

  same_out_bullets <- purrr::map_chr(file_out_dup, function(f_out) {
    src_files <- src_path(file_in[which(file_out == f_out)])
    src_files <- paste(src_files, collapse = " and ")
  })
  names(same_out_bullets) <- rep_len("x", length(same_out_bullets))

  cli::cli_abort(
    c(
      "Rendered articles must have unique names. Rename or relocate:",
      same_out_bullets
    ),
    call = caller_env()
  )
}

test_that("is_pkgdown checks its inputs", {
  expect_snapshot(error = TRUE, {
    as_pkgdown(1)
    as_pkgdown(override = 1)
  })

})

test_that("package_vignettes() doesn't trip over directories", {
  dir <- withr::local_tempdir()
  dir_create(path(dir, "vignettes", "test.Rmd"))
  file_create(path(dir, "vignettes", "test2.Rmd"))

  expect_equal(as.character(package_vignettes(dir)$file_in), "vignettes/test2.Rmd")
})

test_that("check_bootstrap_version() allows 3, 4 (with warning), and 5", {
  pkg <- local_pkgdown_site()

  expect_equal(check_bootstrap_version(3, pkg), 3)
  expect_snapshot(expect_equal(check_bootstrap_version(4, pkg), 5))
  expect_equal(check_bootstrap_version(5, pkg), 5)
})

test_that("check_bootstrap_version() gives informative error otherwise", {
  pkg <- local_pkgdown_site()
  expect_snapshot(check_bootstrap_version(1, pkg), error = TRUE)
})

test_that("package_vignettes() moves vignettes/articles up one level", {
  dir <- withr::local_tempdir()
  dir_create(path(dir, "vignettes", "articles"))
  file_create(path(dir, "vignettes", "articles", "test.Rmd"))

  pkg_vig <- package_vignettes(dir)
  expect_equal(as.character(pkg_vig$file_out), "articles/test.html")
  expect_equal(pkg_vig$depth, 1L)
})

test_that("package_vignettes() detects conflicts in final article paths", {
  dir <- withr::local_tempdir()
  dir_create(path(dir, "vignettes", "articles"))
  file_create(path(dir, "vignettes", "test.Rmd"))
  file_create(path(dir, "vignettes", "articles", "test.Rmd"))

  expect_error(package_vignettes(dir))
})

test_that("package_vignettes() sorts articles alphabetically by file name", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/a.Rmd")
  pkg <- pkg_add_file(pkg, "vignettes/b.Rmd")
  pkg <- pkg_add_file(pkg, "vignettes/c.Rmd")

  expect_equal(
    path_file(pkg$vignettes$file_out),
    c("a.html", "b.html", "c.html")
  )
})

test_that("override works correctly for as_pkgdown", {
  pkg1 <- local_pkgdown_site(meta = list(figures = list(dev = "jpeg")))
  pkg2 <- as_pkgdown(pkg1, override = list(figures = list(dev = "png")))
  expect_equal(pkg2$meta$figures$dev, "png")
})
# titles ------------------------------------------------------------------

test_that("multiline titles are collapsed", {
  rd <- rd_text("\\title{
    x
  }", fragment = FALSE)

  expect_equal(extract_title(rd), "x")
})

test_that("titles can contain other markup", {
  rd <- rd_text("\\title{\\strong{x}}", fragment = FALSE)
  expect_equal(extract_title(rd), "<strong>x</strong>")
})

test_that("titles don't get autolinked code", {
  rd <- rd_text("\\title{\\code{foo()}}", fragment = FALSE)
  expect_equal(extract_title(rd), "<code>foo()</code>")
})

test_that("read_meta() errors gracefully if _pkgdown.yml failed to parse", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "_pkgdown.yml", c(
    "url: https://pkgdown.r-lib.org",
    "  title: Build websites for R packages"
  ))
  expect_snapshot(
    as_pkgdown(pkg$src_path),
    error = TRUE,
    transform = function(x) gsub(pkg$src_path, "<src>", x, fixed = TRUE)
  )
})

# lifecycle ---------------------------------------------------------------

test_that("can extract lifecycle badges from description", {
  rd_desc <- rd_text(
    paste0("\\description{", lifecycle::badge("deprecated"), "}"),
    fragment = FALSE
  )
  rd_param <- rd_text(
    paste0("\\arguments{\\item{pkg}{", lifecycle::badge("deprecated"), "}}"),
    fragment = FALSE
  )

  expect_equal(extract_lifecycle(rd_desc), "deprecated")
  expect_equal(extract_lifecycle(rd_param), NULL)
})

test_that("malformed figures fail gracefully", {
  rd_lifecycle <- function(x) extract_lifecycle(rd_text(x))

  expect_null(rd_lifecycle("{\\figure{deprecated.svg}}"))
  expect_null(rd_lifecycle("{\\figure{}}"))
})

test_that("can get info about external function", {
  expect_snapshot(str(ext_topics("base::mean")))

  # and column names match
  pkg <- as_pkgdown(test_path("assets/reference"))
  expect_equal(names(ext_topics("base::mean")), names(pkg$topics))
})

test_that("fails if documentation not available", {
  expect_snapshot(ext_topics("base::doesntexist"), error = TRUE)
})

# File level tweaks --------------------------------------------
tweak_page <- function(html, name, pkg = list(bs_version = 3)) {
  # Syntax highlighting and linking
  if (name == "reference-topic") {
    # Reference topic takes a minimal approach since some is
    # already handled by Rd processing
    tweak_reference_highlighting(html)
    tweak_extra_logo(html)
  } else {
    downlit::downlit_html_node(html)

    # Rescue highlighting of non-collapsed output - needed for ANSI escapes
    pre <- xml2::xml_find_all(html, ".//pre[not(contains(@class, 'downlit'))]")
    is_wrapped <- is_wrapped_pre(pre)
    purrr::walk(pre[!is_wrapped], tweak_highlight_r)
  }

  tweak_anchors(html)
  tweak_link_md(html)
  tweak_link_external(html, pkg = pkg)
  tweak_img_src(html)
  tweak_strip(html, !identical(pkg$development$mode, "release"))

  # BS3 uses table for layout of reference-index
  if (name != "reference-index") {
    tweak_tables(html)
  }

  if (pkg$bs_version > 3) {
    tweak_footnotes(html)
    tweak_tabsets(html)
    tweak_useless_toc(html)
  }

  if (!is.null(pkg$desc) && pkg$desc$has_dep("R6")) {
    tweak_link_R6(html, pkg$package)
  }
}

tweak_rmarkdown_html <- function(html, input_path, pkg = list(bs_version = 3)) {
  # Tweak classes of navbar
  toc <- xml2::xml_find_all(html, ".//div[@id='tocnav']//ul")
  xml2::xml_attr(toc, "class") <- "nav nav-pills nav-stacked"

  # Make sure all images use relative paths
  img <- xml2::xml_find_all(html, "//img")
  src <- xml2::xml_attr(img, "src")
  abs_src <- is_absolute_path(src)
  if (any(abs_src)) {
    purrr::walk2(
      img[abs_src],
      path_rel(src[abs_src], input_path),
      xml2::xml_set_attr,
      attr = "src"
    )
  }

  # If top-level headings use h1, move everything down one level
  h1 <- xml2::xml_find_all(html, "//h1")
  if (length(h1) > 1) {
    tweak_section_levels(html)
  }

  # Has to occur after path normalisation
  # This get called twice on the contents of content-article.html, but that
  # should be harmless
  tweak_page(html, "article", pkg = pkg)

  invisible()
}

# Strip off #toc if it's not needed; easier to do this here than in js
tweak_useless_toc <- function(html) {
  contents <- xml2::xml_find_all(html, ".//main")
  headings <- xml2::xml_find_all(contents, ".//h2|.//h3|.//h4|.//h5|.//h6")

  if (length(headings) > 1) {
    return()
  }

  toc <- xml2::xml_find_first(html, '//nav[@id="toc"]')
  sidebar <- xml2::xml_parent(toc)
  if (length(xml2::xml_children(sidebar)) == 1) {
    xml2::xml_remove(sidebar)
  } else {
    xml2::xml_remove(toc)
  }
}


# Update file on disk -----------------------------------------------------

update_html <- function(path, tweak, ...) {

  raw <- read_file(path)
  # Following the xml 1.0 spec, libxml2 drops low-bit ASCII characters
  # so we convert to \u2029, relying on downlit to convert back in
  # token_escape().
  raw <- gsub("\033", "\u2029", raw, fixed = TRUE)
  html <- xml2::read_html(raw, encoding = "UTF-8")
  tweak(html, ...)

  xml2::write_html(html, path, format = FALSE)
  path
}

#' Build tutorials section
#'
#' learnr tutorials must be hosted elsewhere as they require an
#' R execution engine. Currently, pkgdown will not build or publish tutorials
#' for you, but makes it easy to embed (using `<iframe>`s) published tutorials.
#' Tutorials are automatically discovered from published tutorials in
#' `inst/tutorials` and `vignettes/tutorials`. Alternatively, you can
#' list in `_pkgdown.yml` as described below.
#'
#' # YAML config
#' To override the default discovery process, you can provide a `tutorials`
#' section. This should be a list where each element specifies:
#'
#' * `name`: used for the generated file name
#' * `title`: used in page heading and in navbar
#' * `url`: which will be embedded in an iframe
#' * `source`: optional, but if present will be linked to
#'
#' ```yaml
#' tutorials:
#' - name: 00-setup
#'   title: Setting up R
#'   url: https://jjallaire.shinyapps.io/learnr-tutorial-00-setup/
#' - name: 01-data-basics
#'   title: Data basics
#'   url: https://jjallaire.shinyapps.io/learnr-tutorial-01-data-basics/
#' ```
#' @inheritParams build_articles
#' @family site components
#' @export
build_tutorials <- function(pkg = ".", override = list(), preview = FALSE) {
  pkg <- section_init(pkg, "tutorials", override = override)

  tutorials <- pkg$tutorials

  if (nrow(tutorials) == 0) {
    return(invisible())
  }

  cli::cli_rule("Building tutorials")

  data <- purrr::transpose(tutorials)

  # Build index
  render_page(pkg, "tutorial-index",
    data = list(
      pagetitle = tr_("Tutorials"),
      tutorials = purrr::transpose(list(
        path = path_rel(tutorials$file_out, "tutorials"),
        title = tutorials$title
      ))
    ),
    path = "tutorials/index.html"
  )

  purrr::pwalk(
    list(data = data, path = tutorials$file_out),
    render_page,
    pkg = pkg,
    name = "tutorial"
  )

  preview_site(pkg, "tutorials", preview = preview)
}

package_tutorials <- function(path = ".", meta = list()) {
  # Look first in meta data
  tutorials <- purrr::pluck(meta, "tutorials")

  # Then scan tutorials directories
  if (length(tutorials) == 0) {
    tutorials <- c(
      find_tutorials(path(path, "inst", "tutorials")),
      find_tutorials(path(path, "vignettes", "tutorials"))
    )
  }

  name <- purrr::map_chr(tutorials, "name")
  title <- purrr::map_chr(tutorials, "title")

  tibble::tibble(
    name = name,
    file_out = as.character(path("tutorials", name, ext = "html")),
    title = title,
    pagetitle = title,
    url = purrr::map_chr(tutorials, "url")
  )
}

find_tutorials <- function(path = ".") {
  if (!dir_exists(path)) {
    return(character())
  }

  check_installed("rsconnect", "to find published tutorials")

  rmds <- unname(dir_ls(path, recurse = TRUE, regexp = "\\.[Rrq]md$", type = "file"))
  info <- purrr::map(rmds, tutorial_info, base_path = path)
  purrr::compact(info)
}

tutorial_info <- function(path, base_path) {
  yaml <- rmarkdown::yaml_front_matter(path)
  title <- yaml$title
  if (is.null(title)) {
    return()
  }

  # Must have "runtime: shiny". Partial implementation of full algorithm:
  # https://github.com/rstudio/rmarkdown/blob/master/R/shiny.R#L72-L100
  runtime <- yaml$runtime
  if (is.null(runtime) || !grepl("^shiny", runtime)) {
    return()
  }

  # Find deployment url. Use excludeOrphaned = FALSE since we don't need
  # to worry about redeploying and this makes test infrastructure a little
  # simpler
  deploys <- rsconnect::deployments(path, excludeOrphaned = FALSE)
  if (!is.null(deploys) && nrow(deploys) >= 1) {
    latest <- which.max(deploys$when)
    url <- deploys$url[[latest]]
  } else {
    return()
  }

  list(
    name = as.character(path_ext_remove(path_file(path))),
    title = yaml$title,
    url = url
    # source = as.character(path_rel(path, base_path))
  )
}

#' Generate YAML templates
#'
#' Use these function to generate the default YAML that pkgdown uses for
#' the different parts of `_pkgdown.yml`. This are useful starting
#' points if you want to customise your site.
#'
#' @param path Path to package root
#' @rdname templates
#' @examples
#' \dontrun{
#' pkgdown::template_navbar()
#' }
#'
#' @export
template_navbar <- function(path = ".") {
  pkg <- as_pkgdown(path)

  print_yaml(list(
    navbar = list(
      structure = navbar_structure(),
      components = navbar_components(pkg)
    )
  ))
}

#' @rdname templates
#' @examples
#' \dontrun{
#' pkgdown::template_reference()
#' }
#'
#' @export
template_reference <- function(path = ".") {
  print_yaml(list(reference = default_reference_index(path)))
}

#' @rdname templates
#' @examples
#' \dontrun{
#' pkgdown::template_articles()
#' }
#'
#' @export
template_articles <- function(path = ".") {
  print_yaml(list(articles = default_articles_index(path)))
}

#' Build a complete pkgdown website
#'
#' @description
#' `build_site()` is a convenient wrapper around six functions:
#'
#' * [init_site()]
#' * [build_home()]
#' * [build_reference()]
#' * [build_articles()]
#' * [build_tutorials()]
#' * [build_news()]
#' * [build_redirects()]
#' * [build_llm_docs()]
#'
#' See the documentation for the each function to learn how to control
#' that aspect of the site. This page documents options that affect the
#' whole site.
#'
#' # General config
#' *  `destination` controls where the site will be generated, defaulting to
#'    `docs/`. Paths are relative to the package root.
#'
#' *  `url` is optional, but strongly recommended.
#'
#'    ```yaml
#'    url: https://pkgdown.r-lib.org
#'    ```
#'
#'    It specifies where the site will be published and is used to allow other
#'    pkgdown sites to link to your site when needed (`vignette("linking")`),
#'    generate a `sitemap.xml`, automatically generate a `CNAME` when
#'    [deploying to github][build_site_github_pages()], generate the metadata needed
#'    rich social "media cards" (`vignette("metadata")`), and more.
#'
#' *  `title` overrides the default site title, which is the package name.
#'    It's used in the page title and default navbar.
#'
#' # Navbar and footer
#'
#' The `navbar` and `footer` fields control the appearance of the navbar
#' footer which appear on every page. Learn more about these fields in
#' `vignette("customise")`.
#'
#' # Development mode
#' The `development` field allows you to generate different sites for the
#' development and released versions of your package. To use it, you first
#' need to set the development `mode`:
#'
#' ```yaml
#' development:
#'   mode: auto
#' ```
#'
#' ### Setting development mode
#'
#' The development `mode` of a site controls where the built site is placed
#' and how it is styled (i.e. the colour of the package version in the
#' navbar, the version tooltip), and whether or not the site is indexed by
#' search engines. There are four possible modes:
#'
#' * **automatic** (`mode: auto`): determines the mode based on the version:
#'
#'   * `0.0.0.9000` (`0.0.0.*`): unreleased.
#'   * four version components: development.
#'   * everything else -> release.
#'
#' * **release** (`mode: release`), the default. Site is written to `docs/`
#'   and styled like a released package, even if the content is for an
#'   unreleased or development version. Version in navbar gets the default
#'   colouring. Development badges are not shown in the sidebar
#'   (see `?build_home`).
#'
#' * **development** (`mode: devel`). Site is written to `docs/dev/`.
#'   The navbar version gets a "danger" class and a tooltip stating these are
#'   docs for an in-development version of the package. The `noindex` meta tag
#'   is used to ensure that these packages are not indexed by search engines.
#'   Development badges are shown in the sidebar (see `?build_home`).
#'
#' * **unreleased** (`mode: unreleased`). Site is written to `docs/`.
#'   Version in navbar gets the "danger" class, and a message indicating the
#'   package is not yet on CRAN.
#'   Development badges are shown in the sidebar (see `?build_home`).
#'
#' Use `mode: auto` if you want both a released and a dev site, and
#' `mode: release` if you just want a single site. It is very rare that you
#' will need either devel or unreleased modes.
#'
#' You can override the mode specified in the `_pkgdown.yml` by setting
#' by setting `PKGDOWN_DEV_MODE` to `devel` or `release`.
#'
#' ### Selective HTML
#'
#' You can selectively show HTML only on the devel or release site by adding
#' class `pkgdown-devel` or `pkgdown-release`. This is most easily accessed
#' from `.Rmd` files where you can use pandoc's `<div>` syntax to control
#' where a block of markdown will display. For example, you can use the
#' following markdown in your README to only show GitHub install instructions
#' on the development version of your site:
#'
#' ```md
#' ::: {.pkgdown-devel}
#' You can install the development version of pkgdown from GitHub with:
#' `remotes::install_github("r-lib/pkgdown")`
#' :::
#' ```
#'
#' You can use a similar technique to control where badges are displayed.
#' This markdown show the CRAN status badge on the site for the released
#' package and the GitHub check status for the development package:
#'
#' ```md
#' [![CRAN Status](https://www.r-pkg.org/badges/version/pkgdown)]
#'   (https://cran.r-project.org/package=pkgdown){.pkgdown-release}
#' [![R-CMD-check](https://github.com/r-lib/pkgdown/workflows/R-CMD-check/badge.svg)]
#'   (https://github.com/r-lib/pkgdown/actions){.pkgdown-devel}
#' ```
#'
#' ### Other options
#'
#' There are three other options that you can control:
#'
#' ```yaml
#' development:
#'   destination: dev
#'   version_label: danger
#'   version_tooltip: "Custom message here"
#' ```
#'
#' `destination` allows you to override the default subdirectory used for the
#' development site; it defaults to `dev/`. `version_label` allows you to
#' override the style used for development (and unreleased) versions of the
#' package. It defaults to "danger", but you can set to "default", "info", or
#' "warning" instead. (The precise colours are determined by your bootstrap
#' theme, but become progressively more eye catching as you go from default
#' to danger). Finally, you can choose to override the default tooltip with
#' `version_tooltip`.
#'
#' # Template
#' The `template` field is mostly used to control the appearance of the site.
#' See `vignette("customise")` for details. But it's also used to control
#'
#' ## Other aspects
#'
#' There are a few other `template` fields that control other aspects of the
#' site:
#'
#' *   `noindex: true` will suppress indexing of your pages by search engines:
#'
#'     ```yaml
#'     template:
#'       params:
#'         noindex: true
#'     ```
#'
#' * `google_site_verification` allows you to verify your site with google:
#'
#'      ```yaml
#'      template:
#'        params:
#'          google_site_verification: _nn6ile-a6x6lctOW
#'      ```
#'
#' *   `trailing_slash_redirect: true` will automatically redirect
#'     `your-package-url.com` to `your-package-url.com/`, using a JS script
#'      added to the `<head>` of the home page. This is useful in certain
#'      redirect scenarios.
#'
#'      ```yaml
#'      template:
#'        trailing_slash_redirect: true
#'      ```
#'
#' ## Analytics
#'
#' To capture usage of your site with a web analytics tool, you can make
#' use of the `includes` field to add the special HTML they need. This HTML
#' is typically placed `in_header` (actually in the `<head>`), `before_body`,
#' or `after_body`.
#' You can learn more about how includes work in pkgdown at
#' <https://pkgdown.r-lib.org/articles/customise.html#additional-html-and-files>.
#'
#' I include a few examples of popular analytics platforms below, but we
#' recommend getting the HTML directly from the tool:
#'
#' *   [plausible.io](https://plausible.io):
#'
#'     ```yaml
#'     template:
#'       includes:
#'         in_header: |
#'           <script defer data-domain="{YOUR DOMAIN}" src="https://plausible.io/js/plausible.js"></script>
#'     ```
#'
#' *   [Google analytics](https://analytics.google.com/analytics/web/):
#'
#'     ```yaml
#'     template:
#'       includes:
#'         in_header: |
#'            <!-- Global site tag (gtag.js) - Google Analytics -->
#'            <script async src="https://www.googletagmanager.com/gtag/js?id={YOUR MEASUREMENT ID}" ></script>
#'            <script>
#'              window.dataLayer = window.dataLayer || [];
#'              function gtag(){dataLayer.push(arguments);}
#'              gtag('js', new Date());
#'
#'              gtag('config', '{YOUR MEASUREMENT ID}');
#'            </script>
#'            <!-- Google tag (gtag.js) -->
#'     ```
#' *   [GoatCounter](https://www.goatcounter.com):
#'
#'     ```yaml
#'     template:
#'       includes:
#'         after_body: >
#'           <script data-goatcounter="https://{YOUR CODE}.goatcounter.com/count" data-goatcounter-settings="{YOUR SETTINGS}" async src="https://gc.zgo.at/count.js"></script>
#'     ```
#'
#' # Source repository
#'
#' Use the `repo` field to override pkgdown's automatically discovery
#' of your source repository. This is used in the navbar, on the homepage,
#' in articles and reference topics, and in the changelog (to link to issue
#' numbers and user names). pkgdown can automatically figure out the necessary
#' URLs if you link to a GitHub, GitLab or Codeberg repo in your `BugReports`
#' or `URL` field.
#'
#' Otherwise, you can supply your own in the `repo` field:
#'
#' ```yaml
#' repo:
#'   url:
#'     home: https://github.com/r-lib/pkgdown/
#'     source: https://github.com/r-lib/pkgdown/blob/HEAD/
#'     issue: https://github.com/r-lib/pkgdown/issues/
#'     user: https://github.com/
#' ```
#'
#' * `home`: path to package home on source code repository.
#' * `source`: path to source of individual file in default branch
#'   (more on that below).
#' * `issue`: path to individual issue.
#' * `user`: path to user.
#'
#' The varying components (e.g. path, issue number, user name) are pasted on
#' the end of these URLs so they should have trailing `/`s.
#'
#' When creating the link to a package source, we have to link to a specific
#' branch. The default behaviour is to use current branch when in GitHub
#' actions and `HEAD` otherwise. You can overide this default with
#' `repo.branch`:
#'
#' ```yaml
#' repo:
#'   branch: devel
#' ```
#'
#' pkgdown can automatically link to Jira issues as well if specify both a
#' custom `issue` URL as well Jira project names to auto-link in
#' `jira_projects`. You can specify as many projects as you would like:
#'
#' ```yaml
#' repo:
#'   jira_projects: [this_project, another_project]
#'   url:
#'     issue: https://jira.organisation.com/jira/browse/
#' ```
#'
#' # Deployment (`deploy`)
#' There is a single `deploy` field
#'
#' *  `install_metadata` allows you to install package index metadata into
#'    the package itself. Normally this metadata is made available on the
#'    published site; installing it into your package means that it's
#'    available for autolinking even if your website is not reachable at build
#'    time (e.g. because behind a firewall or requires auth).
#'
#'    ```yaml
#'    deploy:
#'      install_metadata: true
#'    ```
#' @inheritParams build_articles
#' @inheritParams build_reference
#' @param lazy If `TRUE`, will only rebuild articles and reference pages
#'   if the source is newer than the destination.
#' @param devel Use development or deployment process?
#'
#'   If `TRUE`, uses lighter-weight process suitable for rapid
#'   iteration; it will run examples and vignettes in the current process,
#'   and will load code with `pkgload::load_all()`.
#'
#'   If `FALSE`, will first install the package to a temporary library,
#'   and will run all examples and vignettes in a new process.
#'
#'   `build_site()` defaults to `devel = FALSE` so that you get high fidelity
#'   outputs when you building the complete site; `build_reference()`,
#'   `build_home()` and friends default to `devel = TRUE` so that you can
#'   rapidly iterate during development.
#' @param new_process If `TRUE`, will run `build_site()` in a separate process.
#'   This enhances reproducibility by ensuring nothing that you have loaded
#'   in the current process affects the build process.
#' @param install If `TRUE`, will install the package in a temporary library
#'   so it is available for vignettes.
#' @param quiet If `FALSE`, generate build messages for build functions that
#'   take `quiet` arguments.
#' @export
#' @examples
#' \dontrun{
#' build_site()
#'
#' build_site(override = list(destination = tempdir()))
#' }
build_site <- function(
  pkg = ".",
  examples = TRUE,
  run_dont_run = FALSE,
  seed = 1014L,
  lazy = FALSE,
  override = list(),
  preview = NA,
  devel = FALSE,
  new_process = !devel,
  install = !devel,
  quiet = TRUE
) {
  pkg <- as_pkgdown(pkg, override = override)
  check_bool(devel)
  check_bool(new_process)
  check_bool(install)

  if (install) {
    withr::local_temp_libpaths()
    cli::cli_rule(
      "Installing package {.pkg {pkg$package}} into temporary library"
    )
    # Keep source, so that e.g. pillar can show the source code
    # of its functions in its articles
    withr::with_options(
      list(keep.source.pkgs = TRUE, keep.parse.data.pkgs = TRUE),
      utils::install.packages(
        pkg$src_path,
        repos = NULL,
        type = "source",
        quiet = TRUE
      )
    )
  }

  if (new_process) {
    build_site_external(
      pkg = pkg,
      examples = examples,
      run_dont_run = run_dont_run,
      seed = seed,
      lazy = lazy,
      override = override,
      preview = preview,
      devel = devel,
      quiet = quiet
    )
  } else {
    build_site_local(
      pkg = pkg,
      examples = examples,
      run_dont_run = run_dont_run,
      seed = seed,
      lazy = lazy,
      override = override,
      preview = preview,
      devel = devel,
      quiet = quiet
    )
  }
}

build_site_external <- function(
  pkg = ".",
  examples = TRUE,
  run_dont_run = FALSE,
  seed = 1014L,
  lazy = FALSE,
  override = list(),
  preview = NA,
  devel = TRUE,
  quiet = TRUE
) {
  pkg <- as_pkgdown(pkg, override = override)
  args <- list(
    pkg = pkg,
    examples = examples,
    run_dont_run = run_dont_run,
    seed = seed,
    lazy = lazy,
    override = override,
    install = FALSE,
    preview = FALSE,
    new_process = FALSE,
    devel = devel,
    quiet = quiet,
    cli_colors = cli::num_ansi_colors(),
    hyperlinks = cli::ansi_has_hyperlink_support()
  )
  callr::r(
    function(..., cli_colors, hyperlinks) {
      options(
        cli.num_colors = cli_colors,
        cli.hyperlink = hyperlinks,
        cli.hyperlink_run = hyperlinks
      )
      pkgdown::build_site(...)
    },
    args = args,
    show = TRUE
  )

  cli::cli_rule(
    "Finished building pkgdown site for package {.pkg {pkg$package}}"
  )

  preview_site(pkg, preview = preview)
  invisible()
}

build_site_local <- function(
  pkg = ".",
  examples = TRUE,
  run_dont_run = FALSE,
  seed = 1014L,
  lazy = FALSE,
  override = list(),
  preview = NA,
  devel = TRUE,
  quiet = TRUE
) {
  pkg <- section_init(pkg, override = override)

  cli::cli_rule("Building pkgdown site for package {.pkg {pkg$package}}")
  cli::cli_inform("Reading from: {src_path(path_abs(pkg$src_path))}")
  cli::cli_inform("Writing to:   {dst_path(path_abs(pkg$dst_path))}")

  pkgdown_sitrep(pkg)

  if (!lazy) {
    # Only force init_site() if `!lazy`
    # if site is not initialized, it will be in build_home()
    init_site(pkg, override)
  }

  build_home(pkg, override = override, quiet = quiet, preview = FALSE)
  build_reference(
    pkg,
    lazy = lazy,
    examples = examples,
    run_dont_run = run_dont_run,
    seed = seed,
    override = override,
    preview = FALSE,
    devel = devel
  )
  build_articles(
    pkg,
    lazy = lazy,
    override = override,
    quiet = quiet,
    preview = FALSE
  )
  build_tutorials(pkg, override = override, preview = FALSE)
  build_news(pkg, override = override, preview = FALSE)
  build_sitemap(pkg)
  if (pkg$bs_version > 3) {
    build_llm_docs(pkg)
  }
  build_redirects(pkg, override = override)
  if (pkg$bs_version == 3) {
    build_docsearch_json(pkg)
  } else {
    build_search(pkg, override = override)
  }

  check_built_site(pkg)

  cli::cli_rule(
    "Finished building pkgdown site for package {.pkg {pkg$package}}"
  )
  preview_site(pkg, preview = preview)
}

#' Open site in browser
#'
#' `preview_site()` opens your pkgdown site in your browser. pkgdown has been
#' carefully designed to work even when served from the file system like 
#' this; the only part that doesn't work is search. You can use `servr::httw("docs/")` 
#'  to create a server to make search work locally.
#' 
#' @inheritParams build_article
#' @param path Path relative to destination
#' @export
preview_site <- function(pkg = ".", path = ".", preview = TRUE) {
  path <- local_path(pkg, path)
  
  check_bool(preview, allow_na = TRUE)
  if (is.na(preview)) {
    preview <- interactive() && !is_testing()
  }

  if (preview) {
    cli::cli_inform(c(i = "Previewing site"))
    utils::browseURL(path)
  }

  invisible()
}

local_path <- function(pkg, path, call = caller_env()) {
  pkg <- as_pkgdown(pkg)
  check_string(path, call = call)

  abs_path <- path_abs(path, pkg$dst_path)
  if (!file_exists(abs_path)) {
    cli::cli_abort("Can't find file {.path {path}}.", call = call)
  }

  if (is_dir(abs_path)) {
    abs_path <- path(abs_path, "index.html")
  }
  abs_path
}

#' Preview a local pkgdown page in the browser
#'
#' @description
#' `r lifecycle::badge("deprecated")` Use [preview_site()] instead.
#'
#' @export
#' @keywords internal
preview_page <- function(path, pkg = ".") {
  lifecycle::deprecate_warn("2.1.0", "preview_page()", "preview_site()")
  preview_site(pkg, path, preview = TRUE)
}

is_testing <- function() {
  identical(Sys.getenv("TESTTHAT"), "true")
}

test_that("checks its inputs", {
  pkg <- local_pkgdown_site()

  expect_snapshot(error = TRUE, {
    preview_site(pkg, path = 1)
    preview_site(pkg, path = "foo")
    preview_site(pkg, preview = 1)
  })
})

test_that("local_path adds index.html if needed", {
  pkg <- local_pkgdown_site()
  file_create(path(pkg$dst_path, "test.html"))
  expect_equal(
    local_path(pkg, "test.html"),
    path(pkg$dst_path, "test.html")
  )

  dir_create(path(pkg$dst_path, "reference"))
  expect_equal(
    local_path(pkg, "reference"),
    path(pkg$dst_path, "reference", "index.html")
  ) 
})

as_html <- function(x, ...) {
  UseMethod("as_html")
}

# Various types of text ------------------------------------------------------

flatten_para <- function(x, ...) {
  if (length(x) == 0) {
    return(character())
  }

  # Look for "\n" TEXT blocks after a TEXT block, and not at end of file
  is_nl <- purrr::map_lgl(x, is_newline, trim = TRUE)
  is_text <- purrr::map_lgl(x, inherits, "TEXT")
  is_text_prev <- c(FALSE, is_text[-length(x)])
  has_next <- c(rep(TRUE, length(x) - 1), FALSE)
  is_para_break <- is_nl & is_text_prev & has_next

  # Or tags that are converted to HTML blocks
  block_tags <- c(
    "tag_preformatted",
    "tag_itemize",
    "tag_enumerate",
    "tag_tabular",
    "tag_describe",
    "tag_subsection"
  )
  is_block <- purrr::map_lgl(x, inherits, block_tags)

  # Break before and after each status change
  before_break <- is_para_break | is_block
  after_break <- c(FALSE, before_break[-length(x)])
  groups <- cumsum(before_break | after_break)

  unwrap_purrr_error(html <- purrr::map(x, as_html, ...))
  # split at line breaks for everything except blocks
  empty <- purrr::map_lgl(x, purrr::is_empty)
  needs_split <- !is_block & !empty
  html[needs_split] <- purrr::map(html[needs_split], split_at_linebreaks)

  blocks <- purrr::map_chr(split(html, groups), function(x) {
    paste(unlist(x), collapse = "")
  })

  # There are three types of blocks:
  # 1. Combined text and inline tags
  # 2. Paragraph breaks (text containing only "\n")
  # 3. Block-level tags
  #
  # Need to wrap 1 in <p>
  needs_p <- purrr::map_lgl(split(!(is_nl | is_block), groups), any)
  blocks[needs_p] <- paste0("<p>", str_trim(blocks[needs_p]), "</p>")

  paste0(blocks, collapse = "")
}

split_at_linebreaks <- function(text) {
  if (length(text) == 0) {
    character()
  } else {
    strsplit(text, "\\n\\s*\\n")[[1]]
  }
}

flatten_text <- function(x, ...) {
  if (length(x) == 0) {
    return("")
  }

  unwrap_purrr_error(html <- purrr::map_chr(x, as_html, ...))
  paste(html, collapse = "")
}

#' @export
as_html.Rd <- function(x, ...) flatten_text(x, ...)

#' @export
as_html.LIST <- flatten_text

# Leaves  -----------------------------------------------------------------

#' @export
as_html.character <- function(x, ..., escape = TRUE) {
  # src_highlight (used by usage, examples, and out) also does escaping
  # so we need some way to turn it off when needed.
  if (escape) {
    escape_html(x)
  } else {
    as.character(x)
  }
}
#' @export
as_html.TEXT <- function(x, ..., escape = TRUE) {
  # tools:::htmlify
  x <- gsub("---", "\u2014", x)
  x <- gsub("--", "\u2013", x)
  x <- gsub("``", "\u201c", x)
  x <- gsub("''", "\u201d", x)

  x <- as_html.character(x, ..., escape = escape)
  x
}
#' @export
as_html.RCODE <- as_html.character
#' @export
as_html.VERB <- as_html.character
#' @export
as_html.COMMENT <- function(x, ...) {
  paste0("<!-- ", flatten_text(x), " -->")
}
# USERMACRO appears first, followed by the rendered macro
#' @export
as_html.USERMACRO <- function(x, ...) ""

#' @export
as_html.tag_subsection <- function(x, ..., subsection_level = 3L) {
  h <- paste0("h", subsection_level)

  title <- flatten_text(x[[1]], ...)
  id <- make_slug(title)
  text <- flatten_para(x[[2]], ..., subsection_level = subsection_level + 1L)

  # fmt: skip
  paste0(
    "<div class='section' id='", id, "'>\n",
    "<", h, ">", title, "</", h, ">\n",
    text, "\n",
    "</div>"
  )
}

# Equations ------------------------------------------------------------------

#' @export
as_html.tag_eqn <- function(x, ...) {
  latex_rep <- x[[1]]
  paste0("\\(", flatten_text(latex_rep, ...), "\\)")
}

#' @export
as_html.tag_deqn <- function(x, ...) {
  latex_rep <- x[[1]]
  paste0("$$", flatten_text(latex_rep, ...), "$$")
}

# Links ----------------------------------------------------------------------
#' @export
as_html.tag_url <- function(x, ...) {
  if (length(x) != 1) {
    if (length(x) == 0) {
      msg <- "Check for empty \\url{{}} tags."
    } else {
      msg <- "This may be caused by a \\url tag that spans a line break."
    }
    stop_bad_tag("url", msg)
  }

  text <- flatten_text(x[[1]])
  a(text, href = text)
}
#' @export
as_html.tag_href <- function(x, ...) {
  a(flatten_text(x[[2]]), href = flatten_text(x[[1]]))
}
#' @export
as_html.tag_email <- function(x, ...) {
  if (length(x) != 1) {
    stop_bad_tag("email", "empty {}")
  }
  paste0("<a href='mailto:", x[[1]], "'>", x[[1]], "</a>")
}

# If single, need to look up alias to find file name and package
#' @export
as_html.tag_link <- function(x, ...) {
  opt <- attr(x, "Rd_option")

  in_braces <- flatten_text(x)

  if (is.null(opt)) {
    # \link{topic}
    href <- downlit::href_topic(in_braces)
  } else if (substr(opt, 1, 1) == "=") {
    # \link[=dest]{name}
    href <- downlit::href_topic(substr(opt, 2, nchar(opt)))
  } else {
    match <- regexec('^([^:]+)(?:|:(.*))$', opt)
    parts <- regmatches(opt, match)[[1]][-1]

    if (parts[[2]] == "") {
      # \link[pkg]{foo}
      href <- downlit::href_topic(in_braces, opt)
    } else {
      # \link[pkg:bar]{foo}
      href <- downlit::href_topic(parts[[2]], parts[[1]])
    }
  }

  a(in_braces, href = href)
}

#' @export
as_html.tag_linkS4class <- function(x, ...) {
  if (length(x) != 1) {
    stop_bad_tag("linkS4class")
  }

  text <- flatten_text(x[[1]])
  href <- downlit::href_topic(paste0(text, "-class"))
  a(text, href = href)
}

# Conditionals and Sexprs ----------------------------------------------------

#' @export
as_html.tag_Sexpr <- function(x, ...) {
  code <- flatten_text(x, escape = FALSE)
  options <- parse_opts(attr(x, "Rd_option"))

  # Needs to be package root
  old_wd <- setwd(context_get("src_path"))
  on.exit(setwd(old_wd), add = TRUE)

  # Environment shared across a file
  env <- context_get("sexpr_env")

  results <- options$results %||% "rd"
  if (results == "verbatim") {
    outlines <- utils::capture.output({
      out <- withVisible(eval(parse(text = code), env))
      res <- out$value
      if (out$visible) print(res)
    })
    paste0(
      "<pre>\n",
      paste0(escape_html(outlines), collapse = "\n"),
      "\n</pre>\n"
    )
  } else {
    res <- eval(parse(text = code), env)
    switch(
      results,
      text = as.character(res),
      rd = flatten_text(rd_text(as.character(res))),
      hide = "",
      cli::cli_abort(
        "unknown \\Sexpr option: results={results}",
        call = NULL
      )
    )
  }
}

#' @export
as_html.tag_if <- function(x, ...) {
  if (x[[1]] == "html") {
    as_html(x[[2]], ...)
  } else {
    ""
  }
}

#' @export
as_html.tag_ifelse <- function(x, ...) {
  if (x[[1]] == "html") as_html(x[[2]], ...) else as_html(x[[3]], ...)
}

# Used inside a \usage{} Rd tag to prevent the code from being treated as
# regular R syntax, either because it is not valid R, or because its usage
# intentionally deviates from regular R usage. An example of the former is the
# command line documentation, e.g. `R CMD SHLIB`
# (https://github.com/wch/r-source/blob/trunk/src/library/utils/man/SHLIB.Rd):
#
#    \special{R CMD SHLIB [options] [-o dllname] files}
#
# An example of the latter is the documentation shortcut `?`
# (https://github.com/wch/r-source/blob/trunk/src/library/utils/man/Question.Rd):
#
#    \special{?topic}
#
#' @export
as_html.tag_special <- function(x, ...) {
  flatten_text(x, ...)
}

#' @export
`as_html.#ifdef` <- function(x, ...) {
  os <- trimws(flatten_text(x[[1]]))
  if (os == "unix") {
    flatten_text(x[[2]])
  } else {
    ""
  }
}

#' @export
`as_html.#ifndef` <- function(x, ...) {
  os <- trimws(flatten_text(x[[1]]))
  if (os == "windows") {
    flatten_text(x[[2]])
  } else {
    ""
  }
}

# Tables ---------------------------------------------------------------------

#' @export
as_html.tag_tabular <- function(x, ...) {
  align_abbr <- strsplit(as_html(x[[1]], ...), "")[[1]]
  align_abbr <- align_abbr[!(align_abbr %in% c("|", ""))]
  align <- unname(c("r" = "right", "l" = "left", "c" = "center")[align_abbr])

  contents <- x[[2]]
  class <- purrr::map_chr(contents, ~ class(.x)[[1]])
  cell_contents <- purrr::map_chr(contents, flatten_text, ...)

  row_sep <- class == "tag_cr"
  contents[row_sep] <- list("")
  col_sep <- class == "tag_tab"
  sep <- col_sep | row_sep

  # Identify groups in reverse order (preserve empty cells)
  # Negative maintains correct ordering once reversed
  cell_grp <- rev(cumsum(-rev(sep)))
  cells <- unname(split(contents, cell_grp))
  # Remove trailing content (that does not match the dimensions of the table)
  cells <- cells[seq_len(length(cells) - length(cells) %% length(align))]
  cell_contents <- purrr::map_chr(cells, flatten_text, ...)
  cell_contents <- paste0("<td>", str_trim(cell_contents), "</td>")
  cell_contents <- matrix(cell_contents, ncol = length(align), byrow = TRUE)

  rows <- apply(cell_contents, 1, paste0, collapse = "")

  paste0(
    "<table class='table'>\n",
    paste0("<tr>", rows, "</tr>\n", collapse = ""),
    "</table>\n"
  )
}


# Figures -----------------------------------------------------------------

#' @export
as_html.tag_figure <- function(x, ...) {
  n <- length(x)
  path <- as.character(x[[1]])

  if (n == 1) {
    paste0("<img src='figures/", path, "' alt='' />")
  } else if (n == 2) {
    opt <- paste(trimws(as.character(x[[2]])), collapse = " ")
    if (substr(opt, 1, 9) == "options: ") {
      extra <- substr(opt, 9, nchar(opt))
      paste0("<img src='figures/", path, "'", extra, " />")
    } else {
      paste0("<img src='figures/", path, "' alt='", opt, "' />")
    }
  }
}

# List -----------------------------------------------------------------------

#' @export
as_html.tag_itemize <- function(x, ...) {
  paste0("<ul>\n", parse_items(x[-1], ...), "</ul>")
}
#' @export
as_html.tag_enumerate <- function(x, ...) {
  paste0("<ol>\n", parse_items(x[-1], ...), "</ol>")
}
#' @export
as_html.tag_describe <- function(x, ...) {
  paste0("<dl>\n", parse_descriptions(x[-1], ...), "\n</dl>")
}

# Effectively does nothing: only used by parse_items() to split up
# sequence of tags.
#' @export
as_html.tag_item <- function(x, ...) {
  ""
}

parse_items <- function(rd, ...) {
  separator <- purrr::map_lgl(rd, inherits, "tag_item")
  group <- cumsum(separator)

  # Drop anything before first tag_item
  if (!all(group == 0) && any(group == 0)) {
    rd <- rd[group != 0]
    group <- group[group != 0]
  }

  parse_item <- function(x) {
    x <- trim_ws_nodes(x)
    paste0("<li>", flatten_para(x, ...), "</li>\n")
  }

  paste(purrr::map_chr(split(rd, group), parse_item), collapse = "")
}

parse_descriptions <- function(rd, ..., id_prefix = NULL) {
  if (length(rd) == 0) {
    return(character())
  }

  parse_item <- function(x) {
    if (inherits(x, "tag_item")) {
      term <- flatten_text(x[[1]], ...)
      def <- flatten_para(x[[2]], ...)

      if (!is.null(id_prefix)) {
        id <- paste0(id_prefix, make_slug(term))
        id_attr <- paste0(" id='", id, "'")
        anchor <- anchor_html(id)
      } else {
        id_attr <- ""
        anchor <- ""
      }
      paste0(
        "<dt",
        id_attr,
        ">",
        term,
        anchor,
        "</dt>\n",
        "<dd>",
        def,
        "</dd>\n"
      )
    } else {
      flatten_text(x, ...)
    }
  }

  paste(purrr::map_chr(rd, parse_item), collapse = "")
}

# Marking text ------------------------------------------------------------
# https://cran.rstudio.com/doc/manuals/r-devel/R-exts.html#Marking-text

tag_wrapper <- function(prefix, suffix = NULL) {
  function(x, ...) {
    html <- flatten_text(x, ...)
    paste0(prefix, html, suffix)
  }
}

#' @export
as_html.tag_emph <- tag_wrapper("<em>", "</em>")
#' @export
as_html.tag_strong <- tag_wrapper("<strong>", "</strong>")
#' @export
as_html.tag_bold <- tag_wrapper("<b>", "</b>")

#' @export
as_html.tag_dQuote <- tag_wrapper("&#8220;", "&#8221;")
#' @export
as_html.tag_sQuote <- tag_wrapper("&#8216;", "&#8217;")

#' @export
as_html.tag_code <- function(x, ..., auto_link = TRUE) {
  text <- flatten_text(x, ...)

  if (auto_link) {
    href <- downlit::autolink_url(text)
    text <- a(text, href = href)
  }
  paste0("<code>", text, "</code>")
}

#' @export
as_html.tag_preformatted <- function(x, ...) {
  # the language is stored in a prior \if{}{\out{}} block, so we delay
  # highlighting until we have the complete html page
  pre(flatten_text(x, ...))
}

#' @export
as_html.tag_kbd <- tag_wrapper("<kbd>", "</kbd>")
#' @export
as_html.tag_samp <- tag_wrapper('<samp>', "</samp>")
#' @export
as_html.tag_verb <- tag_wrapper("<code>", "</code>")

#' @export
as_html.tag_pkg <- tag_wrapper('<span class="pkg">', "</span>")
#' @export
as_html.tag_file <- tag_wrapper('<code class="file">', '</code>')

#' @export
as_html.tag_var <- tag_wrapper("<var>", "</var>")
#' @export
as_html.tag_env <- tag_wrapper('<code class="env">', '</code>')
#' @export
as_html.tag_option <- tag_wrapper('<span class="option">', "</span>")
#' @export
as_html.tag_command <- tag_wrapper("<code class='command'>", "</code>")

#' @export
as_html.tag_dfn <- tag_wrapper("<dfn>", "</dfn>")
#' @export
as_html.tag_cite <- tag_wrapper("<cite>", "</cite>")
#' @export
as_html.tag_acronym <- tag_wrapper('<acronym>', '</acronym>')

#' @export
as_html.tag_out <- function(x, ...) flatten_text(x, ..., escape = FALSE)

# Insertions --------------------------------------------------------------

#' @export
as_html.tag_R <- function(x, ...) '<span style="R">R</span>'
#' @export
as_html.tag_dots <- function(x, ...) "..."
#' @export
as_html.tag_ldots <- function(x, ...) "..."
#' @export
as_html.tag_cr <- function(x, ...) "<br>"

# First element of enc is the encoded version (second is the ascii version)
#' @export
as_html.tag_enc <- function(x, ...) {
  if (length(x) == 2) {
    as_html(x[[1]], ...)
  } else {
    stop_bad_tag("enc")
  }
}

# Elements that don't return anything ----------------------------------------

#' @export
as_html.tag_tab <- function(x, ...) ""
#' @export
as_html.tag_newcommand <- function(x, ...) ""
#' @export
as_html.tag_renewcommand <- function(x, ...) ""

#' @export
as_html.tag <- function(x, ...) {
  if (identical(class(x), "tag")) {
    flatten_text(x, ...)
  } else {
    cli::cli_inform("Unknown tag: {.cls {class(x)}}")
    ""
  }
}

# Whitespace helper -------------------------------------------------------

trim_ws_nodes <- function(x, side = c("both", "left", "right")) {
  is_ws <- purrr::map_lgl(x, ~ inherits(., "TEXT") && grepl("^\\s*$", .[[1]]))

  if (!any(is_ws)) {
    return(x)
  }
  if (all(is_ws)) {
    return(x[0])
  }

  which_not <- which(!is_ws)

  side <- match.arg(side)
  if (side %in% c("left", "both")) {
    start <- which_not[1]
  } else {
    start <- 1
  }

  if (side %in% c("right", "both")) {
    end <- which_not[length(which_not)]
  } else {
    end <- length(x)
  }

  x[start:end]
}


# Helpers -----------------------------------------------------------------

parse_opts <- function(string) {
  if (is.null(string)) {
    return(list())
  }

  args <- list("text", "verbatim", "rd", "hide", "build", "install", "render")
  names(args) <- args
  arg_env <- child_env(baseenv(), !!!args)

  args <- strsplit(string, ",")[[1]]
  exprs <- purrr::map(args, parse_expr)

  env <- child_env(arg_env)
  purrr::walk(exprs, eval_bare, env = env)

  as.list(env)
}

stop_bad_tag <- function(tag, msg = NULL) {
  bad_tag <- paste0("\\", tag, "{}")
  msg_abort <- 'Failed to parse tag {.val {bad_tag}}.'
  cli::cli_abort(c(msg_abort, i = msg), call = NULL)
}

is_newline <- function(x, trim = FALSE) {
  if (!inherits(x, "TEXT") && !inherits(x, "RCODE") && !inherits(x, "VERB")) {
    return(FALSE)
  }

  text <- x[[1]]
  if (trim) {
    text <- gsub("^[ \t]+|[ \t]+$", "", text)
  }
  identical(text, "\n")
}

data_navbar <- function(pkg = ".", depth = 0L, call = caller_env()) {
  pkg <- as_pkgdown(pkg)

  navbar <- config_pluck(pkg, "navbar")

  if (uses_lightswitch(pkg)) {
    style <- NULL
  } else {
    style <- navbar_style(
      navbar = navbar,
      theme = get_bslib_theme(pkg),
      bs_version = pkg$bs_version
    )
  }

  links <- navbar_links(pkg, depth = depth, call = call)

  c(style, links)
}

uses_lightswitch <- function(pkg) {
  config_pluck_bool(pkg, "template.light-switch", default = FALSE)
}

# Default navbar ----------------------------------------------------------

navbar_style <- function(navbar = list(), theme = "_default", bs_version = 3) {
  if (bs_version == 3) {
    list(type = navbar$type %||% "default")
  } else {
    # bg is usually light, dark, or primary, but can use any .bg-*
    bg <- navbar$bg %||% bootswatch_bg[[theme]] %||% "light"
    type <- navbar$type %||% if (bg == "light") "light" else "dark"

    list(bg = bg, type = type)
  }
}

navbar_structure <- function() {
  print_yaml(list(
    left = c("intro", "reference", "articles", "tutorials", "news"),
    right = c("search", "github", "lightswitch")
  ))
}

navbar_links <- function(pkg, depth = 0L, call = caller_env()) {
  components <- navbar_link_components(pkg, call = call)

  list(
    left = render_navbar_links(
      components$left,
      depth = depth,
      pkg = pkg,
      side = "left"
    ),
    right = render_navbar_links(
      components$right,
      depth = depth,
      pkg = pkg,
      side = "right"
    )
  )
}

navbar_link_components <- function(pkg, call = caller_env()) {
  # Combine default components with user supplied: must not merge recursively
  components <- navbar_components(pkg)
  components_meta <- config_pluck(pkg, "navbar.components", default = list())
  components[names(components_meta)] <- components_meta
  components <- purrr::compact(components)

  # Combine default structure with user supplied
  # (must preserve NULLs in yaml to mean display nothing)
  pkg$meta$navbar$structure <- modify_list(
    navbar_structure(),
    config_pluck(pkg, "navbar.structure")
  )
  right_comp <- intersect(
    config_pluck_character(pkg, "navbar.structure.right", call = call),
    names(components)
  )
  left_comp <- intersect(
    config_pluck_character(pkg, "navbar.structure.left", call = call),
    names(components)
  )
  # Backward compatibility
  left <- config_pluck(pkg, "navbar.left") %||% components[left_comp]
  right <- config_pluck(pkg, "navbar.right") %||% components[right_comp]

  list(left = left, right = right)
}

render_navbar_links <- function(x, depth = 0L, pkg, side = c("left", "right")) {
  if (!is.list(x)) {
    config_abort(
      pkg,
      c(
        "{.field navbar} is incorrectly specified.",
        i = "See details in {.vignette pkgdown::customise}."
      ),
      call = quote(data_template())
    )
  }
  check_number_whole(depth, min = 0)
  side <- arg_match(side)

  tweak <- function(x) {
    if (!is.null(x$menu)) {
      x$menu <- lapply(x$menu, tweak)
      x
    } else if (!is.null(x$href) && !grepl("://", x$href, fixed = TRUE)) {
      x$href <- paste0(up_path(depth), x$href)
      x
    } else {
      x
    }
  }
  if (depth != 0L) {
    x <- lapply(x, tweak)
  }

  if (pkg$bs_version == 3) {
    rmarkdown::navbar_links_html(x)
  } else {
    navbar_html_list(x, path_depth = depth, side = side)
  }
}

# Components --------------------------------------------------------------

navbar_components <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)

  menu <- list()
  menu$reference <- menu_link(tr_("Reference"), "reference/index.html")

  # in BS3, search is hardcoded in the template
  if (pkg$bs_version == 5) {
    menu$search <- menu_search()
  }

  if (uses_lightswitch(pkg)) {
    menu$lightswitch <- menu_submenu(
      text = NULL,
      icon = "fa-sun",
      label = tr_("Light switch"),
      id = "lightswitch",
      list(
        menu_theme(tr_("Light"), icon = "fa-sun", theme = "light"),
        menu_theme(tr_("Dark"), icon = "fa-moon", theme = "dark"),
        menu_theme(tr_("Auto"), icon = "fa-adjust", theme = "auto")
      )
    )
  }

  if (!is.null(pkg$tutorials)) {
    menu$tutorials <- menu_submenu(
      tr_("Tutorials"),
      menu_links(pkg$tutorials$title, pkg$tutorials$file_out)
    )
  }
  menu$news <- navbar_news(pkg)

  menu$github <- switch(
    repo_type(pkg),
    github = menu_icon("fab fa-github fa-lg", repo_home(pkg), "GitHub"),
    gitlab = menu_icon("fab fa-gitlab fa-lg", repo_home(pkg), "GitLab"),
    codeberg = menu_icon("fas fa-code fa-lg", repo_home(pkg), "Codeberg"),
    NULL
  )

  menu <- c(menu, navbar_articles(pkg))

  print_yaml(menu)
}

navbar_articles <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)

  menu <- list()

  vignettes <- pkg$vignettes
  pkg_intro <- article_is_intro(vignettes$name, pkg$package)
  if (any(pkg_intro)) {
    intro <- vignettes[pkg_intro, , drop = FALSE]

    menu$intro <- menu_link(tr_("Get started"), intro$file_out)
  }

  if (!has_name(pkg$meta, "articles")) {
    vignettes <- vignettes[!pkg_intro, , drop = FALSE]
    menu$articles <- menu_submenu(
      tr_("Articles"),
      menu_links(vignettes$title, vignettes$file_out)
    )
  } else {
    articles_index <- config_pluck(pkg, "articles")
    articles <- data_articles(pkg)

    navbar <- purrr::keep(articles_index, ~ has_name(.x, "navbar"))
    if (length(navbar) == 0) {
      # No articles to be included in navbar so just link to index
      menu$articles <- menu_link(tr_("Articles"), "articles/index.html")
    } else {
      sections <- purrr::imap(navbar, function(section, index) {
        idx <- select_topics(
          section$contents,
          articles,
          error_pkg = pkg,
          error_path = paste0("articles[", index, "].contents")
        )
        vig <- articles[idx, , drop = FALSE]
        vig <- vig[vig$name != pkg$package, , drop = FALSE]
        c(
          if (!is.null(section$navbar)) {
            list(menu_separator(), menu_heading(section$navbar))
          },
          menu_links(vig$title, vig$href)
        )
      })
      children <- unlist(sections, recursive = FALSE, use.names = FALSE)

      if (length(navbar) != length(articles_index)) {
        children <- c(
          children,
          list(
            menu_separator(),
            menu_link(tr_("More articles..."), "articles/index.html")
          )
        )
      }
      menu$articles <- menu_submenu(tr_("Articles"), children)
    }
  }
  print_yaml(menu)
}

# Testing helpers ---------------------------------------------------------
# Simulate minimal package structure so we can more easily test

pkg_navbar <- function(
  meta = NULL,
  vignettes = pkg_navbar_vignettes(),
  github_url = NULL
) {
  structure(
    list(
      package = "test",
      src_path = file_temp(),
      meta = meta,
      vignettes = vignettes,
      repo = list(url = list(home = github_url)),
      bs_version = 5
    ),
    class = "pkgdown"
  )
}

pkg_navbar_vignettes <- function(
  name = character(),
  title = NULL,
  file_out = NULL
) {
  title <- title %||% paste0("Title ", name)
  file_out <- file_out %||% paste0(name, ".html")

  tibble::tibble(name = name, title = title, file_out, description = "desc")
}


# bootswatch defaults -----------------------------------------------------

# Scraped from bootswatch preivews, see code in
# <https://github.com/r-lib/pkgdown/issues/1758>
bootswatch_bg <- list(
  "_default" = "light",
  cerulean = "primary",
  cosmo = "primary",
  cyborg = "dark",
  darkly = "primary",
  flatly = "primary",
  journal = "light",
  litera = "light",
  lumen = "light",
  lux = "light",
  materia = "primary",
  minty = "primary",
  morph = "primary",
  pulse = "primary",
  quartz = "primary",
  sandstone = "primary",
  simplex = "light",
  sketchy = "light",
  slate = "primary",
  solar = "dark",
  spacelab = "light",
  superhero = "dark",
  united = "primary",
  vapor = "primary",
  yeti = "primary",
  zephyr = "primary"
)

test_that("adds github/gitlab link when available", {
  pkg <- pkg_navbar()
  expect_snapshot_output(navbar_components(pkg))

  pkg <- pkg_navbar(github_url = "https://github.com/r-lib/pkgdown")
  expect_snapshot_output(navbar_components(pkg))

  pkg <- pkg_navbar(github_url = "https://gitlab.com/r-lib/pkgdown")
  expect_snapshot_output(navbar_components(pkg))
})

test_that("vignette with package name turns into getting started", {
  vig <- pkg_navbar_vignettes("test")
  pkg <- pkg_navbar(vignettes = vig)
  expect_snapshot_output(navbar_components(pkg))
})

test_that("can control articles navbar through articles meta", {
  pkg <- function(...) {
    vig <- pkg_navbar_vignettes(c("a", "b"))
    pkg_navbar(vignettes = vig, meta = list(...))
  }

  "Default: show all alphabetically"
  expect_snapshot(navbar_articles(pkg()))

  "No navbar sections: link to index"
  expect_snapshot(
    navbar_articles(
      pkg(
        articles = list(list(name = "all", contents = c("a", "b"))
        )
      )
    )
  )

  "navbar without text"
  expect_snapshot(
    navbar_articles(
      pkg(
        articles = list(list(name = "all", contents = c("a", "b"), navbar = NULL))
      )
    )
  )

  "navbar with label"
  expect_snapshot(
    navbar_articles(
      pkg(
        articles = list(list(name = "all", contents = c("a", "b"), navbar = "Label"))
      )
    )
  )

  "navbar with only some articles"
  expect_snapshot(
    navbar_articles(
      pkg(
        articles = list(
          list(name = "a", contents = "a", navbar = NULL),
          list(name = "b", contents = "b")
        )
     )
    )
  )

})

test_that("can control articles navbar through navbar meta", {
  pkg <- local_pkgdown_site(meta = list(
    navbar = list(
      components = list(articles = menu_submenu("Hi!", list(menu_heading("Hi"))))
    )
  ))
  pkg <- pkg_add_file(pkg, "vignettes/a.Rmd", pkg_vignette())
  pkg <- pkg_add_file(pkg, "vignettes/b.Rmd", pkg_vignette())

  navbar <- navbar_link_components(pkg)
  expect_equal(navbar$left$articles$text, "Hi!")
})

test_that("data_navbar() works by default", {
  pkg <- local_pkgdown_site(meta = list(
    repo = list(url = list(home = "https://github.com/r-lib/pkgdown/"))
  ))
  file_touch(path(pkg$src_path, "NEWS.md"))

  expect_snapshot(data_navbar(pkg))
})

test_that("data_navbar() can re-order default elements", {
  pkg <- local_pkgdown_site(meta = list(
    repo = list(url = list(home = "https://github.com/r-lib/pkgdown/")),
    navbar = list(
      structure = list(
        left = c("github", "search"),
        right = "news"
      )
    )
  ))
  file_create(path(pkg$src_path, "NEWS.md"))

  expect_snapshot(data_navbar(pkg)[c("left", "right")])
})

test_that("data_navbar() can remove elements", {
  pkg <- local_pkgdown_site(meta = list(
    navbar = list(
      structure = list(
        left = c("github", "search"),
        right = list()
      )
    )
  ))

  expect_equal(data_navbar(pkg)$right, "")
})

test_that("data_navbar() works with empty side", {
  pkg <- local_pkgdown_site(
    meta = list(navbar = list(structure = list(left = list(), right = list())))
  )
  
   expect_snapshot(data_navbar(pkg))
 })

test_that("data_navbar_() errors with bad yaml specifications", {
  data_navbar_ <- function(...) {
    pkg <- local_pkgdown_site(meta = list(...))
    data_navbar(pkg)
  }

  expect_snapshot(error = TRUE, {
    data_navbar_(navbar = list(structure = list(left = 1)))
    data_navbar_(navbar = list(right = "github"))
  })
})

test_that("for bs4, default bg and type come from bootswatch", {
  style <- navbar_style(bs_version = 5)
  expect_equal(style, list(bg = "light", type = "light"))

  style <- navbar_style(theme = "cyborg", bs_version = 5)
  expect_equal(style, list(bg = "dark", type = "dark"))

  # but can override
  style <- navbar_style(list(bg = "primary"), bs_version = 5)
  expect_equal(style, list(bg = "primary", type = "dark"))

  style <- navbar_style(list(bg = "primary", type = "light"), bs_version = 5)
  expect_equal(style, list(bg = "primary", type = "light"))
})

test_that("render_navbar_links BS3 & BS4 default", {
  x <- list(
    intro =  menu_link("Get started", "articles/pkgdown.html"),
    reference = menu_link("Reference", "reference/index.html"),
    articles = menu_submenu(
      "Articles",
      list(
        menu_link("Auto-linking", "articles/linking.html"),
        menu_link("Search", "articles/search.html"),
        menu_link("Metadata", "articles/metadata.html"),
        menu_link("Customize your pkgdown website", "articles/customization.html"),
        menu_separator(),
        menu_link("More...", "articles/index.html")
      )
    ),
    news = menu_link("News", "news/index.html")
  )

  expect_snapshot(cat(render_navbar_links(x, pkg = list(bs_version = 3))))
  expect_snapshot(cat(render_navbar_links(x, pkg = list(bs_version = 4))))
})

test_that("dropdowns on right are right-aligned", {
  x <- list(
    articles = menu_submenu("Articles", list(menu_heading("A"), menu_heading("B")))
  )
  pkg <- list(bs_version = 5)

  right <- xml2::read_html(render_navbar_links(x, pkg = pkg, side = "right"))
  left <-  xml2::read_html(render_navbar_links(x, pkg = pkg, side = "left"))

  expect_equal(xpath_attr(right, ".//ul", "class"), "dropdown-menu dropdown-menu-end")
  expect_equal(xpath_attr(left, ".//ul", "class"), "dropdown-menu")
})

find_template <- function(type, name, ext = ".html", pkg = ".") {
  pkg <- as_pkgdown(pkg)

  paths <- template_candidates(type = type, name = name, ext = ext, pkg = pkg)

  existing <- paths[file_exists(paths)]

  if (length(existing) == 0) {
    abort(paste0("Can't find template for ", type, "-", name, "."))
  }
  existing[[1]]
}

# Used for testing
read_template_html <- function(type, name, pkg = list()) {
  if (is_list(pkg)) {
    # promote to a shell "pkgdown" object so we don't need a complete pkg
    class(pkg) <- "pkgdown"
  }
  path <- find_template(type = type, name = name, pkg = pkg)
  xml2::read_html(path)
}

template_candidates <- function(type, name, ext = ".html", pkg = list()) {
  paths <- c(
    path(pkg$src_path, "pkgdown", "templates"),
    templates_dir(pkg),
    path_pkgdown(paste0("BS", pkg$bs_version), "templates")
  )
  names <- c(paste0(type, "-", name, ext), paste0(type, ext))
  all <- expand.grid(paths, names)

  path(all[[1]], all[[2]])
}

# Find directory where custom templates might live:
# * path supplied in `template.path`
# * package supplied in `template.package`
# * templates in package itself
templates_dir <- function(pkg = list()) {
  template <- pkg$meta$template

  if (!is.null(template$path)) {
    # Directory specified in yaml doesn't exist, so eagerly error
    if (!dir_exists(template$path)) {
      abort(paste0("Can not find templates path ", src_path(template$path)))
    }
    path_abs(template$path, start = pkg$src_path)
  } else if (!is.null(template$package)) {
    path_package_pkgdown("templates", package = template$package, bs_version = pkg$bs_version)
  } else {
    path(pkg$src_path, "pkgdown", "templates")
  }
}

test_that("template_candidates look for specific first", {
  expect_equal(
    path_file(template_candidates("content", "article")),
    c("content-article.html", "content.html")
  )
})

test_that("template_candidates look in template dir then pkgdown", {
  pkg_dir <- withr::local_tempdir()
  template_dir <- withr::local_tempdir()

  pkg <- list(
    src_path = pkg_dir,
    meta = list(template = list(path  = template_dir)),
    bs_version = 3
  )

  # ensure that templates_dir(pkg) returns the specific template path
  expect_equal(templates_dir(pkg), path(template_dir))

  paths <- template_candidates("content", "article", pkg = pkg)
  dirs <- unique(path_dir(paths))
  expect_equal(
    dirs,
    c(                                       # search for candidates...
      path(pkg_dir, "pkgdown", "templates"), # first in local templates
      path(template_dir),                    # second in global template path
      path_pkgdown("BS3", "templates")       # finally in pkgdown templates
    )
  )
})


test_that("look for templates_dir in right places", {
  dir <- withr::local_tempdir()
  pkg <- list(src_path = dir, meta = list(template = list()))

  # Look in site templates
  expect_equal(templates_dir(pkg), path(dir, "pkgdown", "templates"))

  # Look in specified directory
  pkg$meta$template$path <- path(withr::local_tempdir())
  expect_equal(templates_dir(pkg), pkg$meta$template$path)
})

test_that("find templates in local pkgdown first", {
  pkg <- test_path("assets", "templates-local")

  # local template used over default pkgdown template
  expect_equal(
    find_template("content", "article", pkg = pkg),
    path_abs(path(pkg, "pkgdown", "templates", "content-article.html"))
  )

  expect_equal(
    find_template("footer", "article", pkg = pkg),
    path_abs(path(pkg, "pkgdown", "templates", "footer-article.html"))
  )

  # pkgdown template used (no local template)
  expect_equal(
    find_template("content", "tutorial", pkg = pkg),
    path_pkgdown("BS3", "templates", "content-tutorial.html")
  )

  expect_equal(
    find_template("footer", "ignored", pkg = pkg),
    path_pkgdown("BS3", "templates", "footer.html")
  )
})


# Expected contents -------------------------------------------------------

test_that("BS5 templates have main + aside", {
  names <- dir_ls(path_pkgdown("BS5", "templates"), regexp = "content-")
  names <- path_ext_remove(path_file(names))
  names <- gsub("content-", "", names)

  templates <- lapply(names, read_template_html,
    type = "content",
    pkg = list(bs_version = 5)
  )
  for (i in seq_along(templates)) {
    template <- templates[[i]]
    name <- names[[i]]

    expect_equal(xpath_length(template, ".//div/main"), 1, info = name)
    expect_equal(xpath_attr(template, ".//div/main", "id"), "main", info = name)
    expect_equal(xpath_length(template, ".//div/aside"), 1, info = name)
    expect_equal(xpath_attr(template, ".//div/aside", "class"), "col-md-3", info = name)
  }
})

# Bootstrap version resolution --------------------------------------------
test_that("Bootstrap version in template package under `template.bootstrap`", {
  path_template_package <- local_pkgdown_template_pkg(
    meta = list(template = list(bootstrap = 5))
  )

  pkg <- local_pkgdown_site(meta = list(template = list(package = "templatepackage")))

  expect_equal(pkg$bs_version, 5)
})

test_that("Bootstrap version in template package under `template.bslib.version`", {
  path_template_package <- local_pkgdown_template_pkg(
    meta = list(template = list(bslib = list(version = 5)))
  )

  pkg <- local_pkgdown_site(meta = list(template = list(package = "templatepackage")))

  expect_equal(pkg$bs_version, 5)
})

test_that("Invalid bootstrap version spec in template package", {
  path_template_package <- local_pkgdown_template_pkg(
    meta = list(template = list(bootstrap = 4, bslib = list(version = 5)))
  )

  expect_snapshot(
    error = TRUE,
    local_pkgdown_site(meta = list(template = list(package = "templatepackage")))
  )
})

test_that("Invalid bootstrap version spec in _pkgdown.yml", {
  expect_snapshot(
    error = TRUE,
    local_pkgdown_site(meta = list(template = list(
      bootstrap = 4, bslib = list(version = 5)
    )))
  )
})

test_that("Valid local Bootstrap version masks invalid template package", {
  path_template_package <- local_pkgdown_template_pkg(
    meta = list(template = list(bootstrap = 4, bslib = list(version = 5)))
  )

  expect_no_error(
    local_pkgdown_site(meta = list(template = list(
      package = "templatepackage",
      bootstrap = 5
    )))
  )
})

# Bootstrap theme resolution ----------------------------------------------
test_that("Finds Bootstrap theme in all the places", {
  pkg_sketchy <- local_pkgdown_site(meta = 
    list(template = list(bslib = list(preset = "sketchy", version = 5)))
  )
  pkg_cosmo <- local_pkgdown_site(meta = 
    list(template = list(bootstrap = 5, bootswatch = "cosmo"))
  )
  pkg_yeti <- local_pkgdown_site(meta = 
    list(template = list(bootstrap = 5, params = list(bootswatch = "yeti")))
  )
  
  expect_equal(get_bslib_theme(pkg_sketchy), "sketchy")
  expect_equal(get_bslib_theme(pkg_cosmo), "cosmo")
  expect_equal(get_bslib_theme(pkg_yeti), "yeti")
})

test_that("Warns when Bootstrap theme is specified in multiple locations", {
  pkg <- local_pkgdown_site(meta = list(
    template = list(
      bootstrap = 5,
      bootswatch = "cerulean",
      bslib = list(preset = "flatly", bootswatch = "lux"),
      params = list(bootswatch = "darkly")
    )
  ))
   expect_snapshot(get_bslib_theme(pkg))
})

test_that("Doesn't warn when the same Bootstrap theme is specified in multiple locations", {
  pkg <- local_pkgdown_site(meta = 
    list(template = list(
      bootstrap = 5,
      bootswatch = "cerulean",
      bslib = list(preset = "cerulean")
    ))
  )

  expect_equal(expect_silent(get_bslib_theme(pkg)), "cerulean")
})

build_bslib <- function(pkg = ".", call = caller_env()) {
  pkg <- as_pkgdown(pkg)
  bs_theme <- bs_theme(pkg, call = call)

  cur_deps <- find_deps(pkg)
  cur_digest <- purrr::map_chr(cur_deps, file_digest)

  deps <- c(bslib::bs_theme_dependencies(bs_theme), external_dependencies(pkg))
  deps <- lapply(
    deps,
    htmltools::copyDependencyToDir,
    path(pkg$dst_path, "deps")
  )
  deps <- lapply(deps, htmltools::makeDependencyRelative, pkg$dst_path)

  new_deps <- find_deps(pkg)
  new_digest <- purrr::map_chr(cur_deps, file_digest)

  all_deps <- union(new_deps, cur_deps)
  diff <- (cur_digest[all_deps] == new_digest[all_deps])
  changed <- all_deps[!diff | is.na(diff)]

  if (length(changed) > 0) {
    withr::local_locale(LC_COLLATE = "C")
    purrr::walk(sort(changed), function(dst) {
      cli::cli_inform("Updating {dst_path(path_rel(dst, pkg$dst_path))}")
    })
  }

  head <- htmltools::renderDependencies(deps, srcType = "file")
  write_lines(head, data_deps_path(pkg))
}

data_deps <- function(pkg, depth) {
  if (!file_exists(data_deps_path(pkg))) {
    # this is unlikely to occur after #2439 and #2571
    cli::cli_abort(
      "Run {.fn pkgdown::init_site} first.",
      .internal = TRUE,
      call = caller_env()
    )
  }

  deps_path <- paste0(up_path(depth), "deps")

  data_deps <- read_lines(data_deps_path(pkg))
  data_deps <- gsub('src="deps', sprintf('src="%s', deps_path), data_deps)
  data_deps <- gsub('href="deps', sprintf('href="%s', deps_path), data_deps)

  paste0(data_deps, collapse = "")
}

data_deps_path <- function(pkg) {
  path(pkg$dst_path, "deps", "data-deps.txt")
}

find_deps <- function(pkg) {
  deps_path <- path(pkg$dst_path, "deps")
  if (!dir_exists(deps_path)) {
    character()
  } else {
    dir_ls(deps_path, type = "file", recurse = TRUE)
  }
}

bs_theme <- function(pkg = ".", call = caller_env()) {
  pkg <- as_pkgdown(pkg)

  bs_theme_args <- pkg$meta$template$bslib %||% list()
  bs_theme_args[["version"]] <- pkg$bs_version
  # In bslib >= 0.5.1, bs_theme() takes bootstrap preset theme via `preset`
  bs_theme_args[["preset"]] <- get_bslib_theme(pkg)
  bs_theme_args[["bootswatch"]] <- NULL

  bs_theme <- exec(bslib::bs_theme, !!!bs_theme_args)

  # Drop bs3 compat files added for shiny/RMarkdown
  bs_theme <- bslib::bs_remove(bs_theme, "bs3compat")

  # Add additional pkgdown rules
  rules <- bs_theme_rules(pkg, call = call)
  files <- lapply(rules, sass::sass_file)
  bs_theme <- bslib::bs_add_rules(bs_theme, files)

  # Add dark theme if needed
  if (uses_lightswitch(pkg)) {
    dark_theme <- config_pluck_string(
      pkg,
      "template.theme-dark",
      default = "arrow-dark"
    )
    check_theme(
      dark_theme,
      error_pkg = pkg,
      error_path = "template.theme-dark",
      error_call = call
    )
    path <- highlight_path(dark_theme)
    css <- c('[data-bs-theme="dark"] {', read_lines(path), '}')
    bs_theme <- bslib::bs_add_rules(bs_theme, css)
  }

  bs_theme
}

bs_theme_rules <- function(pkg, call = caller_env()) {
  paths <- path_pkgdown("BS5", "assets", "pkgdown.scss")

  theme <- config_pluck_string(pkg, "template.theme", default = "arrow-light")
  check_theme(
    theme,
    error_pkg = pkg,
    error_path = "template.theme",
    error_call = call
  )
  paths <- c(paths, highlight_path(theme))

  package <- config_pluck_string(pkg, "template.package")
  if (!is.null(package)) {
    package_extra <- path_package_pkgdown("extra.scss", package, pkg$bs_version)
    if (file_exists(package_extra)) {
      paths <- c(paths, package_extra)
    }
  }

  # Also look in site supplied
  site_extra <- path(pkg$src_path, "pkgdown", "extra.scss")
  if (file_exists(site_extra)) {
    paths <- c(paths, site_extra)
  }

  paths
}

check_theme <- function(
  theme,
  error_pkg,
  error_path,
  error_call = caller_env()
) {
  if (theme %in% highlight_styles()) {
    return()
  }
  config_abort(
    error_pkg,
    "{.field {error_path}} uses theme {.val {theme}}",
    call = error_call
  )
}

highlight_path <- function(theme) {
  path_pkgdown("highlight-styles", paste0(theme, ".scss"))
}

highlight_styles <- function() {
  paths <- dir_ls(path_pkgdown("highlight-styles"), glob = "*.scss")
  path_ext_remove(path_file(paths))
}

get_bslib_theme <- function(pkg) {
  themes <- list(
    "template.bslib.preset" = pkg$meta[["template"]]$bslib$preset,
    "template.bslib.bootswatch" = pkg$meta[["template"]]$bslib$bootswatch,
    "template.bootswatch" = pkg$meta[["template"]]$bootswatch,
    # Historically (< 0.2.0), bootswatch wasn't a top-level template field
    "template.params.bootswatch" = pkg$meta[["template"]]$params$bootswatch
  )

  is_present <- !purrr::map_lgl(themes, is.null)
  n_present <- sum(is_present)
  n_unique <- length(unique(themes[is_present]))

  if (n_present == 0) {
    return("default")
  }

  if (n_present > 1 && n_unique > 1) {
    cli::cli_warn(c(
      "Multiple Bootstrap preset themes were set. Using {.val {themes[is_present][[1]]}} from {.field {names(themes)[is_present][1]}}.",
      x = "Found {.and {.field {names(themes)[is_present]}}}.",
      i = "Remove extraneous theme declarations to avoid this warning."
    ))
  }

  field <- names(themes)[which(is_present)[1]]
  check_bslib_theme(themes[[field]], pkg, field)
}

check_bslib_theme <- function(
  theme,
  pkg,
  field = "template.bootswatch",
  bs_version = pkg$bs_version
) {
  bslib_themes <- c(
    bslib::bootswatch_themes(bs_version),
    bslib::builtin_themes(bs_version),
    # bs_theme() recognizes both below as bare bootstrap
    "default",
    "bootstrap"
  )

  if (theme %in% bslib_themes) {
    return(theme)
  }

  config_abort(
    pkg,
    c(
      x = "{.field {field}} contains unknown Bootswatch/bslib theme {.val {theme}}.",
      i = "Using Bootstrap version {.val {bs_version}} ({.field template.bootstrap})."
    ),
    call = caller_env()
  )
}

bs_theme_deps_suppress <- function(deps = list()) {
  # jquery and bootstrap are provided by bslib
  # headr-attrs is included for pandoc 2.7.3 - 2.9.2.1 to improve accessibility
  # but includes javascript that breaks our HTML anchor system
  bs_dep_names <- c("jquery", "bootstrap", "header-attrs")
  bs_deps <- purrr::map(bs_dep_names, function(name) {
    # minimal version of htmltools::htmlDependency() (see suppressDependencies())
    structure(
      list(
        name = name,
        version = "9999",
        src = list(href = ""),
        all_files = TRUE
      ),
      class = "html_dependency"
    )
  })

  c(deps, bs_deps)
}

test_that("check_bslib_theme() works", {
  pkg <- local_pkgdown_site()
  expect_equal(check_bslib_theme("default", pkg, bs_version = 4), "default")
  expect_equal(check_bslib_theme("lux", pkg, bs_version = 4), "lux")
  expect_snapshot(error = TRUE, {
    check_bslib_theme("paper", pkg, bs_version = 4)
  })
})

test_that("get_bslib_theme() works with template.bslib.preset", {
  pkg <- local_pkgdown_site(
    meta = list(
      template = list(bslib = list(preset = "shiny"), bootstrap = 5)
    )
  )
  expect_equal(get_bslib_theme(pkg), "shiny")
  expect_no_error(bs_theme(pkg))

  pkg <- local_pkgdown_site(
    meta = list(
      template = list(bslib = list(preset = "lux"), bootstrap = 5)
    )
  )
  expect_equal(get_bslib_theme(pkg), "lux")
  expect_no_error(bs_theme(pkg))
})

test_that("validations yaml specification", {
  build_bslib_ <- function(...) {
    pkg <- local_pkgdown_site(
      meta = list(template = list(..., bootstrap = 5, `light-switch` = TRUE))
    )
    build_bslib(pkg)
  }

  expect_snapshot(error = TRUE, {
    build_bslib_(theme = 1)
    build_bslib_(theme = "fruit")
    build_bslib_(`theme-dark` = "fruit")
  })
})

# Renders LICENSE text file into html
build_home_license <- function(pkg) {
  license_md <- path_first_existing(pkg$src_path, c("LICENSE.md", "LICENCE.md"))
  if (!is.null(license_md)) {
    render_md(pkg, license_md)
  }

  license_raw <- path_first_existing(pkg$src_path, c("LICENSE", "LICENCE"))
  if (!is.null(license_raw)) {
    render_page(
      pkg,
      "title-body",
      data = list(
        pagetitle = tr_("License"),
        body = paste0("<pre>", escape_html(read_file(license_raw)), "</pre>")
      ),
      path = "LICENSE-text.html"
    )
    return()
  }
}

data_home_sidebar_license <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)

  link <- autolink_license(pkg$desc$get_field("License", ""))

  license_md <- path_first_existing(pkg$src_path, c("LICENSE.md", "LICENCE.md"))
  if (!is.null(license_md)) {
    link <- c(
      a(tr_("Full license"), "LICENSE.html"),
      paste0("<small>", link, "</small>")
    )
  }

  sidebar_section(tr_("License"), link)
}

# helpers -----------------------------------------------------------------

autolink_license <- function(x) {
  db <- licenses_db()

  for (i in seq_len(nrow(db))) {
    match <- paste0("\\b\\Q", db$abbr[[i]], "\\E\\b")
    x <- gsub(match, db$a[[i]], x, perl = TRUE)
  }
  x
}

licenses_db <- function() {
  path <- path(R.home("share"), "licenses", "license.db")
  db <- tibble::as_tibble(read.dcf(path))

  db <- add_missing_sss(db)

  abbr <- ifelse(is.na(db$SSS), db$Abbrev, db$SSS)
  url <- db$URL

  # Add entry for LICENSE file
  abbr <- c(abbr, "LICENSE", "LICENCE")
  url <- c(url, "LICENSE-text.html", "LICENSE-text.html")

  out <- tibble::tibble(abbr, url)
  out$a <- paste0("<a href='", url, "'>", abbr, "</a>")

  out[!is.na(out$abbr), ]
}

# Add missing standard short specification (SSS) for some licenses
# (e.g., Mozilla Public Licences)
# see src/library/tools/R/license.R in R source for details
add_missing_sss <- function(db) {
  needs_sss <- !is.na(db$Abbrev) & !is.na(db$Version) & is.na(db$SSS)
  x <- db[needs_sss, ]

  db[needs_sss, "SSS"] <- paste0(x$Abbrev, "-", x$Version)

  db
}

test_that("link_license matchs exactly", {
  # R 3.1 uses http url
  skip_if_not(getRversion() >= "3.2.0")

  # Shouldn't match first GPL-2
  expect_equal(
    autolink_license("LGPL-2"),
    "<a href='https://www.r-project.org/Licenses/LGPL-2'>LGPL-2</a>"
  )

  expect_equal(
    autolink_license("MPL-2.0"),
    "<a href='https://www.mozilla.org/en-US/MPL/2.0/'>MPL-2.0</a>"
  )
})

test_that("link_license matches LICENSE", {
  expect_equal(
    autolink_license("LICENSE"),
    "<a href='LICENSE-text.html'>LICENSE</a>"
  )
  expect_equal(
    autolink_license("LICENCE"),
    "<a href='LICENSE-text.html'>LICENCE</a>"
  )
})

# Reference page ---------------------------------------------------------------

# For testing
usage2text <- function(x) {
  rd <- rd_text(paste0("\\usage{", x, "}"), FALSE)[[1]]
  strip_html_tags(as_data(rd))
}

#' @export
as_data.tag_usage <- function(x, ...) {
  text <- paste(flatten_text(x, ..., escape = FALSE), collapse = "\n")
  text <- str_trim(text)

  # Look for single line calls to non-syntactic functions (except for `=` 
  # since that's probably a single argument on its own line) and then use
  # deparse1 to convert to standard style. We want to avoid reparsing 
  # any other lines to avoid losing whitespace, comments etc. (These
  # are not generated by roxygen but can be added by the user.)
  lines <- strsplit(text, "\n", fixed = TRUE)[[1]]
  parsed <- lapply(lines, function(x) tryCatch(parse(text = x)[[1]], error = function(e) NULL))
  needs_tweak <- function(x) {
    is_call(x) && !is_call(x, "=") && (is_symbol(x[[1]]) && !is_syntactic(x[[1]]))
  }
  to_tweak <- vapply(parsed, needs_tweak, logical(1))
  lines[to_tweak] <- vapply(parsed[to_tweak], deparse1, character(1))

  text <- paste(lines, collapse = "\n")
  
  highlight_text(text)
}

#' @export
as_html.tag_method <- function(x, ...) method_usage(x, "S3")
#' @export
as_html.tag_S3method <- function(x, ...) method_usage(x, "S3")
#' @export
as_html.tag_S4method <- function(x, ...) method_usage(x, "S4")

method_usage <- function(x, type) {
  # Despite these being called from the as_html() generic, the target isn't
  # actually HTML, but R code, which is turned into HTML by the syntax
  # highlighting in as as_data.tag_usage()
  fun <- as_html(x[[1]], escape = FALSE)
  class <- as_html(x[[2]], escape = FALSE)

  if (x[[2]] == "default") {
    method <- sprintf(tr_("# Default %s method"), type)
  } else {
    method <- sprintf(tr_("# %s method for class '%s'"), type, class)
  }

  if (!is_syntactic(fun)) {
    fun <- paste0("`", fun, "`")
  }
  paste0(method, "\n", fun)
}

# Reference index --------------------------------------------------------------

topic_funs <- function(rd) {
  funs <- parse_usage(rd)

  # Remove all methods for generics documented in this file
  name <- purrr::map_chr(funs, "name")
  type <- purrr::map_chr(funs, "type")

  gens <- name[type == "fun"]
  self_meth <- (name %in% gens) & (type %in% c("s3", "s4"))

  funs <- purrr::map_chr(funs[!self_meth], ~ short_name(.$name, .$type, .$signature))
  unique(funs)
}

parse_usage <- function(x) {
  if (!inherits(x, "tag")) {
    usage <- paste0("\\usage{", x, "}")
    x <- rd_text(usage, fragment = FALSE)
  }

  r <- usage_code(x)
  if (length(r) == 0) {
    return(list())
  }

  exprs <- tryCatch(
    parse_exprs(r),
    error = function(e) {
      cli::cli_warn("Failed to parse usage: {.code {r}}")
      list()
    }
  )
  purrr::map(exprs, usage_type)
}

short_name <- function(name, type, signature) {
  name <- escape_html(name)
  qname <- auto_quote(name)

  if (type == "data") {
    qname
  } else if (type == "fun") {
    if (is_infix(name)) {
      qname
    } else {
      paste0(qname, "()")
    }
  } else {
    sig <- paste0("<i>&lt;", escape_html(signature), "&gt;</i>", collapse = ",")
    paste0(qname, "(", sig, ")")
  }
}

# Given single expression generated from usage_code, extract
usage_type <- function(x) {
  if (is_symbol(x)) {
    list(type = "data", name = as.character(x))
  } else if (is_call(x, "data")) {
    list(type = "data", name = as.character(x[[2]]))
  } else if (is.call(x)) {
    if (identical(x[[1]], quote(`<-`))) {
      replacement <- TRUE
      x <- x[[2]]
    } else {
      replacement <- FALSE
    }

    out <- fun_info(x)
    out$replacement <- replacement
    out$infix <- is_infix(out$name)
    if (replacement) {
      out$name <- paste0(out$name, "<-")
    }

    out
  } else {
    untype <- paste0(typeof(x), " (in ", as.character(x), ")")
    cli::cli_abort(
      "Unknown type: {.val {untype}}",
      call = caller_env()
    )
  }
}

is_infix <- function(x) {
  if (is.null(x)) {
    return(FALSE)
  }

  x <- as.character(x)
  ops <- c(
    "+", "-", "*", "^", "/",
    "==", ">", "<", "!=", "<=", ">=",
    "&", "|",
    "[[", "[", "$"
  )

  grepl("^%.*%$", x) || x %in% ops
}

fun_info <- function(fun) {
  stopifnot(is.call(fun))

  if (is.call(fun[[1]])) {
    x <- fun[[1]]
    if (identical(x[[1]], quote(S3method))) {
      list(
        type = "s3",
        name = as.character(x[[2]]),
        signature = as.character(x[[3]])
      )
    } else if (identical(x[[1]], quote(S4method))) {
      list(
        type = "s4",
        name = as.character(x[[2]]),
        signature = sub("^`(.*)`$", "\\1", as.character(as.list(x[[3]])[-1]))
      )
    } else if (is_call(x, c("::", ":::"))) {
      # TRUE if fun has a namespace, pkg::fun()
      list(
        type = "fun",
        name = call_name(fun)
      )
    } else {
      cli::cli_abort(
        "Unknown call: {.val {as.character(x[[1]])}}",
        call = caller_env()
      )
    }
  } else {
    list(
      type = "fun",
      name = as.character(fun[[1]]),
      signature = NULL
    )
  }
}

# usage_code --------------------------------------------------------------
# Transform Rd embedded inside usage into parseable R code

usage_code <- function(x) {
  UseMethod("usage_code")
}

#' @export
usage_code.Rd <- function(x) {
  usage <- purrr::detect(x, inherits, "tag_usage")
  usage_code(usage)
}

#' @export
usage_code.NULL <- function(x) character()

# Tag without additional class use
#' @export
usage_code.tag <- function(x) {
  if (!identical(class(x), "tag")) {
    cli::cli_abort(
      "Undefined tag in usage: {.val class(x)[[1]]}}",
      call = caller_env()
    )
  }
  paste0(purrr::flatten_chr(purrr::map(x, usage_code)), collapse = "")
}

#' @export
usage_code.tag_special <- function(x) {
  paste0(purrr::flatten_chr(purrr::map(x, usage_code)), collapse = "")
}

#' @export
usage_code.tag_dots <- function(x) "..."
#' @export
usage_code.tag_ldots <- function(x) "..."

#' @export
usage_code.TEXT <-    function(x) as.character(x)
#' @export
usage_code.RCODE <-   function(x) as.character(x)
#' @export
usage_code.VERB <-    function(x) as.character(x)
#' @export
usage_code.COMMENT <- function(x) character()

#' @export
usage_code.tag_S3method <- function(x) {
  generic <- paste0(usage_code(x[[1]]), collapse = "")
  class <- paste0(usage_code(x[[2]]), collapse = "")

  paste0("S3method(`", generic, "`, ", class, ")")
}

#' @export
usage_code.tag_method <- usage_code.tag_S3method

#' @export
usage_code.tag_S4method <- function(x) {
  generic <- paste0(usage_code(x[[1]]), collapse = "")
  class <- strsplit(usage_code(x[[2]]), ",")[[1]]
  class <- paste0("`", class, "`")
  class <- paste0(class, collapse = ",")
  paste0("S4method(`", generic, "`, list(", class, "))")
}
#' @export
usage_code.tag_usage <- function(x) {
  paste0(purrr::flatten_chr(purrr::map(x, usage_code)), collapse = "")
}

# Reference --------------------------------------------------------------------

test_that("usage escapes special characters", {
  # parseable
  expect_equal(usage2text("# <"), "# &lt;")
  #unparseable
  expect_equal(usage2text("<"), "&lt;")
})

test_that("usage re-renders non-syntactic calls", {
  expect_equal(usage2text("`<`(x, y)"), "x &lt; y")
  expect_equal(usage2text("`[`(x, y)"), "x[y]")
})

test_that("usage doesn't re-renders syntactic calls", {
  expect_equal(usage2text("foo(x , y) # hi"), "foo(x , y) # hi")

  multi_line <- "foo(\n  x # x,\n  y = 1 # y,\n)"
  expect_equal(usage2text(multi_line), multi_line)
})

test_that("usage generates user facing code for S3/S4 infix/replacement methods", {
  expect_snapshot({
    cat(usage2text("\\S3method{$}{indexed_frame}(x, name)"))
    cat(usage2text("\\method{[[}{indexed_frame}(x, i) <- value"))
    cat(usage2text("\\S4method{>=}{MyType,numeric}(e1, e2)"))
  })
})

test_that("S4 methods gets comment", {
  out <- rd2html("\\S4method{fun}{class}(x, y)")
  expect_equal(out[1], "# S4 method for class 'class'")
  expect_equal(out[2], "fun(x, y)")
})

test_that("S3 methods gets comment", {
  out <- rd2html("\\S3method{fun}{class}(x, y)")
  expect_equal(out[1], "# S3 method for class 'class'")
  expect_equal(out[2], "fun(x, y)")

  out <- rd2html("\\method{fun}{class}(x, y)")
  expect_equal(out[1], "# S3 method for class 'class'")
  expect_equal(out[2], "fun(x, y)")
})

test_that("Methods for class function work", {
  out <- rd2html("\\S3method{fun}{function}(x, y)")
  expect_equal(out[1], "# S3 method for class 'function'")
  expect_equal(out[2], "fun(x, y)")

  out <- rd2html("\\method{fun}{function}(x, y)")
  expect_equal(out[1], "# S3 method for class 'function'")
  expect_equal(out[2], "fun(x, y)")

  out <- rd2html("\\S4method{fun}{function,function}(x, y)")
  expect_equal(out[1], "# S4 method for class 'function,function'")
  expect_equal(out[2], "fun(x, y)")
})

test_that("default methods get custom text", {
  out <- rd2html("\\S3method{fun}{default}(x, y)")
  expect_equal(out[1], "# Default S3 method")

  out <- rd2html("\\S4method{fun}{default}(x, y)")
  expect_equal(out[1], "# Default S4 method")
})

test_that("non-syntactic functions get backquoted, not escaped", {
  out <- rd2html("\\S3method{<}{foo}(x, y)")
  expect_equal(out[[2]], "`<`(x, y)")

  out <- rd2html("\\S4method{bar<-}{foo}(x, y)")
  expect_equal(out[[2]], "`bar<-`(x, y)")
})

# Reference index --------------------------------------------------------------

test_that("can parse data", {
  usage <- parse_usage("f")[[1]]
  expect_equal(usage, list(type = "data", name = "f"))

  usage <- parse_usage("data(f)")[[1]]
  expect_equal(usage, list(type = "data", name = "f"))
})

test_that("can parse function/methods", {
  usage <- parse_usage("f(x)")[[1]]
  expect_equal(usage$type, "fun")
  expect_equal(usage$name, "f")

  usage <- parse_usage("\\method{f}{bar}(x)")[[1]]
  expect_equal(usage$type, "s3")
  expect_equal(usage$name, "f")
  expect_equal(usage$signature, "bar")

  usage <- parse_usage("\\S3method{f}{bar}(x)")[[1]]
  expect_equal(usage$type, "s3")
  expect_equal(usage$name, "f")
  expect_equal(usage$signature, "bar")

  usage <- parse_usage("\\S3method{f}{`foo bar`}(x)")[[1]]
  expect_equal(usage$type, "s3")
  expect_equal(usage$name, "f")
  expect_equal(usage$signature, "foo bar")

  usage <- parse_usage("\\S4method{f}{bar,baz}(x)")[[1]]
  expect_equal(usage$type, "s4")
  expect_equal(usage$name, "f")
  expect_equal(usage$signature, c("bar", "baz"))

  usage <- parse_usage("\\S4method{f}{NULL}(x)")[[1]]
  expect_equal(usage$type, "s4")
  expect_equal(usage$name, "f")
  expect_equal(usage$signature, c("NULL"))

  usage <- parse_usage("\\S4method{f}{function,function}(x, y)")[[1]]
  expect_equal(usage$type, "s4")
  expect_equal(usage$name, "f")
  expect_equal(usage$signature, c("function", "function"))

  usage <- parse_usage("\\S4method{f}{function,foo bar}(x, y)")[[1]]
  expect_equal(usage$type, "s4")
  expect_equal(usage$name, "f")
  expect_equal(usage$signature, c("function", "foo bar"))

  usage <- parse_usage("pkg::func()")[[1]]
  expect_equal(usage$type, "fun")
  expect_equal(usage$name, "func")

  usage <- parse_usage("pkg:::func()")[[1]]
  expect_equal(usage$type, "fun")
  expect_equal(usage$name, "func")
})

test_that("can parse replacement functions", {
  usage <- parse_usage("f() <- value")[[1]]
  expect_true(usage$replacement)
  expect_equal(usage$name, "f<-")

  usage <- parse_usage("\\S3method{f}{bar}(x) <- value")[[1]]
  expect_true(usage$replacement)
  expect_equal(usage$name, "f<-")

  usage <- parse_usage("\\S4method{f}{bar,baz}(x) <- value")[[1]]
  expect_true(usage$replacement)
  expect_equal(usage$name, "f<-")
})

test_that("can parse infix functions", {
  usage <- parse_usage("x \\%f\\% y")[[1]]
  expect_true(usage$infix)
  expect_equal(usage$name, "%f%")

  usage <- parse_usage("\\S3method{[}{bar}(x)")[[1]]
  expect_true(usage$infix)
  expect_equal(usage$name, "[")

  usage <- parse_usage("\\S4method{[}{bar,baz}(x)")[[1]]
  expect_true(usage$infix)
  expect_equal(usage$name, "[")
})

test_that("can parse infix replacement functions", {
  usage <- parse_usage("\\S3method{[}{bar}(x) <- value")[[1]]
  expect_true(usage$infix)
  expect_true(usage$replacement)
  expect_equal(usage$name, "[<-")

  usage <- parse_usage("\\S4method{[}{bar,baz}(x) <- value")[[1]]
  expect_true(usage$infix)
  expect_true(usage$replacement)
  expect_equal(usage$name, "[<-")
})

test_that("can parse multistatement usages", {
  usage <- parse_usage("f()\n%This is a comment\ng(\n\n)")
  expect_length(usage, 2)

  expect_equal(usage[[1]]$name, "f")
  expect_equal(usage[[2]]$name, "g")
})

test_that("can parse dots", {
  usage <- parse_usage("f(\\dots)")[[1]]
  expect_equal(usage$name, "f")
})

test_that("usage2text can parse symbols (#2727)", {
  expect_no_error(usage2text("viridisLite::viridis(21)"))
})

# short_name --------------------------------------------------------------

test_that("infix functions left as", {
  expect_equal(short_name("%||%", "fun"), "`%||%`")
})

test_that("function name and signature is escaped", {
  expect_equal(short_name("%<%", "fun"), "`%&lt;%`")
  expect_equal(short_name("f", "S3", "<"), "f(<i>&lt;&lt;&gt;</i>)")
})

#' Automatically link references and articles in an HTML page
#'
#' Deprecated: please use [downlit::downlit_html_path] instead.
#'
#' @param input,output Input and output paths for HTML file
#' @param local_packages A named character vector providing relative paths
#'   (value) to packages (name) that can be reached with relative links
#'   from the target HTML document.
#' @export
#' @keywords internal
#' @examples
#' \dontrun{
#' autolink_html("path/to/file.html",
#'   local_packages = c(
#'     shiny = "shiny",
#'     shinydashboard = "shinydashboard"
#'   )
#' )
#' }
autolink_html <- function(input, output = input, local_packages = character()) {
  withr::local_options(list(
    downlit.package = "",
    downlit.local_packages = local_packages
  ))

  html <- xml2::read_html(input, encoding = "UTF-8")
  downlit::downlit_html_node(html)

  xml2::write_html(html, output, format = FALSE)
  invisible()
}

#' Example with HTML dependency
#'
#' @examples
#' a()
#' @export
a <- function() {
  x <- htmltools::tagList(
    htmltools::p("hello"),
    rmarkdown::html_dependency_jquery(),
    rmarkdown::html_dependency_bootstrap("flatly")
  )
  htmltools::browsable(x)
}

has_contributing <- function(path = ".") {
  file_exists(path(path, 'CONTRIBUTING.md')) ||
    file_exists(path(path, '.github', 'CONTRIBUTING.md'))
}

has_coc <- function(path = ".") {
  file_exists(path(path, 'CODE_OF_CONDUCT.md')) ||
    file_exists(path(path, '.github', 'CODE_OF_CONDUCT.md'))
}

has_support <- function(path = ".") {
  file_exists(path(path, 'SUPPORT.md')) ||
    file_exists(path(path, '.github', 'SUPPORT.md'))
}

data_home_sidebar_community <- function(pkg) {
  pkg <- as_pkgdown(pkg)

  links <- NULL

  if (has_contributing(pkg$src_path)) {
    links <- c(links, a(tr_("Contributing guide"), "CONTRIBUTING.html"))
  }

  if (has_coc(pkg$src_path)) {
    links <- c(links, a(tr_("Code of conduct"), "CODE_OF_CONDUCT.html"))
  }

  if (has_support(pkg$src_path)) {
    links <- c(links, a(tr_("Getting help"), "SUPPORT.html"))
  }

  if (is.null(links)) {
    return("")
  }

  sidebar_section(tr_("Community"), links)
}

test_that("handles CoC and SUPPORT if present", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, ".github/SUPPORT.md")
  pkg <- pkg_add_file(pkg, ".github/CODE_OF_CONDUCT.md")

  expect_true(has_coc(pkg$src_path))
  expect_true(has_support(pkg$src_path))

  # And added to sidebar
  text <- data_home_sidebar_community(pkg)
  expect_snapshot_output(cat(text))
})

test_that("empty site doesn't have community asserts", {
  pkg <- local_pkgdown_site()

  expect_false(has_contributing(pkg$src_path))
  expect_false(has_coc(pkg$src_path))
  expect_equal(data_home_sidebar_community(pkg$src_path), "")
})

meta_development <- function(pkg, call = caller_env()) {
  config_pluck_list(pkg, "development", call = call)
  
  mode <- dev_mode(pkg, call = call)
  
  destination <- config_pluck_string(pkg, "development.destination", default = "dev", call = call)
  version_label <- config_pluck_string(pkg, "development.version_label", call = call)
  if (is.null(version_label)) {
    if (mode %in% c("release", "default")) {
      version_label <- if (pkg$bs_version == 3) "default" else "muted"
    } else {
      version_label <- "danger"
    }
  }
  in_dev <- mode == "devel"

  list(
    destination = destination,
    mode = mode,
    version_label = version_label,
    in_dev = in_dev,
    prefix = if (in_dev) paste0(destination, "/") else ""
  )
}

dev_mode <- function(pkg, call = caller_env()) {
  mode <- Sys.getenv("PKGDOWN_DEV_MODE")
  if (identical(mode, "")) {
    mode <- config_pluck_string(pkg, "development.mode", default = "default", call = call)
  }

  if (mode == "auto") {
    mode <- dev_mode_auto(pkg$version)
  } else {
    valid_mode <- c("auto", "default", "release", "devel", "unreleased")
    if (!mode %in% valid_mode) {
      msg <- "{.field development.mode} must be one of {.or {valid_mode}}, not {mode}."
      config_abort(pkg, msg, call = call)
    }
  }

  mode
}

dev_mode_auto <- function(version) {
  version <- unclass(package_version(version))[[1]]

  if (length(version) < 3) {
    "release"
  } else if (length(version) == 3) {
    if (version[3] >= 9000) {
      "devel"
    } else {
      "release"
    }
  } else if (identical(version[1:3], c(0L, 0L, 0L))) {
    "unreleased"
  } else {
    "devel"
  }
}

# Called in render_page() so that LANG env var set up
version_tooltip <- function(mode) {
  switch(mode,
    default = "",
    release = tr_("Released version"),
    devel = tr_("In-development version"),
    unreleased = tr_("Unreleased version")
  )
}

test_that("empty yaml gets correct defaults", {
  pkg <- local_pkgdown_site()
  expect_equal(pkg$development$mode, "default")
  expect_equal(pkg$development$in_dev, FALSE)
  expect_equal(pkg$development$version_label, "muted")
})

test_that("mode = auto uses version", {
  pkg <- local_pkgdown_site(
    desc = list(Version = "0.0.9000"),
    meta = list(development = list(mode = "auto"))
  )
  expect_equal(pkg$development$mode, "devel")
  expect_equal(pkg$development$in_dev, TRUE)
  expect_equal(pkg$development$version_label, "danger")
})

test_that("mode overrides version", {
  pkg <- local_pkgdown_site(
    desc = list(Version = "0.0.9000"),
    meta = list(development = list(mode = "release"))
  )

  expect_equal(pkg$development$mode, "release")
})

test_that("env var overrides mode", {
  withr::local_envvar("PKGDOWN_DEV_MODE" = "devel")
  pkg <- local_pkgdown_site(
    meta = list(development = list(mode = "release"))
  )
  expect_equal(pkg$development$mode, "devel")
})

test_that("dev_mode recognises basic version structure", {
  expect_equal(dev_mode_auto("0.0.0.9000"), "unreleased")

  expect_equal(dev_mode_auto("0.0.1"), "release")

  expect_equal(dev_mode_auto("0.1"), "release")
  expect_equal(dev_mode_auto("0.1.0"), "release")
  expect_equal(dev_mode_auto("0.1.9000"), "devel")

  expect_equal(dev_mode_auto("1.0"), "release")
  expect_equal(dev_mode_auto("1.0.0"), "release")
  expect_equal(dev_mode_auto("1.0.0.9000"), "devel")
})

test_that("validates yaml", {
  data_development_ <- function(...) {
    local_pkgdown_site(meta = list(...) )
  }

  expect_snapshot(error = TRUE, {
    data_development_(development = 1)
    data_development_(development = list(mode = 1))
    data_development_(development = list(mode = "foo"))
    data_development_(development = list(destination = 1))
    data_development_(development = list(version_label = 1))
  })
})

#' Build home section
#'
#' @description
#' `build_home()` function generates pages at the top-level of the site
#' including:
#'
#' * The home page
#' * HTML files from any `.md` files in `./` or `.github/`.
#' * The authors page (from `DESCRIPTION`)
#' * The citation page (from `inst/CITATION`, if present).
#' * The license page
#' * A default 404 page if `.github/404.md` is not found.
#'
#' `build_home_index()` rebuilds just the index page; it's useful for rapidly
#' iterating when experimenting with site styles.
#'
#' # Home page
#'
#' The main content of the home page (`index.html`) is generated from
#' `pkgdown/index.md`, `index.md`, or `README.md`, in that order.
#' Most packages will use `README.md` because that's also displayed by GitHub
#' and CRAN. Use `index.md` if you want your package website to look
#' different to your README, and use `pkgdown/index.md` if you don't want that
#' file to live in your package root directory.
#'
#' If you use `index.Rmd` or `README.Rmd` it's your responsibility to knit
#' the document to create the corresponding `.md`. pkgdown does not do this
#' for you because it only touches files in the `doc/` directory.
#'
#' Extra markdown files in the base directory (e.g. `ROADMAP.md`) or in
#' `.github/` (e.g. `CODE_OF_CONDUCT.md`) are copied by `build_home()` to `docs/` and converted to HTML.
#'
#' The home page also features a sidebar with information extracted from the
#' package. You can tweak it via the configuration file, to help make the home
#' page an as informative as possible landing page.
#'
#' ## Images and figures
#'
#' If you want to include images in your `README.md`, they must be stored
#' somewhere in the package so that they can be displayed on the CRAN website.
#' The best place to put them is `man/figures`. If you are generating figures
#' with R Markdown, make sure you set up `fig.path` as followed:
#'
#' ``` r
#' knitr::opts_chunk$set(
#'   fig.path = "man/figures/"
#' )
#' ```
#'
#' This should usually go in a chunk with `include = FALSE`.
#'
#' ```` markdown
#' ```{r chunk-name, include=FALSE}`r ''`
#' knitr::opts_chunk$set(
#'   fig.path = "man/figures/"
#' )
#' ```
#' ````
#'
#' ## Package logo
#'
#' If you have a package logo, you can include it at the top of your README
#' in a level-one heading:
#'
#' ``` markdown
#' # pkgdown <img src="man/figures/logo.png" align="right" />
#' ```
#'
#' [init_site()] will also automatically create a favicon set from your package logo.
#'
#' ## YAML config - title and description
#'
#' By default, the page title and description are extracted automatically from
#' the `Title` and `Description` fields `DESCRIPTION` (stripping single quotes
#' off quoted words). CRAN ensures that these fields don't contain phrases
#' like "R package" because that's obvious on CRAN. To make your package more
#' findable on search engines, it's good practice to override the `title` and
#' `description`, thinking about what people might search for:
#'
#' ```yaml
#' home:
#'   title: An R package for pool-noodle discovery
#'   description: >
#'     Do you love R? Do you love pool-noodles? If so, you might enjoy
#'     using this package to automatically discover and add pool-noodles
#'     to your growing collection.
#' ```
#'
#' (Note the use of YAML's `>` i.e. "YAML pipes"; this is a convenient way of
#' writing paragraphs of text.)
#'
#' ## Dev badges
#'
#' pkgdown identifies badges in three ways:
#'
#' -   Any image-containing links between `<!-- badges: start -->` and
#'     `<!-- badges: end -->`, as e.g. created by `usethis::use_readme_md()`
#'     or `usethis::use_readme_rmd()`. There should always be an empty line after
#'     the `<!-- badges: end -->` line. If you divide badges into paragraphs,
#'     make sure to add an empty line before the `<!-- badges: end -->` line.
#'
#' -   Any image-containing links within `<div id="badges"></div>`.
#'
#' -   Within the first paragraph, if it only contains image-containing links.
#'
#' Identified badges are **removed** from the _main content_.
#' They are shown or not in the _sidebar_ depending on the development mode and
#' sidebar customization, see the sidebar section.
#'
#' # Authors
#'
#' By default, pkgdown will display author information in three places:
#'
#' * the sidebar,
#' * the left part side of the footer,
#' * the author page.
#'
#' This documentation describes how to customise the overall author display.
#' See `?build_home` and `?build_site` for details about changing the location
#' of the authors information within the home sidebar and the site footer.
#'
#' ## Authors ORCID and bio
#'
#' Author ORCID identification numbers in the `DESCRIPTION` are linked using
#' the ORCID logo:
#'
#' ```r
#' Authors@R: c(
#'     person("Hadley", "Wickham", , "hadley@rstudio.com", role = c("aut", "cre"),
#'       comment = c(ORCID = "0000-0003-4757-117X")
#'     ),
#'     person("Jay", "Hesselberth", role = "aut",
#'       comment = c(ORCID = "0000-0002-6299-179X")
#'     )
#'   )
#' ```
#'
#' If you want to add more details about authors or their involvement with the
#' package, you can use the comment field, which will be rendered on the
#' authors page.
#'
#' ```r
#' Authors@R: c(
#'     person("Hadley", "Wickham", , "hadley@rstudio.com", role = c("aut", "cre"),
#'       comment = c(ORCID = "0000-0003-4757-117X", "Indenter-in-chief")
#'     ),
#'     person("Jay", "Hesselberth", role = "aut",
#'       comment = c(ORCID = "0000-0002-6299-179X")
#'     )
#'   )
#' ```
#'
#' ## Additional control via YAML
#'
#' You can control additinal aspects of the authors display via the `authors`
#' YAML field:
#'
#' * display of each author in the footer, sidebar and authors page,
#' * which authors (by role) are displayed in the sidebar and footer,
#' * text before authors in the footer,
#' * text before and after authors in the sidebar,
#' * text before and after authors on the authors page.
#'
#' You can modify how each author's name is displayed by adding a subsection
#' for `authors`. Each entry in `authors` should be named the author's name
#' (matching `DESCRIPTION`) and can contain `href` and/or `html` fields:
#'
#' * If `href` is provided, the author's name will be linked to this URL.
#' * If `html` is provided, it will be shown instead of the author's name.
#'   This is particularly useful if you want to display the logo of a corporate
#'   sponsor. Use an absolute URL to an image, not a relative link. Use an empty
#'   alternative text rather than no alternative text so a screen-reader would
#' skip over it.
#'
#' ```yaml
#' authors:
#'   firstname lastname:
#'     href: "http://name-website.com"
#'     html: "<img src='https://website.com/name-picture.png' width=72 alt=''>"
#' ```
#'
#'
#' By default, the "developers" list shown in the sidebar and footer is
#' populated by the maintainer ("cre"), authors ("aut"), and funder ("fnd")
#' from the `DESCRIPTION`. You could choose other roles for filtering.
#' With the configuration below:
#'
#' * only the maintainer and funder(s) appear in the footer, after the text
#'   "Crafted by",
#' * all authors and contributors appear in the sidebar,
#' * the authors list on the sidebar is preceded and followed by some text,
#' * the authors list on the authors page is preceded and followed by some text.
#'
#'
#' ```yaml
#' authors:
#'   footer:
#'     roles: [cre, fnd]
#'     text: "Crafted by"
#'   sidebar:
#'     roles: [aut, ctb]
#'     before: "So *who* does the work?"
#'     after: "Thanks all!"
#'   before: "This package is proudly brought to you by:"
#'   after: "See the [changelog](news/index.html) for other contributors. :pray:"
#' ```
#'
#' If you want to filter authors based on something else than their roles,
#' consider using a custom sidebar/footer component
#' (see `?build_home`/`?build_site`, respectively).
#'
#' # Sidebar
#'
#' You can customise the homepage sidebar with the `home.sidebar` field.
#' It's made up of two pieces: `structure`, which defines the overall layout,
#' and `components`, which defines what each piece looks like. This organisation
#' makes it easy to mix and match the pkgdown defaults with your own
#' customisations.
#'
#' This is the default structure:
#'
#' ``` yaml
#' home:
#'   sidebar:
#'     structure: [links, license, community, citation, authors, dev]
#' ```
#'
#' These are drawn from seven built-in components:
#'
#' -   `links`: automated links generated from `URL` and `BugReports` fields
#'     from `DESCRIPTION` plus manual links from the `home.links` field:
#'
#'     ``` yaml
#'     home:
#'       links:
#'       - text: Link text
#'         href: https://website.com
#'       - text: Roadmap
#'         href: /roadmap.html
#'     ```
#'
#' -   `license`: Licensing information if `LICENSE`/`LICENCE` or
#'     `LICENSE.md`/`LICENCE.md` files are present.
#'
#' -   `community`: links to to `.github/CONTRIBUTING.md`,
#'     `.github/CODE_OF_CONDUCT.md`, etc.
#'
#' -   `citation`: link to package citation information. Uses either
#'     `inst/CITATION` or, if absent, information from the `DESCRIPTION`.
#'
#' -   `authors`: selected authors from the `DESCRIPTION`.
#'
#' -   `dev`: development status badges extracted from `README.md`/`index.md`.
#'     This is only shown for "development" versions of websites; see
#'     "Development mode" in `?build_site` for details.
#'
#' -   `toc`: a table of contents for the README (not shown by default).
#'
#' You can also add your own components, where `text` is markdown text:
#'
#' ``` yaml
#' home:
#'   sidebar:
#'     structure: [authors, custom, toc, dev]
#'     components:
#'       custom:
#'         title: Funding
#'         text: We are *grateful* for funding!
#' ```
#'
#' Alternatively, you can provide a ready-made sidebar HTML:
#'
#' ``` yaml
#' home:
#'   sidebar:
#'     html: path-to-sidebar.html
#' ```
#'
#' Or completely remove it:
#'
#' ``` yaml
#' home:
#'   sidebar: FALSE
#' ```
#' @inheritParams build_articles
#' @family site components
#' @export
#' @order 1
build_home <- function(pkg = ".",
                       override = list(),
                       preview = FALSE,
                       quiet = TRUE) {

  pkg <- section_init(pkg, override = override)
  check_bool(quiet)

  cli::cli_rule("Building home")

  build_citation_authors(pkg)

  build_home_md(pkg)
  build_home_license(pkg)
  build_home_index(pkg, quiet = quiet)

  if (!pkg$development$in_dev) {
    build_404(pkg)
  }


  preview_site(pkg, "/", preview = preview)
}

activate_navbar <- function(html, path, pkg = list()) {
  path <- remove_useless_parts(path, pkg = pkg)

  # Get nav items, their links, their similarity to the current path
  navbar_haystack <- navbar_links_haystack(html, pkg, path)

  # Nothing similar
  if (nrow(navbar_haystack) == 0) {
    return()
  }

  # Pick the most similar link, activate the corresponding nav item
  tweak_class_prepend(
    navbar_haystack$nav_item[which.max(navbar_haystack$similar)][[1]],
    "active"
  )
}

navbar_links_haystack <- function(html, pkg, path) {
  # Extract links from the menu items
  html_navbar <- xml2::xml_find_first(html, ".//div[contains(@class, 'navbar')]")
  nav_items <- xml2::xml_find_all(html_navbar,".//li[contains(@class, 'nav-item')]")

  get_hrefs <- function(nav_item, pkg = pkg) {
    href <- xml2::xml_attr(xml2::xml_child(nav_item), "href")

    if (href != "#") {
      links <- href
    } else {
      # links in a drop-down
      hrefs <- xml2::xml_attr(xml2::xml_find_all(nav_item, ".//a"), "href")
      links <- hrefs[hrefs != "#"]
    }

    tibble::tibble(
      nav_item = list(nav_item),
      links = remove_useless_parts(links[is_internal_link(links, pkg = pkg)], pkg = pkg)
    )
  }

  haystack <- do.call(rbind, lapply(nav_items, get_hrefs, pkg = pkg))

  # For each link, calculate similarity to the current path
  separate_path <- function(link) {
    strsplit(link, "/")[[1]]
  }
  get_similarity <- function(stalk, needle) {
    needle <- separate_path(needle)
    stalk <- separate_path(stalk)

    # Active item can't be more precise than current path/needle
    if (length(stalk) > length(needle)) {
      return(0)
    }

    # Active item can however be less precise than current path/needle
    length(stalk) <- length(needle)
    similar <- (needle == stalk)

    # Any difference indicates it's not the active item
    if (any(!similar, na.rm = TRUE)) {
      0
    } else {
      sum(similar, na.rm = TRUE)
    }
  }
  haystack$similar <- purrr::map_dbl(haystack$links, get_similarity, needle = path)

  # Only return rows of links with some similarity to the current path
  haystack[haystack$similar > 0, ]
}

remove_useless_parts <- function(links, pkg) {
  # remove website URL
  if (!is.null(pkg$meta$url)) {
    links <- sub(pkg$meta$url, "", links)
  }
  # remove first slash from path
  links <- sub("^/", "", links)
  # remove /index.html from the end
  links <- sub("/index.html/?", "", links)
  # remove ../ from the beginning
  links <- gsub("\\.\\./", "", links)

  links
}

test_that("navbar_links_haystack()", {
  html <- xml2::read_html(
    '
    <div class="navbar-collapse">
    <ul>
      <li class="nav-item"><a href="articles/pkgdown.html">Get started</a></li>
      <li class="nav-item"><a href="reference/index.html">Reference</a></li>
      <li class="nav-item dropdown">
        <a href="#" class="nav-link dropdown-toggle">Articles</a>
        <div class="dropdown-menu" aria-labelledby="navbarDropdown">
          <a class="dropdown-item" href="articles/linking.html">Auto-linking</a>
          <a class="dropdown-item" href="articles/index.html">More</a>
        </div>
      </li>
    </ul>
    </div>
  '
  )

  best_match <- function(path) {
    navbar_links_haystack(html, pkg = list(), path)[1, c("links", "similar")]
  }

  # Link to exact path if present
  expect_equal(
    best_match("articles/pkgdown.html"),
    tibble::tibble(links = "articles/pkgdown.html", similar = 2)
  )
  # even if in sub-menu
  expect_equal(
    best_match("articles/linking.html"),
    tibble::tibble(links = "articles/linking.html", similar = 2)
  )

  # ignores index.html
  expect_equal(
    best_match("articles/index.html"),
    tibble::tibble(links = "articles", similar = 1)
  )

  # If not present, guess at top-level home
  expect_equal(
    best_match("articles/bla.html"),
    tibble::tibble(links = "articles", similar = 1)
  )
})

test_that("activation sets class of best match", {
  html <- xml2::read_html(
    '
    <div class="navbar-collapse">
    <ul>
      <li class="nav-item"><a href="articles/pkgdown.html">Get started</a></li>
      <li class="nav-item"><a href="reference/index.html">Reference</a></li>
      <li class="nav-item dropdown">
        <a href="#" class="nav-link dropdown-toggle">Articles</a>
        <div class="dropdown-menu" aria-labelledby="navbarDropdown">
          <a class="dropdown-item" href="articles/linking.html">Auto-linking</a>
          <a class="dropdown-item" href="articles/index.html">More</a>
        </div>
      </li>
    </ul>
    </div>
  '
  )

  activate_navbar(html, "articles/linking.html")
  expect_equal(
    xpath_attr(html, "//li", "class"),
    c("nav-item", "nav-item", "active nav-item dropdown")
  )
})

ext_topics <- function(match_strings) {
  pieces <- strsplit(match_strings, "::", fixed = TRUE)
  pkg <- purrr::map_chr(pieces, 1)
  fun <- sub("\\(\\)$", "", purrr::map_chr(pieces, 2))

  unwrap_purrr_error(
    ext_rd <- purrr::map2(pkg, fun, get_rd_from_help)
  )
  ext_title <- purrr::map_chr(ext_rd, extract_title)
  ext_href <- purrr::map2_chr(fun, pkg, downlit::href_topic)
  ext_funs <- purrr::map(ext_rd, topic_funs)

  tibble::tibble(
    name = match_strings,
    file_in = NA_character_,
    file_out = ext_href,
    alias = list(character()), # used to find icons,
    funs = ext_funs,           # used list alternative names
    title = sprintf("%s (from %s)", ext_title, pkg),
    rd = list(character()),
    source = NA_character_,
    keywords = list(character()), # used for has_keyword()
    concepts = list(character()), # used for has_concept()
    internal = FALSE,
    lifecycle = list(NULL)        # used for has_lifecycle
  )
}

# Adapted from roxygen2::get_rd_from_help
get_rd_from_help <- function(package, alias) {
  call <- quote(build_reference_index())
  check_installed(package, "as it's used in the reference index.", call = call)

  help <- utils::help((alias), (package))
  if (length(help) == 0) {
    fun <- paste0(package, "::", alias)
    cli::cli_abort(
      "Could not find documentation for {.fn {fun}}.",
      call = call
    )
    return()
  }

  out <- get(".getHelpFile", envir = asNamespace("utils"))(help)
  set_classes(out)
}

tweak_homepage_html <- function(html,
                                strip_header = FALSE,
                                sidebar = TRUE,
                                show_badges = TRUE,
                                bs_version = 3,
                                logo = NULL) {

  html <- tweak_sidebar_html(html, sidebar = sidebar, show_badges = show_badges)

  # Always remove dummy page header
  header <- xml2::xml_find_all(html, ".//div[contains(@class, 'page-header')]")
  if (length(header) > 0)
    xml2::xml_remove(header, free = TRUE)

  header <- xml2::xml_find_first(html, ".//h1")
  if (strip_header) {
    page_header <- xml2::xml_remove(header, free = TRUE)
  } else {
    page_header <- xml2::xml_add_parent(header, "div", class = "page-header")
  }

  if (!is.null(logo) && bs_version > 3) {
    # Remove logo if added to h1
    # Bare image
    xml2::xml_remove(xml2::xml_find_all(html, ".//h1/img[contains(@src, 'logo')]"))

    # Image in link
    xml2::xml_remove(
      xml2::xml_parent(
        xml2::xml_find_all(html, ".//h1/a/img[contains(@src, 'logo')]")
      )
    )

    # Add back to header
    xml2::xml_add_sibling(page_header, "img",
      src = logo,
      class = "logo",
      alt = "",
      .where = "before"
    )
  }

  invisible()
}

tweak_sidebar_html <- function(html, sidebar = TRUE, show_badges = TRUE) {
  if (!sidebar) {
    return(html)
  }

  # this extracts *and removes* badges from HTML
  badges <- badges_extract(html)

  dev_status_html <- xml2::xml_find_first(html, ".//div[@class='dev-status']")
  if (inherits(dev_status_html, "xml_missing")) {
    return(html)
  }
  if (!show_badges || length(badges) == 0) {
    xml2::xml_remove(dev_status_html)
  } else {
    list <- sidebar_section(tr_("Dev status"), badges)
    list_html <- xml2::xml_find_first(xml2::read_html(list, encoding = "UTF-8"), ".//div")
    xml2::xml_replace(dev_status_html, list_html)
  }

  html
}

# Mutates `html`, removing the badge container
badges_extract <- function(html) {
  # First try specially named element;
  x <- xml2::xml_find_first(html, "//div[@id='badges']")
  strict <- FALSE

  # then try usethis-readme-like more complex structure;
  if (length(x) == 0) {
    # Find start comment, then all elements after
    # which are followed by the end comment.
    x <- xml2::xml_find_all(html, "
      //comment()[contains(., 'badges: start')][1]
      /following-sibling::*[following-sibling::comment()[contains(., 'badges: end')]]
    ")

  }

  # then try usethis-readme-like paragraph;
  # where the badges: end comment is inside the paragraph after badges: start
  if (length(x) == 0) {
    x <- xml2::xml_find_all(html, ".//*/comment()[contains(., 'badges: start')]/following-sibling::p[1]")
  }

  # finally try first paragraph
  if (length(x) == 0) {
    # BS5 (main) and BS3 (div)
    x <- xml2::xml_find_first(html, "//main/p|//div[@class='contents col-md-9']/p")
    strict <- TRUE
  }

  # No paragraph
  if (length(x) == 0) {
    return(character())
  }

  # If we guessed the element,
  # we only proceed if there is no text
  if (strict && any(xml2::xml_text(x, trim = TRUE) != "")) {
    return(character())
  }

  # Proceed if we find image-containing links
  badges <- xml2::xml_find_all(x, ".//a[img]")
  if (length(badges) == 0) {
    return(character())
  }

  xml2::xml_remove(x)

  as.character(badges)
}

badges_extract_text <- function(x) {
  html <- xml2::read_html(x, encoding = "UTF-8")
  badges_extract(html)
}

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom utils installed.packages
#' @import rlang
#' @import fs
#' @importFrom lifecycle deprecated
## usethis namespace: end
NULL

release_bullets <- function() {
  c(
    "Check that [test/widget.html](https://pkgdown.r-lib.org/dev/articles/) responds to mouse clicks on 5/10/50",
    "Update translations with `potools::po_extract()` + `potools::po_update()`",
    "Use an LLM to proofread / fill in missing translations (see #2926)",
    "Compile po files with `potools::po_compile()`",
    "Update `vignette/translations.Rmd` with any new languages"
  )
}

skip_if_no_pandoc <- function(version = "1.12.3") {
  skip_if_not(rmarkdown::pandoc_available(version), "pandoc not available")
}
skip_if_no_quarto <- function() {
  skip_on_os("windows") # quarto set up currently broken?
  skip_if(is.null(quarto::quarto_path()), "quarto not available")
  skip_if_not(quarto::quarto_version() >= "1.5", "quarto 1.5 not available")
}

# Simulate a package --------------------------------------------------------

local_pkgdown_site <- function(path = NULL,
                               meta = list(),
                               desc = list(),
                               env = caller_env()) {
  check_string(path, allow_null = TRUE)

  dst_path <- path_real(
    withr::local_tempdir(.local_envir = env, pattern = "pkgdown-dst")
  )
  # Simulate init_site() so we only have to run it if we care about <head>
  file_create(path(dst_path, "pkgdown.yml"))
  dir_create(path(dst_path, "deps"))
  file_create(path(dst_path, "deps", "data-deps.txt"))

  meta <- modify_list(meta, list(destination = dst_path))

  if (is.null(path)) {
    path <- path_real(
      withr::local_tempdir(.local_envir = env, pattern = "pkgdown-src")
    )

    description <- desc::desc("!new")
    description$set("Package", "testpackage")
    description$set("Title", "A test package")
    if (length(desc) > 0)
      inject(description$set(!!!desc))
    description$write(file = path(path, "DESCRIPTION"))

    # Default to BS5 only if template not specified
    meta$template <- meta$template %||% list(bootstrap = 5)

    # Record meta in case we re-run as_pkgdown()
    yaml::write_yaml(meta, path(path, "_pkgdown.yml"))

    # Make it a bit easier to create other files
    dir_create(path(path, "R"))
    dir_create(path(path, "vignettes"))
  }

  as_pkgdown(path, meta)
}

pkg_add_file <- function(pkg, path, lines = NULL) {
  full_path <- path(pkg$src_path, path)
  dir_create(path_dir(full_path))

  if (is.null(lines)) {
    file_create(full_path)
  } else {
    write_lines(lines, full_path)
  }

  if (path_has_parent(path, "vignettes")) {
    pkg <- as_pkgdown(pkg$src_path)
  }
  pkg
}

pkg_add_kitten <- function(pkg, path) {
  full_path <- path(pkg$src_path, path)
  dir_create(full_path)

  file_copy(test_path("assets/kitten.jpg"), full_path)
  pkg
}

pkg_vignette <- function(..., title = "title") {
  dots <- list2(title = title, ...)
  meta <- dots[have_name(dots)]
  contents <- unlist(dots[!have_name(dots)])

  meta$vignette <- paste0("\n", "  %\\VignetteIndexEntry{", title, "}")
  yaml <- yaml::as.yaml(
    meta,
    handlers = list(logical = yaml::verbatim_logical)
  )

  c("---", yaml, "---", contents)
}

r_code_block <- function(...) c("```{r}", ..., "```")

# Simulate a template package ------------------------------------------------

local_pkgdown_template_pkg <- function(path = NULL, meta = NULL, env = parent.frame()) {
  if (is.null(path)) {
    path <- withr::local_tempdir(.local_envir = env)
    desc <- desc::desc("!new")
    desc$set("Package", "templatepackage")
    desc$set("Title", "A test template package")
    desc$write(file = path(path, "DESCRIPTION"))
  }
  
  if (!is.null(meta)) {
    path_pkgdown_yml <- path(path, "inst", "pkgdown", "_pkgdown.yml")
    dir_create(path_dir(path_pkgdown_yml))
    yaml::write_yaml(meta, path_pkgdown_yml)
  }
  
  pkgload::load_all(path, quiet = TRUE)
  withr::defer(pkgload::unload("templatepackage"), envir = env)
  
  path
}
  

# index -------------------------------------------------------------------

test_that("can build site even if no Authors@R present", {
  skip_if_no_pandoc()

  pkg <- local_pkgdown_site(desc = list(
    Author = "Hadley Wickham",
    Maintainer = "Hadley Wickham <hadley@rstudio.com>",
    `Authors@R` = NA
  ))

  expect_no_error(suppressMessages(build_home_index(pkg)))
})

test_that("can build package without any index/readme", {
  pkg <- local_pkgdown_site()
  expect_no_error(suppressMessages(build_home_index(pkg)))
})

# .github files -----------------------------------------------------------

test_that(".github files are copied and linked", {
  skip_if_no_pandoc()

  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, ".github/404.md")
  pkg <- pkg_add_file(pkg, ".github/CODE_OF_CONDUCT.md")

  suppressMessages(build_home(pkg))

  lines <- read_lines(path(pkg$dst_path, "index.html"))
  expect_match(lines, 'href="CODE_OF_CONDUCT.html"', fixed = TRUE, all = FALSE)
  expect_true(file_exists(path(pkg$dst_path, "404.html")))
})

test_that("can build all quarto article", {
  skip_if_no_quarto()

  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/vig1.qmd")
  pkg <- pkg_add_file(pkg, "vignettes/vig2.qmd")

  suppressMessages(build_articles(pkg))

  expect_true(file_exists(path(pkg$dst_path, "articles/vig1.html")))
  expect_true(file_exists(path(pkg$dst_path, "articles/vig2.html")))
})

test_that("can build a single quarto article", {
  skip_if_no_quarto()

  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/vig1.qmd")
  pkg <- pkg_add_file(pkg, "vignettes/vig2.qmd")

  suppressMessages(build_article("vig1", pkg))

  expect_true(file_exists(path(pkg$dst_path, "articles/vig1.html")))
  expect_false(file_exists(path(pkg$dst_path, "articles/vig2.html")))
})

test_that("doesn't do anything if no quarto articles", {
  pkg <- local_pkgdown_site()
  expect_no_error(suppressMessages(build_quarto_articles(pkg)))
})

test_that("can render a pdf qmd", {
  skip_if_no_quarto()

  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/vig1.qmd", pkg_vignette(
    format = list(pdf = list(toc = TRUE))
  ))

  expect_equal(pkg$vignettes$type, "qmd")
  expect_equal(pkg$vignettes$file_out, "articles/vig1.pdf")

  suppressMessages(build_article("vig1", pkg))
  expect_true(file_exists(path(pkg$dst_path, "articles/vig1.pdf")))
})

test_that("auto-adjusts heading levels", {
  skip_if_no_quarto()

  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/vig1.qmd", pkg_vignette(
    "# Heading 1",
    "# Heading 2"
  ))

  suppressMessages(build_article("vig1", pkg))

  html <- xml2::read_html(path(pkg$dst_path, "articles/vig1.html"))
  expect_equal(xpath_text(html, "//h1"), "title")
  expect_equal(xpath_text(html, "//h2"), c("Heading 1\n", "Heading 2\n"))
})

test_that("we find out if quarto styles change", {
  skip_if_no_quarto()

  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/vig1.qmd")

  output_dir <- quarto_render(pkg, path(pkg$src_path, "vignettes", "vig1.qmd"))

  data <- data_quarto_article(pkg, path(output_dir, "vig1.html"), "vig1.qmd")
  expect_snapshot(cat(data$includes$style))
})

test_that("quarto articles are included in the index", {
  skip_if_no_quarto()

  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/vig1.qmd", pkg_vignette(
    "## Heading 1",
    "Some text"
  ))

  suppressMessages(build_article("vig1", pkg))
  index <- build_search_index(pkg)
  
  expect_equal(index[[1]]$path, "/articles/vig1.html")
  expect_equal(index[[1]]$what, "Heading 1")
  expect_equal(index[[1]]$text, "text") # some is a stop word
})

test_that("quarto headings get anchors", {
  skip_if_no_quarto()

  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/vig1.qmd", pkg_vignette(
    "## Heading 1",
    "### Heading 2"
  ))

  suppressMessages(build_article("vig1", pkg))
  html <- xml2::read_html(path(pkg$dst_path, "articles/vig1.html"))
  headings <- xpath_xml(html, "//h2|//h3")
  expect_equal(xpath_attr(headings, "./a", "href"), c("#heading-1", "#heading-2"))
})

test_that("can build article that uses html_vignette", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/test.Rmd", pkg_vignette(
    output = "rmarkdown::html_vignette",
    pkgdown = list(as_is = TRUE)
  ))

  # theme is not set since html_vignette doesn't support it
  suppressMessages(expect_no_error(build_article("test", pkg)))
})

test_that("can override html_document() options", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/test.Rmd", pkg_vignette(
    output = list(html_document = list(number_sections = TRUE)),
    pkgdown = list(as_is = TRUE),
    "# Heading 1",
    "# Heading 2"
  ))
  suppressMessages(path <- build_article("test", pkg))

  # Check that number_sections is respected
  html <- xml2::read_html(path)
  expect_equal(xpath_text(html, ".//h2//span"), c("1", "2"))

  # But title isn't affected
  expect_equal(xpath_text(html, ".//h1"), "title")

  # And no links or scripts are inlined
  expect_equal(xpath_length(html, ".//body//link"), 0)
  expect_equal(xpath_length(html, ".//body//script"), 0)
})

test_that("can set width", {
  pkg <- local_pkgdown_site(meta = list(code = list(width = 50)))
  pkg <- pkg_add_file(pkg, "vignettes/test.Rmd", pkg_vignette(
    r_code_block("getOption('width')")
  ))

  suppressMessages(path <- build_article("test", pkg))
  html <- xml2::read_html(path)
  expect_equal(xpath_text(html, ".//pre")[[2]], "## [1] 50")
})

test_that("bad width gives nice error", {
  pkg <- local_pkgdown_site(meta = list(code = list(width = "abc")))
  expect_snapshot(rmarkdown_setup_pkgdown(pkg), error = TRUE)
})

test_that("BS5 article laid out correctly with and without TOC", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/toc-true.Rmd", pkg_vignette(
    "## Heading 1",
    "## Heading 2"
  ))
  pkg <- pkg_add_file(pkg, "vignettes/toc-false.Rmd", pkg_vignette(
    toc = FALSE,
    "## Heading 1",
    "## Heading 2"
  ))

  suppressMessages(toc_true_path <- build_article("toc-true", pkg))
  suppressMessages(toc_false_path <- build_article("toc-false", pkg))

  toc_true <- xml2::read_html(toc_true_path)
  toc_false <- xml2::read_html(toc_false_path)

  # Always has class col-md-9
  expect_equal(xpath_attr(toc_true, ".//main", "class"), "col-md-9")
  expect_equal(xpath_attr(toc_false, ".//main", "class"), "col-md-9")

  # The no sidebar without toc
  expect_equal(xpath_length(toc_true, ".//aside"), 1)
  expect_equal(xpath_length(toc_false, ".//aside"), 0)
})

test_that("BS5 article gets correctly activated navbar", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/article.Rmd", pkg_vignette())
  suppressMessages(article_path <- build_article("article", pkg))

  html <- xml2::read_html(article_path)
  navbar <- xml2::xml_find_first(html, ".//div[contains(@class, 'navbar')]")

  expect_equal(
    xpath_text(navbar,".//li[contains(@class, 'active')]//button"),
    "Articles"
  )
})

test_that("titles are escaped when needed", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/test.Rmd", pkg_vignette(title = "a <-> b"))
  suppressMessages(path <- build_article("test", pkg))

  html <- xml2::read_html(path)
  expect_equal(xpath_text(html, "//title", trim = TRUE), "a <-> b • testpackage")
  expect_equal(xpath_text(html, "//h1", trim = TRUE), "a <-> b")
})

test_that("output is reproducible by default, i.e. 'seed' is respected", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/test.Rmd", pkg_vignette(
    r_code_block("runif(5L)")
  ))
  suppressMessages(path <- build_article("test", pkg))

  html <- xml2::read_html(path)
  output <- xpath_text(html, "//main//pre")[[2]]
  expect_snapshot(cat(output))
})

test_that("reports on bad open graph meta-data", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/test.Rmd", pkg_vignette(
    opengraph = list(twitter = 1)
  ))
  expect_snapshot(build_article("test", pkg), error = TRUE)
})

test_that("can control math mode", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/math.Rmd", "$1 + 1$")

  pkg$meta$template$`math-rendering` <- "mathml"
  suppressMessages(init_site(pkg))
  suppressMessages(path <- build_article("math", pkg))
  html <- xml2::read_html(path)
  expect_equal(xpath_length(html, ".//math"), 1)

  pkg$meta$template$`math-rendering` <- "mathjax"
  suppressMessages(init_site(pkg))
  suppressMessages(path <- build_article("math", pkg))
  html <- xml2::read_html(path)
  expect_equal(xpath_length(html, ".//span[contains(@class, 'math')]"), 1)

  pkg$meta$template$`math-rendering` <- "katex"
  suppressMessages(init_site(pkg))
  suppressMessages(path <- build_article("math", pkg))
  html <- xml2::read_html(path)
  expect_equal(xpath_length(html, ".//span[contains(@class, 'math')]"), 1)
  expect_contains(
    path_file(xpath_attr(html, ".//script", "src")),
    c("katex-auto.js", "katex.min.js")
  )
})

test_that("rmarkdown_template cleans up after itself", {
  pkg <- local_pkgdown_site()
  path <- NULL

  local({
    path <<- rmarkdown_template(pkg)
    expect_true(file_exists(path))
  })
  expect_false(file_exists(path))
})

test_that("build_article styles ANSI escapes", {
  skip_if_no_pandoc()

  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/test.Rmd", pkg_vignette(
    r_code_block("cat(cli::col_red('X'), '\n')")
  ))

  suppressMessages(path <- build_article("test", pkg))
  html <- xml2::read_html(path)
  expect_snapshot_output(xpath_xml(html, ".//code//span[@class='co']"))
})

# Errors -----------------------------------------------------------------------

test_that("build_article yields useful error if pandoc fails", {
  skip_on_cran() # fragile due to pandoc dependency
  skip_if_no_pandoc("2.18")

  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/test.Rmd", "Hi")

  expect_snapshot(
    build_article("test", pkg, pandoc_args = "--fail-if-warnings"),
    error = TRUE
  )
})

test_that("build_article yields useful error if R fails", {
  skip_if_no_pandoc()

  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/test.Rmd", c(
    "---",
    "title: title",
    "---",
    "```{r}",
    "f <- function() g()",
    "g <- function() h()",
    "h <- function() rlang::abort('Error!')",
    "f()",
    "```"
  ))

  # check that error looks good
  expect_snapshot(build_article("test", pkg), error = TRUE)
  # check that traceback looks good - need extra work because rlang
  # avoids tracebacks in snapshots
  expect_snapshot(summary(expect_error(build_article("test", pkg))))
})

# Images -----------------------------------------------------------------------

test_that("build_article copies image files in subdirectories", {
  skip_if_no_pandoc()
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/test.Rmd", c(
    "```{r}",
    "#| fig-alt: alt-text",
    "knitr::include_graphics('test/kitten.jpg')",
    "```"
  ))
  pkg <- pkg_add_kitten(pkg, "vignettes/test")

  expect_snapshot(build_article("test", pkg))
  expect_equal(path_file(dir_ls(path(pkg$dst_path, "articles", "test"))), "kitten.jpg")
})

test_that("finds external resources referenced by R code", {
  # weird path differences that I don't have the energy to dig into
  skip_on_cran()
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/test.Rmd", c(
    "![external dependency](`r 'kitten.jpg'`)"
  ))
  pkg <- pkg_add_kitten(pkg, "vignettes")
  suppressMessages(path <- build_article("test", pkg))

  # ensure that we the HTML references `<img src="external.png" />` directly
  html <- xml2::read_html(path)
  expect_equal(xpath_attr(html, ".//img", "src"), "kitten.jpg")

  # expect that `external.png` was copied to the rendered article directory
  expect_true(file_exists(path(path_dir(path), "kitten.jpg")))
})

test_that("image links relative to output", {
  pkg <- local_pkgdown_site(test_path("assets/articles-images"))

  suppressMessages(init_site(pkg))
  suppressMessages(copy_figures(pkg))
  suppressMessages(build_article("kitten", pkg))

  html <- xml2::read_html(path(pkg$dst_path, "articles", "kitten.html"))
  src <- xpath_attr(html, "//main//img", "src")

  expect_equal(src, c(
    # knitr::include_graphics()
    "../reference/figures/kitten.jpg",
    "another-kitten.jpg",
    # rmarkdown image
    "../reference/figures/kitten.jpg",
    "another-kitten.jpg",
    # magick::image_read()
    "kitten_files/figure-html/magick-1.png",
    # figure
    "kitten_files/figure-html/plot-1.jpg"
  ))

  # And files aren't copied
  expect_false(dir_exists(path(pkg$dst_path, "man")))
})

test_that("warns about missing images", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/kitten.Rmd", "![foo](kitten.jpg)")

  expect_snapshot(build_article("kitten", pkg))
})

test_that("spaces in sorce paths do work", {
  # create simulated package
  pkg0 <- local_pkgdown_site()
  pkg0 <- pkg_add_file(pkg0, "vignettes/kitten.Rmd", "![Kitten](kitten.jpg)")
  pkg0 <- pkg_add_kitten(pkg0, "vignettes")

  # copy simulated pkg to path that contains spaces
  pkg1 <- fs::dir_copy(pkg0$src_path, fs::file_temp(pattern = "beware of spaces-"))

  # check that pkgdown site builds anyways
  expect_no_error(suppressMessages(
    build_article("kitten", as_pkgdown(pkg1))
  ))
})

test_that("warns about missing alt-text", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/kitten.Rmd", "![](kitten.jpg)")
  pkg <- pkg_add_kitten(pkg, "vignettes")
  expect_snapshot(build_article("kitten", pkg))
})

# External dependencies --------------------------------------------------------

test_that("pkgdown deps are included only once in articles", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/test.Rmd", pkg_vignette(
    # Some code that adds jquery/bootstrap
    r_code_block(
      'htmltools::tagList(
        htmltools::p("hello"),
        rmarkdown::html_dependency_jquery(),
        rmarkdown::html_dependency_bootstrap("flatly")
      )'
    )
  ))

  # Rely on default init_site() from local_pkgdown_site() setting all
  # the default includes to empty
  suppressMessages(path <- build_article("test", pkg))
  html <- xml2::read_html(path)
  expect_equal(path_file(xpath_attr(html, ".//script", "src")), "pkgdown.js")
  expect_equal(path_file(xpath_attr(html, ".//link", "href")), character())
})

test_that("html widgets get needed css/js", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/test.Rmd", pkg_vignette(
    r_code_block('
      path1 <- tempfile()
      writeLines(letters, path1)
      path2 <- tempfile()
      writeLines(letters[-(10:11)], path2)

      diffviewer::visual_diff(path1, path2)
    ')
  ))

  suppressMessages(path <- build_article("test", pkg))

  html <- xml2::read_html(path)
  css <- xpath_attr(html, ".//body//link", "href")
  js <- xpath_attr(html, ".//body//script", "src")

  expect_true("diffviewer.css" %in% path_file(css))
  expect_true("diffviewer.js" %in% path_file(js))
})

test_that("both versions of build_site have same arguments", {
  expect_equal(formals(build_site_local), formals(build_site_external))
})

test_that("build_site can be made unquiet", {
  skip_if_no_pandoc()

  # `quiet = FALSE` from build_site() should get passed to
  # build_articles(), which will include some rmarkdown build out in the
  # messages, including "pandoc", which won't be there normally
  pkg <- local_pkgdown_site(test_path("assets/figure"))
  output_unquiet <- suppressMessages(
    capture.output(
      build_site(
        pkg,
        quiet = FALSE,
        preview = FALSE
      )
    )
  )
  expect_match(paste(output_unquiet, collapse = ""), "pandoc")

  output_quiet <- suppressMessages(
    capture.output(
      build_site(
        pkg,
        preview = FALSE
      )
    )
  )

  expect_no_match(paste(output_quiet, collapse = ""), "pandoc")
})

test_that("informative print method", {
  pkg <- local_pkgdown_site()
  expect_snapshot(init_site(pkg))
})

test_that("extra.css and extra.js copied and linked", {
  pkg <- local_pkgdown_site()
  dir_create(path(pkg$src_path, "pkgdown"))
  file_create(path(pkg$src_path, "pkgdown", "extra.css"))
  file_create(path(pkg$src_path, "pkgdown", "extra.js"))

  suppressMessages(init_site(pkg))
  expect_true(file_exists(path(pkg$dst_path, "extra.css")))
  expect_true(file_exists(path(pkg$dst_path, "extra.js")))

  skip_if_no_pandoc()
  # Now check they actually get used .
  suppressMessages(build_home(pkg))

  html <- xml2::read_html(path(pkg$dst_path, "index.html"))
  paths <- xpath_attr(html, ".//link", "href")

  expect_true("extra.css" %in% paths)
})

test_that("single extra.css correctly copied", {
  pkg <- local_pkgdown_site()
  dir_create(path(pkg$src_path, "pkgdown"))
  file_create(path(pkg$src_path, "pkgdown", "extra.css"))
  suppressMessages(init_site(pkg))

  expect_true(file_exists(path(pkg$dst_path, "extra.css")))
})

test_that("asset subdirectories are copied", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "pkgdown/assets/subdir1/file1.txt")
  pkg <- pkg_add_file(pkg, "pkgdown/assets/subdir1/subdir2/file2.txt")
  suppressMessages(init_site(pkg))

  expect_true(file_exists(path(pkg$dst_path, "subdir1", "file1.txt")))
  expect_true(file_exists(path(
    pkg$dst_path,
    "subdir1",
    "subdir2",
    "file2.txt"
  )))
})

test_that("site meta doesn't break unexpectedly", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/a.Rmd")
  pkg <- pkg_add_file(pkg, "vignettes/b.Rmd")

  # null out components that will vary
  yaml <- site_meta(pkg)
  yaml$pkgdown <- "{version}"
  yaml$pkgdown_sha <- "{sha}"
  yaml$pandoc <- "{version}"
  yaml$last_built <- timestamp(as.POSIXct("2020-01-01", tz = "UTC"))

  expect_snapshot(yaml)
})

test_that("site meta includes vignette subdirectories", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "vignettes/a/a.Rmd")
  pkg <- pkg_add_file(pkg, "vignettes/a/b.Rmd")

  meta <- site_meta(pkg)
  expect_equal(meta$articles, list("a/a" = "a/a.html", "a/b" = "a/b.html"))
})

test_that("can autodetect tutorials", {
  pkg <- local_pkgdown_site()
  base_path <- "vignettes/tutorials/test-1"
  pkg <- pkg_add_file(pkg, path(base_path, "test-1.Rmd"), c(
    '---',
    'title: "Tutorial"',
    'output: learnr::tutorial',
    'runtime: shiny_prerendered',
    '---'
  ))
  path <- path(base_path, "rsconnect/documents/test-1.Rmd/shinyapps.io/hadley/tutorial-test-1.dcf")
  pkg <- pkg_add_file(pkg, path, c(
    "name: tutorial-test-1",
    "title: tutorial-test-1",
    "hostUrl: https://api.shinyapps.io/v1",
    "when: 1521734722.72611",
    "url: https://hadley.shinyapps.io/tutorial-test-1/"
  ))

  out <- package_tutorials(pkg$src_path)
  expect_equal(out$name, "test-1")
  expect_equal(out$file_out, "tutorials/test-1.html")
  expect_equal(out$url, "https://hadley.shinyapps.io/tutorial-test-1/")

  # and aren't included in vignettes
  out <- package_vignettes(pkg$src_path)
  expect_equal(nrow(out), 0)
})

test_that("can manually supply tutorials", {
  meta <- list(
    tutorials = list(
      list(name = "1-name", title = "1-title", url = "1-url"),
      list(name = "2-name", title = "2-title", url = "2-url")
    )
  )

  pkg <- local_pkgdown_site()
  out <- package_tutorials(pkg, meta)
  expect_equal(out$name, c("1-name", "2-name"))
  expect_equal(out$file_out, c("tutorials/1-name.html", "tutorials/2-name.html"))
  expect_equal(out$url, c("1-url", "2-url"))
})

test_that("can override defaults in _pkgdown.yml", {
  skip_if_no_pandoc()
  skip_if_no_quarto()
  withr::local_temp_libpaths()

  pkg <- local_pkgdown_site(test_path("assets/figure"))
  suppressMessages(init_site(pkg))

  callr::rcmd("INSTALL", pkg$src_path, show = FALSE, fail_on_status = TRUE)

  suppressMessages(build_reference(pkg, devel = FALSE))
  img <- path_file(dir_ls(path(pkg$dst_path, "reference"), glob = "*.jpg"))
  expect_setequal(img, c("figure-1.jpg", "figure-2.jpg"))

  suppressMessages(build_articles(pkg))
  img <- path_file(dir_ls(path(pkg$dst_path, "articles"), glob = "*.jpg", recurse = TRUE))
  expect_equal(img, "unnamed-chunk-1-1.jpg")
})

test_that("check images in readme", {
  pkg <- local_pkgdown_site()
  pkg <- pkg_add_file(pkg, "README.md", "![foo](vignettes/kitten.jpg)")
  suppressMessages(build_home_index(pkg))

  # no image, so should warn
  expect_snapshot(check_built_site(pkg))

  # create and build vignette that uses image, so no warning
  pkg <- pkg_add_file(pkg, "vignettes/kitten.Rmd", "![foo](kitten.jpg)")
  pkg <- pkg_add_kitten(pkg, "vignettes")

  suppressMessages(build_article("kitten", pkg))
  suppressMessages(expect_no_warning(check_built_site(pkg)))
})

#' A
#' @export
#' @keywords foo
#' @param a a letter
#' @param b a a number
#' @param c a logical
a <- function(a, b, c) {}

#' B
#' @export
#' @concept graphics
b <- function() {}

#' C
#' @export
c <- function() {}

#' D
#' @usage
#' \special{?topic}
#' @export
`?` <- function() {}

#' E
#' @name e
NULL

#' F
#' @keywords internal
#' @examples
#' testpackage:::f()
f <- function() {
  runif(5L)
}


#' g <-> h
#' @keywords internal
g <- function() 1

#' Initialise favicons from package logo
#'
#' @description
#' This function auto-detects the location of your package logo (with the name
#' `logo.svg` (recommended format) or `logo.png`, created with `usethis::use_logo()`)
#' and runs it through the <https://realfavicongenerator.net> API to build a
#' complete set of favicons with different sizes, as needed for modern web usage.
#'
#' You only need to run the function once. The favicon set will be stored in
#' `pkgdown/favicon` and copied by [init_site()] to the relevant location when
#' the website is rebuilt.
#'
#' Once complete, you should add `pkgdown/` to `.Rbuildignore ` to avoid a NOTE
#' during package checking. (`usethis::use_logo()` does this for you!)
#'
#' @inheritParams as_pkgdown
#' @param overwrite If `TRUE`, re-create favicons from package logo.
#' @export
build_favicons <- function(pkg = ".", overwrite = FALSE) {
  rlang::check_installed("openssl")
  pkg <- as_pkgdown(pkg)

  cli::cli_rule("Building favicons")

  logo_path <- find_logo(pkg$src_path)

  if (is.null(logo_path)) {
    cli::cli_abort(c(
      "Can't find package logo PNG or SVG to build favicons.",
      "i" = "See {.fun usethis::use_logo} for more information."
    ))
  }

  if (has_favicons(pkg) && !overwrite) {
    cli::cli_inform(c(
      "Favicons already exist in {.path pkgdown}",
      "i" = "Set {.code overwrite = TRUE} to re-create."
    ))
    return(invisible())
  }

  cli::cli_inform(c(
    i = "Building favicons with {.url https://realfavicongenerator.net}..."
  ))

  logo <- readBin(logo_path, what = "raw", n = file_info(logo_path)$size)
  json_request <- list(
    "favicon_generation" = list(
      "api_key" = "87d5cd739b05c00416c4a19cd14a8bb5632ea563",
      "master_picture" = list(
        "type" = "inline",
        "content" = openssl::base64_encode(logo)
      ),
      "favicon_design" = list(
        "desktop_browser" = list(),
        "ios" = list(
          "picture_aspect" = "no_change",
          "assets" = list(
            "ios6_and_prior_icons" = FALSE,
            "ios7_and_later_icons" = TRUE,
            "precomposed_icons" = FALSE,
            "declare_only_default_icon" = TRUE
          )
        )
      )
    )
  )
  req <- httr2::request("https://realfavicongenerator.net/api/favicon")
  req <- httr2::req_body_json(req, json_request)

  withCallingHandlers(
    resp <- httr2::req_perform(req),
    error = function(e) {
      cli::cli_abort("API request failed.", parent = e)
    }
  )

  content <- httr2::resp_body_json(resp)
  result <- content$favicon_generation_result

  if (!identical(result$result$status, "success")) {
    cli::cli_abort("API request failed.", .internal = TRUE)
  }

  tmp <- withr::local_tempfile()
  req <- httr2::request(result$favicon$package_url)
  resp <- httr2::req_perform(req, tmp)

  withCallingHandlers(
    paths <- utils::unzip(tmp, exdir = path(pkg$src_path, "pkgdown", "favicon")),
    warning = function(e) {
      cli::cli_abort("Your logo file couldn't be processed and may be corrupt.", parent = e)
    },
    error = function(e) {
      cli::cli_abort("Your logo file couldn't be processed and may be corrupt.", parent = e)
    }
  )
  cli::cli_inform(c("v" = "Added {.path {sort(path_file(paths))}}."))

  invisible()
}

copy_favicons <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)

  dir_copy_to(
    src_dir = path(pkg$src_path, "pkgdown", "favicon"),
    src_root = pkg$src_path,
    dst_dir = pkg$dst_path,
    dst_root = pkg$dst_path
  )
}

has_favicons <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)
  unname(file_exists(path_favicons(pkg)))
}

path_favicons <- function(pkg) {
  path(pkg$src_path, "pkgdown", "favicon")
}

test_that("missing logo generates message", {
  pkg <- local_pkgdown_site()

  expect_snapshot(error = TRUE,
    expect_output(build_favicons(pkg), "Building favicons")
  )
})

test_that("existing logo generates message", {
  pkg <- local_pkgdown_site()
  dir_create(path(pkg$src_path, "pkgdown", "favicon"))
  file_create(path(pkg$src_path, "pkgdown", "favicon", "favicon.ico"))
  file_create(path(pkg$src_path, "logo.png"))

  expect_true(has_favicons(pkg))
  expect_snapshot(build_favicons(pkg))
})

#' @order 2
#' @export
#' @rdname build_articles
#' @param name Name of article to render. This should be either a path
#'   relative to `vignettes/` *without extension*, or `index` or `README`.
#' @param new_process Build the article in a clean R process? The default,
#'   `TRUE`, ensures that every article is build in a fresh environment, but
#'   you may want to set it to `FALSE` to make debugging easier.
#' @param pandoc_args Pass additional arguments to pandoc. Used for testing.
build_article <- function(
  name,
  pkg = ".",
  lazy = FALSE,
  seed = 1014L,
  new_process = TRUE,
  pandoc_args = character(),
  override = list(),
  quiet = TRUE
) {
  pkg <- section_init(pkg, "articles", override = override)

  # Look up in pkg vignette data - this allows convenient automatic
  # specification of depth, output destination, and other parameters that
  # allow code sharing with building of the index.
  vig <- match(name, pkg$vignettes$name)
  if (is.na(vig)) {
    cli::cli_abort("Can't find article {.file {name}}")
  }

  input <- pkg$vignettes$file_in[vig]
  output_file <- pkg$vignettes$file_out[vig]
  depth <- pkg$vignettes$depth[vig]
  type <- pkg$vignettes$type[vig]

  input_path <- path_abs(input, pkg$src_path)
  output_path <- path_abs(output_file, pkg$dst_path)

  if (lazy && !out_of_date(input_path, output_path)) {
    return(invisible())
  }

  if (type == "rmd") {
    build_rmarkdown_article(
      pkg = pkg,
      input_file = input,
      input_path = input_path,
      output_file = output_file,
      output_path = output_path,
      depth = depth,
      seed = seed,
      new_process = new_process,
      pandoc_args = pandoc_args,
      quiet = quiet
    )
  } else {
    build_quarto_articles(pkg = pkg, article = name, quiet = quiet)
  }
}

build_rmarkdown_article <- function(
  pkg,
  input_file,
  input_path,
  output_file,
  output_path,
  depth,
  seed = NULL,
  new_process = TRUE,
  pandoc_args = character(),
  quiet = TRUE,
  call = caller_env()
) {
  cli::cli_inform("Reading {src_path(input_file)}")
  digest <- file_digest(output_path)

  data <- data_article(pkg, input_file, call = call)
  if (data$as_is) {
    if (identical(data$ext, "html")) {
      setup <- rmarkdown_setup_custom(
        pkg,
        input_path,
        depth = depth,
        data = data
      )
    } else {
      setup <- list(format = NULL, options = NULL)
    }
  } else {
    setup <- rmarkdown_setup_pkgdown(
      pkg,
      depth = depth,
      data = data,
      pandoc_args = pandoc_args
    )
  }

  local_envvar_pkgdown(pkg)
  local_texi2dvi_envvars(input_path)
  withr::local_envvar(R_CLI_NUM_COLORS = 256)

  args <- list(
    input = input_path,
    output_file = path_file(output_path),
    output_dir = path_dir(output_path),
    intermediates_dir = tempdir(),
    encoding = "UTF-8",
    seed = seed,
    output_format = setup$format,
    output_options = setup$options,
    quiet = quiet
  )
  if (new_process) {
    path <- withCallingHandlers(
      callr::r_safe(rmarkdown_render_with_seed, args = args, show = !quiet),
      error = function(cnd) wrap_rmarkdown_error(cnd, input_file, call)
    )
  } else {
    path <- inject(rmarkdown_render_with_seed(!!!args))
  }

  is_html <- identical(path_ext(path)[[1]], "html")
  if (is_html) {
    local_options_link(pkg, depth = depth)
    update_html(
      path,
      tweak_rmarkdown_html,
      input_path = path_dir(input_path),
      pkg = pkg
    )
    # Need re-active navbar now that we now the target path
    update_html(path, function(html) {
      activate_navbar(html, path_rel(path, pkg$dst_path), pkg)
    })
  }
  if (digest != file_digest(output_path)) {
    writing_file(path_rel(output_path, pkg$dst_path), output_file)
  }
  if (is_html) {
    copy_article_images(path, input_path, output_path)
    check_missing_images(pkg, input_path, output_file)
  }

  invisible(path)
}


data_article <- function(pkg, input, call = caller_env()) {
  yaml <- rmarkdown::yaml_front_matter(path_abs(input, pkg$src_path))

  opengraph <- check_open_graph(pkg, yaml$opengraph, input, call = call)
  opengraph$description <- opengraph$description %||% yaml$description

  list(
    opengraph = opengraph,
    pagetitle = escape_html(yaml$title),
    toc = yaml$toc %||% TRUE,
    source = repo_source(pkg, input),
    filename = path_file(input),
    as_is = isTRUE(purrr::pluck(yaml, "pkgdown", "as_is")),
    ext = purrr::pluck(yaml, "pkgdown", "extension", .default = "html")
  )
}

rmarkdown_setup_custom <- function(
  pkg,
  input_path,
  depth = 1L,
  data = list(),
  env = caller_env()
) {
  template <- rmarkdown_template(pkg, depth = depth, data = data, env = env)

  # Override defaults & values supplied in metadata
  options <- list(
    template = template,
    self_contained = FALSE
  )

  output <- rmarkdown::default_output_format(input_path)
  if (output$name != "rmarkdown::html_vignette") {
    # Force to NULL unless overridden by user
    options$theme <- output$options$theme
  }

  list(format = NULL, options = options)
}

rmarkdown_setup_pkgdown <- function(
  pkg,
  depth = 1L,
  data = list(),
  pandoc_args = character(),
  env = caller_env()
) {
  template <- rmarkdown_template(pkg, depth = depth, data = data, env = env)

  format <- rmarkdown::html_document(
    self_contained = FALSE,
    theme = NULL,
    template = template,
    anchor_sections = FALSE,
    math_method = config_math_rendering(pkg),
    extra_dependencies = bs_theme_deps_suppress(),
    pandoc_args = pandoc_args
  )
  format$knitr$opts_chunk <- fig_opts_chunk(
    pkg$figures,
    format$knitr$opts_chunk
  )

  # Add knitr hook to inject CSS class into plot img tags
  format$knitr$knit_hooks <- format$knitr$knit_hooks %||% list()
  format$knitr$knit_hooks$plot <- function(x, options) {
    # Get the default plot hook output
    hook_output <- knitr::hook_plot_md(x, options)

    # Add the fig.class to img tags if specified
    if (!is.null(options$fig.class)) {
      # Match img tags and add class attribute
      hook_output <- gsub(
        '<img src="([^"]+)"',
        sprintf('<img src="\\1" class="%s"', options$fig.class),
        hook_output
      )
    }

    hook_output
  }

  width <- config_pluck_number_whole(pkg, "code.width", default = 80)
  old_pre <- format$pre_knit
  format$pre_knit <- function(...) {
    options(width = width)
    if (is.function(old_pre)) {
      old_pre(...)
    }
  }

  list(format = format, options = NULL)
}

# Generates pandoc template by rendering templates/content-article.html
rmarkdown_template <- function(
  pkg,
  data = list(),
  depth = 1L,
  env = caller_env()
) {
  path <- withr::local_tempfile(
    pattern = "pkgdown-rmd-template-",
    fileext = ".html",
    .local_envir = env
  )
  render_page(pkg, "article", data, path, depth = depth, quiet = TRUE)

  path
}

copy_article_images <- function(built_path, input_path, output_path) {
  ext_src <- rmarkdown::find_external_resources(input_path)

  # temporarily copy the rendered html into the input path directory and scan
  # again for additional external resources that may be been included by R code
  tempfile <- path(path_dir(input_path), "--find-assets.html")
  withr::defer(try(file_delete(tempfile)))
  file_copy(built_path, tempfile)
  ext_post <- rmarkdown::find_external_resources(tempfile)

  ext <- rbind(ext_src, ext_post)
  ext <- ext[!duplicated(ext$path), ]

  # copy web + explicit files beneath vignettes/
  is_child <- path_has_parent(ext$path, ".")
  ext_path <- ext$path[(ext$web | ext$explicit) & is_child]

  src <- path(path_dir(input_path), ext_path)
  dst <- path(path_dir(output_path), ext_path)
  # Make sure destination paths exist before copying files there
  dir_create(unique(path_dir(dst)))
  file_copy(src, dst, overwrite = TRUE)
}

wrap_rmarkdown_error <- function(cnd, input, call = caller_env()) {
  lines <- strsplit(gsub("^\r?\n", "", cnd$stderr), "\r?\n")[[1]]
  lines <- lines[nchar(lines) > 0]
  # Feeding random text back into cli, so have to escape
  lines <- gsub("{", "{{", lines, fixed = TRUE)
  lines <- gsub("}", "}}", lines, fixed = TRUE)

  cli::cli_abort(
    c(
      "!" = "Failed to render {.path {input}}.",
      set_names(cli_escape(lines), "x")
    ),
    parent = cnd$parent %||% cnd,
    trace = cnd$parent$trace,
    call = call
  )
}

rmarkdown_render_with_seed <- function(..., seed = NULL) {
  if (!is.null(seed)) {
    set.seed(seed)
    if (requireNamespace("htmlwidgets", quietly = TRUE)) {
      htmlwidgets::setWidgetIdSeed(seed)
    }
  }

  # Ensure paths from output are not made relative to input
  # https://github.com/yihui/knitr/issues/2171
  options(knitr.graphics.rel_path = FALSE)

  rmarkdown::render(envir = globalenv(), ...)
}

a <- function(text, href) {
  ifelse(is.na(href), text, paste0("<a href='", href, "'>", text, "</a>"))
}

link_url <- function(text, href) {
  if (!is.null(href)) {
    paste0("<a href='", href, "'>", text, "</a>")
  }
}

linkify <- function(text) {
  text <- escape_html(text)
  text <- gsub(
    "&lt;doi:([^&]+)&gt;", # DOIs with < > & are not supported
    "&lt;<a href='https://doi.org/\\1'>doi:\\1</a>&gt;",
    text,
    ignore.case = TRUE
  )
  text <- gsub(
    "&lt;arXiv:([^&]+)&gt;",
    "&lt;<a href='https://arxiv.org/abs/\\1'>arXiv:\\1</a>&gt;",
    text,
    ignore.case = TRUE
  )
  text <- gsub(
    "&lt;((http|ftp)[^&]+)&gt;", # URIs with & are not supported
    "&lt;<a href='\\1'>\\1</a>&gt;",
    text
  )
  text
}

dont_index <- function(x) {
  paste0("<div class='dont-index'>", x, "</div>")
}

escape_html <- function(x) {
  x <- gsub("&", "&amp;", x)
  x <- gsub("<", "&lt;", x)
  x <- gsub(">", "&gt;", x)
  # x <- gsub("'", "&#39;", x)
  # x <- gsub("\"", "&quot;", x)
  x
}

strip_html_tags <- function(x) gsub("<.*?>", "", x)

test_that("Stripping HTML tags", {
  expect_identical(
    strip_html_tags("<p>some text about <code>data</code>"),
    "some text about data"
  )
})

# links and references in the package description -------------------------

test_that("references in angle brackets are converted to HTML", {
  ## URL
  expect_identical(
    linkify("see <https://CRAN.R-project.org/view=SpatioTemporal>."),
    "see &lt;<a href='https://CRAN.R-project.org/view=SpatioTemporal'>https://CRAN.R-project.org/view=SpatioTemporal</a>&gt;."
  )
  ## DOI
  expect_identical(
    linkify("M & H (2017) <doi:10.1093/biostatistics/kxw051>"),
    "M &amp; H (2017) &lt;<a href='https://doi.org/10.1093/biostatistics/kxw051'>doi:10.1093/biostatistics/kxw051</a>&gt;"
  )
  ## arXiv
  expect_identical(
    linkify("see <arXiv:1802.03967>."),
    "see &lt;<a href='https://arxiv.org/abs/1802.03967'>arXiv:1802.03967</a>&gt;."
  )
  ## unsupported formats are left alone (just escaping special characters)
  unsupported <- c(
    "<doi:10.1002/(SICI)1097-0258(19980930)17:18<2045::AID-SIM943>3.0.CO;2-P>",
    "<https://scholar.google.com/citations?view_op=top_venues&hl=de&vq=phy_probabilitystatistics>"
  )
  expect_identical(linkify(unsupported), escape_html(unsupported))
})

rd_text <- function(x, fragment = TRUE) {
  con <- textConnection(x)
  on.exit(close(con), add = TRUE)

  set_classes(tools::parse_Rd(con, fragment = fragment, encoding = "UTF-8"))
}

rd_file <- function(path, pkg_path = NULL) {
  set_classes(tools::parse_Rd(path, encoding = "UTF-8"))
}

#' Translate an Rd string to its HTML output
#'
#' @param x Rd string. Backslashes must be double-escaped ("\\\\").
#' @param fragment logical indicating whether this represents a complete Rd file
#' @param ... additional arguments for as_html
#'
#' @examples
#' rd2html("a\n%b\nc")
#'
#' rd2html("a & b")
#'
#' rd2html("\\strong{\\emph{x}}")
#'
#' @export
rd2html <- function(x, fragment = TRUE, ...) {
  html <- as_html(rd_text(x, fragment = fragment), ...)
  str_trim(strsplit(str_trim(html), "\n")[[1]])
}

print.Rd <- function(x, ...) {
  utils::str(x)
}
#' @export
print.tag <- function(x, ...) {
  utils::str(x)
}

# Convert RD attributes to S3 classes -------------------------------------

set_classes <- function(rd) {
  if (is.list(rd)) {
    rd[] <- lapply(rd, set_classes)
  }
  set_class(rd)
}

set_class <- function(x) {
  structure(x,
    class = c(attr(x, "class"), tag(x), "tag"),
    Rd_tag = NULL,
    srcref = NULL,
    macros = NULL
  )
}

tag <- function(x) {
  tag <- attr(x, "Rd_tag")
  if (is.null(tag)) return()

  gsub("\\", "tag_", tag, fixed = TRUE)
}

#' @export
`[.tag` <- function(x, ...) {
  structure(NextMethod(), class = class(x))
}

test_that("special characters are escaped", {
  out <- rd2html("a & b")
  expect_equal(out, "a &amp; b")
})

test_that("converts Rd unicode shortcuts", {
  expect_snapshot(rd2html("``a -- b --- c''"))
})

test_that("simple tags translated to known good values", {
  # Simple insertions
  expect_equal(rd2html("\\ldots"), "...")
  expect_equal(rd2html("\\dots"), "...")
  expect_equal(rd2html("\\R"), "<span style=\"R\">R</span>")
  expect_equal(rd2html("\\cr"), "<br>")

  "Macros"
  expect_equal(rd2html("\\newcommand{\\f}{'f'} \\f{}"), "'f'")
  expect_equal(rd2html("\\renewcommand{\\f}{'f'} \\f{}"), "'f'")
})

test_that("comments converted to html", {
  expect_equal(rd2html("a\n%b\nc"), c("a", "<!-- %b -->", "c"))
})

test_that("simple wrappers work as expected", {
  expect_equal(rd2html("\\strong{x}"), "<strong>x</strong>")
  expect_equal(rd2html("\\strong{\\emph{x}}"), "<strong><em>x</em></strong>")
})

test_that("subsection generates h3", {
  expect_snapshot(cli::cat_line(rd2html("\\subsection{A}{B}")))
})

test_that("subsection generates h3", {
  expect_snapshot(cli::cat_line(rd2html("\\subsection{A}{
    p1

    p2
  }")))
})

test_that("subsection generates generated anchor", {
  text <- c("<body>", rd2html("\\subsection{A}{B}"), "</body>")
  html <- xml2::read_xml(paste0(text, collapse = "\n"))
  tweak_anchors(html)

  expect_equal(xpath_attr(html, ".//h3", "id"), "a")
  expect_equal(xpath_attr(html, ".//a", "href"), "#a")
})

test_that("nested subsection generates h4", {
  expect_snapshot(cli::cat_line(rd2html("\\subsection{H3}{\\subsection{H4}{}}")))
})

test_that("if generates html", {
  expect_equal(rd2html("\\if{html}{\\bold{a}}"), "<b>a</b>")
  expect_equal(rd2html("\\if{latex}{\\bold{a}}"), character())
})

test_that("ifelse generates html", {
  expect_equal(rd2html("\\ifelse{html}{\\bold{a}}{x}"), "<b>a</b>")
  expect_equal(rd2html("\\ifelse{latex}{x}{\\bold{a}}"), "<b>a</b>")
})

test_that("out is for raw html", {
  expect_equal(rd2html("\\out{<hr />}"), "<hr />")
})

test_that("support platform specific code", {
  os_specific <- function(command, os, output) {
    rd2html(paste0(
      "#", command, " ", os, "\n",
      output, "\n",
      "#endif"
    ))
  }

  expect_equal(os_specific("ifdef", "windows", "X"), character())
  expect_equal(os_specific("ifdef", "unix", "X"), "X")
  expect_equal(os_specific("ifndef", "windows", "X"), "X")
  expect_equal(os_specific("ifndef", "unix", "X"), character())
})


# tables ------------------------------------------------------------------

test_that("tabular generates complete table html", {
  table <- "\\tabular{ll}{a \\tab b \\cr}"
  expectation <- c("<table class='table'>", "<tr><td>a</td><td>b</td></tr>", "</table>")
  expect_equal(rd2html(table), expectation)
})

test_that("internal \\crs are stripped", {
  table <- "\\tabular{l}{a \\cr b \\cr c \\cr}"
  expectation <- c("<table class='table'>", "<tr><td>a</td></tr>", "<tr><td>b</td></tr>", "<tr><td>c</td></tr>", "</table>")
  expect_equal(rd2html(table), expectation)
})

test_that("can convert single row", {
  expect_equal(
    rd2html("\\tabular{lll}{A \\tab B \\tab C \\cr}")[[2]],
    "<tr><td>A</td><td>B</td><td>C</td></tr>"
  )
})


test_that("don't need internal whitespace", {
  expect_equal(
    rd2html("\\tabular{lll}{\\tab\\tab C\\cr}")[[2]],
    "<tr><td></td><td></td><td>C</td></tr>"
  )
  expect_equal(
    rd2html("\\tabular{lll}{\\tab B \\tab\\cr}")[[2]],
    "<tr><td></td><td>B</td><td></td></tr>"
  )
  expect_equal(
    rd2html("\\tabular{lll}{A\\tab\\tab\\cr}")[[2]],
    "<tr><td>A</td><td></td><td></td></tr>"
  )

  expect_equal(
    rd2html("\\tabular{lll}{\\tab\\tab\\cr}")[[2]],
    "<tr><td></td><td></td><td></td></tr>"
  )
})

test_that("can skip trailing \\cr", {
  expect_equal(
    rd2html("\\tabular{lll}{A \\tab B \\tab C}")[[2]],
    "<tr><td>A</td><td>B</td><td>C</td></tr>"
  )
})

test_that("code blocks in tables render (#978)", {
  expect_equal(
    rd2html('\\tabular{ll}{a \\tab \\code{b} \\cr foo \\tab bar}')[[2]],
    "<tr><td>a</td><td><code>b</code></td></tr>"
  )
})

test_that("tables with tailing \n (#978)", {
  expect_equal(
    rd2html('
      \\tabular{ll}{
        a   \\tab     \\cr
        foo \\tab bar
      }
    ')[[2]],
    "<tr><td>a</td><td></td></tr>"
  )
})

# sexpr  ------------------------------------------------------------------

test_that("code inside Sexpr is evaluated", {
  local_context_eval()
  expect_equal(rd2html("\\Sexpr{1 + 2}"), "3")
})

test_that("can control \\Sexpr output", {
  local_context_eval()
  expect_equal(rd2html("\\Sexpr[results=hide]{1}"), character())
  expect_equal(rd2html("\\Sexpr[results=text]{1}"), "1")
  expect_equal(rd2html("\\Sexpr[results=rd]{\"\\\\\\emph{x}\"}"), "<em>x</em>")
  expect_equal(
    rd2html("\\Sexpr[results=verbatim]{1 + 2}"),
    c("<pre>", "[1] 3", "</pre>")
  )
  expect_equal(
    rd2html("\\Sexpr[results=verbatim]{cat(42)}"),
    c("<pre>", "42", "</pre>")
  )
  expect_equal(
    rd2html("\\Sexpr[results=verbatim]{cat('42!\n'); 3}"),
    c("<pre>", "42!", "[1] 3", "</pre>")
  )
})

test_that("Sexpr can contain multiple expressions", {
  local_context_eval()
  expect_equal(rd2html("\\Sexpr{a <- 1; a}"), "1")
})

test_that("Sexprs with multiple args are parsed", {
  local_context_eval()
  expect_equal(rd2html("\\Sexpr[results=hide,stage=build]{1}"), character())
})

test_that("Sexprs in file share environment", {
  local_context_eval()
  expect_equal(rd2html("\\Sexpr{x <- 1}\\Sexpr{x}"), c("1", "1"))

  local_context_eval()
  expect_error(rd2html("\\Sexpr{x}"), "not found")
})

test_that("Sexprs run from package root", {
  local_context_eval(src_path = test_path("assets/reference"))

  # \packageTitle is built in macro that uses DESCRIPTION
  expect_equal(
    rd2html("\\packageTitle{testpackage}"),
    "A test package"
  )
})

# links -------------------------------------------------------------------

test_that("simple links generate <a>", {
  expect_equal(
    rd2html("\\href{http://bar.com}{BAR}"),
    "<a href='http://bar.com'>BAR</a>"
  )
  expect_equal(
    rd2html("\\email{foo@bar.com}"),
    "<a href='mailto:foo@bar.com'>foo@bar.com</a>"
  )
  expect_equal(
    rd2html("\\url{http://bar.com}"),
    "<a href='http://bar.com'>http://bar.com</a>"
  )
})

test_that("can convert cross links to online documentation url", {
  expect_equal(
    rd2html("\\link[base]{library}"),
    a("library", href = "https://rdrr.io/r/base/library.html")
  )
})

test_that("can convert cross links to the same package (#242)", {
  withr::local_options(list(
    "downlit.package" = "test",
    "downlit.topic_index" = c(x = "y", z = "z"),
    "downlit.rdname" = "z"
  ))

  expect_equal(rd2html("\\link{x}"), "<a href='y.html'>x</a>")
  expect_equal(rd2html("\\link[test]{x}"), "<a href='y.html'>x</a>")
  # but no self links
  expect_equal(rd2html("\\link[test]{z}"), "z")
})

test_that("can parse local links with topic!=label", {
  withr::local_options(list(
    "downlit.topic_index" = c(x = "y")
  ))
  expect_equal(rd2html("\\link[=x]{z}"), "<a href='y.html'>z</a>")
})

test_that("functions in other packages generates link to rdrr.io", {
  withr::local_options(list(
    "downlit.package" = "test",
    "downlit.topic_index" = c(x = "y", z = "z")
  ))

  expect_equal(
    rd2html("\\link[stats:acf]{xyz}"),
    a("xyz", downlit::href_topic("acf", "stats"))
  )

  # Unless it's the current package
  expect_equal(rd2html("\\link[test:x]{xyz}"), "<a href='y.html'>xyz</a>")
})

test_that("link to non-existing functions return label", {
  expect_equal(rd2html("\\link[xyzxyz:xyzxyz]{abc}"), "abc")
  expect_equal(rd2html("\\link[base:xyzxyz]{abc}"), "abc")
})

test_that("code blocks autolinked to vignettes", {
  withr::local_options(list(
    "downlit.package" = "test",
    "downlit.article_index" = c("abc" = "abc.html")
  ))

  expect_equal(
    rd2html("\\code{vignette('abc')}"),
    "<code><a href='abc.html'>vignette('abc')</a></code>"
  )
})

test_that("link to non-existing functions return label", {
  withr::local_options(list(
    "downlit.package" = "test",
    "downlit.topic_index" = c("TEST-class" = "test")
  ))
  expect_equal(rd2html("\\linkS4class{TEST}"), "<a href='test.html'>TEST</a>")
})

test_that("bad specs throw errors", {
  expect_snapshot(error = TRUE, {
    rd2html("\\url{}")
    rd2html("\\url{a\nb}")
    rd2html("\\email{}")
    rd2html("\\linkS4class{}")
  })
})

# Paragraphs --------------------------------------------------------------

test_that("empty input gives empty output", {
  expect_equal(flatten_para(character()), character())
})

test_that("empty lines break paragraphs", {
  expect_equal(
    flatten_para(rd_text("a\nb\n\nc")),
    "<p>a\nb</p>\n<p>c</p>"
  )
})

test_that("indented empty lines break paragraphs", {
  expect_equal(
    flatten_para(rd_text("a\nb\n  \nc")),
    "<p>a\nb</p>  \n<p>c</p>"
  )
})

test_that("block tags break paragraphs", {
  out <- flatten_para(rd_text("a\n\\itemize{\\item b}\nc"))
  expect_equal(out, "<p>a</p><ul>\n<li><p>b</p></li>\n</ul><p>c</p>")
})

test_that("inline tags + empty line breaks", {
  out <- flatten_para(rd_text("a\n\n\\code{b}"))
  expect_equal(out, "<p>a</p>\n<p><code>b</code></p>")
})

test_that("single item can have multiple paragraphs", {
  out <- flatten_para(rd_text("\\itemize{\\item a\n\nb}"))
  expect_equal(out, "<ul>\n<li><p>a</p>\n<p>b</p></li>\n</ul>\n")
})

test_that("nl after tag doesn't trigger paragraphs", {
  out <- flatten_para(rd_text("One \\code{}\nTwo"))
  expect_equal(out, "<p>One <code></code>\nTwo</p>")
})

test_that("cr generates line break", {
  out <- flatten_para(rd_text("a \\cr b"))
  expect_equal(out, "<p>a <br> b</p>")
})


# lists -------------------------------------------------------------------

test_that("simple lists work", {
  expect_equal(
    rd2html("\\itemize{\\item a}"),
    c("<ul>", "<li><p>a</p></li>", "</ul>")
  )
  expect_equal(
    rd2html("\\enumerate{\\item a}"),
    c("<ol>", "<li><p>a</p></li>", "</ol>")
  )
})

test_that("\\describe items can contain multiple paragraphs", {
  out <- rd2html("\\describe{
    \\item{Label 1}{Contents 1}
    \\item{Label 2}{Contents 2}
  }")
  expect_snapshot_output(cat(out, sep = "\n"))
})

test_that("can add ids to descriptions", {
  out <- rd2html("\\describe{
    \\item{abc}{Contents 1}
    \\item{xyz}{Contents 2}
  }", id_prefix = "foo")
  expect_snapshot_output(cat(out, sep = "\n"))
})

test_that("\\describe items can contain multiple paragraphs", {
  out <- rd2html("\\describe{
    \\item{Label}{
      Paragraph 1

      Paragraph 2
    }
  }")
  expect_snapshot_output(cat(out, sep = "\n"))
})

test_that("nested item with whitespace parsed correctly", {
  out <- rd2html("
    \\describe{
    \\item{Label}{

      This text is indented in a way pkgdown doesn't like.
  }}")
  expect_snapshot_output(cat(out, sep = "\n"))
})

# Verbatim ----------------------------------------------------------------

test_that("preformatted blocks aren't double escaped", {
  out <- flatten_para(rd_text("\\preformatted{\\%>\\%}"))
  expect_equal(out, "<pre><code>%&gt;%</code></pre>\n")
})

test_that("newlines are preserved in preformatted blocks", {
  out <- flatten_para(rd_text("\\preformatted{^\n\nb\n\nc}"))
  expect_equal(out, "<pre><code>^\n\nb\n\nc</code></pre>\n")
})

test_that("spaces are preserved in preformatted blocks", {
  out <- flatten_para(rd_text("\\preformatted{^\n\n  b\n\n  c}"))
  expect_equal(out, "<pre><code>^\n\n  b\n\n  c</code></pre>\n")
})

# Other -------------------------------------------------------------------

test_that("eqn", {
  out <- rd2html(" \\eqn{\\alpha}{alpha}")
  expect_equal(out, "\\(\\alpha\\)")
  out <- rd2html(" \\eqn{x}")
  expect_equal(out, "\\(x\\)")
})

test_that("deqn", {
  out <- rd2html(" \\deqn{\\alpha}{alpha}")
  expect_equal(out, "$$\\alpha$$")
  out <- rd2html(" \\deqn{x}")
  expect_equal(out, "$$x$$")
})

test_that("special", {
  out <- rd2html("\\special{( \\dots )}")
  expect_equal(out, "( ... )")
})

# figures -----------------------------------------------------------------

test_that("figures are converted to img", {
  expect_equal(rd2html("\\figure{a}"), "<img src='figures/a' alt='' />")
  expect_equal(rd2html("\\figure{a}{b}"), "<img src='figures/a' alt='b' />")
  expect_equal(
    rd2html("\\figure{a}{options: height=1}"),
    "<img src='figures/a' height=1 />"
  )
})

test_that("figures with multilines alternative text can be parsed", {
  expect_equal(rd2html("\\figure{a}{blabla
    blop}"), "<img src='figures/a' alt='blabla blop' />")
})

fig_save <- function(plot,
                     name,
                     dev = "ragg::agg_png",
                     dpi = 96L,
                     dev.args = list(),
                     fig.ext = "png",
                     fig.width = 700 / 96,
                     fig.height = NULL,
                     fig.retina = 2L,
                     fig.asp = 1 / 1.618, # golden ratio
                     bg = NULL,
                     other.parameters = list()
                     ) {

  path <- paste0(name, ".", fig.ext)
  dev <- match_fun(dev)

  if (is.null(fig.height)) {
    fig.height <- fig.width * fig.asp
  } else if (is.null(fig.width)) {
    fig.width <- fig.height / fig.asp
  }
  width <- round(dpi * fig.width)
  height <- round(dpi * fig.height)

  has_res <- "res" %in% names(formals(dev))
  if (has_res) {
    # raster device; units in pixels, need to rescale for retina
    args <- list(
      path, # some devices use file and some use filename
      width = width * fig.retina,
      height = height * fig.retina,
      res = dpi * fig.retina
    )
  } else {
    # vector device; units in inches; no need to rescale
    args <- list(
      path,
      width = fig.width,
      height = fig.height
    )
  }

  # NB: bg is always set to transparent here; it takes effect during
  # recording in highlight_examples()
  dev.args$bg <- NA

  with_device(dev, c(args, dev.args), plot)

  list(path = path, width = width, height = height)
}

fig_save_args <- function() {
  # Avoid having multiple copies of the default settings
  default <- formals(fig_save)
  default$plot <- NULL
  default$name <- NULL
  default <- lapply(default, eval, baseenv())
  default
}

meta_figures <- function(meta = list()) {
  default <- fig_save_args()
  figures <- purrr::pluck(meta, "figures", .default = list())

  print_yaml(utils::modifyList(default, figures))
}

#' Get current settings for figures
#'
#' @description
#' You will generally not need to use this function unless you are handling
#' custom plot output.
#'
#' Packages needing custom parameters should ask users to place them within
#' the `other.parameters` entry under the package name, e.g.
#' ```
#' figures:
#'   other.parameters:
#'     rgl:
#'       fig.asp: 1
#' ```
#'
#' @return
#' A list containing the entries from the `figures` field in `_pkgdown.yml`
#' (see [build_reference()]), with default values added. Computed `width` and
#' `height` values (in pixels) are also included.
#' @export
#' @keywords internal
fig_settings <- function() {
  result <- fig_save_args()

  # The context might not be initialized.
  settings <- tryCatch(context_get("figures"), error = function(e) NULL)
  result[names(settings)] <- settings

  if (is.null(result$fig.height)) {
    result$fig.height <- result$fig.width * result$fig.asp
  } else if (is.null(result$fig.width)) {
    result$fig.width <- result$fig.height / result$fig.asp
  }
  result
}

with_device <- function(dev, dev.args, plot) {
  do.call(dev, dev.args)
  on.exit(grDevices::dev.off())

  print(plot)
}

fig_opts_chunk <- function(figures, default) {
  figures$dev <- fun_name(figures$dev)

  # fig.asp beats fig.height in knitr, so if it's provided only use
  # it to override the default height
  if (!is.null(figures$fig.asp) && is.null(figures$fig.height)) {
    figures$fig.height <- figures$fig.width * figures$fig.asp
    figures$fig.asp <- NULL
  }

  # Copy background into dev.args
  figures$dev.args <- figures$dev.args %||% list()
  figures$dev.args$bg <- figures$bg %||% NA

  utils::modifyList(default, figures)
}

# Find graphics device ----------------------------------------------------

match_fun <- function(x) {
  if (is.function(x)) {
    x
  } else if (is.character(x) && length(x) == 1) {
    e <- parse_expr(x)
    f <- eval(e, globalenv())

    if (!is.function(f)) {
      stop("`x` must evaluate to a function", call. = FALSE)
    }

    f
  } else {
    stop("`x` must be a function or string", call. = FALSE)
  }
}

# knitr only takes a function name - user will need to load package
fun_name <- function(x) {
  if (x == "ragg::agg_png") {
    return("ragg_png")
  }

  expr <- parse_expr(x)
  if (is_symbol(expr)) {
    x
  } else if (is_call(expr, "::")) {
    as.character(expr[[3]])
  } else {
    stop("Unknown input", call. = FALSE)
  }
}

# Reading -----------------------------------------------------------------

read_file <- function(path) {
  lines <- read_lines(path)
  paste0(lines, "\n", collapse = "")
}


# Inspired by roxygen2 utils-io.R (https://github.com/klutometis/roxygen/) --------
read_lines <- function(path, n = -1L) {
  base::readLines(path, n = n, encoding = "UTF-8", warn = FALSE) # nolint
}

write_lines <- function(text, path) {
  base::writeLines(enc2utf8(text), path, useBytes = TRUE) # nolint
}

# Other -------------------------------------------------------------------

file_equal <- function(src, dst) {
  if (!file_exists(dst)) {
    return(FALSE)
  }

  src_hash <- rlang::hash_file(src)
  dst_hash <- rlang::hash_file(dst)

  identical(src_hash, dst_hash)
}

#' Deploy a pkgdown site on Travis-CI to Github Pages
#'
#' @description
#' `r lifecycle::badge('superseded')`
#'
#' `deploy_site_github()` was designed to deploy your site from Travis CI,
#' which we no longer recommend, so this function is deprecated. There are
#' two replacements:
#'
#' * [usethis::use_pkgdown_github_pages()] will setup a GitHub action to
#'   automatically build and deploy your package website to GitHub pages.
#'
#' * [deploy_to_branch()] can be called locally to build and deploy your
#'   website to any desired branch.
#'
#' @inheritParams build_site
#' @param install Optionally, opt-out of automatic installation. This is
#'   necessary if the package you're documenting is a dependency of pkgdown
#' @param tarball The location of the built package tarball. The default Travis
#'   configuration for R packages sets `PKG_TARBALL` to this path.
#' @param ssh_id The private id to use, a base64 encoded content of the private
#'   pem file. This should _not_ be your personal private key. Instead create a
#'   new keypair specifically for deploying the site. The easiest way is to use
#'   `travis::use_travis_deploy()`.
#' @param commit_message The commit message to be used for the commit.
#' @param clean Clean all files from old site.
#' @param verbose Print verbose output
#' @param ... Additional arguments passed to [build_site()].
#' @param host The GitHub host url.
#' @param repo_slug The `user/repo` slug for the repository.
#' @export
#' @keywords internal
deploy_site_github <- function(
  pkg = ".",
  install = TRUE,
  tarball = Sys.getenv("PKG_TARBALL", ""),
  ssh_id = Sys.getenv("id_rsa", ""),
  commit_message = construct_commit_message(pkg),
  clean = FALSE,
  verbose = FALSE,
  host = "github.com",
  ...,
  repo_slug = Sys.getenv("TRAVIS_REPO_SLUG", "")) {
  rlang::check_installed("openssl")
  if (!nzchar(tarball)) {
    cli::cli_abort("No built tarball detected, please provide the location of one with {.var tarball}", call = caller_env())
  }

  if (!nzchar(ssh_id)) {
    cli::cli_abort("No deploy key found, please setup with {.fn travis::use_travis_deploy}", call = caller_env())
  }

  if (!nzchar(repo_slug)) {
    cli::cli_abort("No repo detected, please supply one with {.var repo_slug}", call = caller_env())
  }

  cli::cli_alert("Deploying site to GitHub")
  if (install) {
    cli::cli_inform("Installing package")
    callr::rcmd("INSTALL", tarball, show = verbose, fail_on_status = TRUE)
  }

  ssh_id_file <- "~/.ssh/id_rsa"
  cli::cli_inform("Setting up SSH id")
  cli::cli_inform("Copying private key to {.file ssh_id_file}")
  write_lines(rawToChar(openssl::base64_decode(ssh_id)), ssh_id_file)
  cli::cli_inform("Setting private key permissions to 0600")
  file_chmod(ssh_id_file, "0600")

  cli::cli_inform("Setting remote to use the ssh url")

  git("remote", "set-url", "origin", sprintf("git@%s:%s.git", host, repo_slug))

  deploy_to_branch(
    pkg,
    commit_message = commit_message,
    clean = clean,
    branch = "gh-pages",
    ...
  )

  cli::cli_inform(c(v = "Deploy completed"))
}

#' Build and deploy a site locally
#'
#' Assumes that you're in a git clone of the project, and the package is
#' already installed. Use [usethis::use_pkgdown_github_pages()] to automate
#' this process using GitHub actions.
#'
#' @param branch The git branch to deploy to
#' @param remote The git remote to deploy to
#' @param github_pages Is this a GitHub pages deploy. If `TRUE`, adds a `CNAME`
#'   file for custom domain name support, and a `.nojekyll` file to suppress
#'   jekyll rendering.
#' @param ... Additional arguments passed to [build_site()].
#' @param subdir The sub-directory where the site should be built on the branch.
#'   This argument can be used to support a number of site configurations.
#'   For example, you could build version-specific documentation by setting
#'   `subdir = "v1.2.3"`; `deploy_to_branch()` will build and deploy the
#'   package documentation in the `v.1.2.3/` directory of your site.
#' @inheritParams build_site
#' @inheritParams deploy_site_github
#' @export
deploy_to_branch <- function(pkg = ".",
                         commit_message = construct_commit_message(pkg),
                         clean = TRUE,
                         branch = "gh-pages",
                         remote = "origin",
                         github_pages = (branch == "gh-pages"),
                         ...,
                         subdir = NULL) {
  dest_dir <- dir_create(file_temp())
  on.exit(dir_delete(dest_dir))

  if (!git_has_remote_branch(remote, branch)) {
    old_branch <- git_current_branch()

    # If no remote branch, we need to create it
    git("checkout", "--orphan", branch)
    git("rm", "-rf", "--quiet", ".")
    git("commit", "--allow-empty", "-m", sprintf("Initializing %s branch", branch))
    git("push", remote, paste0("HEAD:", branch))

    # checkout the previous branch
    git("checkout", old_branch)
  }

  # Explicitly set the branches tracked by the origin remote.
  # Needed if we are using a shallow clone, such as on travis-CI
  git("remote", "set-branches", "--add", remote, branch)

  git("fetch", remote, branch)

  github_worktree_add(dest_dir, remote, branch)
  on.exit(github_worktree_remove(dest_dir), add = TRUE)

  site_dest_dir <-
    if (!is.null(subdir)) {
      dir_create(path(dest_dir, subdir))
    } else {
      dest_dir
    }

  pkg <- as_pkgdown(pkg, override = list(destination = site_dest_dir))

  if (!is.null(subdir) && !is.null(pkg$meta$url)) {
    pkg$meta$url <- path(pkg$meta$url, subdir)
  }

  build_site_github_pages(pkg, ..., clean = clean)

  github_push(dest_dir, commit_message, remote, branch)

  invisible()
}

git_has_remote_branch <- function(remote, branch) {
  has_remote_branch <- git("ls-remote", "--quiet", "--exit-code", remote, branch, echo = FALSE, echo_cmd = FALSE, error_on_status = FALSE)$status == 0
}

git_current_branch <- function() {
  branch <- git("rev-parse", "--abbrev-ref", "HEAD", echo = FALSE, echo_cmd = FALSE)$stdout
  sub("\n$", "", branch)
}

github_worktree_add <- function(dir, remote, branch) {
  cli::cli_inform("Adding worktree")
  git("worktree",
    "add",
    "--track", "-B", branch,
    dir,
    paste0(remote, "/", branch)
  )
}

github_worktree_remove <- function(dir) {
  cli::cli_inform("Removing worktree")
  git("worktree", "remove", dir)
}

github_push <- function(dir, commit_message, remote, branch) {
  # force execution before changing working directory
  force(commit_message)

  cli::cli_inform("Commiting updated site")

  withr::with_dir(dir, {
    git("add", "-A", ".")
    git("commit", "--allow-empty", "-m", commit_message)

    cli::cli_alert("Deploying to GitHub Pages")
    git("remote", "-v")
    git("push", "--force", remote, paste0("HEAD:", branch))
  })
}

git <- function(..., echo_cmd = TRUE, echo = TRUE, error_on_status = TRUE) {
  callr::run("git", c(...), echo_cmd = echo_cmd, echo = echo, error_on_status = error_on_status)
}

construct_commit_message <- function(pkg = ".", commit = ci_commit_sha()) {
  pkg <- as_pkgdown(pkg)
  cli::format_inline("Built site for {pkg$package}@{pkg$version}: {substr(commit, 1, 7)}")
}

ci_commit_sha <- function() {
  env_vars <- c(
    # https://docs.travis-ci.com/user/environment-variables/#default-environment-variables
    "TRAVIS_COMMIT",
    # https://help.github.com/en/actions/configuring-and-managing-workflows/using-environment-variables#default-environment-variables
    "GITHUB_SHA"
  )

  for (var in env_vars) {
    commit_sha <- Sys.getenv(var, "")
    if (commit_sha != "")
      return(commit_sha)
  }

  ""
}

# ci_commit_sha() ------------------------------------------------------------

test_that("commit sha can be retrieved from travis or GitHub action env vars", {
  sha <- "XYZ"

  withr::with_envvar(
    c("TRAVIS_COMMIT" = sha, "GITHUB_SHA" = ""),
    expect_equal(ci_commit_sha(), sha)
  )
  withr::with_envvar(
    c("TRAVIS_COMMIT" = "", "GITHUB_SHA" = sha),
    expect_equal(ci_commit_sha(), sha)
  )
  withr::with_envvar(
    c("TRAVIS_COMMIT" = "", "GITHUB_SHA" = ""),
    expect_equal(ci_commit_sha(), "")
  )
})

# highligh_text() and highlight_examples() are only used for usage
# and examples, and are specifically excluded in tweak_reference_highlighting()
highlight_text <- function(text) {
  out <- downlit::highlight(text, classes = downlit::classes_pandoc())
  if (!is.na(out)) {
    sourceCode(pre(out, r_code = TRUE))
  } else {
    sourceCode(pre(escape_html(text)))
  }
}

highlight_examples <- function(code, topic, env = globalenv()) {
  bg <- fig_settings()$bg %||% NA

  # some options from testthat::local_reproducible_output()
  # https://github.com/r-lib/testthat/blob/47935141d430e002070a95dd8af6dbf70def0994/R/local.R#L86
  withr::local_options(list(
    device = function(...) ragg::agg_png(..., bg = bg),
    rlang_interactive = FALSE,
    cli.num_colors = 256,
    cli.dynamic = FALSE
  ))
  withr::local_envvar(RSTUDIO = NA)
  withr::local_collate("C")

  fig_save_topic <- function(plot, id) {
    name <- paste0(topic, "-", id)
    do.call(fig_save, c(list(plot, name), fig_settings()))
  }

  out <- downlit::evaluate_and_highlight(
    code,
    fig_save = fig_save_topic,
    env = child_env(env),
    output_handler = evaluate::new_output_handler(value = pkgdown_print)
  )
  structure(
    sourceCode(pre(out, r_code = TRUE)),
    dependencies = attr(out, "dependencies")
  )
}

pre <- function(x, r_code = FALSE) {
  paste0(
    "<pre", if (r_code) " class='sourceCode r'", ">", "<code>",
    x,
    "</code>","</pre>"
  )
}

sourceCode <- function(x) {
  paste0("<div class='sourceCode'>", x, "</div>")
}

test_that("highlight_examples captures dependencies", {
  withr::defer(try(file_delete(test_path("Rplot001.png")), TRUE))

  dummy_dep <- htmltools::htmlDependency("dummy", "1.0.0", "dummy.js")
  widget <- htmlwidgets::createWidget("test", list(), dependencies = dummy_dep)
  out <- highlight_examples("widget", env = environment())

  # htmlwidgets always get dependency on htmlwidgets.js
  expect_equal(attr(out, "dependencies")[-1], list(dummy_dep))
})

test_that("highlight_examples runs and hides DONTSHOW calls()", {
  out <- highlight_examples("DONTSHOW(x <- 1)\nx")
  expect_snapshot(cat(strip_html_tags(out)))
})

test_that("highlight_text & highlight_examples include sourceCode div", {
  withr::defer(try(file_delete(test_path("Rplot001.png")), TRUE))

  html <- xml2::read_html(highlight_examples("a + a", "x"))
  expect_equal(xpath_attr(html, "./body/div", "class"), "sourceCode")

  html <- xml2::read_html(highlight_text("a + a"))
  expect_equal(xpath_attr(html, "./body/div", "class"), "sourceCode")
})

test_that("pre() can produce needed range of outputs", {
  expect_snapshot({
    cat(pre("x"))
    cat(pre("x", r_code = TRUE))
  })
})

test_that("tweak_highlight_other() renders generic code blocks for roxygen2 >= 7.2.0", {
  html <- xml2::read_html('<div class="sourceCode"><pre><code>1+1\n</code></pre></div>')
  div <- xml2::xml_find_first(html, "//div")
    
  tweak_highlight_other(div)
  expect_equal(xpath_text(div, "pre/code"), "1+1")
})

test_that("tweak_highlight_other() renders nested code blocks for roxygen2 >= 7.2.0", {
  html <- xml2::read_html(dedent("
    <div class='sourceCode markdown'><pre><code>
    blablabla

    ```{r results='asis'}
    lalala
    ```

    </code></pre></div>
  "))
  div <- xml2::xml_find_first(html, "//div")

  tweak_highlight_other(div)
  expect_snapshot(cat(xpath_text(div, "pre/code")))
})

#' Initialise site infrastructure
#'
#' @description
#' `init_site()`:
#'
#' * creates the output directory (`docs/`),
#' * generates a machine readable description of the site, used for autolinking,
#' * copies CSS/JS assets and extra files, and
#' * runs `build_favicons()`, if needed.
#'
#' Typically, you will not need to call this function directly, as all `build_*()`
#' functions will run `init_site()` if needed.
#'
#' The only good reasons to call `init_site()` directly are the following:
#' * If you add or modify a package logo.
#' * If you add or modify `pkgdown/extra.scss`.
#' * If you modify `template.bslib` variables in `_pkgdown.yml`.
#'
#' See `vignette("customise")` for the various ways you can customise the
#' display of your site.
#'
#' # Build-ignored files
#' We recommend using [usethis::use_pkgdown_github_pages()] to build-ignore `docs/` and
#' `_pkgdown.yml`. If use another directory, or create the site manually,
#' you'll need to add them to `.Rbuildignore` yourself. A `NOTE` about
#' an unexpected file during `R CMD CHECK` is an indication you have not
#' correctly ignored these files.
#'
#' @inheritParams build_articles
#' @export
init_site <- function(pkg = ".", override = list()) {
  # This is the only user facing function that doesn't call section_init()
  # because section_init() can conditionally call init_site()
  rstudio_save_all()
  cache_cli_colours()
  pkg <- as_pkgdown(pkg, override = override)

  cli::cli_rule("Initialising site")
  dir_create(pkg$dst_path)

  copy_assets(pkg)
  if (pkg$bs_version > 3) {
    build_bslib(pkg)
  }

  # Building favicons is expensive, so we hopefully only do it once, locally
  if (has_logo(pkg) && !has_favicons(pkg) && !on_ci()) {
    build_favicons(pkg)
  }
  copy_favicons(pkg)
  copy_logo(pkg)

  build_site_meta(pkg)

  invisible()
}

copy_assets <- function(pkg = ".") {
  pkg <- as_pkgdown(pkg)
  template <- config_pluck(pkg, "template")

  # pkgdown assets
  if (!identical(template$default_assets, FALSE)) {
    copy_asset_dir(
      pkg,
      path_pkgdown(paste0("BS", pkg$bs_version, "/", "assets")),
      src_root = path_pkgdown(),
      src_label = "<pkgdown>/"
    )
  }

  # package assets
  if (!is.null(template$package)) {
    copy_asset_dir(
      pkg,
      path_package_pkgdown("assets", template$package, pkg$bs_version),
      src_root = system_file(package = template$package),
      src_label = paste0("<", template$package, ">/")
    )
  }

  # extras
  copy_asset_dir(pkg, "pkgdown", file_regexp = "^extra")
  # site assets
  copy_asset_dir(pkg, "pkgdown/assets")

  invisible()
}

copy_asset_dir <- function(pkg,
                           dir,
                           src_root = pkg$src_path,
                           src_label = "",
                           file_regexp = NULL) {
  src_dir <- path_abs(dir, pkg$src_path)
  if (!file_exists(src_dir)) {
    return(character())
  }

  src_paths <- dir_ls(src_dir, recurse = TRUE)
  src_paths <- src_paths[!is_dir(src_paths)]
  if (!is.null(file_regexp)) {
    src_paths <- src_paths[grepl(file_regexp, path_file(src_paths))]
  }
  src_paths <- src_paths[path_ext(src_paths) != "scss"] # Handled in bs_theme()

  dst_paths <- path(pkg$dst_path, path_rel(src_paths, src_dir))

  file_copy_to(
    src_paths = src_paths,
    src_root = src_root,
    src_label = src_label,
    dst_paths = dst_paths,
    dst_root = pkg$dst_path
  )
}

timestamp <- function(time = Sys.time()) {
  attr(time, "tzone") <- "UTC"
  strftime(time, "%Y-%m-%dT%H:%MZ", tz = "UTC")
}

# Generate site meta data file (available to website viewers)
build_site_meta <- function(pkg = ".") {
  meta <- site_meta(pkg)

  # Install pkgdown.yml to ./inst if requested,
  install_metadata <- pkg$install_metadata %||% FALSE
  if (install_metadata) {
    path_meta <- path(pkg$src_path, "inst", "pkgdown.yml")

    dir_create(path_dir(path_meta))
    write_yaml(meta, path_meta)
  }

  path_meta <- path(pkg$dst_path, "pkgdown.yml")
  write_yaml(meta, path_meta)
  invisible()
}

site_meta <- function(pkg) {
  article_index <- article_index(pkg)

  yaml <- list(
    pandoc = as.character(rmarkdown::pandoc_version()),
    pkgdown = as.character(utils::packageDescription("pkgdown", fields = "Version")),
    pkgdown_sha = utils::packageDescription("pkgdown")$GithubSHA1,
    articles = as.list(article_index),
    last_built = timestamp()
  )

  url <- config_pluck_string(pkg, "url")
  if (!is.null(url)) {
    yaml$urls <- list(
      reference = paste0(url, "/reference"),
      article = paste0(url, "/articles")
    )
  }

  print_yaml(yaml)
}

---
title: "Accessibility"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Accessibility}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

pkgdown automates as many accessibility details as possible, so that your package website is readable by as many people as possible. This vignette describes the additional details that can't be automated away and you need to be aware of.

```{r}
#| label: setup
library(pkgdown)
```

## Theming

* If you adjust any colours from the default theme (including the syntax highlighting theme), you should double check that the contrast between the background and foreground doesn't make any text difficult to read. A good place to start is running a representative page of your site through <https://wave.webaim.org>.

* The default colour of the development version label makes a slightly too low contrast against the pale grey background of the navbar. This colour comes from the bootstrap "danger" colour, so you can fix it by overriding that variable in your `_pkgdown.yml`:

    ```yaml
    template:
      bootstrap: 5
      bslib:
        danger: "#A6081A"
    ```

* If you use custom navbar entries that only display an icon, make sure to also use the `aria-label` field to provide an accessible label that describes the icon.

    ```yaml
    cran:
      icon: fab fa-r-project
      href: https://cloud.r-project.org/package=pkgdown
      aria-label: View on CRAN
    ```

## Images

To make your site fully accessible, the place where you are likely to need to do the most work is adding alternative text to any images that you create. Unfortunately, there's currently no way to do this for plots you generate in examples, but you can and should add alternative text to plots in vignettes using the `fig.alt` chunk option:

````{verbatim}
```{r}
#| fig.alt: >
#|   Histogram of time between eruptions for Old Faithful. 
#|   It is a bimodal distribution with peaks at 50-55 and 
#|   80-90 minutes.
hist(faithful$waiting)
```
````

If you forget to add alt text to your vignettes, pkgdown will automatically remind you.

---
title: "Translations"
output: rmarkdown::html_vignette
description: >
  If your documentation is written in a language other than English, you can
  automatically translate the text generated by pkgdown by setting the `lang`
  field. If your language isn't currently supported, we'll show you how you can
  provide translations.
vignette: >
  %\VignetteIndexEntry{Translations}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

If your documentation (`.Rd` and `.Rmd`) is written in a language other than English, declare it by setting setting `lang` to the [language code](https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes) for your language:

``` yaml
lang: es
```

This will be used to set the language of the web page and to translate the English words that pkgdown generates on your site.
Current available translations are:

-   `ca`: Catalan
-   `de`: German
-   `dk`: Danish
-   `es`: Spanish
-   `fr`: French
-   `ja`: Japanese
-   `ko`: Korean
-   `pt`: Portuguese
-   `tr`: Turkish
-   `zh_CN`: Chinese (simplified)

As you can see, most language codes are two letters, but if a language has multiple variants, it gets a longer form which can be used to disambiguate the options. For example, Chinese can use one of two forms: simplified (used in China and Singapore) or traditional (used in Taiwan and Hong Kong). Another example would be providing specific French Canadian translations by using code `fr_CN`.

If you deploy your site on Github Actions, you will need to update the locale and set the `env` during `pkgdown::build_site()` to the same language code you set in `_pkgdown.yml`, including the country code. For example, for `lang: es`, you would use `es_ES.UTF-8`:

``` yaml
      - name: Set locale to Spanish
        run: |
          sudo locale-gen es_ES.UTF-8
          sudo update-locale LANG=es_ES.UTF-8
        shell: bash

      - name: Build site
        run: pkgdown::build_site_github_pages(new_process = FALSE, install = FALSE)
        shell: Rscript {0}
        env:
          LANG: es_ES.UTF-8
```
## Translations

Translations are contributed by community members so if your language is not currently available, you could be the one to add it!

To get started, first check the [pkgdown issues](https://github.com/r-lib/pkgdown/issues) to see if anyone has filed an existing issue. If so, the person who filed the issue might make for a great collaborator 😀.

Next, install [potools](https://michaelchirico.github.io/potools/) and [usethis](https://usethis.r-lib.org):

```{r}
#| eval: false
install.packages(c("potools", "usethis"))
```

You'll then need to familiarise yourself with the basics of [translations with potools](https://michaelchirico.github.io/potools/articles/translators.html) and [creating pull requests](https://usethis.r-lib.org/articles/pr-functions.html) with usethis.

If you don't already know it, you'll need to look up the ISO 639-1 abbreviation for [your language](https://en.wikipedia.org/wiki/List_of_ISO_639_language_codes). In the examples below, I'll pretend I'm providing translations for Zulu, which has code `zu`.

Start by initialising a pull request:

```{r}
#| eval: false
usethis::pr_init("translation-zu")
```

Then create the translation file by running `potools::po_create("zu")`, open `po/R-zu.po`, and starting filling in the translations.

If you have access to Claude or similar, you can try prepopulating the translations, with a prompt like this:

```
You are an experienced translator. Create new a new translation for Hindi in po/R-hi.po. These are GNU gettext translations for pkgdown, a tool used to generate websites for R packages.

Translation advice:

- For languages that use genedered nouns, do your best to generate inclusive translations, e.g. "Beitragende:r" or "auteur·rice".
- If you need to change the order of substititions (e.g. %s) from English version, use %1$s to identify the first substituion %2$s, etc. Only use this form when the translated word order is different.
- You should choose to be succinct and contextually-correct even when there is not a precise translation.
- Use the "Last-Translator" field to document the tool (e.g. "Positron Assistant") and language model used for the translation (e.g. "Claude Sonnet 4.5")
```

> You are an R developer who is fluent in English and Zulu. You love to do careful, high quality translations in your spare time. Complete the following po file for the R pkgdown package (which creates websites for R packages) by supplying translations for Zulu. Use inclusive gender forms where possible.

Then copy and paste the complete contents of the `.po` file .

You can check your work by adding `lang: zu` to your `_pkgdown.yml` then running:

```{r}
#| eval: false
potools::po_compile()
devtools::load_all()
build_site("~/path/to/your/site")
```

Once you're happy with your work, make sure to compile the changes:

```{r}
#| eval: false
potools::po_compile()
```

Then commit your changes to Git and submit your pull request for review:

```{r}
#| eval: false
usethis::pr_push()
```

---
title: "Accessibility"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Accessibility}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

pkgdown automates as many accessibility details as possible, so that your package website is readable by as many people as possible. This vignette describes the additional details that can't be automated away and you need to be aware of.

```{r setup}
library(pkgdown)
```

## Theming

* If you adjust any colours from the default theme (including the syntax highlighting theme), you should double check that the contrast between the background and foreground doesn't make any text difficult to read. A good place to start is running a representative page of your site through <https://wave.webaim.org>.

* The default colour of the development version label makes a slightly too low contrast against the pale grey background of the navbar. This colour comes from the bootstrap "danger" colour, so you can fix it by overriding that variable in your `_pkgdown.yml`:

    ```yaml
    template:
      bootstrap: 5
      bslib:
        danger: "#A6081A"
    ```

* If you use custom navbar entries that only display an icon, make sure to also use the `aria-label` field to provide an accessible label that describes the icon.

    ```yaml
    cran:
      icon: fab fa-r-project
      href: https://cloud.r-project.org/package=pkgdown
      aria-label: View on CRAN
    ```

## Images

To make your site fully accessible, the place where you are likely to need to do the most work is adding alternative text to any images that you create. Unfortunately, there's currently no way to do this for plots you generate in examples, but you can and should add alternative text to plots in vignettes using the `fig.alt` chunk option:

````{verbatim}
```{r}
#| fig.alt: >
#|   Histogram of time between eruptions for Old Faithful. 
#|   It is a bimodal distribution with peaks at 50-55 and 
#|   80-90 minutes.
hist(faithful$waiting)
```
````

If you forget to add alt text to your vignettes, pkgdown will automatically remind you.

---
title: "Customise your site"
output: rmarkdown::html_vignette
description: >
  Learn how to change the look and feel of your pkgdown site.
vignette: >
  %\VignetteIndexEntry{Customise your site}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

This vignette teaches you how to customise the style/design of your pkgdown site.
We'll start by discussing two techniques that only require tweaks to your `_pkgdown.yml`: theming (colours and fonts) and layout (content of the navbar, sidebar, footer, ...).
We'll then discuss how to add additional HTML and other files.
Next, we'll discuss how to give multiple sites the same style using a package, then finish up with some workflow advice.

In terms of your `_pkgdown.yml`, this vignette focusses on the most important fields nested under `template` and `navbar`. To learn more about customising other aspects of the site, see the documentation for the indiviudal functions like `build_reference()`, `build_articles()`,  `build_home()`, `build_redirects()`, and `init_site()`. To learn about less important fields nested under `template`, see `build_site()`.

```{r}
#| label: setup
library(pkgdown)
```

## Getting started

Most theming features work only with Bootstrap 5, so first update your site by adding the following lines to your `_pkgdown.yml`:

``` yaml
template:
  bootstrap: 5
```

Overall, the site should look pretty similar, but you will notice a number of small improvements.
Most importantly, the default font is much bigger, making it considerably easier to read.
Upgrading to Bootstrap 5 has a low chance of breaking your site unless you were using your [own pkgdown templates](#template-packages) or custom CSS.

## Theming

There are two ways to change the visual style of your site from `_pkgdown.yml`: using a pre-packaged bootswatch theme or customising theme variables with [bslib](https://rstudio.github.io/bslib/).
The following sections show you how.

Please note that pkgdown's default theme has been carefully optimised to be accessible, so if you make changes, make sure to also read `vignette("accessibility")` to learn about potential accessibility pitfalls.

### Light switch {#light-switch}

You can provide a "light switch" to allow your users to switch between dark and light themes by setting the `light-switch` template option to true:

```yaml
template:
  light-switch: true
```

This will add a `lightswitch` component to the navbar, which by default appears at the far right. This allows the user to select light mode, dark mode, or auto mode (which follows the system setting). The modes are applied using Bootstrap 5.3's [colours modes](https://getbootstrap.com/docs/5.3/customize/color-modes/) so are not separate themes, but a thin layer of colour customisation applied via CSS.

### Bootswatch themes

The easiest way to change the entire appearance of your website is to use a [Bootswatch theme](https://bootswatch.com):

``` yaml
template:
  bootstrap: 5
  bootswatch: materia
```

(Themes are unlikely to work with the light switch, but you can try it and see.)

Changing the bootswatch theme affects both the HTML (via the navbar, more on that below) and the CSS, so you'll need to re-build your complete site with `build_site()` to fully appreciate the changes.
While you're experimenting, you can speed things up by just rebuilding the home page and the CSS by running `build_home_index()`; `init_site()` (and then refreshing the browser).

Bootswatch templates with tall navbars (e.g. lux, pulse) also require that you set the `pkgdown-nav-height` bslib variable.
Because Bootswatch themes are provided by the [bslib](https://rstudio.github.io/bslib/) R package, you can also nest the `bootswatch` field under the `bslib` field.

``` yaml
template:
  bootstrap: 5
  bslib:
    bootswatch: lux
    pkgdown-nav-height: 100px
```

You can find the correct height by running `$(".navbar").outerHeight()` in the [javascript console](https://firefox-source-docs.mozilla.org/devtools-user/web_console/index.html).

### bslib variables

Instead of picking a complete theme, you can tweak fonts and colours individually using bslib variables.
[bslib](https://rstudio.github.io/bslib/) is an R package that wraps sass, the tool that Boostrap uses to produce CSS from a special language called [scss](https://sass-lang.com).
The primary advantage of scss over CSS is that it's more programmable, so you can have a few key bslib variables that affect appearance of many HTML elements.

There are three key variables that affect the colour:

-   `bg` (background) determines the page background.
-   `fg` (foreground) determines the text colour. `bg` and `fg` are mixed to yield `gray-100`, `gray-200`, ..., `grey-900`, which are used to style other elements to match the overall colour scheme.
-   `primary` sets the link colour and the (translucent) hover colour in the navbar and sidebar.

``` yaml
template:
  bootstrap: 5
  bslib:
    bg: "#202123"
    fg: "#B8BCC2"
    primary: "#306cc9"
```

You can customise other components by setting more specific bslib variables, taking advantage of inheritance where possible.
For example, `table-border-color` defaults to `border-color` which defaults to `gray-300`.
If you want to change the colour of all borders, you can set `border-color` and if you just want to change the colour of table borders, you can set `table-border-color`.
You can find a full list of variables in the [bslib docs](https://rstudio.github.io/bslib/articles/bs5-variables/index.html).

If you're using the light switch, [many colours](https://getbootstrap.com/docs/5.3/customize/color-modes/#sass-variables) are available for customisation specifically for the dark theme.

Theming with bslib is powered by `bslib::bs_theme()` and the `bslib` field is a direct translation of the arguments to that function.
As a result, you can fully specify a bslib theme using the `template.bslib` field, making it easy to share YAML with the `output.html_document.theme` field [of an R Markdown document](https://rstudio.github.io/bslib/articles/theming/index.html).

``` yaml
template:
  bslib:
    version: 5
    bg: "#202123"
    fg: "#B8BCC2"
    primary: "#306cc9"
```

While iterating on colours and other variables you only need to rerun `init_site()` and refresh your browser to see the changes.

### Fonts

You can also override the default fonts used for the majority of the text (`base_font`), for headings (`heading_font`) and for code (`code_font`).
The easiest way is to supply the name of a [Google font](https://fonts.google.com) with the following syntax:

``` yaml
template:
  bootstrap: 5
  bslib:
    base_font: {google: "Roboto"}
    heading_font: {google: "Roboto Slab"}
    code_font: {google: "JetBrains Mono"}
```

If you want to use a non-Google font, you'll need to do a bit more work. There are two steps: you need to first configure the font with CSS and then use it in your `_pkgdown.yml`. There are two ways you might get the CSS:

* As a block of CSS which you should put in `pkgdown/extra.scss` or `pkgdown/extra.css`. The CSS will look something like this:

    ```css
    @font-face {
      font-family: "proxima-nova";
      src: 
        local("Proxima Nova Regular"), 
        local("ProximaNova-Regular"),
        url("https://example.com/ProximaNova-Regular.eot?#iefix") format("embedded-opentype"),
        url("https://example.com/fonts/proxima/ProximaNova-Regular.woff2") format("woff2"),
        url("https://example.com/fonts/proxima/ProximaNova-Regular.woff") format("woff"),
        url("https://example.com/fonts/proxima/ProximaNova-Regular.ttf") format("truetype");
      font-weight: normal;
      font-style: normal;
      font-display: fallback;
    }
    ```

* As a link to a style file, which you'll need to add to the `<head>` using this syntax:

    ```yaml
    template:
    includes:
      in_header: <link rel="stylesheet" type="text/css" href="https://..." />
    ```

Then in `_pkgdown.yml` you can use the name of the font you just specified:

```yaml
template:
  bslib:
    base_font: proxima-nova
```

Depending on where the font is from (and if you purchased it), you may need to take additional steps to ensure that it can only be used from your site, and/or make sure that it can still be used when you're previewing locally. If you're having problems getting a custom font to work, looking for errors in the [browser developer console](https://developer.mozilla.org/en-US/docs/Glossary/Developer_Tools) is a good place to start.

When iterating on fonts, you'll need to run `build_home_index(); init_site()` then refresh your browser to see the update.


### Syntax highlighting

The colours used for syntax highlighting in code blocks are controlled by the `theme` setting:

``` yaml
template:
  bootstrap: 5
  theme: breeze-light
```

You can choose from any of the following options: `r paste0(pkgdown:::highlight_styles(), collapse = ", ")`.

Bootswatch themes with a dark background (e.g. cyborg, darkly, solar) will need a dark syntax highlighting `theme`, e.g. `arrow-dark`:

``` yaml
template:
  bootstrap: 5
  bootswatch: cyborg
  theme: arrow-dark
```

If you're using the light switch, you will want to provide a `theme` and a `theme-dark`:

```yaml
template:
  light-switch: true
  theme: gruvbox-light
  theme-dark: gruvbox-dark
```

The foreground and background colours used for inline code are controlled by `code-color` and `code-bg` bslib variables.
If you want inline code to match code blocks, you'll need to override the variables yourself, e.g.:

``` yaml
template: 
  bootstrap: 5
  theme: arrow-dark  
  bslib:
    code-bg: "#2b2b2b"
```

### Math rendering

By default, pkgdown will render math using [mathml](https://w3c.github.io/mathml/). mathml is the official standard for rendering math on the web, and requires no additional javascript or css dependencies. However, browser support for complex math is not always that good, so if you are including complex equations in your documentation, you may want to switch to either [`katex`](https://katex.org), which we recommend for display of complex math expressions, or [`mathjax`](https://www.mathjax.org) by using the `template.math-rendering` field:

```yaml
template:
  math-rendering: katex
```

On Linux the default symbol font is _Standard Symbols PS_. Unfortunately, this font does not render plotmath expressions correctly under pkgdown's default graphics device `ragg::png`. There are several simple solutions to this:

* Before building your site, use the systemfonts package to change the symbol font to one which does render plotmath expressions correctly under ragg such as _Liberation Sans_, e.g.: 
  
  ```r
  systemfonts::register_variant("symbol", "Liberation Sans")
  ```

* Configure pkgdown to use R's `grDevices::png` device by specifying the following in `_pkgdown.yml`:

  ```yaml
  figures:
    dev: grDevices::png
  ```

### Navbar style

When [light-switch](#light-switch) is disabled, the primary navbar colours are determined by HTML classes, not CSS, and can be customized using the `navbar` fields `bg` and `type` which control the background and foreground colours respectively.
Typically `bg` will be one of `light`, `dark`, or `primary`:

``` yaml
navbar:
  bg: primary
```

You generally don't need to set `bg` if you use a bootswatch theme, as pkgdown will pick the `bg` used on the [Bootswatch preview](https://bootswatch.com/).
Similarly, you don't usually need to set `type` because bootstrap will guess it for you.
If the guess is wrong, you can override with `type: light` or `type: dark` depending on whether the background colour is light (so you need dark text) or `type: dark` if the background is dark (so you need light text).
Unfortunately, these are defined relative to the page background, so if you have a dark site you'll need to flip `light` and `dark` (a little experimentation should quickly determine what looks best).

When [light-switch](#light-switch) is enabled, navbar background colours can be customized via Sass variables provided to `template.bslib`.
Use `$navbar-bg` to set a constant navbar colour in both light and dark mode, or `$navbar-light-bg` and `$navbar-dark-bg` to choose separate colours.

```yaml
template:
  bslib:
    # navbar-bg: "#306CC9"
    navbar-light-bg: "#9CBAE7"
    navbar-dark-bg: "#183663"
```

Because the navbar is styled with HTML, you'll need to `build_home_index(); init_site()` to see the effect of changing this parameter.

## Layout {#layout}

You can customise the contents of the navbar, footer, using the `navbar` and `footer` fields.
See `?build_home` for how to customise the sidebar on the homepage.
They all use a similar structure that separately defines the overall `structure` and the individual `components`.

### Navbar {#navbar-heading}

You can customise the navigation bar that appears at the top of the page with the `navbar` field.
It's made up of two pieces: `structure`, which defines the overall layout, and `components`, which defines what each piece looks like.
This organisation makes it easy to mix and match pkgdown defaults with your own customisations.

This is the default structure:

``` yaml
navbar:
  structure:
    left:  [intro, reference, articles, tutorials, news]
    right: [search, github, lightswitch]
```

It makes use of the the following built-in components:

-   `intro`: "Get Started", which links to a vignette or article with the same name as the package[^dots].
-   `reference`: if there are any `.Rd` files.
-   `articles`: if there are any vignettes or articles.
-   `tutorials`: if there any tutorials.
-   `news`: if `NEWS.md` exists.
-   `search`: the search box (see `?build_search` for more details).
-   `github`: a link to the source repository (with an icon), if it can be automatically determined from the `DESCRIPTION`.
-   `lightswitch`; a ["light switch"](#light-switch) to select light mode, dark mode, or auto mode.

Note that customising `navbar` like this comes with a downside: if pkgdown later changes the defaults, you'll have to update your `_pkgdown.yml`.

[^dots]: Note that dots (`.`) in the package name need to be replaced by hyphens (`-`) in the vignette filename to be recognized as the intro. That means for a
    package `foo.bar` the intro needs to be named `foo-bar.Rmd`.

You can use the `structure` field to reorganise the navbar without changing the default contents:

``` yaml
navbar:
  structure:
    left:  [search]
    right: [reference, articles]
```

You can use `components` to override the default content.
For example, this yaml provides a custom articles menu:

``` yaml
navbar:
 components:
   articles:
    text: Articles
    menu:
    - text: Category A
    - text: Title A1
      href: articles/a1.html
    - text: Title A2
      href: articles/a2.html
    - text: -------
    - text: "Category B"
    - text: Article B1
      href: articles/b1.html
```

Components uses the same syntax as [RMarkdown menus](https://bookdown.org/yihui/rmarkdown/rmarkdown-site.html#site-navigation).
The elements of `menu` can be:

-   Linked text (`text`, `href`, and optional `target`, `class`, and `id`.). 

-   A linked icon (`icon`, `aria-label`, `href`, and an optional `target`). 
    You can find a list of available icons at [fontawesome](https://fontawesome.com/icons?d=gallery). Provide a text description of the icon in the `aria-label` field
    for screenreader users.

-   A heading (just `text`).

-   A separator (`text: ——–`).

To add a new component to the navbar, you need to modify both `structure` and `components`.
For example, the following yaml adds a new "twitter" component that appears to the left of the github icon.

``` yaml
navbar:
  structure:
    right: [search, twitter, github, lightswitch]
  components:
    twitter:
      icon: fa-twitter
      href: http://twitter.com/hadleywickham
      aria-label: Twitter
```

Finally, you can add arbitrary HTML to three locations in the navbar:

```yaml
template:
  includes:
    before_title: <!-- inserted before the package title in the header ->
    before_navbar: <!-- inserted before the navbar links -->
    after_navbar: <!-- inserted after the navbar links -->
```

These inclusions will appear on all screen sizes, and will not be collapsed into the the navbar drop down.

You can also customise the colour scheme of the navbar by using the `type` and `bg` parameters. See above for details.

### Footer

You can customise the footer with the `footer` field.
It's made up of two pieces: `structure`, which defines the overall layout, and `components`, which defines what each piece looks like.
This organisation makes it easy to mix and match the pkgdown defaults with your own customisations.

This is the default structure:

``` yaml
footer:
  structure: 
    left: developed_by
    right: built_with
```

Which uses two of the three built-in components:

-   `developed_by`: a sentence describing the main authors of the package. (See `?build_home` if you want to tweak *which* authors appear in the footer.)
-   `built_with`: a sentence advertising pkgdown.
-   `package`: the name of the package.

You can override these defaults with the `footer` field.
The example below puts the author's information on the right along with a legal disclaimer, and puts the pkgdown link on the left.

``` yaml
footer:
  structure: 
    left: pkgdown
    right: [developed_by, legal]
  components:
    legal: Provided without **any warranty**.
```

Each side is pasted into a single string (separated by `" "`) and then converted from markdown to HTML.

## Additional HTML and files

If you need to include additional HTML, you can add it in the following locations:

``` yaml
template:
  includes:
    in_header: <!-- inserted at the end of the head -->
    before_body: <!-- inserted at the beginning of the body -->
    after_body: <!-- inserted at the end of the body -->
    before_title: <!-- inserted before the package title in the header ->
    before_navbar: <!-- inserted before the navbar links -->
    after_navbar: <!-- inserted after the navbar links -->
```

You can include additional files by putting them in the right place:

-   `pkgdown/extra.css` and `pkgdown/extra.js` will be copied to the rendered site and linked from `<head>` (after the pkgdown defaults).

-   `pkgdown/extra.scss` will be added to the scss ruleset used to generate the site CSS.

-   Any files in `pkgdown/assets` will be copied to the website root directory.

-   For expert users: template files in `pkgdown/templates` will override layout templates provided by pkgdown or [template packages](#template-packages).

Use `init_site()` to update your rendered website after making changes to these files.

## Template packages {#template-packages}

To share a pkgdown style across several packages, the best workflow is to create... a package!
It can contain any of the following:

-   A configuration file in `inst/pkgdown/_pkgdown.yml`. This can be used to set (e.g.) author definitions, Bootstrap version and variables, the sidebar, footer, navbar, etc.
-   Templates in `inst/pkgdown/templates/` will override the default templates.
-   Assets in `inst/pkgdown/assets/` will be copied in to the destination directory.
    (Note these files are only copied; you'll need to reference them in your stylesheet or elsewhere in order for them to be actually used.)
-   `inst/pkgdown/extra.scss` will be added to the bslib ruleset.
    (Note that `extra.css` is not supported in templates.)

The pkgdown defaults will be overriden by these template files, which are in turn overridden by package specific settings.

Once you have created your template package `theverybest`, you need to set it as your site's theme:

``` yaml
template:
  package: theverybest
```

You then also need to make sure it's available when your site is built. Typically, you won't want to publish this package to CRAN, but you will want to publish to GitHub. Once you've done that, and assuming you're using the [usethis workflow](https://usethis.r-lib.org/reference/use_pkgdown.html), add the following line to your `DESCRIPTION`:

```yaml
Config/Needs/website: myorg/theverybest
```

This will ensure that the GitHub action will automatically install it from GitHub when building your pkgdown site.

To get some sense of how a theming package works, you can look at:

-   [tidytemplate](https://tidytemplate.tidyverse.org/) used for tidyverse and tidymodels packages;
-   [quillt](https://pkgs.rstudio.com/quillt) used for R Markdown packages;
-   [rotemplate](https://github.com/ropensci-org/rotemplate) used for rOpenSci packages.

But please note that these templates aren't suitable for use with your own package as they're all designed to give a common visual identity to a specific family of packages.

### Porting a template package

If you are updating a template package that works with pkgdown 1.0.0, create directories `inst/pkgdown/BS5/templates` and `inst/pkgdown/BS5/assets` (if you don't have any templates/assets make sure to a add dummy file to ensure that git tracks them).
The `templates` and `assets` directories directly under `inst/pkgdown` will be used by pkgdown 1.0.0 and by pkgdown 2.0.0 if `boostrap: 3`.
The directories under `inst/pkgdown/BS5/` will be used for pkgdown 2.0.0 with `boostrap: 5`.
This lets your package support both versions of Bootstrap and pkgdown.

## PR previews

Lastly, it might be useful for you to get a preview of the website in internal pull requests.
For that, you could use Netlify and GitHub Actions (or apply a similar logic to your toolset):

-   Create a new Netlify website (either from scratch by dragging and dropping a simple index.html, or by creating a site from a GitHub repository and then unlinking that repository); from the site settings get its ID to be saved as `NETLIFY_SITE_ID` in your repo secrets; from your account developer settings get a token to be saved as `NETLIFY_AUTH_TOKEN` in your repo secrets.
-   Starting from the standard pkgdown workflow `usethis::use_github_action("pkgdown")`, add some logic to build the site and deploy it to Netlify for pull requests from inside the repository, not pull requests from forks. [Example workflow](https://github.com/r-lib/pkgdown/blob/master/.github/workflows/pkgdown.yaml).

## Conclusion

In this vignette we explained how to change the theming and layout of pkgdown websites.
Further work to improve user experience will involve:

-   Working on the article (`?build_articles`) and reference indexes (`?build_reference`).
-   Writing a compelling README that explains why your package is so cool/useful/fun.
-   Improving the contents of the individual articles and reference topics 😉.

---
title: "Introduction to pkgdown"
description: >
  Learn how to get started with the basics of pkgdown.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to pkgdown}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

The goal of pkgdown is to make it easy to make an elegant and useful package website with a minimum of work.
You can get a basic website up and running in just a couple of minutes. If you're using GitHub, we recommend setting up pkgdown and GitHub actions to automatically build and publish your site:

```{r}
#| eval: false
# Run this once to publish your site regularly
usethis::use_pkgdown_github_pages()
```

If you are not using GitHub, you will have to run `pkgdown::build_site()` manually everytime you want to update the site.

```{r}
#| eval: false
# Run once
# Remove docs/ from gitignore to ensure it is checked into git.
usethis::use_pkgdown()
# Run everytime you want to update your site
pkgdown::build_site()
```


While you'll get a decent website without any additional work, if you want a website that really pops, you'll need to read the rest of this vignette.
It starts by showing you how to configure pkgdown with a `_pkgdown.yml`.
You'll learn about the main components of the site (the home page, reference, articles, and news), and then how to publish and promote your site.

## Metadata

You can override pkgdown's defaults with a YAML file called `_pkgdown.yml`[^1].
The most important field is `url`, which gives the final location of the site:

[^1]: You can also put it in `pkgdown/_pkgdown.yml` if you want to keep the package root clutter-free, or in `inst/_pkgdown.yml` if you want to make it available when your package is installed. You can also use `.yaml` as the extension if desired.

``` yaml
url: https://pkgdown.r-lib.org
```

`url` is used throughout the site to generate absolute urls where they are needed.
`url` is also part of what enables auto-links to your help topics or vignettes from sites external to your package, such as from other pkgdown sites or from Quarto websites.
See `vignette("linking")` for more.

Another important option is `template`, which allows you to control the overall appearance of your site:

``` yaml
template:
  bootstrap: 5
  bootswatch: cerulean
```

You can learn more about controlling the appearance of your site in `vignette("customise")`.

## Accessibility

pkgdown's defaults work to ensure that your site is accessible to as many people as possible. But there are some accessibilty issues that only a human can solve, so make sure to also read `vignette("accessibility")` to learn about them.

## Home page

The contents of the home page are automatically generated from `index.md` or `README.md`.
pkgdown tries to put them in order, so it's possible to have a different display on GitHub and pkgdown by providing both files.
The homepage also includes a sidebar full of useful links (see `?build_home` for how these are generated and how you can customise them).

## Reference

pkgdown creates a function reference in `reference/` that includes one page for each `.Rd` help topic in `man/`.
The translation of individual help topics from Rd to HTML is generally straightforward, but there are a couple of things you should bear in mind:

-   pkgdown does its best to autolink all references to help topics and articles described in `vignette("linking")`.

-   pkgdown executes all examples, inserting the rendered results in the generated HTML files.

By default, pkgdown generates a reference index that is just an alphabetically-ordered list of functions.
The index is much more useful with human curation because functions can be grouped and described in categories.
To override the default, provide a `reference` field in `_pkgdown.yml`.

Each entry in `reference` can take one of three forms:

-   A title, defined by `title` and optional `desc` (description) fields.
-   A subtitle, defined by `subtitle` and optional `desc` (description) fields.
-   A list of topics defined by a `contents` field.

``` yaml
reference:
- title: "Connecting to Spark"
  desc: >
    Functions for installing Spark components and managing
    connections to Spark
  contents: 
  - spark_config
  - spark_connect
  - spark_disconnect
  - spark_install
  - spark_log
- title: "Reading and Writing Data"
  desc: "Functions for reading and writing Spark DataFrames."
  contents:
  - starts_with("spark_read")
  - starts_with("spark_write")
  - matches("saveload")
```

Note the use of `starts_with()` to select all functions with a common prefix.
You can also use `ends_with()` and `matches()`.
See complete details in `?build_reference`, including other topic matching helper functions.

While iterating on the reference index you might want to run `pkgdown::build_reference_index()`.
It just re-builds the index page, making it faster to quickly change `_pkgdown.yml` and see how it affects your site.

## Articles

pkgdown will automatically build all vignettes found in `vignettes/`, translating them to HTML files in `articles/`.
It is recommended to name your intro article with your package name to generate a "Get Started" page automatically.

Due to the way that pkgdown has to integrate R Markdown generated HTML with its own HTML, relatively little control is available over the output format.
You can see the details in `?build_articles`.

If you want to include an [article](https://r-pkgs.org/website.html#non-vignette-articles) on the website but not in the package (e.g., because it's large), you can use `usethis::use_article()` to set it up.

## News

If `NEWS.md` is present, it will be rendered into a single-page changelog based on markdown level headings.
pkgdown assumes your `NEWS.md` is formatted using level one headings (`#`) to specify package name and version number, and level two headings (`##`) to provide topical organization for each release.

``` markdown
# pkgdown 1.1.0

## Bug Fixes

* Lots of them

# pkgdown 1.0.0

* This is the first release of pkgdown.
```

See more suggestions for writing news bullets in the [tidyverse style guide](https://style.tidyverse.org/news.html).

See `?build_news` for more customisation options including how to:

-   Create one page for each major version and related minor versions.
-   Add release announcements to the news navbar drop-down.

## Publishing

If you use GitHub, the easiest way to build and publish your site is via GitHub actions.
Using GitHub actions automatically builds and publishes the site every time you make a change.
The easiest way to set this up is to run `usethis::use_pkgdown_github_pages()`, and if you need to customize the action, see [README.md r-lib/actions](https://github.com/r-lib/actions/tree/v2-branch/examples#build-pkgdown-site).

## Promoting

Once your finalized site is built and published on the web, you should publicize its URL in a few places:

1.  The `URL` field of your package `DESCRIPTION`, alongside a link to its source:

        URL: https://pkgdown.r-lib.org, https://github.com/r-lib/pkgdown

    (`usethis::use_pkgdown_github_pages()` does this for you.)

2.  Your repository description on GitHub.

    (`usethis::use_pkgdown_github_pages()` does this for you.)

3.  On social media (make sure to include `#rstats`).

---
title: quarto vignettes
description: > 
  Learn how quarto vignettes work with pkgdown, including currently supported
  features and known limitations.
vignette: >
  %\VignetteIndexEntry{quarto vignettes}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
knitr:
  opts_chunk: 
    collapse: true
    comment: '#>'
---

pkgdown effectively uses quarto only to generate HTML and then supplies its own CSS and JS. This means that when quarto introduces new features, pkgdown may lag behind in their support. If you're trying out something that doesn't work (and isn't mentioned explicitly below), please [file an issue](https://github.com/r-lib/pkgdown/issues) so we can look into it.

## Operation

pkgdown turns your articles directory into a quarto project by temporarily adding a `_quarto.yml` to your articles. You can also add your own if you want to control options for all quarto articles. If you do so, and you have a mix of `.qmd` and `.Rmd` files, you'll need to include the following yaml so that RMarkdown can continue to handle the .Rmd files:

```yaml
project:
  render: ['*.qmd']
```

### GitHub Actions

The `setup-r-dependencies` action will [automatically](https://github.com/r-lib/actions/tree/v2-branch/setup-r-dependencies#usage) install Quarto in your GitHub Actions if a .qmd file is present in your repository (see the `install-quarto` parameter for more details). 


## Limitations

* Callouts are not currently supported (<https://github.com/quarto-dev/quarto-cli/issues/9963>).

* pkgdown assumes that you're using [quarto vignette style](https://quarto-dev.github.io/quarto-r/articles/hello.html), or more generally an html format with [`minimal: true`](https://quarto.org/docs/output-formats/html-basics.html#minimal-html). Specifically, only HTML vignettes are currently supported.

* You can't customise mermaid styles with quarto mermaid themes. If you want to change the colours, you'll need to provide your own custom CSS as shown in [the quarto docs](https://quarto.org/docs/authoring/diagrams.html#customizing-mermaid).

* pkgdown will pass the `lang` setting on to quarto, but the set of available languages is not perfectly matched. Learn more in <https://quarto.org/docs/authoring/language.html>, including how to supply your own translations.

## Supported features

The following sections demonstrate a bunch of useful quarto features so that we can make sure that they work. 

### Inline formatting

* [Small caps]{.smallcaps}

* Here is a footnote reference[^1]

[^1]: And here is the footnote.

### Code

```{r}
#| fig.alt: A plot of the numbers 1, 2, and 3
1 + 1
2 + 2

plot(1:3)
```

### Figures

::: {#fig-puppies layout-ncol=2}
![A sketch of a pitbull puppy](pitbull.jpg){#fig-pitbull}

![A sketch of a sharpei puppy](shar-pei.jpg){#fig-shar-pei}

Cute puppies
:::

### Equations
$$
\frac{\partial \mathrm C}{ \partial \mathrm t } + \frac{1}{2}\sigma^{2} \mathrm S^{2}
\frac{\partial^{2} \mathrm C}{\partial \mathrm C^2}
  + \mathrm r \mathrm S \frac{\partial \mathrm C}{\partial \mathrm S}\ =
  \mathrm r \mathrm C 
$$ {#eq-black-scholes}


### Cross references

See @fig-puppies for two cute puppies.

Black-Scholes (@eq-black-scholes) is a mathematical model that seeks to explain the behavior of financial derivatives, most commonly options.

## To do

* [ ] Code annotations
* [ ] Tabsets
* [x] Citations
* [x] Task/to do lists
* [x] Figures
* [x] Equations
* [x] Cross-references
* [x] Footnotes
* [x] Callouts

---
title: "How to update a released site"
output: rmarkdown::html_vignette
description: >
  A guide to updating the "released" version of your pkgdown site without
  having to release a new version of your package.
vignette: >
  %\VignetteIndexEntry{How to update a released site}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE,
  purl = FALSE
)
```

This vignette shows you how to update the released version of your site to match the dev version of the site, so the first step is to ensure that the dev site looks the way that you want it.

This site assumes that you're using a recent version of our recommended [pkgdown action](https://github.com/r-lib/actions/blob/HEAD/examples/pkgdown.yaml). If your workflow does not contain `workflow_dispatch`, you will need to update `.github/actions/pkgdown.yaml` by running `use_github_action("pkgdown")`.

## Process

If you're up to speed with the basic idea and just want some code to follow, here it is.
Otherwise, read more below.

### Setup

First, make sure you're in the `main` branch, and you have the latest version:

```{r}
gert::git_branch_checkout("main")
gert::git_pull()
```

Next figure out the released version that you're updating:

```{r}
ver <- desc::desc_get_version()[1, 1:3]
```

You will use this to create and checkout the branch that you'll work in:

```{r}
gert::git_branch_create(
  branch = paste0("pkgdown-v", ver),
  ref = paste0("v", ver)
)
```

This assumes that the `ver` version of your package, e.g. version X.Y.Z, has been marked with the the git tag 'vX.Y.Z'.
This happens automatically if you use `usethis::use_github_release()`.

### Backport changes

Now you need to backport changes from the dev site into this branch.
Run this R code to generate the git code to pull changes for the most common locations:

```{r}
files <- c(
  # overall site config
  "_pkgdown.yml",
  # the workflow that builds the site
  ".github/workflows/pkgdown.yaml",
  # readme and vignettes
  "README.md", "vignettes",
  # logo and favicon
  "man/figures/", "pkgdown/",
  # Author metadata and Config/Needs/Website
  "DESCRIPTION"
)

glue::glue("git checkout main -- {files}")
```

If you backport `DESCRIPTION`, you'll also need undo the change to the `Version`:

```{r}
desc::desc_set_version(ver)
```

Now build the site locally and check that it looks as expected:

```{r}
pkgdown::build_site()
```

Make sure to commit these changes.

### Publish

Now you need to publish the site.
First push your branch to GitHub:

```{r}
usethis:::git_push_first()
```

Then trigger the pkgdown workflow:

1.  Go to your package's GHA page, e.g. with `usethis::browse_github_actions()`.
2.  Select the pkgdown workflow.
3.  Click *Run workflow* and select the branch you just pushed.

If there's no dropdown menu for this, that means your pkgdown workflow config is not current.

## Context

Now that we've sketched **how** to update a released site, we discuss **why** you might need to do this.
What is a released site?
What other kind of pkgdown site could you have?
Why does updating a released site take special effort?

### Automatic development mode

Every pkgdown site has a so-called [*development mode*](https://pkgdown.r-lib.org/reference/build_site.html#development-mode), which can be specified via the `development` field in `_pkgdown.yml`.
If unspecified, the default is `mode: release`, which results in a single pkgdown site.
Despite the name, this single site reflects "the current package state", which could be either a released state or a development state.

For packages with a substantial user base, it is recommended instead to specify `mode: auto` like so:

``` yaml
development:
  mode: auto
```

This directs pkgdown to "generate different sites for the development and released versions of your package."

The readr package demonstrates what happens in automatic development mode:

[readr.tidyverse.org](https://readr.tidyverse.org) documents the released version, i.e. what `install.packages()` will deliver.\
[readr.tidyverse.org/dev/](https://readr.tidyverse.org/dev/) documents the dev version, i.e. what you'd get by installing from GitHub.

In this mode, `pkgdown::build_site()`, consults DESCRIPTION to learn the package's version number.
For a development version number, the rendered site is written to `docs/dev/`.
For a released version number, the site is written to `docs/`.
(There are also other signals to alert users that they are reading documentation for a dev version vs. a released version.)

Automatic development mode is recommended for packages with a broad user base because it maximizes the chance that a user will read web-based documentation which reflects the package version that is locally installed.

### Publishing

Now that we've established the meaning of a released (vs dev) site, we have to consider how the site is built (i.e. how the HTML is generated) and deployed (i.e. how the HTML is published to a website so people can see it.).

We recommend `usethis::use_pkgdown_github_pages()` to do basic pkgdown setup and configure a GitHub Actions (GHA) workflow to automatically render and publish the site to GitHub Pages. This function is basically a shortcut for calling the following functions individually:

* `use_pkgdown()`
* `use_github_pages()`
* `use_github_action("pkgdown")`

It then adds the pkgdown site's URL to `_pkgdown.yml`, the `URL` field of DESCRIPTION, and the GitHub repo.

As a result, the publishing cadence of many pkgdown sites is governed by the workflow maintained at [`r-lib/actions/examples/pkgdown.yaml`](https://github.com/r-lib/actions/blob/v2/examples/pkgdown.yaml).
(Do not confuse `_pkgdown.yml`, which gives instructions to the pkgdown package, with `.github/workflows/pkgdown.yaml`, which gives instructions to GHA.)

Here are some of the most important bits of the GHA workflow config:

``` yaml
on:
  push:
    branches: [main, master]
  pull_request:
    branches: [main, master]
  release:
    types: [published]
  workflow_dispatch:

<snip, snip>

      - name: Build site
        run: Rscript -e 'pkgdown::build_site_github_pages(...)'

      - name: Deploy to GitHub pages 🚀
        if: github.event_name != 'pull_request'
        uses: JamesIves/github-pages-deploy-action@4.1.4

        <snip, snip>
```

Altogether this means that we:

* Build and deploy for pushes to `main`.

* Build, but don't deploy, for pull requests against `main`.
  This reveals any pkgdown errors, but ensures the live site isn't
  updated until the pull request is merged (because the code is
  pushed to `main`).

* Build and deploy when we publish a GitHub release.
  By convention, we assume that a GitHub release coincides with a CRAN release.
  So this is the **primary mechanism for building the released pkgdown site**.

`pkgdown::build_site_github_pages()` consults the version in DESCRIPTION to detect whether it's building from a released version or a dev version.
That determines the `dest_dir`, e.g. `docs/` for released and `docs/dev/` for dev.
For a package in automatic development mode, this means that almost all of your pushes trigger an update to the dev site.
The released site is only updated when you push a state with a non-development version number or when you publish a GitHub release.

So how do you tweak things about the released site *in between* releases?

That brings us to `workflow_dispatch:`.
(Yes that dangling colon is correct.)

The inclusion of `workflow_dispatch` as a trigger means the pkgdown workflow can be [run on demand](https://docs.github.com/en/actions/using-workflows/manually-running-a-workflow), most importantly from the browser.

## Construct a branch for the update

The overall goal is to create a branch that combines some features of the released website (e.g. the released version) and the development version (e.g. improvements to your `_pkgdown.yml`). The easiest way is to start your branch using the latest release tag, then bring in selected changes or files from the development version.

For example, if readr's latest release is 2.1.1:

```
git checkout -b update-pkgdown-2-1-1 v2.1.1
```

And here is the general pattern:

```
git checkout -b NEW-BRANCH-NAME NAME-OF-RELEASE-TAG
```

Now you should port innovations from the development site that you want to apply to your released site.

Files you must update:

-   `.github/workflows/pkgdown.yaml`
-   `_pkgdown.yml`
-   `Config/Needs/website` field of DESCRIPTION (And, probably, only this field! In particular, do not mess with the version number.)

Other likely candidates:

-   `README.Rmd` + `README.md`, e.g., if you've updated badges.
-   Any documentation fixes that **apply to the released version**. This is the only reason to touch anything below `R/` and even then it should only affect roxygen comments. Don't forget to `document()` if you do this!
-   Any new vignettes or articles that apply to the released version.

Here are some tips on backporting specific changes into this branch.
If you are lucky, there are specific commits in your default branch that contain all the necessary changes.
In that case, we can cherry pick such a commit by its SHA:

```
git cherry-pick SHA
```

If that doesn't cover everything for each file you want to update, identify a Git reference (meaning: a SHA, tag, or branch) where the file is in the desired state.
Checkout that specific file path from that specific ref:

```
git checkout main -- path/to/the/file
```

For example, readr recently gained a new vignette that applies to the released version of readr, i.e. it does not document any dev-only features or functions.
We can bring that into the current branch with:

```
git checkout main -- vignettes/column-types.Rmd
```

Commit and push this new branch to GitHub.
`usethis::pr_push()` can be handy for this.
Just don't bother opening a pull request (the branch will still be pushed).

Now we will use the `workflow_dispatch` GHA trigger:

1. Go to the Actions page of your repo, maybe via `usethis::browse_github_actions()`.
1. Click on the `pkgdown` workflow.
1. Click the "Run workflow".
1. In the "Use workflow from" dropdown menu, select the branch you've just made and pushed, then click "Run workflow".

This should kick off a pkgdown build-and-deploy and, specifically, it should cause updates to the **released** site.

You can keep this branch around for a while, in case you didn't get everything right the first time or if more things crop up that you'd like backport to the released site, before your next CRAN release.

## Problem-solving

Another great problem solving technique is to get a bunch of other people's `_pkgdown.yml` files in front of your eyeballs. There are two ways to do this: [GitHub search](https://github.com/search?q=path%3A%22_pkgdown.yml%22+AND+%28org%3Atidyverse+OR+org%3Ar-lib%29&type=code) or [Michael Chirico's `r-ci-samples` repo](https://github.com/MichaelChirico/r-ci-samples/tree/master/pkgdown). For any given `_pkgdown.yml` file, remember that its History and Blame can be helpful for seeing how it has evolved over time.

---
title: "Metadata"
description: >
  Customise metadata and social media cards for pkgdown websites.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Metadata}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

Package authors can customize the metadata used by Twitter and the [Open Graph protocol][ogp] for rich social media cards. In addition to specifying an alternate description for the package and any individual articles, you may also choose the preview image shown and the style of the card used on Twitter.

You can preview and validate the appearance of the social media cards with online tools:

* [Google Rich Results Test][ogp-validator]
* Drafting a tweet to yourself
* Sending yourself a Slack message

## Necessary configuration

Metadata can only be produced correctly if your pkgdown website URL is indicated in its configuration file.

```yaml
url: https://example.com
```

## Site-wide customization

Metadata for the entire pkgdown website can be specified in the site's `_pkgdown.yml` configuration file in the `home` and `template: opengraph` sections:

```yaml
home:
  title: An R package for pool-noodle discovery
  description: Discover and add pool-noodles to your growing collection.

template:
  opengraph:
    image:
      src: man/figures/card.png
      alt: "Pool noodles configured to form the word poolnoodlr"
    twitter:
      creator: "@hadleywickham"
      site: "@rstudio"
      card: summary_large_image
```

The `home: title` and `home: description` fields override the `Title` and `Description` fields in the package `DESCRIPTION`. It's good practice to set these fields to make your package documentation easier to find via search, rather than sticking with the title and description needed by CRAN.

The `template: opengraph` section allows you to further customize the social media card.

*   `image`: By default, pkgdown uses the package's logo for the card image
    (if one exists). Use `image` to specify an alternative image for the social
    media cards of pages in your pkgdown site.

    * `src`: A fully qualified URL to a media card image e.g. `src: https://avatars.githubusercontent.com/u/22618716?v=4`;
      or a relative path to an image in the rendered website e.g. `src: articles/test/image.jpg`;
      or a relative path to an image stored in `man/figures` in the package e.g. `src: man/figures/cards.png`.
      The `src` field is required if `image` is specified.

    * `alt`: Alternative text describing the image for screen readers and
      other situations where your social media card image cannot be displayed.

*  `twitter`: You can specify the Twitter accounts associated with your package
    and the [style of social media card][twitter-card] that Twitter will
    display.

    * `creator`: Typically, the Twitter handle of the author of the package or
      article.

    * `site`: The Twitter handle of the organization affiliated with the
      package author or sponsoring the package development.

    * If only one of either `creator` or `site` is included, the provided value will
      be used for both fields.

    * `card`: The [style of social media card][twitter-card] that Twitter will
      display. For pkgdown sites, the most relevant options are
      `summary_large_image`, featuring a large image over the page title and
      description, or `summary`, featuring a small square image inline and to
      the left of the page title and description.

## Article metadata

Articles and vignettes rendered as articles by pkgdown can have individually customized metadata and social media cards.

```yaml
title: "Introduction to poolnoodlr"
description: "A brief introduction to pool noodles in R."
author: "Mara Averick"
opengraph:
  image:
    src: "https://example.com/pkg/batpig.png"
  twitter:
    card: summary
    creator: "@dataandme"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to poolnoodlr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
```

Use the `title`, `description`, and `author` fields to specify the title, description, and (optional) author of the vignette or article.

* The `title` field is used as the title of your article in your pkgdown site
  and should always be included.

* Both `title` and `description` are used by pkgdown for the page's social
  media card. If `description` is not included in the article's YAML front
  matter, then the name of the package is used instead. The `description`
  is also displayed on the articles index.

- The `author` field is only used in the text of the vignette or article.
  How the author name is displayed depends on the `output` format.

In articles, the `opengraph` section works in the same way as the site-wide `template: opengraph` settings, but is only applied to the article or vignette. This allows you to specify social media card preview images for individual articles, or to associate an article with a particular Twitter account. If not specified, the `opengraph` settings from the site-wide configuration are used.

[ogp]: https://ogp.me/
[ogp-validator]: https://search.google.com/test/rich-results
[twitter-card]: https://developer.x.com/en/docs/x-for-websites/cards/overview/abouts-cards

---
title: "Search"
description: >
  Learn how to set up search for your pkgdown site.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Search}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

Depending on the Bootstrap version used for your pkgdown website, search will be either built-in (Bootstrap 5 and up) or will need some more configuration of an external service (Bootstrap 3).
We'd recommend switching to Bootstrap 5, see `vignette("customise")`.

## Bootstrap 5: built-in search

With BS5, search is built-in so building your pkgdown website will automatically build a search index that's available in navbar (using [fuse.js](https://fusejs.io/)).
Currently the only available customisation is excluding some paths from the search index:

``` yaml
search:
  exclude: ['news/index.html']
```

Note that search requires a "real" server, not a local `file://` based preview, so search won't work with the default preview.
To test search with a local preview, you'll need to use `servr::httw("docs")` or similar.

## Bootstrap 3: Algolia

pkgdown websites can integrate search capability using [DocSearch](https://docsearch.algolia.com/) from Algolia.
DocSearch is a powerful search engine that is free for documentation websites.
There are only two steps needed to enable DocSearch on a pkgdown website.

### Indexing

Once you have published your pkgdown website, submit the [pkgdown site URL to Docsearch](https://docsearch.algolia.com/).
Docsearch will contact you via e-mail to confirm you are the website owner.

Docsearch will set up a crawler configuration that indexes your site every 24 hours.
pkgdown builds a suggested Docsearch crawler configuration in `docsearch.json` and you should point the Docsearch team to this configuration as a starting point.
If you want to optimize your search, Docsearch will accept pull requests to the configuration that incorporate [additional options](https://github.com/algolia/docsearch-configs#introduction) to fine tune the scraping.

### Configuration

The Docsearch team will e-mail you some JavaScript to integrate into your website.

``` js
<script type="text/javascript">
  docsearch({ 
    apiKey: 'API_KEY',  // a long hex string
    indexName: 'INDEX_NAME', 
    inputSelector: '### REPLACE ME ####', 
    debug: false // Set debug to true if you want to inspect the dropdown 
}); 
```

Put the value of the `apiKey` and `indexName` parameters into your site `_pkgdown.yml` under `template: params`:

``` yaml
template:
  params:
    docsearch:
      api_key: API_KEY
      index_name: INDEX_NAME
```

You also need to add a `url:` field to `_pkgdown.yml` that specifies the location of your documentation on the web.
For pkgdown, the URL field is:

``` yaml
url: https://pkgdown.r-lib.org
```

If you are building your own custom Docsearch index, you can also include your Docsearch `app_id` in `_pkgdown.yml`.

See the [pkgdown configuration](https://github.com/r-lib/pkgdown/blob/main/pkgdown/_pkgdown.yml#L7-L11) for a functional search configuration.

Once this configuration is complete, you should find a search bar after re-building your site.
After search is enabled, pressing `shift` + `/` (i.e., "?") will move the focus to the search bar.

---
title: "Metadata"
description: >
  Customise metadata and social media cards for pkgdown websites.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Metadata}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

Package authors can customize the metadata used by Twitter and the [Open Graph protocol][ogp] for rich social media cards. In addition to specifying an alternate description for the package and any individual articles, you may also choose the preview image shown and the style of the card used on Twitter.

You can preview and validate the appearance of the social media cards with online tools:

* [Google Rich Results Test][ogp-validator];
* Drafting a tweet to yourself;
* Sending yourself a Slack message.

## Necessary configuration

Metadata can only be produced correctly if your pkgdown website URL is indicated in its configuration file.

```yaml
url: https://example.com
```

## Site-wide customization

Metadata for the entire pkgdown website can be specified in the site's `_pkgdown.yml` configuration file in the `home` and `template: opengraph` sections:

```yaml
home:
  title: An R package for pool-noodle discovery
  description: Discover and add pool-noodles to your growing collection.
  
template:
  opengraph:
    image:
      src: man/figures/card.png
      alt: "Pool noodles configured to form the word poolnoodlr"
    twitter:
      creator: "@hadleywickham"
      site: "@rstudio"
      card: summary_large_image
```

The `home: title` and `home: description` fields override the `Title` and `Description` fields in the package `DESCRIPTION`. It's good practice to set these fields to make your package documentation easier to find via search, rather than sticking with the title and description needed by CRAN.

The `template: opengraph` section allows you to further customize the social media card.

*   `image`: By default, pkgdown uses the package's logo for the card image 
    (if one exists). Use `image` to specify an alternative image for the social 
    media cards of pages in your pkgdown site.

    * `src`: A fully qualified URL to a media card image e.g. `src: https://avatars.githubusercontent.com/u/22618716?v=4`; 
      or a relative path to an image in the rendered website e.g. `src: articles/test/image.jpg`;
      or a relative path to an image stored in `man/figures` in the package e.g. `src: man/figures/cards.png`.
      The `src` field is required if `image` is specified.
    
    * `alt`: Alternative text describing the image for screen readers and 
      other situations where your social media card image cannot be displayed.

*  `twitter`: You can specify the Twitter accounts associated with your package
    and the [style of social media card][twitter-card] that Twitter will 
    display.
    
    * `creator`: Typically, the Twitter handle of the author of the package or 
      article.
    
    * `site`: The Twitter handle of the organization affiliated with the
      package author or sponsoring the package development. 
      
    * If only one of `creator` or `site` is included, the provided value will 
      be used for both fields.
    
    * `card`: The [style of social media card][twitter-card] that Twitter will 
      display. For pkgdown sites, the most relevant options are 
      `summary_large_image`, featuring a large image over the page title and
      description, or `summary`, featuring a small square image inline and to
      the left of the page title and description.

## Article metadata

Articles and vignettes rendered as articles by pkgdown can have individually customized metadata and social media cards.

```yaml
title: "Introduction to poolnoodlr"
description: "A brief introduction to pool noodles in R."
author: "Mara Averick"
opengraph:
  image: 
    src: "https://example.com/pkg/batpig.png"
  twitter:
    card: summary
    creator: "@dataandme"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to poolnoodlr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
```

Use the `title`, `description`, and `author` fields to specify the title, description, and (optional) author of the vignette or article.

* The `title` field is used as the title of your article in your pkgdown site
  and should always be included.

* Both `title` and `description` are used by pkgdown for the page's social 
  media card. If `description` is not included in the article's YAML front 
  matter, then the name of the package is used instead. The `description`
  is also displayed on the articles index.

- The `author` field is only used in the text of the vignette or article.
  How the author name is displayed depends on the `output` format.

In articles, the `opengraph` section works in the same way as the site-wide `template: opengraph` settings, but is only applied to the article or vignette. This allows you to specify social media card preview images for individual articles, or to associate an article with a particular Twitter account. If not specified, the `opengraph` settings from the site-wide configuration are used.

[ogp]: https://ogp.me/
[ogp-validator]: https://search.google.com/test/rich-results
[twitter-card]: https://developer.twitter.com/en/docs/twitter-for-websites/cards/overview/abouts-cards

---
title: quarto vignettes
description: > 
  Learn how quarto vignettes work with pkgdown, including currently supported
  features and known limitations.
vignette: >
  %\VignetteIndexEntry{quarto vignettes}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
knitr:
  opts_chunk: 
    collapse: true
    comment: '#>'
---

pkgdown effectively uses quarto only to generate HTML and then supplies its own CSS and JS. This means that when quarto introduces new features, pkgdown may lag behind in their support. If you're trying out something that doesn't work (and isn't mentioned explicitly below), please [file an issue](https://github.com/r-lib/pkgdown/issues) so we can look into it.

## Operation

pkgdown turns your articles directory into a quarto project by temporarily adding a `_quarto.yml` to your articles. You can also add your own if you want to control options for all quarto articles. If you do so, and you have a mix of `.qmd` and `.Rmd` files, you'll need to include the following yaml so that RMarkdown can continue to handle the .Rmd files:

```yaml
project:
  render: ['*.qmd']
```

### GitHub Actions

The `setup-r-dependencies` action will [automatically](https://github.com/r-lib/actions/tree/v2-branch/setup-r-dependencies#usage) install Quarto in your GitHub Actions if a .qmd file is present in your repository (see the `install-quarto` parameter for more details). 


## Limitations

* Callouts are not currently supported (<https://github.com/quarto-dev/quarto-cli/issues/9963>).

* pkgdown assumes that you're using [quarto vignette style](https://quarto-dev.github.io/quarto-r/articles/hello.html), or more generally an html format with [`minimal: true`](https://quarto.org/docs/output-formats/html-basics.html#minimal-html). Specifically, only HTML vignettes are currently supported.

* You can't customise mermaid styles with quarto mermaid themes. If you want to change the colours, you'll need to provide your own custom CSS as shown in [the quarto docs](https://quarto.org/docs/authoring/diagrams.html#customizing-mermaid).

* pkgdown will pass the `lang` setting on to quarto, but the set of available language is not perfectly matched. Learn more in <https://quarto.org/docs/authoring/language.html>, including how to supply your own translations.

## Supported features

The following sections demonstrate a bunch of useful quarto features so that we can make sure that they work. 

### Inline formatting

* [Small caps]{.smallcaps}

* Here is a footnote reference[^1]

[^1]: And here is the footnote.

### Code

```{r}
#| fig.alt: A plot of the numbers 1, 2, and 3
1 + 1
2 + 2

plot(1:3)
```

### Figures

::: {#fig-puppies layout-ncol=2}
![A sketch of a pitbull puppy](pitbull.jpg){#fig-pitbull}

![A sketch of a sharpei puppy](shar-pei.jpg){#fig-shar-pei}

Cute puppies
:::

### Equations
$$
\frac{\partial \mathrm C}{ \partial \mathrm t } + \frac{1}{2}\sigma^{2} \mathrm S^{2}
\frac{\partial^{2} \mathrm C}{\partial \mathrm C^2}
  + \mathrm r \mathrm S \frac{\partial \mathrm C}{\partial \mathrm S}\ =
  \mathrm r \mathrm C 
$$ {#eq-black-scholes}


### Cross references

See @fig-puppies for two cute puppies.

Black-Scholes (@eq-black-scholes) is a mathematical model that seeks to explain the behavior of financial derivatives, most commonly options.

## To do

* [ ] Code annotations
* [ ] Tabsets
* [x] Citations
* [x] Task/to do lists
* [x] Figures
* [x] Equations
* [x] Cross-references
* [x] Footnotes
* [x] Callouts

If you change the structure of your documentation (by renaming vignettes or help topics) you can setup redirects from the old content to the new content.
One or several now-absent pages can be redirected to a new page (or to a new section of a new page).
This works by creating a html page that performs a "meta refresh", which isn't the best way of doing a redirect but works everywhere that you might deploy your site.

The syntax is the following, with old paths on the left, and new paths or URLs on the right.

```yaml
redirects:
  - ["articles/old-vignette-name.html", "articles/new-vignette-name.html"]
  - ["articles/another-old-vignette-name.html", "articles/new-vignette-name.html"]
  - ["articles/yet-another-old-vignette-name.html", "https://pkgdown.r-lib.org/dev"]
```

If for some reason you choose to redirect an existing page make sure to exclude it from the search index, see `?build_search`.

# Authors

By default, pkgdown will display author information in three places:

* the sidebar,
* the left part side of the footer,
* the author page.

This documentation describes how to customise the overall author display. See `?build_home` and `?build_site` for details about changing the location of the authors information within the home sidebar and the site footer.

## Authors ORCID and bio

Author ORCID identification numbers in the `DESCRIPTION` are linked using the ORCID logo:

```r
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@rstudio.com", role = c("aut", "cre"),
      comment = c(ORCID = "0000-0003-4757-117X")
    ),
    person("Jay", "Hesselberth", role = "aut",
      comment = c(ORCID = "0000-0002-6299-179X")
    )
  )
```

If you want to add more details about authors or their involvement with the package,
you can use the comment field, which will be rendered on the authors page.

```r
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@rstudio.com", role = c("aut", "cre"),
      comment = c(ORCID = "0000-0003-4757-117X", "Indenter-in-chief")
    ),
    person("Jay", "Hesselberth", role = "aut",
      comment = c(ORCID = "0000-0002-6299-179X")
    )
  )
```

## YAML config - authors

You can tweak a few things via the `authors` YAML field:

* display of each author in the footer, sidebar and authors page,
* which authors (by role) are displayed in the sidebar and footer,
* text before authors in the footer, 
* text before and after authors in the sidebar,
* text before and after authors on the authors page.


You can modify how each author's name is displayed
by adding a subsection for `authors`. 
Each entry in `authors` should be named 
with the author's name (matching `DESCRIPTION`)
and can contain `href` and/or `html` fields:

* If `href` is provided, the author's name will be linked to this URL.
* If `html` is provided, it will be shown instead of the author's name.
  This is particularly useful if you want to display the logo of a corporate
  sponsor. Use an absolute URL to an image, not a relative link.

```yaml
authors:
  firstname lastname:
    href: "http://name-website.com"
    html: "<img src='https://website.com/name-picture.png' width=72>"
```


By default, the "developers" list shown in the sidebar and footer is populated by the maintainer ("cre"), authors
("aut"), and funder ("fnd") from the `DESCRIPTION`. 
You could choose other roles for filtering.
With the configuration below:

* only the maintainer and funder(s) appear in the footer, after the text "Crafted by",
* all authors and contributors appear in the sidebar,
* the authors list on the sidebar is preceded and followed by some text,
* the authors list on the authors page is preceded and followed by some text.


```yaml
authors:
  footer:
    roles: [cre, fnd]
    text: "Crafted by"
  sidebar:
    roles: [aut, ctb]
    before: "So *who* does the work?"
    after: "Thanks all!"
  before: "This package is proudly brought to you by:"
  after: "See the [changelog](news/index.html) for other contributors. :pray:"
```

If you want to filter authors based on something else than their roles, 
consider using a custom sidebar/footer component 
(see `?build_home`/`?build_site`, respectively).

# Home page

The main content of the home page (`index.html`) is generated from `pkgdown/index.md`, `index.md`, or `README.md`, in that order.
Most packages will use `README.md` because that's also displayed by GitHub and CRAN.
Use `index.md` if you want your package website to look different to your README, and use `pkgdown/index.md` if you don't want that file to live in your package root directory.

If you use `index.Rmd` or `README.Rmd` it's your responsibility to knit the document to create the corresponding `.md`.
pkgdown does not do this for you because it only touches files in the `doc/` directory.

Extra markdown files in the base directory (e.g. `ROADMAP.md`) or in `.github/` (e.g. `CODE_OF_CONDUCT.md`) are copied by `build_home()` to `docs/` and converted to HTML.

The home page also features a sidebar with information extracted from the package.
You can tweak it via the configuration file, to help make the home page an as informative as possible landing page.

## Images and figures

If you want to include images in your `README.md`, they must be stored somewhere in the package so that they can be displayed on the CRAN website.
The best place to put them is `man/figures`.
If you are generating figures with R Markdown, make sure you set up `fig.path` as followed:

``` r
knitr::opts_chunk$set(
  fig.path = "man/figures/"
)
```

This should usually go in a chunk with `include = FALSE`.

```` markdown
```{r chunk-name, include=FALSE}`r ''`
knitr::opts_chunk$set(
  fig.path = "man/figures/"
)
```
````

## Package logo

If you have a package logo, you can include it at the top of your README in a level-one heading:

``` markdown
# pkgdown <img src="man/figures/logo.png" align="right" />
```

[init_site()] will also automatically create a favicon set from your package logo.

## YAML config - home

To tweak the home page, you need a section of the configuration file called `home`.

### Page title and description

By default, the page title and description are extracted automatically from the `Title` and `Description` fields `DESCRIPTION` (stripping single quotes off quoted words).
CRAN ensures that these fields don't contain phrases like "R package" because that's obvious on CRAN.
To make your package more findable on search engines, it's good practice to override the `title` and `description`, thinking about what people might search for:

``` yaml
home:
  title: An R package for pool-noodle discovery
  description: >
    Do you love R? Do you love pool-noodles? If so, you might enjoy
    using this package to automatically discover and add pool-noodles
    to your growing collection.
```

(Note the use of YAML's `>` i.e. "YAML pipes"; this is a convenient way of writing paragraphs of text.)

### README header

READMEs usually start with an `<h1>` containing the package name.
If that feels duplicative with the package name in the navbar you can remove it with `strip_header: true`:

``` yaml
home:
  strip_header: true
```

## Dev badges

pkgdown identifies badges in three ways:

-   Any image-containing links between `<!-- badges: start -->` and `<!-- badges: end -->`, as e.g. created by `usethis::use_readme_md()` or `usethis::use_readme_rmd()`.
    There should always be an empty line after the `<!-- badges: end -->` line.
    If you divide badges into paragraphs, make sure to add an empty line before the `<!-- badges: end -->` line.

-   Any image-containing links within `<div id="badges"></div>`.

-   Within the first paragraph, if it only contains image-containing links.

Identified badges are **removed** from the _main content_.
They are shown or not in the _sidebar_ depending on the development mode and sidebar customization, see the sidebar section.

You can customise the footer with the `footer` field.
It's made up of two pieces: `structure`, which defines the overall layout, and `components`, which defines what each piece looks like.
This organisation makes it easy to mix and match the pkgdown defaults with your own customisations.

This is the default structure:

``` yaml
footer:
  structure: 
    left: developed_by
    right: built_with
```

Which uses two of the three built-in components:

-   `developed_by`, a sentence describing the main authors of the package. (See `?build_home` if you want to tweak *which* authors appear in the footer.)
-   `built_with`, a sentence advertising pkgdown.
-   `package`, the name of the package.

You can override these defaults with the `footer` field.
The example below puts the authors' information on the right along with a legal disclaimer, and puts the pkgdown link on the left.

``` yaml
footer:
  structure: 
    left: pkgdown
    right: [authors, legal]
  components:
    legal: Provided without **any warranty**.
```

Each side is pasted into a single string (separated by `" "`) and then converted from markdown to HTML.

---
title: "How to update a released site"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{How to update a released site}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Checklist

If you're already up-to-speed, here's the checklist:

* Make sure your dev site looks the way you want before embarking on an update
  of your released site. Here are good instructions for updating an existing
  site for pkgdown 2.0.0, Bootstrap 5, and "tidyverse/tidytemplate" (only
  applies to certain packages):  
  <https://tidytemplate.tidyverse.org>
* Make sure you are using automatic development mode. In `_pkgdown.yml`:

  ``` yaml
  development:
    mode: auto
  ```
* Push/pull to sync your default branch.
* Create and checkout a branch based on the tag of your most recent GitHub/CRAN
  release.
  
  Example for readr, whose latest release is 2.1.1:  
  `git checkout -b update-pkgdown-2-1-1 v2.1.1`
  
  General pattern:  
  `git checkout -b NEW-BRANCH-NAME NAME-OF-RELEASE-TAG`
* Backport innovations from the future. Example of a useful opening move:  
  `git checkout main -- _pkgdown.yml .github/workflows/pkgdown.yaml`

  Most likely candidates:
  - `_pkgdown.yml`
  - `.github/workflows/pkgdown.yaml`
  - `Config/Needs/website: tidyverse/tidytemplate` in DESCRIPTION
  - Any fixes to `README.Rmd` / `README.md` or released roxygen comments. Don't
    forget to `devtools::build_readme()` or `devtools::document()`, in that
    case.
  
  These are the two main Git commands you'll need:
  ```
  git checkout REF -- path/to/a/specific/file
  git cherry-pick SHA
  ```
* Monitor your progress locally with `pkgdown::build_site()`.
* Push this branch, maybe using `usethis::pr_push()`. Don't bother opening
  a pull request (the branch will still be created and pushed).
* Go to your package's GHA page, maybe using `usethis::browse_github_actions()`.
  Select the pkgdown workflow. Click *Run workflow* and select the branch you
  just pushed. If there's no dropdown menu for this, that means your pkgdown
  workflow config is not current.
* Scrutinize your newly published **released** site and make sure things look
  right.
* You can let this branch sit around for a while, in case you need to iterate
  on this or if you'd like to backport more niceties before your next CRAN
  release. Or you can delete immediately, if you're happy with the released
  site.

## Context

Before we talk about **how** to update a released site, we first establish **why** you might need to do this.
What is a released site? What other kind of pkgdown site could you have?
Why does updating a released site take special effort?

### Automatic development mode

Every pkgdown site has a so-called [*development mode*](https://pkgdown.r-lib.org/reference/build_site.html#development-mode), which can be specified via the `development` field in `_pkgdown.yml`.
If unspecified, the default is `mode: release`, which results in a single pkgdown site.
Despite the name, this single site reflects "the current package state", which could be either a released state or a development state.

For packages with a substantial user base, it is recommended instead to specify `mode: auto` like so:

``` yaml
development:
  mode: auto
```

This directs pkgdown to "generate different sites for the development and released versions of your package."

The readr package demonstrates what happens in automatic development mode:

[readr.tidyverse.org](https://readr.tidyverse.org) documents the released version, i.e. what `install.packages()` would deliver.  
[readr.tidyverse.org/dev/](https://readr.tidyverse.org/dev/) documents the dev version, i.e. what you'd get by installing from GitHub.

In this mode, `pkgdown::build_site()`, consults DESCRIPTION to learn the package's version number.
For a development version number, the rendered site is written to `docs/dev/`.
For a released version number, the site is written to `docs/`.
(There are also other signals to alert users that they are reading documentation for a dev version vs. a released version.)

Automatic development mode is recommended for packages with a broad user base, because it maximizes the chance that a user will read web-based documentation that reflects the package version that is locally installed.

### Publishing

Now that we've established the meaning of a released (vs dev) site, we have to consider how the site gets rendered and deployed.

Many people use `usethis::use_pkgdown_github_pages()` to do basic pkgdown setup and configure a GitHub Actions (GHA) workflow to automatically render and publish the site to GitHub Pages.
Here's an overview of what this function does:

```
usethis::use_pkgdown_github_pages() =
  use_pkgdown() +
  use_github_pages() + 
  use_github_action("pkgdown") +
  add the pkgdown site's URL to _pkgdown.yml, URL field of DESCRIPTION,
    and to the GitHub repo +
  (if owning org is one of: tidyverse, r-lib, tidymodels, rstudio) 
  add "tidyverse/tidytemplate" to Config/Needs/website in DESCRIPTION
```

As a result, the publishing cadence of many pkgdown sites is governed by the workflow maintained at [`r-lib/actions/examples/pkgdown.yaml`](https://github.com/r-lib/actions/blob/v2/examples/pkgdown.yaml).
(Do not confuse `_pkgdown.yml`, which gives instructions to the pkgdown package, with `.github/workflows/pkgdown.yaml`, which gives instructions to GHA.)

Here are some of the most important bits of the GHA workflow config:

*NOTE: this is from v2 of `r-lib/actions`, which is important! If you use usethis to add/update workflow config, make sure you're using the dev version of usethis, which will pull from the `v2` tag.*

``` yaml
on:
  push:
    branches: [main, master]
  pull_request:
    branches: [main, master]
  release:
    types: [published]
  workflow_dispatch:
  
<snip, snip>

      - name: Build site
        run: Rscript -e 'pkgdown::build_site_github_pages(...)'
        
      - name: Deploy to GitHub pages 🚀
        if: github.event_name != 'pull_request'
        uses: JamesIves/github-pages-deploy-action@4.1.4
        
        <snip, snip>
```

We build and deploy for pushes to `main` (or `master`).    
We build for pull requests against `main` (or `master`).  
But we don't deploy for pull requests.

We build and deploy when we publish a GitHub release.  
By convention, we assume that a GitHub release coincides with a CRAN release.  
**This is the primary mechanism for building the released pkgdown site.**  

`pkgdown::build_site_github_pages()` consults the version in DESCRIPTION to detect whether it's building from a released version or a dev version.
That determines the `dest_dir`, e.g. `docs/` for released and `docs/dev/` for dev.
For a package in automatic development mode, this means that almost all of your pushes trigger an update to the dev site. 
The released site is only updated when you push a state with a non-development version number or when you publish a GitHub release.

So how do you tweak things about the released site *in between* releases?

That brings us to `workflow_dispatch:`. (Yes that dangling colon is correct.)

The inclusion of `workflow_dispatch` as a trigger means the pkgdown workflow can be run on demand, either manually from the browser or via the GitHub REST API.
We're going to show how to update a released site from a GitHub branch or tag, using the browser method.
In the future, we might build some tooling around the API method.

Places to learn more about triggering GHA workflows:

* <https://docs.github.com/en/actions/learn-github-actions/events-that-trigger-workflows#workflow_dispatch>
* <https://docs.github.com/en/actions/managing-workflow-runs/manually-running-a-workflow>
* <https://docs.github.com/en/rest/reference/actions/#create-a-workflow-dispatch-event>

Before we move on, I repeat:  
It is important that your `main` branch have the latest pkgdown workflow.
(Or whatever your default branch is called.)
At the time of writing, this means the `v2` tag in `r-lib/actions`.
The easiest way to get this is to install dev usethis and run `usethis::use_github_action("pkgdown")`.

## Construct a branch for the update

The overall goal is to create a package state from which to update the released site that corresponds to (HEAD of) a Git branch.
This package state should:

* Be based on the most recent GitHub and CRAN release.
* Incorporate all desired updates to your documentation, such as pkgdown config.

Create and checkout a branch based on the tag of the most recent CRAN release.
For example, if readr's latest release is 2.1.1:

```
git checkout -b update-pkgdown-2-1-1 v2.1.1
```

And here is the general pattern:

```
git checkout -b NEW-BRANCH-NAME NAME-OF-RELEASE-TAG
```

Now you should backport innovations from the future that you would like to retroactively apply to your released site.

Files you must update:

* `.github/workflows/pkgdown.yaml`
* `_pkgdown.yml`
* `Config/Needs/website` field of DESCRIPTION (And, probably, only this field!
  In particular, do not mess with the version number.)

Other likely candidates:  

* `README.Rmd` + `README.md`, e.g., if you've updated badges.
* Any documentation fixes that **apply to the released version**. This is the
  only reason to touch anything below `R/` and even then it should only affect
  roxygen comments. Don't forget to `document()` if you do this!
* Any new vignettes or articles that apply to the released version.

Here are some tips on backporting specific changes into this branch.
If you are lucky, there are specific commits in your default branch that contain all the necessary changes.
In that case, we can cherry pick such a commit by its SHA:

```
git cherry-pick SHA
```

If that doesn't cover everything, for each file you want to update, identify a Git reference (meaning: a SHA, tag, or branch) where the file is in the desired state.
Checkout that specific file path from that specific ref:

```
git checkout REF -- path/to/the/file
```
  
For example, readr recently gained a new vignette that applies to the released version of readr, i.e. it does not document any dev-only features or functions.
Here's how to introduce the new vignette from HEAD of `main` into the current branch:

```
git checkout main -- vignettes/column-types.Rmd
```

Commit and push this new branch to GitHub.
`usethis::pr_push()` can be handy for this.
Just don't bother opening a pull request (the branch will still be pushed).

Now we will use the `workflow_dispatch` GHA trigger.
Go to the Actions page of your repo, maybe via `usethis::browse_github_actions()`.
Click on the `pkgdown` workflow.

You should see "This workflow has a workflow_dispatch event trigger."  
(If you don't, that means you didn't do one of the pre-requisites, which is to update to `v2` of the pkgdown workflow.)
  
See the "Run workflow" button?
CLICK IT.
In the "Use workflow from" dropdown menu, select the branch you've just made and pushed.
This should kick off a pkgdown build-and-deploy and, specifically, it should cause updates to the **released** site.
  
You can keep this branch around for a while, in case you didn't get everything right the first time or if more things crop up that you'd like backport to the released site, before your next CRAN release.

## Problem-solving

Remember GitHub search is a great way to get other people's `_pkgdown.yml` files in front of your eyeballs:

Example query: [`filename:_pkgdown.yml org:tidyverse org:r-lib`](https://github.com/search?q=filename%3A_pkgdown.yml+org%3Atidyverse+org%3Ar-lib)

For any given `_pkgdown.yml` file on GitHub, remember that its History and Blame can be helpful for seeing how another package's config has evolved over time.

You can customise the navigation bar that appears at the top of the page with the `navbar` field.
It's made up of two pieces: `structure`, which defines the overall layout, and `components`, which defines what each piece looks like.
This organisation makes it easy to mix and match pkgdown defaults with your own customisations.

This is the default structure:

``` yaml
navbar:
  structure:
    left:  [intro, reference, articles, tutorials, news]
    right: [search, github]
```

It makes use of the the six built-in components:

-   `intro`: "Get Started", which links to a vignette with the same name as the package.
-   `reference`, if there are any `.Rd` files.
-   `articles`, if there are any vignettes or articles.
-   `tutorials`, if there any tutorials.
-   `news`, if `NEWS.md` exists.
-   `search`, the search box (see `vignette("search")` for more details).
-   `github`, a link to the source repository (with an icon), if it can be automatically determined from the `DESCRIPTION`.

You can use the `structure` field to reorganise the navbar without changing the default contents:

``` yaml
navbar:
  structure:
    left:  [search]
    right: [reference, articles]
```

You can use `components` to override the default content.
For example, this yaml provides a custom articles menu:

``` yaml
navbar:
 components:
   articles:
    text: Articles
    menu:
    - text: Category A
    - text: Title A1
      href: articles/a1.html
    - text: Title A2
      href: articles/a2.html
    - text: -------
    - text: "Category B"
    - text: Article B1
      href: articles/b1.html
```

Components uses the same syntax as [RMarkdown menus](https://bookdown.org/yihui/rmarkdown/rmarkdown-site.html#site-navigation).
The elements of `menu` can be:

-   A link (`text` + `href`)

-   A heading (just `text`)

-   A separator (`text: ——–`)

Instead of text, you can also use the name of an `icon`s from [fontawesome](https://fontawesome.com/icons?d=gallery).
You should also provide a textual description in the `aria-label` field for screenreader users.

To add a new component to the navbar, you need to modify both `structure` and `components`.
For example, the following yaml adds a new "twitter" component that appears to the left of the github icon.

``` yaml
navbar:
  structure:
    right: [twitter, github]
  components:
    twitter:
      icon: fa-twitter
      href: http://twitter.com/hadleywickham
      aria-label: Twitter
```

Finally, you can add arbitrary HTML to three locations in the navbar:

```yaml
template:
  includes:
    before_title: <!-- inserted before the package title in the header ->
    before_navbar: <!-- inserted before the navbar links -->
    after_navbar: <!-- inserted after the navbar links -->
```

These includes will appear on all screen sizes, and will not be collapsed into the the navbar drop down.

You can customise the homepage sidebar with the `home.sidebar` field.
It's made up of two pieces: `structure`, which defines the overall layout, and `components`, which defines what each piece looks like.
This organisation makes it easy to mix and match the pkgdown defaults with your own customisations.

This is the default structure:

``` yaml
home:
  sidebar:
    structure: [links, license, community, citation, authors, dev]
```

These are drawn from seven built-in components:

-   `links`: automated links generated from `URL` and `BugReports` fields from `DESCRIPTION` plus manual links from the `home.links` field:

    ``` yaml
    home:
      links:
      - text: Link text
        href: https://website.com
      - text: Roadmap
        href: /roadmap.html
    ```

-   `license`: Licensing information if `LICENSE`/`LICENCE` or `LICENSE.md`/`LICENCE.md` files are present.

-   `community`: links to to `.github/CONTRIBUTING.md`, `.github/CODE_OF_CONDUCT.md`, etc.

-   `citation`: link to package citation information. Uses either `inst/CITATION` or, if absent, information from the `DESCRIPTION`.

-   `authors`: selected authors from the `DESCRIPTION`.

-   `dev`: development status badges extracted from `README.md`/`index.md`. This is only shown for "development" versions of websites; see "Development mode" in `?build_site` for details. 

-   `toc`: a table of contents for the README (not shown by default).

You can also add your own components, where `text` is markdown text:

``` yaml
home:
  sidebar:
    structure: [authors, custom, toc, dev]
    components:
      custom:
        title: Funding
        text: We are *grateful* for funding!
```

Alternatively, you can provide a ready-made sidebar HTML:

``` yaml
home:
  sidebar:
    html: path-to-sidebar.html
```

Or completely remove it:

``` yaml
home:
  sidebar: FALSE
```

# bad width gives nice error

    Code
      rmarkdown_setup_pkgdown(pkg)
    Condition
      Error in `rmarkdown_setup_pkgdown()`:
      ! In _pkgdown.yml, code.width must be a whole number, not the string "abc".

# output is reproducible by default, i.e. 'seed' is respected

    Code
      cat(output)
    Output
      ## [1] 0.080750138 0.834333037 0.600760886 0.157208442 0.007399441

# reports on bad open graph meta-data

    Code
      build_article("test", pkg)
    Message
      Reading vignettes/test.Rmd
    Condition
      Error in `build_article()`:
      ! In vignettes/test.Rmd, opengraph.twitter must be a list, not the number 1.

# build_article styles ANSI escapes

    <span class="co">## <span style="color: #BB0000;">X</span></span>

# build_article copies image files in subdirectories

    Code
      build_article("test", pkg)
    Message
      Reading vignettes/test.Rmd
      Writing `articles/test.html`

# warns about missing images

    Code
      build_article("kitten", pkg)
    Message
      Reading vignettes/kitten.Rmd
      Writing `articles/kitten.html`
      Missing images in 'vignettes/kitten.Rmd': 'kitten.jpg'
      i pkgdown can only use images in 'man/figures' and 'vignettes'

# warns about missing alt-text

    Code
      build_article("kitten", pkg)
    Message
      Reading vignettes/kitten.Rmd
      Writing `articles/kitten.html`
      x Missing alt-text in 'vignettes/kitten.Rmd'
      * kitten.jpg
      i Learn more in `vignette(pkgdown::accessibility)`.

# checks its inputs

    Code
      preview_site(pkg, path = 1)
    Condition
      Error in `preview_site()`:
      ! `path` must be a single string, not the number 1.
    Code
      preview_site(pkg, path = "foo")
    Condition
      Error in `preview_site()`:
      ! Can't find file 'foo'.
    Code
      preview_site(pkg, preview = 1)
    Condition
      Error in `preview_site()`:
      ! `preview` must be `TRUE`, `FALSE`, or `NA`, not the number 1.

# usage generates user facing code for S3/S4 infix/replacement methods

    Code
      cat(usage2text("\\S3method{$}{indexed_frame}(x, name)"))
    Output
      # S3 method for class 'indexed_frame'
      x$name
    Code
      cat(usage2text("\\method{[[}{indexed_frame}(x, i) <- value"))
    Output
      # S3 method for class 'indexed_frame'
      x[[i]] &lt;- value
    Code
      cat(usage2text("\\S4method{>=}{MyType,numeric}(e1, e2)"))
    Output
      # S4 method for class 'MyType,numeric'
      e1 &gt;= e2

# sections with class .tabset are converted to tabsets

    <div class="section level1 tabset tabset-pills">
    <h1 id="tabset">Tabset<a class="anchor" aria-label="anchor" href="#tabset"></a>
    </h1>
    
    
    <ul class="nav nav-pills" id="tabset" role="tablist">
    <li role="presentation" class="nav-item"><button data-bs-toggle="tab" data-bs-target="#tab-1" id="tab-1-tab" type="button" role="tab" aria-controls="tab-1" aria-selected="true" class="active nav-link">Tab 1</button></li>
    <li role="presentation" class="nav-item"><button data-bs-toggle="tab" data-bs-target="#tab-2" id="tab-2-tab" type="button" role="tab" aria-controls="tab-2" aria-selected="false" class="nav-link">Tab 2</button></li>
    </ul>
    <div class="tab-content">
    <div class="active  tab-pane" id="tab-1" role="tabpanel" aria-labelledby="tab-1-tab">
    
    <p>Contents 1</p>
    </div>
    <div class="tab-pane" id="tab-2" role="tabpanel" aria-labelledby="tab-2-tab">
    
    <p>Contents 2</p>
    </div>
    </div>
    </div>

---
title: No Description
opengraph:
  twitter:
    card: summary
    creator: "@dataandme"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{No Description}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Content goes here

# can generate three types of row

    Code
      data_reference_index(pkg)
    Output
      pagetitle: Package index
      rows:
      - title: A
        slug: a
        desc: ~
        is_internal: no
      - subtitle: B
        slug: b
        desc: ~
        is_internal: no
      - topics:
        - path: a.html
          title: A
          lifecycle: ~
          aliases: a()
          icon: ~
        - path: b.html
          title: B
          lifecycle: ~
          aliases: b()
          icon: ~
        - path: c.html
          title: C
          lifecycle: ~
          aliases: c()
          icon: ~
        - path: e.html
          title: E
          lifecycle: ~
          aliases: e
          icon: ~
        - path: help.html
          title: D
          lifecycle: ~
          aliases: '`?`()'
          icon: ~
        names:
        - a
        - b
        - c
        - e
        - '?'
        row_has_icons: no
        is_internal: no
      has_icons: no
      

# warns if missing topics

    Code
      data_reference_index(pkg)
    Condition
      Error:
      ! In _pkgdown.yml, 3 topics missing from index: "c", "e", and "?".
      i Either use `@keywords internal` to drop from index, or

# gives informative errors

    Code
      data_reference_index_(1)
    Condition
      Error in `config_pluck_reference()`:
      ! In _pkgdown.yml, reference must be a list, not the number 1.
    Code
      data_reference_index_(list(1))
    Condition
      Error in `data_reference_index_()`:
      ! In _pkgdown.yml, reference[1] must be a list, not the number 1.
    Code
      data_reference_index_(list(list(title = 1)))
    Condition
      Error in `data_reference_index_()`:
      ! In _pkgdown.yml, reference[1].title must be a string, not the number 1.
    Code
      data_reference_index_(list(list(title = "a\n\nb")))
    Condition
      Error in `data_reference_index_()`:
      ! In _pkgdown.yml, reference[1].title must be inline markdown.
    Code
      data_reference_index_(list(list(subtitle = 1)))
    Condition
      Error in `data_reference_index_()`:
      ! In _pkgdown.yml, reference[1].subtitle must be a string, not the number 1.
    Code
      data_reference_index_(list(list(subtitle = "a\n\nb")))
    Condition
      Error in `data_reference_index_()`:
      ! In _pkgdown.yml, reference[1].subtitle must be inline markdown.
    Code
      data_reference_index_(list(list(title = "bla", contents = 1)))
    Condition
      Error in `data_reference_index_()`:
      ! In _pkgdown.yml, reference[1].contents[1] must be a string.
      i You might need to add '' around special YAML values like 'N' or 'off'
    Code
      data_reference_index_(list(list(title = "bla", contents = NULL)))
    Condition
      Error in `data_reference_index_()`:
      ! In _pkgdown.yml, reference[1].contents is empty.
    Code
      data_reference_index_(list(list(title = "bla", contents = list("a", NULL))))
    Condition
      Error in `data_reference_index_()`:
      ! In _pkgdown.yml, reference[1].contents[2] is empty.
    Code
      data_reference_index_(list(list(title = "bla", contents = list())))
    Condition
      Error in `data_reference_index_()`:
      ! In _pkgdown.yml, reference[1].contents is empty.
    Code
      data_reference_index_(list(list(title = "bla", contents = "notapackage::lala")))
    Condition
      Error in `build_reference_index()`:
      ! The package "notapackage" is required as it's used in the reference index.
    Code
      data_reference_index_(list(list(title = "bla", contents = "rlang::lala")))
    Condition
      Error in `build_reference_index()`:
      ! Could not find documentation for `rlang::lala()`.

# can use a topic from another package

    Code
      data_reference_index(pkg)
    Output
      pagetitle: Package index
      rows:
      - title: bla
        slug: bla
        desc: ~
        is_internal: no
      - topics:
        - path: https://rlang.r-lib.org/reference/is_installed.html
          title: Are packages installed in any of the libraries? (from rlang)
          lifecycle: ~
          aliases:
          - is_installed()
          - check_installed()
          icon: ~
        - path: https://rstudio.github.io/bslib/reference/bs_bundle.html
          title: Add low-level theming customizations (from bslib)
          lifecycle: ~
          aliases:
          - bs_add_variables()
          - bs_add_rules()
          - bs_add_functions()
          - bs_add_mixins()
          - bs_bundle()
          icon: ~
        names:
        - rlang::is_installed()
        - bslib::bs_add_rules
        row_has_icons: no
        is_internal: no
      has_icons: no
      

# can use a selector name as a topic name

    Code
      data_reference_index(pkg)
    Output
      pagetitle: Package index
      rows:
      - title: bla
        slug: bla
        desc: ~
        is_internal: no
      - topics:
        - path: matches.html
          title: matches
          lifecycle: ~
          aliases: matches()
          icon: ~
        - path: A.html
          title: A
          lifecycle: ~
          aliases: A()
          icon: ~
        names:
        - matches
        - A
        row_has_icons: no
        is_internal: no
      has_icons: no
      

# adds github/gitlab link when available

    reference:
      text: Reference
      href: reference/index.html
    search:
      search: []
    

---

    reference:
      text: Reference
      href: reference/index.html
    search:
      search: []
    github:
      icon: fab fa-github fa-lg
      href: https://github.com/r-lib/pkgdown
      aria-label: GitHub
    

---

    reference:
      text: Reference
      href: reference/index.html
    search:
      search: []
    github:
      icon: fab fa-gitlab fa-lg
      href: https://gitlab.com/r-lib/pkgdown
      aria-label: GitLab
    

# vignette with package name turns into getting started

    reference:
      text: Reference
      href: reference/index.html
    search:
      search: []
    intro:
      text: Get started
      href: test.html
    

# can control articles navbar through articles meta

    Code
      navbar_articles(pkg())
    Output
      articles:
        text: Articles
        menu:
        - text: Title a
          href: a.html
        - text: Title b
          href: b.html
      

---

    Code
      navbar_articles(pkg(articles = list(list(name = "all", contents = c("a", "b")))))
    Output
      articles:
        text: Articles
        href: articles/index.html
      

---

    Code
      navbar_articles(pkg(articles = list(list(name = "all", contents = c("a", "b"),
      navbar = NULL))))
    Output
      articles:
        text: Articles
        menu:
        - text: Title a
          href: a.html
        - text: Title b
          href: b.html
      

---

    Code
      navbar_articles(pkg(articles = list(list(name = "all", contents = c("a", "b"),
      navbar = "Label"))))
    Output
      articles:
        text: Articles
        menu:
        - text: '---------'
        - text: Label
        - text: Title a
          href: a.html
        - text: Title b
          href: b.html
      

---

    Code
      navbar_articles(pkg(articles = list(list(name = "a", contents = "a", navbar = NULL),
      list(name = "b", contents = "b"))))
    Output
      articles:
        text: Articles
        menu:
        - text: Title a
          href: a.html
        - text: '---------'
        - text: More articles...
          href: articles/index.html
      

# data_navbar() works by default

    Code
      data_navbar(pkg)
    Output
      $bg
      [1] "light"
      
      $type
      [1] "light"
      
      $left
      [1] "<li class=\"nav-item\"><a class=\"nav-link\" href=\"reference/index.html\">Reference</a></li>\n<li class=\"nav-item\"><a class=\"nav-link\" href=\"news/index.html\">Changelog</a></li>"
      
      $right
      [1] "<li class=\"nav-item\"><form class=\"form-inline\" role=\"search\">\n <input class=\"form-control\" type=\"search\" name=\"search-input\" id=\"search-input\" autocomplete=\"off\" aria-label=\"Search site\" placeholder=\"Search for\" data-search-index=\"search.json\"> \n</form></li>\n<li class=\"nav-item\"><a class=\"nav-link\" href=\"https://github.com/r-lib/pkgdown/\" aria-label=\"GitHub\"><span class=\"fa fab fa-github fa-lg\"></span></a></li>"
      

# data_navbar() can re-order default elements

    Code
      data_navbar(pkg)[c("left", "right")]
    Output
      $left
      [1] "<li class=\"nav-item\"><a class=\"nav-link\" href=\"https://github.com/r-lib/pkgdown/\" aria-label=\"GitHub\"><span class=\"fa fab fa-github fa-lg\"></span></a></li>\n<li class=\"nav-item\"><form class=\"form-inline\" role=\"search\">\n <input class=\"form-control\" type=\"search\" name=\"search-input\" id=\"search-input\" autocomplete=\"off\" aria-label=\"Search site\" placeholder=\"Search for\" data-search-index=\"search.json\"> \n</form></li>"
      
      $right
      [1] "<li class=\"nav-item\"><a class=\"nav-link\" href=\"news/index.html\">Changelog</a></li>"
      

# data_navbar() works with empty side

    Code
      data_navbar(pkg)
    Output
      $bg
      [1] "light"
      
      $type
      [1] "light"
      
      $left
      [1] ""
      
      $right
      [1] ""
      

# data_navbar_() errors with bad yaml specifications

    Code
      data_navbar_(navbar = list(structure = list(left = 1)))
    Condition
      Error in `data_navbar_()`:
      ! In _pkgdown.yml, navbar.structure.left must be a character vector, not the number 1.
    Code
      data_navbar_(navbar = list(right = "github"))
    Condition
      Error in `data_template()`:
      ! In _pkgdown.yml, navbar is incorrectly specified.
      i See details in `vignette(pkgdown::customise)`.

# render_navbar_links BS3 & BS4 default

    Code
      cat(render_navbar_links(x, pkg = list(bs_version = 3)))
    Output
      <li>
        <a href="articles/pkgdown.html">Get started</a>
      </li>
      <li>
        <a href="reference/index.html">Reference</a>
      </li>
      <li class="dropdown">
        <a href="#" class="dropdown-toggle" data-toggle="dropdown" role="button" data-bs-toggle="dropdown" aria-expanded="false">
          Articles
           
          <span class="caret"></span>
        </a>
        <ul class="dropdown-menu" role="menu">
          <li>
            <a href="articles/linking.html">Auto-linking</a>
          </li>
          <li>
            <a href="articles/search.html">Search</a>
          </li>
          <li>
            <a href="articles/metadata.html">Metadata</a>
          </li>
          <li>
            <a href="articles/customization.html">Customize your pkgdown website</a>
          </li>
          <li class="divider"></li>
          <li>
            <a href="articles/index.html">More...</a>
          </li>
        </ul>
      </li>
      <li>
        <a href="news/index.html">News</a>
      </li>

---

    Code
      cat(render_navbar_links(x, pkg = list(bs_version = 4)))
    Output
      <li class="nav-item"><a class="nav-link" href="articles/pkgdown.html">Get started</a></li>
      <li class="nav-item"><a class="nav-link" href="reference/index.html">Reference</a></li>
      <li class="nav-item dropdown">
        <button class="nav-link dropdown-toggle" type="button" id="dropdown-articles" data-bs-toggle="dropdown" aria-expanded="false" aria-haspopup="true">Articles</button>
        <ul class="dropdown-menu" aria-labelledby="dropdown-articles">
          <li><a class="dropdown-item" href="articles/linking.html">Auto-linking</a></li>
          <li><a class="dropdown-item" href="articles/search.html">Search</a></li>
          <li><a class="dropdown-item" href="articles/metadata.html">Metadata</a></li>
          <li><a class="dropdown-item" href="articles/customization.html">Customize your pkgdown website</a></li>
          <li><hr class="dropdown-divider"></li>
          <li><a class="dropdown-item" href="articles/index.html">More...</a></li>
        </ul>
      </li>
      <li class="nav-item"><a class="nav-link" href="news/index.html">News</a></li>

# Invalid bootstrap version spec in template package

    Code
      local_pkgdown_site(meta = list(template = list(package = "templatepackage")))
    Condition
      Error in `as_pkgdown()`:
      ! In _pkgdown.yml, must set only one of template.bootstrap and template.bslib.version.
      i Specified locally and in template package templatepackage.

# Invalid bootstrap version spec in _pkgdown.yml

    Code
      local_pkgdown_site(meta = list(template = list(bootstrap = 4, bslib = list(
        version = 5))))
    Condition
      Error in `as_pkgdown()`:
      ! In _pkgdown.yml, must set only one of template.bootstrap and template.bslib.version.

# Warns when Bootstrap theme is specified in multiple locations

    Code
      get_bslib_theme(pkg)
    Condition
      Warning:
      Multiple Bootstrap preset themes were set. Using "flatly" from template.bslib.preset.
      x Found template.bslib.preset, template.bslib.bootswatch, template.bootswatch, and template.params.bootswatch.
      i Remove extraneous theme declarations to avoid this warning.
    Output
      [1] "flatly"

---
title: "HTML widgets"
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Widgets

Test spacing above widget.

```{r, echo=FALSE}
# set seed for reproducible widget id
if (requireNamespace("htmltools", quietly = TRUE)) {
  htmlwidgets::setWidgetIdSeed(42)
}

path1 <- tempfile()
writeLines(letters, path1)
path2 <- tempfile()
writeLines(letters[-(10:11)], path2)

diffviewer::visual_diff(path1, path2)
```

Test spacing below widget.

# handles CoC and SUPPORT if present

    <div class='community'>
    <h2 data-toc-skip>Community</h2>
    <ul class='list-unstyled'>
    <li><a href='CODE_OF_CONDUCT.html'>Code of conduct</a></li>
    <li><a href='SUPPORT.html'>Getting help</a></li>
    </ul>
    </div>

# validates yaml

    Code
      data_development_(development = 1)
    Condition
      Error in `as_pkgdown()`:
      ! In _pkgdown.yml, development must be a list, not the number 1.
    Code
      data_development_(development = list(mode = 1))
    Condition
      Error in `as_pkgdown()`:
      ! In _pkgdown.yml, development.mode must be a string, not the number 1.
    Code
      data_development_(development = list(mode = "foo"))
    Condition
      Error in `as_pkgdown()`:
      ! In _pkgdown.yml, development.mode must be one of auto, default, release, devel, or unreleased, not foo.
    Code
      data_development_(development = list(destination = 1))
    Condition
      Error in `as_pkgdown()`:
      ! In _pkgdown.yml, development.destination must be a string, not the number 1.
    Code
      data_development_(development = list(version_label = 1))
    Condition
      Error in `as_pkgdown()`:
      ! In _pkgdown.yml, development.version_label must be a string, not the number 1.

---
title: "html_vignette + as_is"
output: rmarkdown::html_vignette
pkgdown:
  as_is: true
vignette: >
  %\VignetteIndexEntry{html_vignette + as_is}
---

```{r include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

This should not break page css/js (i.e. does the navbar look ok?)

# Heading 1

# Heading 2

# messages about reading and writing

    Code
      build_home_index(pkg)
    Message
      Reading 'DESCRIPTION'
      Writing `index.html`
    Code
      build_home_index(pkg)
    Message
      Reading 'DESCRIPTION'

# data_home() validates yaml metadata

    Code
      data_home_(home = 1)
    Condition
      Error in `data_home_()`:
      ! In _pkgdown.yml, home must be a list, not the number 1.
    Code
      data_home_(home = list(title = 1))
    Condition
      Error in `data_home_()`:
      ! In _pkgdown.yml, home.title must be a string, not the number 1.
    Code
      data_home_(home = list(description = 1))
    Condition
      Error in `data_home_()`:
      ! In _pkgdown.yml, home.description must be a string, not the number 1.
    Code
      data_home_(template = list(trailing_slash_redirect = 1))
    Condition
      Error in `data_home_()`:
      ! In _pkgdown.yml, template.trailing_slash_redirect must be true or false, not the number 1.

# data_home_sidebar() works by default

    Code
      cat(data_home_sidebar(pkg))
    Output
      <div class='links'>
      <h2 data-toc-skip>Links</h2>
      <ul class='list-unstyled'>
      <li><a href='{{ BugReports }}'>Report a bug</a></li>
      </ul>
      </div>
      
      <div class='license'>
      <h2 data-toc-skip>License</h2>
      <ul class='list-unstyled'>
      <li>{{ License }}</li>
      </ul>
      </div>
      
      
      <div class='citation'>
      <h2 data-toc-skip>Citation</h2>
      <ul class='list-unstyled'>
      <li><a href='authors.html#citation'>Citing testpackage</a></li>
      </ul>
      </div>
      
      <div class='developers'>
      <h2 data-toc-skip>Developers</h2>
      <ul class='list-unstyled'>
      <li>Jo Doe <br />
      <small class = 'roles'> Author, maintainer </small>  </li>
      </ul>
      </div>
      
      <div class='dev-status'>
      <h2 data-toc-skip>Dev Status</h2>
      <ul class='list-unstyled'>
      <li>placeholder</li>
      </ul>
      </div>

---

    <div class="developers">
    <h2 data-toc-skip>Developers</h2>
    <ul class="list-unstyled">
    <li>Hadley Wickham <br><small class="roles"> Author, maintainer </small>  </li>
    <li>RStudio <br><small class="roles"> Copyright holder, funder </small>  </li>
    <li><a href="authors.html">More about authors...</a></li>
    </ul>
    </div>

# data_home_sidebar() can be defined by a HTML file

    Code
      data_home_sidebar(pkg)
    Condition
      Error:
      ! In _pkgdown.yml, home.sidebar.html specifies a file that doesn't exist ('sidebar.html').

# data_home_sidebar() can get a custom markdown formatted component

    <div class="fancy-section">
    <h2 data-toc-skip>Fancy section</h2>
    <ul class="list-unstyled">
    <li><p>How <em>cool</em> is pkgdown?!</p></li>
    </ul>
    </div>

# data_home_sidebar() can add a TOC

    <div class="table-of-contents">
    <h2 data-toc-skip>Table of contents</h2>
    <ul class="list-unstyled">
    <li><nav id="toc"></nav></li>
    </ul>
    </div>

# data_home_sidebar() outputs informative error messages

    Code
      data_home_sidebar_(html = 1)
    Condition
      Error in `data_home_sidebar_()`:
      ! In _pkgdown.yml, home.sidebar.html must be a string, not the number 1.
    Code
      data_home_sidebar_(structure = 1)
    Condition
      Error in `data_home_sidebar_()`:
      ! In _pkgdown.yml, home.sidebar.structure must be a character vector, not the number 1.
    Code
      data_home_sidebar_(structure = "fancy")
    Condition
      Error in `data_home_sidebar_()`:
      ! In _pkgdown.yml, home.sidebar.components must have component "fancy".
      1 missing component: "fancy".
    Code
      data_home_sidebar_(structure = c("fancy", "cool"))
    Condition
      Error in `data_home_sidebar_()`:
      ! In _pkgdown.yml, home.sidebar.components must have components "fancy" and "cool".
      2 missing components: "fancy" and "cool".
    Code
      data_home_sidebar_(structure = "fancy", components = list(fancy = list(text = "bla")))
    Condition
      Error in `data_home_sidebar_()`:
      ! In _pkgdown.yml, home.sidebar.components.fancy must have components "title" and "text".
      1 missing component: "title".
    Code
      data_home_sidebar_(structure = "fancy", components = list(fancy = list()))
    Condition
      Error in `data_home_sidebar_()`:
      ! In _pkgdown.yml, home.sidebar.components.fancy must have components "title" and "text".
      2 missing components: "title" and "text".

# allow email in BugReports

    Code
      xpath_xml(html, ".//li/a")
    Output
      <a href="mailto:me@tidyverse.com">Report a bug</a>
      <a href="authors.html#citation">Citing testpackage</a>

# can construct menu with children

    Code
      cat(navbar_html(menu))
    Output
      <li class="nav-item dropdown">
        <button class="nav-link dropdown-toggle" type="button" id="dropdown-title" data-bs-toggle="dropdown" aria-expanded="false" aria-haspopup="true">Title</button>
        <ul class="dropdown-menu" aria-labelledby="dropdown-title">
          <li><h6 class="dropdown-header" data-toc-skip>Heading</h6></li>
          <li><hr class="dropdown-divider"></li>
          <li><a class="dropdown-item" href="https://example.com">Link</a></li>
        </ul>
      </li>

# bad inputs give clear error

    Code
      navbar_html(1)
    Condition
      Error in `menu_type()`:
      ! Navbar components must be named lists, not the number 1.
    Code
      navbar_html(list(foo = 1))
    Condition
      Error in `menu_type()`:
      ! Unknown navbar component with names foo.
    Code
      navbar_html(submenu)
    Condition
      Error in `menu_type()`:
      ! Nested menus are not supported.

# can construct bullets

    Code
      cat(navbar_html(menu_icon("fa-question", "https://example.com", "label")))
    Output
      <li class="nav-item"><a class="nav-link" href="https://example.com" aria-label="label"><span class="fa fa-question"></span></a></li>
    Code
      cat(navbar_html(menu_heading("Hi")))
    Output
      <li class="nav-item"><h6 class="dropdown-header" data-toc-skip>Hi</h6></li>
    Code
      cat(navbar_html(menu_link("Hi", "https://example.com")))
    Output
      <li class="nav-item"><a class="nav-link" href="https://example.com">Hi</a></li>

# icons warn if no aria-label

    Code
      . <- navbar_html(menu_icon("fa-question", "https://example.com", NULL))
    Message
      x Icon "fa-question" lacks an `aria-label`.
      i Specify `aria-label` to make the icon accessible to screen readers.
      i Learn more in `vignette(pkgdown::accessibility)`.
      This message is displayed once every 8 hours.

# can construct theme menu

    Code
      cat(navbar_html(lightswitch))
    Output
      <li class="nav-item dropdown">
        <button class="nav-link dropdown-toggle" type="button" id="dropdown-lightswitch" data-bs-toggle="dropdown" aria-expanded="false" aria-haspopup="true" aria-label="Light switch"><span class="fa fa-sun"></span></button>
        <ul class="dropdown-menu" aria-labelledby="dropdown-lightswitch">
          <li><button class="dropdown-item" data-bs-theme-value="light"><span class="fa fa-sun"></span> Light</button></li>
          <li><button class="dropdown-item" data-bs-theme-value="dark"><span class="fa fa-moon"></span> Dark</button></li>
          <li><button class="dropdown-item" data-bs-theme-value="auto"><span class="fa fa-adjust"></span> Auto</button></li>
        </ul>
      </li>

# simple components don't change without warning

    Code
      cat(navbar_html(menu_heading("a")))
    Output
      <li class="nav-item"><h6 class="dropdown-header" data-toc-skip>a</h6></li>
    Code
      cat(navbar_html(menu_link("a", "b")))
    Output
      <li class="nav-item"><a class="nav-link" href="b">a</a></li>
    Code
      cat(navbar_html(menu_separator()))
    Output
      <li class="nav-item"><hr class="dropdown-divider"></li>
    Code
      cat(navbar_html(menu_search()))
    Output
      <li class="nav-item"><form class="form-inline" role="search">
       <input class="form-control" type="search" name="search-input" id="search-input" autocomplete="off" aria-label="Search site" placeholder="Search for" data-search-index="search.json"> 
      </form></li>

---
title: "html_document + as_is"
output: 
  html_document:
    number_sections: true
pkgdown:
  as_is: true
---

```{r include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

This should have section numbers on the headings.

# Heading 1

# Heading 2

# sitrep complains about BS3

    Code
      pkgdown_sitrep(pkg)
    Message
      -- Sitrep ----------------------------------------------------------------------
      x Bootstrap 3 is deprecated; please switch to Bootstrap 5.
      i Learn more at <https://www.tidyverse.org/blog/2021/12/pkgdown-2-0-0/#bootstrap-5>.
      v URLs ok.
      v Favicons ok.
      v Open graph metadata ok.
      v Articles metadata ok.
      v Reference metadata ok.

# sitrep reports all problems

    Code
      pkgdown_sitrep(pkg)
    Message
      -- Sitrep ----------------------------------------------------------------------
      x URLs not ok.
        In DESCRIPTION, URL is missing package url (http://test.org).
        See details in `vignette(pkgdown::metadata)`.
      v Favicons ok.
      v Open graph metadata ok.
      v Articles metadata ok.
      x Reference metadata not ok.
        In _pkgdown.yml, 1 topic missing from index: "?".
        Either use `@keywords internal` to drop from index, or

# checks fails on first problem

    Code
      check_pkgdown(pkg)
    Condition
      Error in `check_pkgdown()`:
      ! In DESCRIPTION, URL is missing package url (http://test.org).
      i See details in `vignette(pkgdown::metadata)`.

# both inform if everything is ok

    Code
      pkgdown_sitrep(pkg)
    Message
      -- Sitrep ----------------------------------------------------------------------
      v URLs ok.
      v Favicons ok.
      v Open graph metadata ok.
      v Articles metadata ok.
      v Reference metadata ok.
    Code
      check_pkgdown(pkg)
    Message
      v No problems found.

# check_urls reports problems

    Code
      check_urls(pkg)
    Condition
      Error:
      ! In _pkgdown.yml, url is missing.
      i See details in `vignette(pkgdown::metadata)`.

---

    Code
      check_urls(pkg)
    Condition
      Error:
      ! In DESCRIPTION, URL is missing package url (https://testpackage.r-lib.org).
      i See details in `vignette(pkgdown::metadata)`.

# check_favicons reports problems

    Code
      check_favicons(pkg)
    Condition
      Error in `check_favicons()`:
      ! Found package logo but not favicons.
      i Do you need to run `build_favicons()`?

---

    Code
      check_favicons(pkg)
    Condition
      Error in `check_favicons()`:
      ! Package logo is newer than favicons.
      i Do you need to rerun `build_favicons()`?

# missing logo generates message

    Code
      expect_output(build_favicons(pkg), "Building favicons")
    Message
      -- Building favicons -----------------------------------------------------------
    Condition
      Error in `build_favicons()`:
      ! Can't find package logo PNG or SVG to build favicons.
      i See `usethis::use_logo()` for more information.

# existing logo generates message

    Code
      build_favicons(pkg)
    Message
      -- Building favicons -----------------------------------------------------------
      Favicons already exist in 'pkgdown'
      i Set `overwrite = TRUE` to re-create.

# can get info about external function

    Code
      str(ext_topics("base::mean"))
    Output
      tibble [1 x 12] (S3: tbl_df/tbl/data.frame)
       $ name     : chr "base::mean"
       $ file_in  : chr NA
       $ file_out : chr "https://rdrr.io/r/base/mean.html"
       $ alias    :List of 1
        ..$ : chr(0) 
       $ funs     :List of 1
        ..$ : chr "mean()"
       $ title    : chr "Arithmetic Mean (from base)"
       $ rd       :List of 1
        ..$ : chr(0) 
       $ source   : chr NA
       $ keywords :List of 1
        ..$ : chr(0) 
       $ concepts :List of 1
        ..$ : chr(0) 
       $ internal : logi FALSE
       $ lifecycle:List of 1
        ..$ : NULL

# fails if documentation not available

    Code
      ext_topics("base::doesntexist")
    Condition
      Error in `build_reference_index()`:
      ! Could not find documentation for `base::doesntexist()`.

# validates articles yaml

    Code
      data_articles_index_(1)
    Condition
      Error in `data_articles_index_()`:
      ! In _pkgdown.yml, articles must be a list, not the number 1.
    Code
      data_articles_index_(list(1))
    Condition
      Error in `data_articles_index_()`:
      ! In _pkgdown.yml, articles[1] must be a list, not the number 1.
    Code
      data_articles_index_(list(list()))
    Condition
      Error in `data_articles_index_()`:
      ! In _pkgdown.yml, articles[1] must have components "title" and "contents".
      2 missing components: "title" and "contents".
    Code
      data_articles_index_(list(list(title = 1, contents = 1)))
    Condition
      Error in `data_articles_index_()`:
      ! In _pkgdown.yml, articles[1].title must be a string, not the number 1.
    Code
      data_articles_index_(list(list(title = "a\n\nb", contents = 1)))
    Condition
      Error in `data_articles_index_()`:
      ! In _pkgdown.yml, articles[1].title must be inline markdown.
    Code
      data_articles_index_(list(list(title = "a", contents = 1)))
    Condition
      Error in `data_articles_index_()`:
      ! In _pkgdown.yml, articles[1].contents[1] must be a string.
      i You might need to add '' around special YAML values like 'N' or 'off'

# validates external-articles

    Code
      data_articles_(1)
    Condition
      Error in `data_articles_()`:
      ! In _pkgdown.yml, external-articles must be a list, not the number 1.
    Code
      data_articles_(list(1))
    Condition
      Error in `data_articles_()`:
      ! In _pkgdown.yml, external-articles[1] must be a list, not the number 1.
    Code
      data_articles_(list(list(name = "x")))
    Condition
      Error in `data_articles_()`:
      ! In _pkgdown.yml, external-articles[1] must have components "name", "title", "href", and "description".
      3 missing components: "title", "href", and "description".
    Code
      data_articles_(list(list(name = 1, title = "x", href = "x", description = "x")))
    Condition
      Error in `data_articles_()`:
      ! In _pkgdown.yml, external-articles[1].name must be a string, not the number 1.
    Code
      data_articles_(list(list(name = "x", title = 1, href = "x", description = "x")))
    Condition
      Error in `data_articles_()`:
      ! In _pkgdown.yml, external-articles[1].title must be a string, not the number 1.
    Code
      data_articles_(list(list(name = "x", title = "x", href = 1, description = "x")))
    Condition
      Error in `data_articles_()`:
      ! In _pkgdown.yml, external-articles[1].href must be a string, not the number 1.
    Code
      data_articles_(list(list(name = "x", title = "x", href = "x", description = 1)))
    Condition
      Error in `data_articles_()`:
      ! In _pkgdown.yml, external-articles[1].description must be a string, not the number 1.

# articles in vignettes/articles/ are unnested into articles/

    Code
      build_redirects(pkg)
    Message
      -- Building redirects ----------------------------------------------------------
      Adding redirect from articles/articles/nested.html to articles/nested.html.

# warns about articles missing from index

    Code
      . <- data_articles_index(pkg)
    Condition
      Error:
      ! In _pkgdown.yml, 1 vignette missing from index: "c".

# data_authors validates yaml inputs

    Code
      data_authors_(authors = 1)
    Condition
      Error in `data_authors_()`:
      ! In _pkgdown.yml, authors must be a list, not the number 1.
    Code
      data_authors_(authors = list(before = 1))
    Condition
      Error in `data_authors_()`:
      ! In _pkgdown.yml, authors.before must be a string, not the number 1.
    Code
      data_authors_(authors = list(after = 1))
    Condition
      Error in `data_authors_()`:
      ! In _pkgdown.yml, authors.after must be a string, not the number 1.

# data_home_sidebar_authors validates yaml inputs

    Code
      data_home_sidebar_authors_(authors = list(sidebar = list(roles = 1)))
    Condition
      Error in `data_home_sidebar_authors_()`:
      ! In _pkgdown.yml, authors.sidebar.roles must be a character vector, not the number 1.
    Code
      data_home_sidebar_authors_(authors = list(sidebar = list(before = 1)))
    Condition
      Error in `data_home_sidebar_authors_()`:
      ! In _pkgdown.yml, authors.sidebar.before must be a string, not the number 1.
    Code
      data_home_sidebar_authors_(authors = list(sidebar = list(before = "x\n\ny")))
    Condition
      Error in `data_home_sidebar_authors_()`:
      ! In _pkgdown.yml, authors.sidebar.before must be inline markdown.

# sidebar can accept additional before and after text

    Code
      cat(data_home_sidebar_authors(pkg))
    Output
      <div class='developers'>
      <h2 data-toc-skip>Developers</h2>
      <ul class='list-unstyled'>
      <li>BEFORE</li>
      <li>Jo Doe <br />
      <small class = 'roles'> Author, maintainer </small>   </li>
      <li>AFTER</li>
      </ul>
      </div>

# role has multiple fallbacks

    Code
      role_lookup("unknown")
    Condition
      Warning:
      Unknown MARC role abbreviation: unknown
    Output
      [1] "unknown"

# multiple citations all have HTML and BibTeX formats

    [[1]]
    [[1]]$html
    [1] "<p>A &amp; B (2021): Proof of b &lt; a &gt; c.</p>"
    
    [[1]]$bibtex
    [1] "@Misc{,\n  title = {Proof of b < a > c},\n  author = {{A} and {B}},\n  year = {2021},\n}"
    
    
    [[2]]
    [[2]]$html
    [1] "<p>Two A (2022).\n&ldquo;Title Two.&rdquo; \n</p>"
    
    [[2]]$bibtex
    [1] "@Misc{,\n  title = {Title Two},\n  author = {Author Two},\n  year = {2022},\n}"
    
    

# bad inputs give informative warnings

    Code
      select_topics_("x + ", topics)
    Condition
      Error in `select_topics_()`:
      ! In _pkgdown.yml, reference[1].contents[1] (x + ) must be valid R code.
    Code
      select_topics_("y", topics)
    Condition
      Error in `select_topics_()`:
      ! In _pkgdown.yml, reference[1].contents[1] (y) must be a known topic name or alias.
    Code
      select_topics_("paste(1)", topics)
    Condition
      Error in `select_topics_()`:
      ! In _pkgdown.yml, reference[1].contents[1] (paste(1)) failed to evaluate.
      Caused by error in `paste()`:
      ! could not find function "paste"
    Code
      select_topics_("starts_with", topics)
    Condition
      Error in `select_topics_()`:
      ! In _pkgdown.yml, reference[1].contents[1] (starts_with) must be a known topic name or alias.
    Code
      select_topics_("1", topics)
    Condition
      Error in `select_topics_()`:
      ! In _pkgdown.yml, reference[1].contents[1] (1) must be a string or function call.
    Code
      select_topics_("starts_with('y')", topics)
    Condition
      Error in `select_topics_()`:
      ! In _pkgdown.yml, reference[1].contents failed to match any topics.

# selector functions validate their inputs

    Code
      select_topics_("starts_with('x', 'y')", topics)
    Condition
      Error in `select_topics_()`:
      ! In _pkgdown.yml, reference[1].contents[1] (starts_with('x', 'y')) failed to evaluate.
      Caused by error in `starts_with()`:
      ! `internal` must be `TRUE` or `FALSE`, not the string "y".
    Code
      select_topics_("starts_with(c('x', 'y'))", topics)
    Condition
      Error in `select_topics_()`:
      ! In _pkgdown.yml, reference[1].contents[1] (starts_with(c('x', 'y'))) failed to evaluate.
      Caused by error in `starts_with()`:
      ! `x` must be a single string, not a character vector.

# can select by name or alias

    Code
      select_topics_("a4", topics)
    Condition
      Error in `select_topics_()`:
      ! In _pkgdown.yml, reference[1].contents[1] (a4) must be a known topic name or alias.
    Code
      select_topics_("c::a", topics)
    Condition
      Error in `select_topics_()`:
      ! In _pkgdown.yml, reference[1].contents[1] (c::a) must be a known topic name or alias.

# can combine positive and negative selections

    Code
      select_topics_("c(a, -x)", topics)
    Condition
      Error in `select_topics_()`:
      ! In _pkgdown.yml, reference[1].contents[1] (c(a, -x)) must be all negative or all positive.

# an unmatched selection generates a warning

    Code
      select_topics_(c("a", "starts_with('unmatched')"), topics)
    Condition
      Error in `select_topics_()`:
      ! In _pkgdown.yml, reference[1].contents (starts_with('unmatched')) must match a function or concept.

---
author: "Hadley Wickham"
title: "Introduction to Open Graph"
description: "The Open Graph protocol is a standard for web page metadata."
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to Open Graph}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Content goes here

# converts Rd unicode shortcuts

    Code
      rd2html("``a -- b --- c''")
    Output
      [1] "“a – b — c”"

# subsection generates h3

    Code
      cli::cat_line(rd2html("\\subsection{A}{B}"))
    Output
      <div class='section' id='a'>
      <h3>A</h3>
      <p>B</p>
      </div>

---

    Code
      cli::cat_line(rd2html("\\subsection{A}{\n    p1\n\n    p2\n  }"))
    Output
      <div class='section' id='a'>
      <h3>A</h3>
      <p>p1</p>
      <p>p2</p>
      </div>

# nested subsection generates h4

    Code
      cli::cat_line(rd2html("\\subsection{H3}{\\subsection{H4}{}}"))
    Output
      <div class='section' id='h-'>
      <h3>H3</h3>
      <div class='section' id='h-'>
      <h4>H4</h4>
      
      </div>
      </div>

# Sexprs in file share environment

    Code
      rd2html("\\Sexpr{x}")
    Condition
      Error:
      ! object 'x' not found

# bad specs throw errors

    Code
      rd2html("\\url{}")
    Condition
      Error:
      ! Failed to parse tag "\\url{}".
      i Check for empty \url{} tags.
    Code
      rd2html("\\url{a\nb}")
    Condition
      Error:
      ! Failed to parse tag "\\url{}".
      i This may be caused by a \url tag that spans a line break.
    Code
      rd2html("\\email{}")
    Condition
      Error:
      ! Failed to parse tag "\\email{}".
      i empty
    Code
      rd2html("\\linkS4class{}")
    Condition
      Error:
      ! Failed to parse tag "\\linkS4class{}".

# \describe items can contain multiple paragraphs

    <dl>
    <dt>Label 1</dt>
    <dd><p>Contents 1</p></dd>
    
    <dt>Label 2</dt>
    <dd><p>Contents 2</p></dd>
    
    
    </dl>

---

    <dl>
    <dt>Label</dt>
    <dd><p>Paragraph 1</p>
    <p>Paragraph 2</p></dd>
    
    
    </dl>

# can add ids to descriptions

    <dl>
    <dt id='fooabc'>abc<a class='anchor' aria-label='anchor' href='#fooabc'></a></dt>
    <dd><p>Contents 1</p></dd>
    
    <dt id='fooxyz'>xyz<a class='anchor' aria-label='anchor' href='#fooxyz'></a></dt>
    <dd><p>Contents 2</p></dd>
    
    
    </dl>

# nested item with whitespace parsed correctly

    <dl>
    <dt>Label</dt>
    <dd>
    
    <p>This text is indented in a way pkgdown doesn't like.</p></dd>
    
    </dl>

# missing template package yields custom error

    Code
      path_package_pkgdown("x", "missing", 3)
    Condition
      Error:
      ! Template package "missing" is not installed.
      i Please install before continuing.

# out_of_date works as expected

    Code
      out_of_date("doesntexist", temp1)
    Condition
      Error:
      ! 'doesntexist' does not exist

# is_pkgdown checks its inputs

    Code
      as_pkgdown(1)
    Condition
      Error in `as_pkgdown()`:
      ! `pkg` must be a single string, not the number 1.
    Code
      as_pkgdown(override = 1)
    Condition
      Error in `as_pkgdown()`:
      ! `override` must be a list, not the number 1.

# check_bootstrap_version() allows 3, 4 (with warning), and 5

    Code
      expect_equal(check_bootstrap_version(4, pkg), 5)
    Condition
      Warning:
      In _pkgdown.yml, `template.bootstrap: 4` no longer supported
      i Using `template.bootstrap: 5` instead

# check_bootstrap_version() gives informative error otherwise

    Code
      check_bootstrap_version(1, pkg)
    Condition
      Error in `check_bootstrap_version()`:
      ! In _pkgdown.yml, template.bootstrap must be 3 or 5, not 1.

# package_vignettes() detects conflicts in final article paths

    Code
      package_vignettes(dir)
    Condition
      Error in `package_vignettes()`:
      ! Rendered articles must have unique names. Rename or relocate:
      x vignettes/articles/test.Rmd and vignettes/test.Rmd

# read_meta() errors gracefully if _pkgdown.yml failed to parse

    Code
      as_pkgdown(pkg$src_path)
    Condition
      Error in `as_pkgdown()`:
      ! Could not parse config file at '<src>/_pkgdown.yml'.
      Caused by error in `yaml.load()`:
      ! Scanner error: mapping values are not allowed in this context at line 2, column 8

# parse failures include file name

    Code
      build_reference(pkg)
    Message
      -- Building function reference -------------------------------------------------
      Writing `reference/index.html`
      Reading man/f.Rd
    Condition
      Error in `build_reference()`:
      ! Failed to parse Rd in 'f.Rd'
      Caused by error:
      ! Failed to parse tag "\\url{}".
      i Check for empty \url{} tags.

# examples are reproducible by default, i.e. 'seed' is respected

    Code
      cat(examples)
    Output
      #> [1] 0.600760886 0.157208442 0.007399441 0.466393497 0.497777389

# check images in readme

    Code
      check_built_site(pkg)
    Message
      -- Checking for problems -------------------------------------------------------
      Missing images in 'README.md': 'articles/kitten.jpg'
      i pkgdown can only use images in 'man/figures' and 'vignettes'

# capture data_template()

    package:
      name: testpackage
      version: 1.0.0
    site:
      root: ''
      title: testpackage
    year: <year>
    lang: en
    translate:
      skip: Skip to contents
      toggle_nav: Toggle navigation
      on_this_page: On this page
      source: Source
      abstract: Abstract
      authors: Authors
      version: Version
      examples: Examples
      citation: Citation
      author_details: Additional details
      toc: Table of contents
      site_nav: Site navigation
    has_favicons: no
    opengraph: []
    extra:
      css: ~
      js: ~
    yaml:
      .present: yes
    headdeps: ''
    development:
      destination: dev
      mode: default
      version_label: muted
      in_dev: no
      prefix: ''
      version_tooltip: ''
    navbar:
      bg: light
      type: light
      left: <li class="nav-item"><a class="nav-link" href="reference/index.html">Reference</a></li>
      right: "<li class=\"nav-item\"><form class=\"form-inline\" role=\"search\">\n <input
        class=\"form-control\" type=\"search\" name=\"search-input\" id=\"search-input\"
        autocomplete=\"off\" aria-label=\"Search site\" placeholder=\"Search for\" data-search-index=\"search.json\">
        \n</form></li>"
    footer:
      left: <p>Developed by Jo Doe.</p>
      right: <p>Site built with <a href="https://pkgdown.r-lib.org/">pkgdown</a> {version}.</p>
    lightswitch: no
    uses_katex: no
    

# check_opengraph validates inputs

    Code
      data_open_graph_(list(foo = list()))
    Condition
      Warning in `data_open_graph_()`:
      In _pkgdown.yml, template.opengraph contains unsupported fields "foo".
    Code
      data_open_graph_(list(foo = list(), bar = list()))
    Condition
      Warning in `data_open_graph_()`:
      In _pkgdown.yml, template.opengraph contains unsupported fields "foo" and "bar".
    Code
      data_open_graph_(list(twitter = 1))
    Condition
      Error in `data_open_graph_()`:
      ! In _pkgdown.yml, template.opengraph.twitter must be a list, not the number 1.
    Code
      data_open_graph_(list(twitter = list()))
    Condition
      Error in `data_open_graph_()`:
      ! In _pkgdown.yml, opengraph.twitter must include either creator or site.
    Code
      data_open_graph_(list(image = 1))
    Condition
      Error in `data_open_graph_()`:
      ! In _pkgdown.yml, template.opengraph.image must be a list, not the number 1.

# highlight_examples runs and hides DONTSHOW calls()

    Code
      cat(strip_html_tags(out))
    Output
      x
      #&gt; [1] 1

# pre() can produce needed range of outputs

    Code
      cat(pre("x"))
    Output
      <pre><code>x</code></pre>
    Code
      cat(pre("x", r_code = TRUE))
    Output
      <pre class='sourceCode r'><code>x</code></pre>

# tweak_highlight_other() renders nested code blocks for roxygen2 >= 7.2.0

    Code
      cat(xpath_text(div, "pre/code"))
    Output
      
      blablabla
      
      ```{r results='asis'}
      lalala
      ```

# we find out if quarto styles change

    Code
      cat(data$includes$style)
    Output
      
          code{white-space: pre-wrap;}
          span.smallcaps{font-variant: small-caps;}
          div.columns{display: flex; gap: min(4vw, 1.5em);}
          div.column{flex: auto; overflow-x: auto;}
          div.hanging-indent{margin-left: 1.5em; text-indent: -1.5em;}
          ul.task-list{list-style: none;}
          ul.task-list li input[type="checkbox"] {
            width: 0.8em;
            margin: 0 0.8em 0.2em -1em; /* quarto-specific, see https://github.com/quarto-dev/quarto-cli/issues/4556 */ 
            vertical-align: middle;
          }
        

# build_redirect() works

    Code
      build_redirect(c("old.html", "new.html#section"), 1, pkg = pkg)
    Message
      Adding redirect from old.html to new.html#section.

# build_redirect() errors if one entry is not right.

    Code
      data_redirects_(redirects = "old.html")
    Condition
      Error in `data_redirects_()`:
      ! In _pkgdown.yml, redirects must be a list, not the string "old.html".
    Code
      data_redirects_(redirects = list("old.html"))
    Condition
      Error in `data_redirects_()`:
      ! In _pkgdown.yml, redirects[1] must be a character vector of length 2, not the string "old.html".

# check_bslib_theme() works

    Code
      check_bslib_theme("paper", pkg, bs_version = 4)
    Condition
      Error:
      x In _pkgdown.yml, template.bootswatch contains unknown Bootswatch/bslib theme "paper".
      i Using Bootstrap version 4 (template.bootstrap).

# validations yaml specification

    Code
      build_bslib_(theme = 1)
    Condition
      Error in `bs_theme_rules()`:
      ! In _pkgdown.yml, template.theme must be a string, not the number 1.
    Code
      build_bslib_(theme = "fruit")
    Condition
      Error in `build_bslib_()`:
      ! In _pkgdown.yml, template.theme uses theme "fruit"
    Code
      build_bslib_(`theme-dark` = "fruit")
    Condition
      Error in `build_bslib_()`:
      ! In _pkgdown.yml, template.theme-dark uses theme "fruit"

# pkgdown_field(s) produces useful description

    Code
      pkgdown_field(pkg, c("a", "b"))
    Output
      [1] "a.b in '_pkgdown.yml'"
    Code
      pkgdown_fields(pkg, list(c("a", "b"), "c"))
    Output
      [1] "a.b, c in '_pkgdown.yml'"

---

    Code
      check_yaml_has("x", where = "a", pkg = pkg)
    Condition
      Error in `check_yaml_has()`:
      ! Can't find component a.x in '_pkgdown.yml'.
    Code
      check_yaml_has(c("x", "y"), where = "a", pkg = pkg)
    Condition
      Error in `check_yaml_has()`:
      ! Can't find components a.x, a.y in '_pkgdown.yml'.

# a CNAME record is built if a url exists in metadata

    Code
      build_github_pages(pkg)
    Message
      -- Extra files for GitHub pages ------------------------------------------------
      Writing `.nojekyll`
      Writing `CNAME`

# works by default

    $left
    [1] "<p>Developed by First Last.</p>"
    
    $right
    [1] "<p>Site built with <a href=\"https://pkgdown.r-lib.org/\">pkgdown</a> {version}.</p>"
    

# validates meta components

    Code
      data_footer_(footer = 1)
    Condition
      Error in `data_footer_()`:
      ! In _pkgdown.yml, footer must be a list, not the number 1.
    Code
      data_footer_(footer = list(structure = 1))
    Condition
      Error in `data_footer_()`:
      ! In _pkgdown.yml, footer.structure must be a list, not the number 1.
    Code
      data_footer_(footer = list(components = 1))
    Condition
      Error in `data_footer_()`:
      ! In _pkgdown.yml, footer.components must be a list, not the number 1.
    Code
      data_footer_(authors = list(footer = list(roles = 1)))
    Condition
      Error in `data_footer_()`:
      ! In _pkgdown.yml, authors.footer.roles must be a character vector, not the number 1.
    Code
      data_footer_(authors = list(footer = list(text = 1)))
    Condition
      Error in `data_footer_()`:
      ! In _pkgdown.yml, authors.footer.text must be a string, not the number 1.

---
title: "Output Rendering"
subtitle: "Subtitle is skipped from the TOC"
author: "Author is skipped from the TOC"
date: "Date is skipped from the TOC"
---

```{r include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

This vignette tests pkgdown output rendering for several use cases.

## Footnotes

Yay[^footnote]

[^footnote]: Including **footnotes**! 😁

## Figures

```{r}
plot(1:10)
```

## External files

```{r}
x <- readLines("test.txt")
x
```

![](bacon.jpg)

## Details tag

<details>
This should only be shown when required
</details>

<details>
<summary>Multiple paragraphs</summary>
First paragraph

Second paragraph
</details>

<details>
<summary>Some R code</summary>
  
```{r}
1 + 2
```
</details>

## Tables

| col 1 | col 2 | col 3 | col 4 |
|:---------------|:-----------|:-------:|:---------:|
| Brightness     | Total brightness, total reflectance, spectral intensity | $$y = x^2$$ | test |

## Mathjax

$$f(x) = \dfrac{1}{\sqrt{2\pi\sigma^2}} e^{-\frac{(x-\mu^2)}{2\sigma^2}}$$

Inline equations: $y=x^2$

## Code

### Line width

```{r}
pkgdown:::ruler()

cat(rep("x ", 100), sep = "")
cat(rep("xy", 100), sep = "")
```

### Should be highlighted

Valid R code in `\preformatted{}`:

```
mean(a + 1)
```

R code in `R` block:

```R
mean(a + 1)
```

R code in `r` block:

```R
mean(a + 1)
```

Yaml

```yaml
yaml: [a, 1]
```

### Shouldn't be highlighted

Non-R code in `\preformatted{}`

```
yaml: [a, b, c]
```

### Crayon

```{r}
cat(cli::col_red("This is red"), "\n")
cat(cli::col_blue("This is blue\n"), "\n")

message(cli::col_green("This is green"))

warning(cli::style_bold("This is bold"))
```
Some text

```{r, error = TRUE}
stop(cli::style_italic("This is italic"))
```

Some more text

## Quoted text

> Single-line quote about something miscellaneous.

| Flush
|  1 space indent
|   2 space indent
|    3 space indent

## This section is unnumbered {-}

There should however be no bug here!

## Tabsets

### Tabset with pills {.tabset .tabset-pills}

#### Tab 1

blablablabla

```r
1 + 1
```

Should be "cool" heading below

##### cool

Stuff

#### Tab 2

blop

### Tabset without pils {.tabset}

#### Tab 1

something nice

```{r}
plot(1:42)
```

#### Tab 2 {.active}

This tab should be active

### Fading tabset {.tabset .tabset-fade}

#### English

Hello!

#### French

Bonjour!

#### German

Guten tag.

## Deep headings

### Heading 3

#### Heading 4

##### Heading 5

# build_search_index() has expected structure

    Code
      str(build_search_index(pkg))
    Output
      List of 1
       $ :List of 8
        ..$ path             : chr "https://example.com/index.html"
        ..$ id               : chr "my-package"
        ..$ dir              : chr ""
        ..$ previous_headings: chr ""
        ..$ what             : chr "A test package"
        ..$ title            : chr "A test package"
        ..$ text             : chr "pakage "
        ..$ code             : chr ""

# build sitemap only messages when it updates

    Code
      build_sitemap(pkg)
    Message
      -- Building sitemap ------------------------------------------------------------
      Writing `sitemap.xml`
    Code
      build_sitemap(pkg)
    Message
      -- Building sitemap ------------------------------------------------------------

---
author: "Hadley Wickham"
title: "R Markdown Vignette with an Image"
description: "The image is in a subdirectory, and we must be sure to copy it."
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{R Markdown Vignette with an Image}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Some words, and then an image like this:

```{r, echo = FALSE}
knitr::include_graphics("open-graph/logo.png")
```

---
title: "Long TOC"
---

```{r include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

This vignette tests has a very long TOC. You should be able to navigate to a heading near the bottom of the doc without having to scroll all the way down.

```{r, echo = FALSE}
h2 <- paste0("## Heading ", t(outer(letters, 1:9, paste0)))
knitr::asis_output(paste(h2, collapse = "\n\n"))
```

# markdown_text_inline() works with inline markdown

    Code
      markdown_text_inline(pkg, "x\n\ny", error_path = "title")
    Condition
      Error:
      ! In _pkgdown.yml, title must be inline markdown.

# validates math yaml

    Code
      config_math_rendering_(`math-rendering` = 1)
    Condition
      Error in `config_math_rendering_()`:
      ! In _pkgdown.yml, template.math-rendering must be a string, not the number 1.
    Code
      config_math_rendering_(`math-rendering` = "math")
    Condition
      Error in `config_math_rendering_()`:
      ! In _pkgdown.yml, template.math-rendering must be one of mathml, mathjax, and katex, not math.

# preserves ANSI characters

    Code
      markdown_text(pkg, sprintf("prefer %s", cli::col_blue("a")))
    Output
      {html_document}
      <html>
      [1] <body><p>prefer \u2029[34ma\u2029[39m</p></body>

# repo_source() truncates automatically

    Code
      cat(repo_source(pkg, character()))
      cat(repo_source(pkg, "a"))
    Output
      Source: <a href='https://github.com/r-lib/pkgdown/blob/HEAD/a'><code>a</code></a>
    Code
      cat(repo_source(pkg, letters[1:10]))
    Output
      Source: <a href='https://github.com/r-lib/pkgdown/blob/HEAD/a'><code>a</code></a>, <a href='https://github.com/r-lib/pkgdown/blob/HEAD/b'><code>b</code></a>, <a href='https://github.com/r-lib/pkgdown/blob/HEAD/c'><code>c</code></a>, and 7 more

# config_pluck_character generates informative error

    Code
      config_pluck_character(pkg, "x")
    Condition
      Error:
      ! In _pkgdown.yml, x must be a character vector, not the number 1.

# config_pluck_string generates informative error

    Code
      config_pluck_string(pkg, "x")
    Condition
      Error:
      ! In _pkgdown.yml, x must be a string, not the number 1.

# config_check_list gives informative errors

    Code
      config_check_list_(1, has_names = "x")
    Condition
      Error in `config_check_list_()`:
      ! In _pkgdown.yml, path must be a list, not the number 1.
    Code
      config_check_list_(list(x = 1, y = 1), has_names = c("y", "z"))
    Condition
      Error in `config_check_list_()`:
      ! In _pkgdown.yml, path must have components "y" and "z".
      1 missing component: "z".

---
title: quarto features
knitr:
  opts_chunk: 
    collapse: true
    comment: '#>'
bibliography: jss.bib
---

## Citations

* Citation: @JSSv059i10

## Code annotations

```r
library(tidyverse)
library(palmerpenguins)
penguins |>                                      # <1>
  mutate(                                        # <2>
    bill_ratio = bill_depth_mm / bill_length_mm, # <2>
    bill_area  = bill_depth_mm * bill_length_mm  # <2>
  )                                              # <2>
```
1. Take `penguins`, and then,
2. add new columns for the bill ratio and bill area.

## Diagrams

```{mermaid}
flowchart LR
  A[Hard edge] --> B(Round edge)
  B --> C{Decision}
  C --> D[Result one]
  C --> E[Result two]
```

## HTML widgets

```{r, echo=FALSE}
dir <- tempdir()
path1 <- file.path(dir, "a.txt")
writeLines(letters, path1)
path2 <- file.path(dir, "b.txt")
writeLines(letters[-(10:11)], path2)

diffviewer::visual_diff(path1, path2)
```

## Keyboard

* Keyboard shortcut: {{< kbd Shift-Ctrl-P >}}

## References

---
title: "HTML Deps"
---

```{r}
# Call some code that adds jquery/bootstrap
htmltools::tagList(
  htmltools::p("hello"),
  rmarkdown::html_dependency_jquery(),
  rmarkdown::html_dependency_bootstrap("flatly")
)
```


```{r}
knitr::knit_hooks$set(document = function(x) {
  if (interactive()) {
    browser()
  }
  x
})
```

---
title: "Tutorial"
output: learnr::tutorial
runtime: shiny_prerendered
---

```{r setup, include=FALSE}
library(learnr)
knitr::opts_chunk$set(echo = FALSE)
```


## Topic 1

### Exercise 

*Here's a simple exercise with an empty code chunk provided for entering the answer.*

Write the R code required to add two plus two:

```{r two-plus-two, exercise=TRUE}

```

### Exercise with Code

*Here's an exercise with some prepopulated code as well as `exercise.lines = 5` to provide a bit more initial room to work.*

Now write a function that adds any two numbers and then call it:

```{r add-function, exercise=TRUE, exercise.lines = 5}
add <- function() {
  
}
```

## Topic 2

### Exercise with Hint

*Here's an exercise where the chunk is pre-evaulated via the `exercise.eval` option (so the user can see the default output we'd like them to customize). We also add a "hint" to the correct solution via the chunk immediate below labeled `print-limit-hint`.*

Modify the following code to limit the number of rows printed to 5:

```{r print-limit, exercise=TRUE, exercise.eval=TRUE}
mtcars
```

```{r print-limit-hint}
head(mtcars)
```

### Quiz

*You can include any number of single or multiple choice questions as a quiz. Use the `question` function to define a question and the `quiz` function for grouping multiple questions together.*

Some questions to verify that you understand the purposes of various base and recommended R packages:

```{r quiz}
quiz(
  question("Which package contains functions for installing other R packages?",
    answer("base"),
    answer("tools"),
    answer("utils", correct = TRUE),
    answer("codetools")
  ),
  question("Which of the R packages listed below are used to create plots?",
    answer("lattice", correct = TRUE),
    answer("tools"),
    answer("stats"),
    answer("grid", correct = TRUE)
  )
)
```

# check integrity validates integrity

    Code
      check_integrity(temp, "sha123-abc")
    Condition
      Error in `check_integrity()`:
      ! integrity must use SHA-256, SHA-384, or SHA-512
      i This is an internal error that was detected in the pkgdown package.
        Please report it at <https://github.com/r-lib/pkgdown/issues> with a reprex (<https://tidyverse.org/help/>) and the full backtrace.
    Code
      check_integrity(temp, "sha256-abc")
    Condition
      Error in `check_integrity()`:
      ! Downloaded asset does not match known integrity
      i This is an internal error that was detected in the pkgdown package.
        Please report it at <https://github.com/r-lib/pkgdown/issues> with a reprex (<https://tidyverse.org/help/>) and the full backtrace.

# @examplesIf

    Code
      expect_equal(strtrim(rd2ex(rd3), 40), strtrim(exp3, 40))
    Condition
      Warning:
      @examplesIf condition "TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && TRUE && FALSE" is "FALSE"

# anchor html added to headings

    <h1 id="x">abc<a class="anchor" aria-label="anchor" href="#x"/></h1>

# can process footnote with code

    Code
      xpath_attr(html, ".//a", "data-bs-content")
    Output
      [1] "<p>Including code:</p>\n<pre><code>1 +\n2 +</code></pre>\n<p>And more text</p>"

# can remove logo

    Code
      xpath_xml(html, ".//div")
    Output
      <div class="page-header">
      <img src="mylogo.png" class="logo" alt=""><h1>First </h1>
      </div>

---

    Code
      xpath_xml(html, ".//div")
    Output
      <div class="page-header">
      <img src="mylogo.png" class="logo" alt=""><h1>First </h1>
      </div>

# can move badges to sidebar

    Code
      xpath_xml(html, ".//div")
    Output
      <div class="dev-status">
      <h2 data-toc-skip>Dev status</h2>
      <ul class="list-unstyled">
      <li><a href="x"><img src="y"></a></li>
      </ul>
      </div>

# remove badges even if no dev-status div

    Code
      html
    Output
      {html_document}
      <html>
      [1] <body>\n<h1>Title</h1>\n    </body>

# data_news works as expected for h1 & h2

    # A tibble: 2 x 3
      version    page  anchor             
      <chr>      <chr> <chr>              
    1 1.0.0.9000 dev   testpackage-1009000
    2 1.0.0      1.0   testpackage-100    

---

    # A tibble: 2 x 3
      version    page  anchor             
      <chr>      <chr> <chr>              
    1 1.0.0.9000 dev   testpackage-1009000
    2 1.0.0      1.0   testpackage-100    

# multi-page news are rendered

    Code
      data_news(pkg)[c("version", "page", "anchor")]
    Output
      # A tibble: 4 x 3
        version page  anchor         
        <chr>   <chr> <chr>          
      1 2.0     2.0   testpackage-20 
      2 1.1     1.1   testpackage-11 
      3 1.0.1   1.0   testpackage-101
      4 1.0.0   1.0   testpackage-100

---

    Code
      build_news(pkg)
    Message
      -- Building news ---------------------------------------------------------------
      Writing `news/news-2.0.html`
      Writing `news/news-1.1.html`
      Writing `news/news-1.0.html`
      Writing `news/index.html`

# news headings get class and release date

    <div>
      <h2 class="page-header" data-toc-text="1.0">
        <small>2020-01-01</small>
      </h2>
    </div>

---

    <div>
      <h2 class="pkg-version" data-toc-text="1.0"/>
      <p class="text-muted">CRAN release: 2020-01-01</p>
    </div>

# clear error for bad hierarchy - bad nesting

    Code
      data_news(pkg)
    Condition
      Error:
      ! In NEWS.md, inconsistent use of section headings.
      i Top-level headings must be either all <h1> or all <h2>.
      i See `?pkgdown::build_news()` for more details.

# clear error for bad hierarchy - h3

    Code
      data_news(pkg)
    Condition
      Error:
      ! In NEWS.md, inconsistent use of section headings.
      i Top-level headings must be either all <h1> or all <h2>.
      i See `?pkgdown::build_news()` for more details.

# news can contain footnotes

    Code
      x <- data_news(pkg)
    Condition
      Warning:
      Footnotes in NEWS.md are not currently supported

# data_news warns if no headings found

    Code
      . <- data_news(pkg)
    Condition
      Warning:
      In NEWS.md, no version headings found
      i See `?pkgdown::build_news()` for expected structure.

---
author:
  - name: FirstName LastName
    affiliation: University/Company
    address: >
      First line
      Second line
    email: \email{name@company.com}
    url: https://rstudio.com
  - name: Second Author
    affiliation: Affiliation
title:
  plain:     "JSS article"
  formatted: "JSS article"
  short:     "JSS article"
pagetitle:   "JSS article"
abstract: >
  The abstract of the article.
keywords:
  plain:     [keywords, not capitalized, Java]
  formatted: [keywords, not capitalized, "\\proglang{Java}"]
output: rticles::jss_article
documentclass: jss
classoption: nojss
bibliography: jss.bib
pkgdown:
  as_is: true
  extension: pdf
---

# Introduction

Some text and a reference [@JSSv059i10].

```{r}
1 +
  2
```

---
title: "kitten"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{kitten}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## `knitr::include_graphics()`

``` {r}
knitr::include_graphics("../man/figures/kitten.jpg")
```

``` {r}
knitr::include_graphics("another-kitten.jpg")
```

## rmarkdown

![](../man/figures/kitten.jpg)

![](another-kitten.jpg)

## External package

```{r magick}
magick::image_read("another-kitten.jpg")
```

## Plot

```{r plot}
plot(1:3)
```

# informative print method

    Code
      init_site(pkg)
    Message
      -- Initialising site -----------------------------------------------------------
      Copying <pkgdown>/BS5/assets/katex-auto.js to katex-auto.js
      Copying <pkgdown>/BS5/assets/lightswitch.js to lightswitch.js
      Copying <pkgdown>/BS5/assets/link.svg to link.svg
      Copying <pkgdown>/BS5/assets/pkgdown.js to pkgdown.js
      Updating deps/bootstrap-5.3.1/bootstrap.bundle.min.js
      Updating deps/bootstrap-5.3.1/bootstrap.bundle.min.js.map
      Updating deps/bootstrap-5.3.1/bootstrap.min.css
      Updating deps/bootstrap-toc-1.0.1/bootstrap-toc.min.js
      Updating deps/clipboard.js-2.0.11/clipboard.min.js
      Updating deps/font-awesome-6.4.2/css/all.css
      Updating deps/font-awesome-6.4.2/css/all.min.css
      Updating deps/font-awesome-6.4.2/css/v4-shims.css
      Updating deps/font-awesome-6.4.2/css/v4-shims.min.css
      Updating deps/font-awesome-6.4.2/webfonts/fa-brands-400.ttf
      Updating deps/font-awesome-6.4.2/webfonts/fa-brands-400.woff2
      Updating deps/font-awesome-6.4.2/webfonts/fa-regular-400.ttf
      Updating deps/font-awesome-6.4.2/webfonts/fa-regular-400.woff2
      Updating deps/font-awesome-6.4.2/webfonts/fa-solid-900.ttf
      Updating deps/font-awesome-6.4.2/webfonts/fa-solid-900.woff2
      Updating deps/font-awesome-6.4.2/webfonts/fa-v4compatibility.ttf
      Updating deps/font-awesome-6.4.2/webfonts/fa-v4compatibility.woff2
      Updating deps/headroom-0.11.0/headroom.min.js
      Updating deps/headroom-0.11.0/jQuery.headroom.min.js
      Updating deps/jquery-3.6.0/jquery-3.6.0.js
      Updating deps/jquery-3.6.0/jquery-3.6.0.min.js
      Updating deps/jquery-3.6.0/jquery-3.6.0.min.map
      Updating deps/search-1.0.0/autocomplete.jquery.min.js
      Updating deps/search-1.0.0/fuse.min.js
      Updating deps/search-1.0.0/mark.min.js

# site meta doesn't break unexpectedly

    Code
      yaml
    Output
      pandoc: '{version}'
      pkgdown: '{version}'
      pkgdown_sha: '{sha}'
      articles:
        a: a.html
        b: b.html
      last_built: 2020-01-01T00:00Z
      

# build_article yields useful error if R fails

    Code
      build_article("test", pkg)
    Message
      Reading vignettes/test.Rmd
    Condition
      Error in `build_article()`:
      ! Failed to render 'vignettes/test.Rmd'.
      x Quitting from test.Rmd:4-10 [unnamed-chunk-1]
      x ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      x <error/rlang_error>
      x Error in `h()`:
      x ! Error!
      x ---
      x Backtrace:
      x  ▆
      x  1. └─global f()
      x  2.  └─global g()
      x  3.  └─global h()
      x ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      Caused by error:
      ! Error!

---

    Code
      summary(expect_error(build_article("test", pkg)))
    Message
      Reading vignettes/test.Rmd
    Output
      <error/rlang_error>
      Error in `build_article()`:
      ! Failed to render 'vignettes/test.Rmd'.
      x Quitting from test.Rmd:4-10 [unnamed-chunk-1]
      x ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      x <error/rlang_error>
      x Error in `h()`:
      x ! Error!
      x ---
      x Backtrace:
      x  ▆
      x  1. └─global f()
      x  2.  └─global g()
      x  3.  └─global h()
      x ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      Caused by error:
      ! Error!
      ---
      Backtrace:
          x
       1. \-global f()
       2.   \-global g()
       3.     \-global h()

# dd with block elements simplifies correctly

    Code
      xpath_xml(html, ".//li")
    Output
      <li>
      <p>a: </p>
                <p>b</p>
                <p>c</p>
              </li>

# warns if not applied

    Code
      . <- simplify_dls(html)
    Condition
      Warning:
      Skipping this <dl>: not a simple term-definition list

---
author: "Mara Averick"
title: "Introduction to Open Graph"
description: "The Open Graph protocol is a standard for web page metadata."
opengraph:
  description: This should be ignored
  image: 
    src: "http://example.com/pkg/batpig.png"
  twitter:
    card: summary
    creator: "@dataandme"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to Open Graph}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Content goes here

# multiple citations all have HTML and BibTeX formats

    [[1]]
    [[1]]$html
    [1] "<p>A &amp; B (2021): Proof of b &lt; a &gt; c.</p>"
    
    [[1]]$bibtex
    [1] "@Misc{,\n  title = {Proof of b < a > c},\n  author = {{A} and {B}},\n  year = {2021},\n}"
    
    
    [[2]]
    [[2]]$html
    [1] "<p>Two A (2022).\n&ldquo;Title Two.&rdquo; \n</p>"
    
    [[2]]$bibtex
    [1] "@Misc{,\n  title = {Title Two},\n  author = {Author Two},\n  year = {2022},\n}"
    
    

# render_rmarkdown yields useful error

    Code
      render_rmarkdown(pkg, "assets/pandoc-fail.Rmd", "test.html", output_format = rmarkdown::html_document(
        pandoc_args = "--fail-if-warnings"))
    Output
      Reading 'assets/pandoc-fail.Rmd'
      -- RMarkdown error -------------------------------------------------------------
      [WARNING] Could not fetch resource path-to-image.png
      Failing because there were warnings.
      --------------------------------------------------------------------------------
    Condition
      Error in `render_rmarkdown()`:
      ! Failed to render RMarkdown
      Caused by error:
      ! in callr subprocess.
      Caused by error:
      ! pandoc document conversion failed with error 3

# render_rmarkdown styles ANSI escapes

    <span class="co">#&gt; <span style="color: #BB0000;">X</span></span>

# warns about missing images

    Code
      build_home(pkg)
    Output
      -- Building home ---------------------------------------------------------------
      Writing 'authors.html'
    Condition
      Warning:
      Missing images in 'README.md': 'foo.png'
      i pkgdown can only use images in 'man/figures' and 'vignettes'
    Output
      Writing '404.html'

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build-search-docs.R
\name{build_search}
\alias{build_search}
\title{Build search index}
\usage{
build_search(pkg = ".", override = list())
}
\arguments{
\item{pkg}{Path to package.}

\item{override}{An optional named list used to temporarily override
values in \verb{_pkgdown.yml}}
}
\description{
Build a JSON file encompassing all HTML pages, for use by the search script.
}
\section{YAML config}{

You can exclude some paths from the search index.
Below we exclude the changelog from the search index.

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{search:
  exclude: ['news/index.html']
}\if{html}{\out{</div>}}
}

\section{Debugging and local testing}{


Locally (as opposed to on GitHub Pages or Netlify for instance),
search won't work if you simply use pkgdown preview of the static files.
You can use \code{servr::httw("docs")} instead.

If search is not working, run \code{pkgdown::pkgdown_sitrep()} to eliminate
common issues such as the absence of URL in the pkgdown configuration file
of your package.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build-articles.R, R/build-article.R
\name{build_articles}
\alias{build_articles}
\alias{build_article}
\alias{build_articles_index}
\title{Build articles section}
\usage{
build_articles(
  pkg = ".",
  quiet = TRUE,
  lazy = TRUE,
  seed = 1014L,
  override = list(),
  preview = FALSE
)

build_article(
  name,
  pkg = ".",
  lazy = FALSE,
  seed = 1014L,
  new_process = TRUE,
  pandoc_args = character(),
  override = list(),
  quiet = TRUE
)

build_articles_index(pkg = ".", override = list())
}
\arguments{
\item{pkg}{Path to package.}

\item{quiet}{Set to \code{FALSE} to display output of knitr and
pandoc. This is useful when debugging.}

\item{lazy}{If \code{TRUE}, will only re-build article if input file has been
modified more recently than the output file.}

\item{seed}{Seed used to initialize random number generation in order to
make article output reproducible. An integer scalar or \code{NULL} for no seed.}

\item{override}{An optional named list used to temporarily override
values in \verb{_pkgdown.yml}}

\item{preview}{If \code{TRUE}, or \code{is.na(preview) && interactive()}, will preview
freshly generated section in browser.}

\item{name}{Name of article to render. This should be either a path
relative to \verb{vignettes/} without extension, or \code{index} or \code{README}.}

\item{new_process}{Build the article in a clean R process? The default,
\code{TRUE}, ensures that every article is build in a fresh environment, but
you may want to set it to \code{FALSE} to make debugging easier.}

\item{pandoc_args}{Pass additional arguments to pandoc. Used for testing.}
}
\description{
\code{build_articles()} renders each R Markdown file underneath \verb{vignettes/} and
saves it to \verb{articles/}. There are two exceptions:
\itemize{
\item Files that start with \verb{_} (e.g., \verb{_index.Rmd}) are ignored,
enabling the use of child documents.
\item Files in \code{vignettes/tutorials} are handled by \code{\link[=build_tutorials]{build_tutorials()}}
}

Vignettes are rendered using a special document format that reconciles
\code{\link[rmarkdown:html_document]{rmarkdown::html_document()}} with the pkgdown template. This means articles
behave slightly differently to vignettes, particularly with respect to
external files, and custom output formats. See below for more details.

Note that when you run \code{build_articles()} directly (outside of
\code{\link[=build_site]{build_site()}}) vignettes will use the currently installed version of the
package, not the current source version. This makes iteration quicker when
you are primarily working on the text of an article.
}
\section{Index and navbar}{
You can control the articles index and navbar with a \code{articles} field in
your \verb{_pkgdown.yml}. If you use it, pkgdown will check that all articles
are included, and will error if you have missed any.

The \code{articles} field defines a list of sections, each of which
can contain four fields:
\itemize{
\item \code{title} (required): title of section, which appears as a heading on the
articles index.
\item \code{desc} (optional): An optional markdown description displayed underneath
the section title.
\item \code{navbar} (optional): A couple of words used to label this section in
the navbar. If omitted, this section of vignettes will not appear in the
navbar.
\item \code{contents} (required): a list of article names to include in the
section. This can either be names of individual vignettes or a
call to \code{starts_with()}. The name of a vignette includes its
path under \code{vignettes} without extension so that the name of the vignette
found at \code{vignettes/pizza/slice.Rmd} is \code{pizza/slice}.
}

The title and description of individual vignettes displayed on the index
comes from \code{title} and \code{description} fields of the YAML header in the Rmds.

For example, this yaml might be used for some version of dplyr:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{articles:
- title: Main verbs
  navbar: ~
  contents:
  - one-table
  - two-table
  - rowwise
  - colwise

- title: Developer
  desc: Vignettes aimed at package developers
  contents:
  - programming
  - packages
}\if{html}{\out{</div>}}

Note the use of the \code{navbar} fields. \verb{navbar: ~} means that the "Main verbs"
will appear in the navbar without a heading; the absence of the \code{navbar}
field in the developer vignettes means that they will only be
accessible via the articles index.

The navbar will include a link to the articles index if one or more
vignettes are not available through the navbar. If some vignettes appear
in the navbar drop-down list and others do not, the list will automatically
include a "More ..." link at the bottom; if no vignettes appear in the
the navbar, it will link directly to the articles index instead of
providing a drop-down.
\subsection{Get started}{

Note that a vignette with the same name as the package (e.g.,
\code{vignettes/pkgdown.Rmd} or \code{vignettes/articles/pkgdown.Rmd}) automatically
becomes a top-level "Get started" link, and will not appear in the articles
drop-down.

(If your package name includes a \code{.}, e.g. \code{pack.down}, use a \code{-} in the
vignette name, e.g. \code{pack-down.Rmd}.)
}

\subsection{Missing articles}{

pkgdown will warn if there are (non-internal) articles that aren't listed
in the articles index. You can suppress such warnings by listing the
affected articles in a section with \code{title: internal} (case sensitive);
this section will not be displayed on the index page.
}

\subsection{External articles}{

You can link to arbitrary additional articles by adding an
\code{external-articles} entry to \verb{_pkgdown.yml}. It should contain an array
of objects with fields \code{name}, \code{title}, \code{href}, and \code{description}.

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{external-articles:
- name: subsampling
  title: Subsampling for Class Imbalances
  description: Improve model performance in imbalanced data sets through undersampling or oversampling.
  href: https://www.tidymodels.org/learn/models/sub-sampling/
}\if{html}{\out{</div>}}

If you've defined a custom articles index, you'll need to include the name
in one of the \code{contents} fields.
}
}

\section{External files}{
pkgdown differs from base R in its handling of external files. When building
vignettes, R assumes that vignettes are self-contained (a reasonable
assumption when most vignettes were PDFs) and only copies files explicitly
listed in \code{.install_extras}. pkgdown takes a different approach based on
\code{\link[rmarkdown:find_external_resources]{rmarkdown::find_external_resources()}}, and it will also copy any images that
you link to. If for some reason the automatic detection doesn't work, you
will need to add a \code{resource_files} field to the yaml metadata, e.g.:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{---
title: My Document
resource_files:
  - data/mydata.csv
  - images/figure.png
---
}\if{html}{\out{</div>}}

Note that you can not use the \code{fig.path} to change the output directory of
generated figures as its default value is a strong assumption of rmarkdown.
}

\section{Embedding Shiny apps}{
If you would like to embed a Shiny app into an article, the app will have
to be hosted independently, (e.g. \url{https://www.shinyapps.io}). Then, you
can embed the app into your article using an \verb{<iframe>}, e.g.
\verb{<iframe src = "https://gallery.shinyapps.io/083-front-page" class="shiny-app">}.

See \url{https://github.com/r-lib/pkgdown/issues/838#issuecomment-430473856} for
some hints on how to customise the appearance with CSS.
}

\section{Output formats}{
By default, pkgdown builds all articles using the
\code{\link[rmarkdown:html_document]{rmarkdown::html_document()}} \code{output} format, ignoring whatever is set in
your YAML metadata. This is necessary because pkgdown has to integrate the
HTML/CSS/JS from the vignette with the HTML/CSS/JS from rest of the site.
Because of the challenges of combining two sources of HTML/CSS/JS, there is
limited support for other output formats and you have to opt-in by setting
the \code{as_is} field in your \code{.Rmd} metadata:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{pkgdown:
  as_is: true
}\if{html}{\out{</div>}}

If the output format produces a PDF, you'll also need to specify the
\code{extension} field:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{pkgdown:
  as_is: true
  extension: pdf
}\if{html}{\out{</div>}}

To work with pkgdown, the output format must accept \code{template}, \code{theme}, and
\code{self_contained} arguments, and must work without any additional CSS or
JSS files. Note that if you use
\href{https://bookdown.org/yihui/rmarkdown/html-document.html#shared-options}{\verb{_output.yml}}
or \href{https://rmarkdown.rstudio.com/docs/reference/render_site.html}{\verb{_site.yml}}
you'll still need to add \code{as_is: true} to each individual vignette.

Additionally, htmlwidgets do not work when \code{as_is: true}.
}

\section{Suppressing vignettes}{
If you want \href{https://r-pkgs.org/vignettes.html#sec-vignettes-article}{articles}
that are not vignettes, use \code{usethis::use_article()} to create it. An articles link will be
automatically added to the default navbar if the vignettes directory is
present: if you do not want this, you will need to customise the navbar. See
\code{\link[=build_site]{build_site()}} details.
}

\section{Figures}{
You can control the default rendering of figures by specifying the \code{figures}
field in \verb{_pkgdown.yml}. The default settings are equivalent to:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{figures:
  dev: ragg::agg_png
  dpi: 96
  dev.args: []
  fig.ext: png
  fig.width: 7.2916667
  fig.height: ~
  fig.retina: 2
  fig.asp: 1.618
  bg: NA
  other.parameters: []
}\if{html}{\out{</div>}}

Most of these parameters are interpreted similarly to knitr chunk
options. \code{other.parameters} is a list of parameters
that will be available to custom graphics output devices such
as HTML widgets.
}

\seealso{
Other site components: 
\code{\link{build_home}()},
\code{\link{build_news}()},
\code{\link{build_reference}()},
\code{\link{build_tutorials}()}
}
\concept{site components}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/init.R
\name{init_site}
\alias{init_site}
\title{Initialise site infrastructure}
\usage{
init_site(pkg = ".")
}
\arguments{
\item{pkg}{Path to package.}
}
\description{
\code{init_site()}:
\itemize{
\item creates the output directory (\verb{docs/}),
\item generates a machine readable description of the site, used for autolinking,
\item copies CSS/JS assets and extra files, and
\item runs \code{build_favicons()}, if needed.
}

See \code{vignette("customise")} for the various ways you can customise the
display of your site.
}
\section{Build-ignored files}{

We recommend using \code{\link[usethis:use_pkgdown]{usethis::use_pkgdown()}} to build-ignore \verb{docs/} and
\verb{_pkgdown.yml}. If use another directory, or create the site manually,
you'll need to add them to \code{.Rbuildignore} yourself. A \code{NOTE} about
an unexpected file during \verb{R CMD CHECK} is an indication you have not
correctly ignored these files.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test.R
\name{test-crayon}
\alias{test-crayon}
\title{Test case: crayon}
\description{
Test case: crayon
}
\examples{
cat(cli::col_red("This is red"), "\n")
cat(cli::col_blue("This is blue"), "\n")

message(cli::col_green("This is green"))

warning(cli::style_bold("This is bold"))
}
\seealso{
Other tests: 
\code{\link{index}},
\code{\link{test-dont}},
\code{\link{test-figures}},
\code{\link{test-links}},
\code{\link{test-lists}},
\code{\link{test-long-lines}},
\code{\link{test-output-styles}},
\code{\link{test-params}},
\code{\link{test-sexpr-title}},
\code{\link{test-verbatim}}
}
\concept{tests}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build-tutorials.R
\name{build_tutorials}
\alias{build_tutorials}
\title{Build tutorials section}
\usage{
build_tutorials(pkg = ".", override = list(), preview = NA)
}
\arguments{
\item{pkg}{Path to package.}

\item{override}{An optional named list used to temporarily override
values in \verb{_pkgdown.yml}}

\item{preview}{If \code{TRUE}, or \code{is.na(preview) && interactive()}, will preview
freshly generated section in browser.}
}
\description{
learnr tutorials must be hosted elsewhere as they require an
R execution engine. Currently, pkgdown will not build or publish tutorials
for you, but makes it easy to embed (using \verb{<iframe>}s) published tutorials.
Tutorials are automatically discovered from published tutorials in
\code{inst/tutorials} and \code{vignettes/tutorials}. Alternatively, you can
list in \verb{_pkgdown.yml} as described below.
}
\section{YAML config}{

To override the default discovery process, you can provide a \code{tutorials}
section. This should be a list where each element specifies:
\itemize{
\item \code{name}: used for the generated file name
\item \code{title}: used in page heading and in navbar
\item \code{url}: which will be embedded in an iframe
\item \code{source}: optional, but if present will be linked to
}

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{tutorials:
- name: 00-setup
  title: Setting up R
  url: https://jjallaire.shinyapps.io/learnr-tutorial-00-setup/
- name: 01-data-basics
  title: Data basics
  url: https://jjallaire.shinyapps.io/learnr-tutorial-01-data-basics/
}\if{html}{\out{</div>}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{build_site}
\alias{build_site}
\title{Build a complete pkgdown website}
\usage{
build_site(
  pkg = ".",
  examples = TRUE,
  run_dont_run = FALSE,
  seed = 1014L,
  lazy = FALSE,
  override = list(),
  preview = NA,
  devel = FALSE,
  new_process = !devel,
  install = !devel,
  quiet = TRUE
)
}
\arguments{
\item{pkg}{Path to package.}

\item{examples}{Run examples?}

\item{run_dont_run}{Run examples that are surrounded in \\dontrun?}

\item{seed}{Seed used to initialize random number generation in order to
make article output reproducible. An integer scalar or \code{NULL} for no seed.}

\item{lazy}{If \code{TRUE}, will only rebuild articles and reference pages
if the source is newer than the destination.}

\item{override}{An optional named list used to temporarily override
values in \verb{_pkgdown.yml}}

\item{preview}{If \code{TRUE}, or \code{is.na(preview) && interactive()}, will preview
freshly generated section in browser.}

\item{devel}{Use development or deployment process?

If \code{TRUE}, uses lighter-weight process suitable for rapid
iteration; it will run examples and vignettes in the current process,
and will load code with \code{pkgload::load_all()}.

If \code{FALSE}, will first install the package to a temporary library,
and will run all examples and vignettes in a new process.

\code{build_site()} defaults to \code{devel = FALSE} so that you get high fidelity
outputs when you building the complete site; \code{build_reference()},
\code{build_home()} and friends default to \code{devel = TRUE} so that you can
rapidly iterate during development.}

\item{new_process}{If \code{TRUE}, will run \code{build_site()} in a separate process.
This enhances reproducibility by ensuring nothing that you have loaded
in the current process affects the build process.}

\item{install}{If \code{TRUE}, will install the package in a temporary library
so it is available for vignettes.}

\item{quiet}{If \code{FALSE}, generate build messages for build functions that
take \code{quiet} arguments.}
}
\description{
\code{build_site()} is a convenient wrapper around six functions:
\itemize{
\item \code{\link[=init_site]{init_site()}}
\item \code{\link[=build_home]{build_home()}}
\item \code{\link[=build_reference]{build_reference()}}
\item \code{\link[=build_articles]{build_articles()}}
\item \code{\link[=build_tutorials]{build_tutorials()}}
\item \code{\link[=build_news]{build_news()}}
\item \code{\link[=build_redirects]{build_redirects()}}
\item \code{\link[=build_llm_docs]{build_llm_docs()}}
}

See the documentation for the each function to learn how to control
that aspect of the site. This page documents options that affect the
whole site.
}
\section{General config}{
\itemize{
\item \code{destination} controls where the site will be generated, defaulting to
\verb{docs/}. Paths are relative to the package root.
\item \code{url} is optional, but strongly recommended.

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{url: https://pkgdown.r-lib.org
}\if{html}{\out{</div>}}

It specifies where the site will be published and is used to allow other
pkgdown sites to link to your site when needed (\code{vignette("linking")}),
generate a \code{sitemap.xml}, automatically generate a \code{CNAME} when
\link[=build_site_github_pages]{deploying to github}, generate the metadata needed
rich social "media cards" (\code{vignette("metadata")}), and more.
\item \code{title} overrides the default site title, which is the package name.
It's used in the page title and default navbar.
}
}

\section{Navbar and footer}{
The \code{navbar} and \code{footer} fields control the appearance of the navbar
footer which appear on every page. Learn more about these fields in
\code{vignette("customise")}.
}

\section{Development mode}{
The \code{development} field allows you to generate different sites for the
development and released versions of your package. To use it, you first
need to set the development \code{mode}:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{development:
  mode: auto
}\if{html}{\out{</div>}}
\subsection{Setting development mode}{

The development \code{mode} of a site controls where the built site is placed
and how it is styled (i.e. the colour of the package version in the
navbar, the version tooltip), and whether or not the site is indexed by
search engines. There are four possible modes:
\itemize{
\item \strong{automatic} (\code{mode: auto}): determines the mode based on the version:
\itemize{
\item \verb{0.0.0.9000} (\verb{0.0.0.*}): unreleased.
\item four version components: development.
\item everything else -> release.
}
\item \strong{release} (\code{mode: release}), the default. Site is written to \verb{docs/}
and styled like a released package, even if the content is for an
unreleased or development version. Version in navbar gets the default
colouring. Development badges are not shown in the sidebar
(see \code{?build_home}).
\item \strong{development} (\code{mode: devel}). Site is written to \verb{docs/dev/}.
The navbar version gets a "danger" class and a tooltip stating these are
docs for an in-development version of the package. The \code{noindex} meta tag
is used to ensure that these packages are not indexed by search engines.
Development badges are shown in the sidebar (see \code{?build_home}).
\item \strong{unreleased} (\code{mode: unreleased}). Site is written to \verb{docs/}.
Version in navbar gets the "danger" class, and a message indicating the
package is not yet on CRAN.
Development badges are shown in the sidebar (see \code{?build_home}).
}

Use \code{mode: auto} if you want both a released and a dev site, and
\code{mode: release} if you just want a single site. It is very rare that you
will need either devel or unreleased modes.

You can override the mode specified in the \verb{_pkgdown.yml} by setting
by setting \code{PKGDOWN_DEV_MODE} to \code{devel} or \code{release}.
}

\subsection{Selective HTML}{

You can selectively show HTML only on the devel or release site by adding
class \code{pkgdown-devel} or \code{pkgdown-release}. This is most easily accessed
from \code{.Rmd} files where you can use pandoc's \verb{<div>} syntax to control
where a block of markdown will display. For example, you can use the
following markdown in your README to only show GitHub install instructions
on the development version of your site:

\if{html}{\out{<div class="sourceCode md">}}\preformatted{::: \{.pkgdown-devel\}
You can install the development version of pkgdown from GitHub with:
`remotes::install_github("r-lib/pkgdown")`
:::
}\if{html}{\out{</div>}}

You can use a similar technique to control where badges are displayed.
This markdown show the CRAN status badge on the site for the released
package and the GitHub check status for the development package:

\if{html}{\out{<div class="sourceCode md">}}\preformatted{[![CRAN Status](https://www.r-pkg.org/badges/version/pkgdown)]
  (https://cran.r-project.org/package=pkgdown)\{.pkgdown-release\}
[![R-CMD-check](https://github.com/r-lib/pkgdown/workflows/R-CMD-check/badge.svg)]
  (https://github.com/r-lib/pkgdown/actions)\{.pkgdown-devel\}
}\if{html}{\out{</div>}}
}

\subsection{Other options}{

There are three other options that you can control:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{development:
  destination: dev
  version_label: danger
  version_tooltip: "Custom message here"
}\if{html}{\out{</div>}}

\code{destination} allows you to override the default subdirectory used for the
development site; it defaults to \verb{dev/}. \code{version_label} allows you to
override the style used for development (and unreleased) versions of the
package. It defaults to "danger", but you can set to "default", "info", or
"warning" instead. (The precise colours are determined by your bootstrap
theme, but become progressively more eye catching as you go from default
to danger). Finally, you can choose to override the default tooltip with
\code{version_tooltip}.
}
}

\section{Template}{
The \code{template} field is mostly used to control the appearance of the site.
See \code{vignette("customise")} for details. But it's also used to control
\subsection{Other aspects}{

There are a few other \code{template} fields that control other aspects of the
site:
\itemize{
\item \code{noindex: true} will suppress indexing of your pages by search engines:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{template:
  params:
    noindex: true
}\if{html}{\out{</div>}}
\item \code{google_site_verification} allows you to verify your site with google:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{template:
  params:
    google_site_verification: _nn6ile-a6x6lctOW
}\if{html}{\out{</div>}}
\item \code{trailing_slash_redirect: true} will automatically redirect
\code{your-package-url.com} to \verb{your-package-url.com/}, using a JS script
added to the \verb{<head>} of the home page. This is useful in certain
redirect scenarios.

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{template:
  trailing_slash_redirect: true
}\if{html}{\out{</div>}}
}
}

\subsection{Analytics}{

To capture usage of your site with a web analytics tool, you can make
use of the \code{includes} field to add the special HTML they need. This HTML
is typically placed \code{in_header} (actually in the \verb{<head>}), \code{before_body},
or \code{after_body}.
You can learn more about how includes work in pkgdown at
\url{https://pkgdown.r-lib.org/articles/customise.html#additional-html-and-files}.

I include a few examples of popular analytics platforms below, but we
recommend getting the HTML directly from the tool:
\itemize{
\item \href{https://plausible.io}{plausible.io}:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{template:
  includes:
    in_header: |
      <script defer data-domain="\{YOUR DOMAIN\}" src="https://plausible.io/js/plausible.js"></script>
}\if{html}{\out{</div>}}
\item \href{https://analytics.google.com/analytics/web/}{Google analytics}:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{template:
  includes:
    in_header: |
       <!-- Global site tag (gtag.js) - Google Analytics -->
       <script async src="https://www.googletagmanager.com/gtag/js?id=\{YOUR MEASUREMENT ID\}" ></script>
       <script>
         window.dataLayer = window.dataLayer || [];
         function gtag()\{dataLayer.push(arguments);\}
         gtag('js', new Date());

         gtag('config', '\{YOUR MEASUREMENT ID\}');
       </script>
       <!-- Google tag (gtag.js) -->
}\if{html}{\out{</div>}}
\item \href{https://www.goatcounter.com}{GoatCounter}:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{template:
  includes:
    after_body: >
      <script data-goatcounter="https://\{YOUR CODE\}.goatcounter.com/count" data-goatcounter-settings="\{YOUR SETTINGS\}" async src="https://gc.zgo.at/count.js"></script>
}\if{html}{\out{</div>}}
}
}
}

\section{Source repository}{
Use the \code{repo} field to override pkgdown's automatically discovery
of your source repository. This is used in the navbar, on the homepage,
in articles and reference topics, and in the changelog (to link to issue
numbers and user names). pkgdown can automatically figure out the necessary
URLs if you link to a GitHub, GitLab or Codeberg repo in your \code{BugReports}
or \code{URL} field.

Otherwise, you can supply your own in the \code{repo} field:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{repo:
  url:
    home: https://github.com/r-lib/pkgdown/
    source: https://github.com/r-lib/pkgdown/blob/HEAD/
    issue: https://github.com/r-lib/pkgdown/issues/
    user: https://github.com/
}\if{html}{\out{</div>}}
\itemize{
\item \code{home}: path to package home on source code repository.
\item \code{source}: path to source of individual file in default branch
(more on that below).
\item \code{issue}: path to individual issue.
\item \code{user}: path to user.
}

The varying components (e.g. path, issue number, user name) are pasted on
the end of these URLs so they should have trailing \code{/}s.

When creating the link to a package source, we have to link to a specific
branch. The default behaviour is to use current branch when in GitHub
actions and \code{HEAD} otherwise. You can overide this default with
\code{repo.branch}:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{repo:
  branch: devel
}\if{html}{\out{</div>}}

pkgdown can automatically link to Jira issues as well if specify both a
custom \code{issue} URL as well Jira project names to auto-link in
\code{jira_projects}. You can specify as many projects as you would like:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{repo:
  jira_projects: [this_project, another_project]
  url:
    issue: https://jira.organisation.com/jira/browse/
}\if{html}{\out{</div>}}
}

\section{Deployment (\code{deploy})}{
There is a single \code{deploy} field
\itemize{
\item \code{install_metadata} allows you to install package index metadata into
the package itself. Normally this metadata is made available on the
published site; installing it into your package means that it's
available for autolinking even if your website is not reachable at build
time (e.g. because behind a firewall or requires auth).

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{deploy:
  install_metadata: true
}\if{html}{\out{</div>}}
}
}

\examples{
\dontrun{
build_site()

build_site(override = list(destination = tempdir()))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/render.R
\name{render_page}
\alias{render_page}
\alias{data_template}
\title{Render page with template}
\usage{
render_page(pkg = ".", name, data, path, depth = NULL, quiet = FALSE)

data_template(pkg = ".", depth = 0L)
}
\arguments{
\item{pkg}{Path to package to document.}

\item{name}{Name of the template (e.g. "home", "vignette", "news")}

\item{data}{Data for the template.

This is automatically supplemented with three lists:
\itemize{
\item \code{site}: \code{title} and path to \code{root}.
\item \code{yaml}: the \code{template} key from \verb{_pkgdown.yml}.
\item \code{package}: package metadata including \code{name} and\code{version}.
}

See the full contents by running \code{\link[=data_template]{data_template()}}.}

\item{path}{Location to create file; relative to destination directory.}

\item{depth}{Depth of path relative to base directory.}

\item{quiet}{If \code{quiet}, will suppress output messages}
}
\description{
Each page is composed of four templates: "head", "header", "content", and
"footer". Each of these templates is rendered using the \code{data}, and
then assembled into an overall page using the "layout" template.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test.R
\name{test-figures}
\alias{test-figures}
\title{Test case: figures}
\description{
\figure{bacon.jpg}
}
\examples{
x <- seq(0, 2 * pi, length.out = 25)
plot(x, sin(x))

plot(1:10)
lines(1:10)
text(2, 5, "Hello", srt = 30, cex = 2)

plot(1:3, col = c("red", "green", "blue"), pch = 16, cex = 10)
}
\seealso{
Other tests: 
\code{\link{index}},
\code{\link{test-crayon}},
\code{\link{test-dont}},
\code{\link{test-links}},
\code{\link{test-lists}},
\code{\link{test-long-lines}},
\code{\link{test-math-examples}},
\code{\link{test-output-styles}},
\code{\link{test-params}},
\code{\link{test-sexpr-title}},
\code{\link{test-tables}},
\code{\link{test-verbatim}}
}
\concept{tests}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pkgdown-package.R
\docType{package}
\name{pkgdown-package}
\alias{pkgdown}
\alias{pkgdown-package}
\title{pkgdown: Make Static HTML Documentation for a Package}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Generate an attractive and useful website from a source package. 'pkgdown' converts your documentation, vignettes, 'README', and more to 'HTML' making it easy to share information about your package online.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://pkgdown.r-lib.org/}
  \item \url{https://github.com/r-lib/pkgdown}
  \item Report bugs at \url{https://github.com/r-lib/pkgdown/issues}
}

}
\author{
\strong{Maintainer}: Hadley Wickham \email{hadley@posit.co} (\href{https://orcid.org/0000-0003-4757-117X}{ORCID})

Authors:
\itemize{
  \item Jay Hesselberth (\href{https://orcid.org/0000-0002-6299-179X}{ORCID})
  \item Maëlle Salmon (\href{https://orcid.org/0000-0002-2815-0399}{ORCID})
  \item Olivier Roy
  \item Salim Brüggemann (\href{https://orcid.org/0000-0002-5329-5987}{ORCID})
}

Other contributors:
\itemize{
  \item Posit Software, PBC (\href{https://ror.org/03wc8by49}{ROR}) [copyright holder, funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deploy-site.R
\name{deploy_site_github}
\alias{deploy_site_github}
\title{Deploy a pkgdown site on Travis-CI to Github Pages}
\usage{
deploy_site_github(
  pkg = ".",
  install = TRUE,
  tarball = Sys.getenv("PKG_TARBALL", ""),
  ssh_id = Sys.getenv("id_rsa", ""),
  commit_message = construct_commit_message(pkg),
  clean = FALSE,
  verbose = FALSE,
  host = "github.com",
  ...,
  repo_slug = Sys.getenv("TRAVIS_REPO_SLUG", "")
)
}
\arguments{
\item{pkg}{Path to package.}

\item{install}{Optionally, opt-out of automatic installation. This is
necessary if the package you're documenting is a dependency of pkgdown}

\item{tarball}{The location of the built package tarball. The default Travis
configuration for R packages sets \code{PKG_TARBALL} to this path.}

\item{ssh_id}{The private id to use, a base64 encoded content of the private
pem file. This should \emph{not} be your personal private key. Instead create a
new keypair specifically for deploying the site. The easiest way is to use
\code{travis::use_travis_deploy()}.}

\item{commit_message}{The commit message to be used for the commit.}

\item{clean}{Clean all files from old site.}

\item{verbose}{Print verbose output}

\item{host}{The GitHub host url.}

\item{...}{Additional arguments passed to \code{\link[=build_site]{build_site()}}.}

\item{repo_slug}{The \code{user/repo} slug for the repository.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#superseded}{\figure{lifecycle-superseded.svg}{options: alt='[Superseded]'}}}{\strong{[Superseded]}}

\code{deploy_site_github()} was designed to deploy your site from Travis CI,
which we no longer recommend, so this function is deprecated. There are
two replacements:
\itemize{
\item \code{\link[usethis:use_pkgdown]{usethis::use_pkgdown_github_pages()}} will setup a GitHub action to
automatically build and deploy your package website to GitHub pages.
\item \code{\link[=deploy_to_branch]{deploy_to_branch()}} can be called locally to build and deploy your
website to any desired branch.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test.R
\name{test-links}
\alias{test-links}
\title{Test case: links}
\description{
\if{html}{\out{<div class="sourceCode r">}}\preformatted{jsonlite::minify("\{\}")
#> \{\}
}\if{html}{\out{</div>}}
}
\examples{
jsonlite::minify("{}")

library(jsonlite, warn.conflicts = FALSE)
minify("{}")
}
\seealso{
Other tests: 
\code{\link{index}},
\code{\link{test-crayon}},
\code{\link{test-dont}},
\code{\link{test-figures}},
\code{\link{test-lists}},
\code{\link{test-long-lines}},
\code{\link{test-math-examples}},
\code{\link{test-output-styles}},
\code{\link{test-params}},
\code{\link{test-sexpr-title}},
\code{\link{test-tables}},
\code{\link{test-verbatim}}
}
\concept{tests}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build-llm.R
\name{build_llm_docs}
\alias{build_llm_docs}
\title{Build docs for LLMs}
\usage{
build_llm_docs(pkg = ".")
}
\arguments{
\item{pkg}{Path to package.}
}
\description{
\code{build_llm_docs()} creates an \code{LLMs.txt} at the root of your site
that contains the contents of your \code{README.md}, your reference index,
and your articles index. It also creates a \code{.md} file for every existing
\code{.html} file in your site. Together, this gives an LLM an overview of your
package and the ability to find out more by following links.

If you don't want these files generated for your site, you can opt-out by
adding the following to your \code{pkgdown.yml}:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{llm-docs: false
}\if{html}{\out{</div>}}
}
\seealso{
Other site components: 
\code{\link{build_articles}()},
\code{\link{build_home}()},
\code{\link{build_news}()},
\code{\link{build_reference}()},
\code{\link{build_tutorials}()}
}
\concept{site components}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pkgdown.R
\name{in_pkgdown}
\alias{in_pkgdown}
\title{Determine if code is executed by pkgdown}
\usage{
in_pkgdown()
}
\description{
This is occasionally useful when you need different behaviour by
pkgdown and regular documentation.
}
\examples{
in_pkgdown()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preview.R
\name{preview_page}
\alias{preview_page}
\title{Preview a local pkgdown page in the browser}
\usage{
preview_page(path, pkg = ".")
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} Use \code{\link[=preview_site]{preview_site()}} instead.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pkgdown_print.R
\name{pkgdown_print}
\alias{pkgdown_print}
\title{Print object in pkgdown output}
\usage{
pkgdown_print(x, visible = TRUE)
}
\arguments{
\item{x}{Object to display}

\item{visible}{Whether it is visible or not}
}
\value{
Either a character vector representing printed output (which
will be escaped for HTML as necessary) or literal HTML produced
by the htmltools or htmlwidgets packages.
}
\description{
This lets package authors control how objects are printed just for
pkgdown examples. The default is to call \code{\link[=print]{print()}} apart from htmlwidgets
where the object is returned as is (with sizes tweaked).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test.R
\name{test-math-examples}
\alias{test-math-examples}
\title{Test case: math rendering in examples}
\arguments{
\item{x}{\eqn{f(x) > 0}: inline equation}
}
\description{
Test case: math rendering in examples
}
\details{
Display equation:

\deqn{y = \alpha + \beta X + \varepsilon}

Multi-line equation (correctly rendered by katex only):

\deqn{\text{Minimize } l \\
\text{ subject to } \\
\sum_{i = 1}^{I} x_i r_{ij} + y_j \geq t_j \forall j \in J \\
l \geq \frac{y_j}{t_j} \forall j \in J \\
\sum_{i = 1}^{I} x_i c_i \leq B}{
Minimize l subject to
sum_i^I (xi * rij) + yj >= tj for all j in J &
l >= (yj / tj) for all j in J &
sum_i^I (xi * ci) <= B}
}
\seealso{
Other tests: 
\code{\link{index}},
\code{\link{test-crayon}},
\code{\link{test-dont}},
\code{\link{test-figures}},
\code{\link{test-links}},
\code{\link{test-lists}},
\code{\link{test-long-lines}},
\code{\link{test-output-styles}},
\code{\link{test-params}},
\code{\link{test-sexpr-title}},
\code{\link{test-tables}},
\code{\link{test-verbatim}}
}
\concept{tests}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check.R
\name{check_pkgdown}
\alias{check_pkgdown}
\alias{pkgdown_sitrep}
\title{Check \verb{_pkgdown.yml}}
\usage{
check_pkgdown(pkg = ".")

pkgdown_sitrep(pkg = ".")
}
\arguments{
\item{pkg}{Path to package.}
}
\description{
This pair of functions checks that your \verb{_pkgdown.yml} is valid without
building the whole site. \code{check_pkgdown()} errors at the first problem;
\code{pkgdown_sitrep()} reports the status of all checks.

Currently they check that:
\itemize{
\item There's a \code{url} in the pkgdown configuration, which is also recorded
in the \code{URL} field of the \code{DESCRIPTION}.
\item All opengraph metadata is valid.
\item All reference topics are included in the index.
\item All articles/vignettes are included in the index.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build-news.R
\name{build_news}
\alias{build_news}
\title{Build news section}
\usage{
build_news(pkg = ".", override = list(), preview = FALSE)
}
\arguments{
\item{pkg}{Path to package.}

\item{override}{An optional named list used to temporarily override
values in \verb{_pkgdown.yml}}

\item{preview}{If \code{TRUE}, or \code{is.na(preview) && interactive()}, will preview
freshly generated section in browser.}
}
\description{
A \code{NEWS.md} will be broken up into versions using level one (\verb{#}) or
level two headings (\verb{##}) that (partially) match one of the following forms
(ignoring case):
\itemize{
\item \verb{\{package name\} 1.3.0}
\item \verb{\{package name\} v1.3.0}
\item \verb{Version 1.3.0}
\item \verb{Changes in 1.3.0}
\item \verb{Changes in v1.3.0}
}
}
\details{
A \href{https://style.tidyverse.org/news.html}{common structure} for news files
is to use a top level heading for each release, and use a second level
heading to break up individual bullets into sections.

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{# foofy 1.0.0

## Major changes

* Can now work with all grooveable grobbles!

## Minor improvements and bug fixes

* Printing scrobbles no longer errors (@githubusername, #100)

* Wibbles are now 55\% less jibbly (#200)
}\if{html}{\out{</div>}}

Issues and contributors will be automatically linked to the corresponding
pages on GitHub if the GitHub repo can be discovered from the \code{DESCRIPTION}
(typically from a \code{URL} entry containing \code{github.com})

If a version is available on CRAN, the release date will automatically
be added to the heading (see below for how to suppress); if not
available on CRAN, "Unreleased" will be added.
}
\section{YAML config}{
To automatically link to release announcements, include a \code{releases}
section.

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{news:
 releases:
 - text: "usethis 1.3.0"
   href: https://www.tidyverse.org/articles/2018/02/usethis-1-3-0/
 - text: "usethis 1.0.0 (and 1.1.0)"
   href: https://www.tidyverse.org/articles/2017/11/usethis-1.0.0/
}\if{html}{\out{</div>}}

Control whether news is present on one page or multiple pages with the
\code{one_page} field. The default is \code{true}.

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{news:
  one_page: false
}\if{html}{\out{</div>}}

Suppress the default addition of CRAN release dates with:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{news:
  cran_dates: false
}\if{html}{\out{</div>}}
}

\seealso{
\href{https://style.tidyverse.org/news.html}{Tidyverse style for News}

Other site components: 
\code{\link{build_articles}()},
\code{\link{build_home}()},
\code{\link{build_llm_docs}()},
\code{\link{build_reference}()},
\code{\link{build_tutorials}()}
}
\concept{site components}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test.R
\name{test-verbatim}
\alias{test-verbatim}
\title{Test case: preformatted blocks & syntax highlighting}
\description{
Manual test cases for various ways of embedding code in sections.
All code blocks should have copy and paste button.
}
\section{Should be highlighted}{
Valid R code in \verb{\preformatted{}}:

\if{html}{\out{<div class="sourceCode">}}\preformatted{mean(a + 1)
}\if{html}{\out{</div>}}

R code in \code{R} block:

\if{html}{\out{<div class="sourceCode R">}}\preformatted{mean(a + 1)
}\if{html}{\out{</div>}}

R code in \code{r} block:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{mean(a + 1)
}\if{html}{\out{</div>}}

Yaml

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{yaml: [a, 1]
}\if{html}{\out{</div>}}
}

\section{Shouldn't be highlighted}{
Non-R code in \verb{\preformatted{}}

\if{html}{\out{<div class="sourceCode">}}\preformatted{yaml: [a, b, c]
}\if{html}{\out{</div>}}
}

\seealso{
Other tests: 
\code{\link{index}},
\code{\link{test-crayon}},
\code{\link{test-dont}},
\code{\link{test-figures}},
\code{\link{test-links}},
\code{\link{test-lists}},
\code{\link{test-long-lines}},
\code{\link{test-math-examples}},
\code{\link{test-output-styles}},
\code{\link{test-params}},
\code{\link{test-sexpr-title}},
\code{\link{test-tables}}
}
\concept{tests}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deploy-site.R
\name{deploy_to_branch}
\alias{deploy_to_branch}
\title{Build and deploy a site locally}
\usage{
deploy_to_branch(
  pkg = ".",
  commit_message = construct_commit_message(pkg),
  clean = TRUE,
  branch = "gh-pages",
  remote = "origin",
  github_pages = (branch == "gh-pages"),
  ...,
  subdir = NULL
)
}
\arguments{
\item{pkg}{Path to package.}

\item{commit_message}{The commit message to be used for the commit.}

\item{clean}{Clean all files from old site.}

\item{branch}{The git branch to deploy to}

\item{remote}{The git remote to deploy to}

\item{github_pages}{Is this a GitHub pages deploy. If \code{TRUE}, adds a \code{CNAME}
file for custom domain name support, and a \code{.nojekyll} file to suppress
jekyll rendering.}

\item{...}{Additional arguments passed to \code{\link[=build_site]{build_site()}}.}

\item{subdir}{The sub-directory where the site should be built on the branch.
This argument can be used to support a number of site configurations.
For example, you could build version-specific documentation by setting
\code{subdir = "v1.2.3"}; \code{deploy_to_branch()} will build and deploy the
package documentation in the \verb{v.1.2.3/} directory of your site.}
}
\description{
Assumes that you're in a git clone of the project, and the package is
already installed. Use \code{\link[usethis:use_pkgdown]{usethis::use_pkgdown_github_pages()}} to automate
this process using GitHub actions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test.R
\name{test-lists}
\alias{test-lists}
\title{Test case: lists}
\description{
\subsection{Bulleted list}{
\itemize{
  \item a
  \item This is an item...

    That spans multiple paragraphs.
}
}

\subsection{Bulleted list (single item)}{
\itemize{\item a}
}

\subsection{Numbered list}{
\enumerate{
  \item a
  \item b
}
}

\subsection{Definition list}{
\describe{
  \item{short}{short}
  \item{short}{Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do
  eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad
  minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip
  ex ea commodo consequat. Duis aute irure dolor in reprehenderit in
  voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur
  sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt
  mollit anim id est laborum.}
  \item{Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do
  eiusmod tempor incididunt ut labore et dolore magna aliqua.}{short}
  \item{Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do
  eiusmod tempor incididunt ut labore et dolore magna aliqua.}{Lorem ipsum
  adipiscing elit, sed do  eiusmod tempor incididunt ut labore et dolore ad
  minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip
  ex ea commodo consequat. Duis aute irure dolor in reprehenderit in
  voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur
  sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt
  mollit anim id est laborum.}
}
}
}
\seealso{
Other tests: 
\code{\link{index}},
\code{\link{test-crayon}},
\code{\link{test-dont}},
\code{\link{test-figures}},
\code{\link{test-links}},
\code{\link{test-long-lines}},
\code{\link{test-output-styles}},
\code{\link{test-params}},
\code{\link{test-sexpr-title}},
\code{\link{test-tables}},
\code{\link{test-verbatim}}
}
\concept{tests}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sitrep.R
\name{pkgdown_sitrep}
\alias{pkgdown_sitrep}
\title{Report package pkgdown situation}
\usage{
pkgdown_sitrep(pkg = ".")
}
\arguments{
\item{pkg}{Path to package.}
}
\description{
\code{pkgdown_sitrep()} reports
\itemize{
\item If there is an \code{url} field in the pkgdown configuration;
\item If that pkgdown website URL is stored in the DESCRIPTION file.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autolink_html.R
\name{autolink_html}
\alias{autolink_html}
\title{Automatically link references and articles in an HTML page}
\usage{
autolink_html(input, output = input, local_packages = character())
}
\arguments{
\item{input, output}{Input and output paths for HTML file}

\item{local_packages}{A named character vector providing relative paths
(value) to packages (name) that can be reached with relative links
from the target HTML document.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

Please use \code{\link[downlit:downlit_html_path]{downlit::downlit_html_path()}} instead.
}
\examples{
\dontrun{
autolink_html("path/to/file.html",
  local_packages = c(
    shiny = "shiny",
    shinydashboard = "shinydashboard"
  )
)
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build-github.R
\name{build_site_github_pages}
\alias{build_site_github_pages}
\title{Build site for GitHub pages}
\usage{
build_site_github_pages(
  pkg = ".",
  ...,
  dest_dir = "docs",
  clean = TRUE,
  install = FALSE,
  new_process = FALSE
)
}
\arguments{
\item{pkg}{Path to package.}

\item{...}{Additional arguments passed to \code{\link[=build_site]{build_site()}}.}

\item{dest_dir}{Directory to build site in.}

\item{clean}{Clean all files from old site.}

\item{install}{If \code{TRUE}, will install the package in a temporary library
so it is available for vignettes.}

\item{new_process}{If \code{TRUE}, will run \code{build_site()} in a separate process.
This enhances reproducibility by ensuring nothing that you have loaded
in the current process affects the build process.}
}
\description{
Designed to be run as part of automated workflows for deploying
to GitHub pages. It cleans out the old site, builds the site into \code{dest_dir}
adds a \code{.nojekyll} file to suppress rendering by Jekyll, and adds a \code{CNAME}
file if needed.

It is designed to be run in CI, so by default it:
\itemize{
\item Cleans out the old site.
\item Does not install the package.
\item Runs \code{\link[=build_site]{build_site()}} in process.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build-reference.R
\name{build_reference}
\alias{build_reference}
\alias{build_reference_index}
\title{Build reference section}
\usage{
build_reference(
  pkg = ".",
  lazy = TRUE,
  examples = TRUE,
  run_dont_run = FALSE,
  seed = 1014,
  override = list(),
  preview = NA,
  devel = TRUE,
  document = "DEPRECATED",
  topics = NULL
)

build_reference_index(pkg = ".")
}
\arguments{
\item{pkg}{Path to package.}

\item{lazy}{If \code{TRUE}, only rebuild pages where the \code{.Rd}
is more recent than the \code{.html}. This makes it much easier to
rapidly prototype. It is set to \code{FALSE} by \code{\link[=build_site]{build_site()}}.}

\item{examples}{Run examples?}

\item{run_dont_run}{Run examples that are surrounded in \\dontrun?}

\item{seed}{Seed used to initialize so that random examples are
reproducible.}

\item{override}{An optional named list used to temporarily override
values in \verb{_pkgdown.yml}}

\item{preview}{If \code{TRUE}, or \code{is.na(preview) && interactive()}, will preview
freshly generated section in browser.}

\item{devel}{Determines how code is loaded in order to run examples.
If \code{TRUE} (the default), assumes you are in a live development
environment, and loads source package with \code{\link[pkgload:load_all]{pkgload::load_all()}}.
If \code{FALSE}, uses the installed version of the package.}

\item{document}{\strong{Deprecated} Use \code{devel} instead.}

\item{topics}{Build only specified topics. If supplied, sets \code{lazy}
and \code{preview} to \code{FALSE}.}
}
\description{
By default, pkgdown will generate an index that lists all functions in
alphabetical order. To override this, provide a \code{reference} section in your
\verb{_pkgdown.yml} as described below.
}
\section{Reference index}{

To tweak the index page, add a section called \code{reference} to \verb{_pkgdown.yml}.
It can contain three different types of element:
\itemize{
\item A \strong{title} (\code{title} + \code{desc}), which generates an row containing an \verb{<h2>}
with optional paragraph description.
\item A \strong{subtitle} (\code{subtitle} + \code{desc}), which generates an row containing an
\verb{<h3>} with optional paragraph description.
\item A \strong{list of topics} (\code{contents}), which generates one row for each topic,
with a list of aliases for the topic on the left, and the topic title
on the right.
}

(For historical reasons you can include \code{contents} with a title or
subtitle, but this is no longer recommended).

Most packages will only need to use \code{title} and \code{contents} components.
For example, here's a snippet from the YAML that pkgdown uses to generate
its own reference index:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{reference:
- title: Build
  desc:  Build a complete site or its individual section components.
- contents:
  - starts_with("build_")
- title: Templates
- contents:
  - template_navbar
  - render_page
}\if{html}{\out{</div>}}

Bigger packages, e.g. ggplot2, may need an additional layer of
structure in order to clearly organise large number of functions:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{reference:
- title: Layers
- subtitle: Geoms
  desc: Geom is short for geometric element
- contents:
  - starts_with("geom")
- subtitle: Stats
  desc: Statistical transformations transform data before display.
  contents:
  - starts_with("stat")
}\if{html}{\out{</div>}}

\code{desc} can use markdown, and if you have a long description it's a good
idea to take advantage of the YAML \code{>} notation:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{desc: >
  This is a very _long_ and **overly** flowery description of a
  single simple function. By using `>`, it's easy to write a description
  that runs over multiple lines.
}\if{html}{\out{</div>}}
\subsection{Topic matching}{

\code{contents} can contain:
\itemize{
\item Individual function/topic names.
\item Weirdly named functions with doubled quoting, once for YAML and once for
R, e.g. \code{"`+.gg`"}.
\item \code{starts_with("prefix")} to select all functions with common prefix.
\item \code{ends_with("suffix")} to select all functions with common suffix.
\item \code{matches("regexp")} for more complex regular expressions.
\item \code{has_keyword("x")} to select all topics with keyword "x";
\code{has_keyword("datasets")} selects all data documentation.
\item \code{has_concept("blah")} to select all topics with concept "blah".
If you are using roxygen2, \code{has_concept()} also matches family tags, because
roxygen2 converts them to concept tags.
\item \code{lacks_concepts(c("concept1", "concept2"))} to select all topics
without those concepts. This is useful to capture topics not otherwise
captured by \code{has_concepts()}.
\item Topics from other installed packages, e.g. \code{rlang::is_installed()} (function name)
or \code{sass::font_face} (topic name).
}

All functions (except for \code{has_keywords()}) automatically exclude internal
topics (i.e. those with \verb{\\keyword\{internal\}}). You can choose to include
with (e.g.) \code{starts_with("build_", internal = TRUE)}.

Use a leading \code{-} to remove topics from a section, e.g. \code{-topic_name},
\code{-starts_with("foo")}.

pkgdown will check that all non-internal topics are included on
the reference index page, and error if you have missed any.
}

\subsection{Missing topics}{

pkgdown will warn if there are (non-internal) topics that not listed
in the reference index. You can suppress these warnings by listing the
topics in section with "title: internal" (case sensitive) which will not be
displayed on the reference index.
}

\subsection{Icons}{

You can optionally supply an icon for each help topic. To do so, you'll need
a top-level \code{icons} directory. This should contain {.png} files that are
either 30x30 (for regular display) or 60x60 (if you want retina display).
Icons are matched to topics by aliases.
}

\subsection{Examples}{

If you need to run extra code before or after all examples are run, you
can create \code{pkgdown/pre-reference.R} and \code{pkgdown/post-reference.R}.
}
}

\section{Figures}{


You can control the default rendering of figures by specifying the \code{figures}
field in \verb{_pkgdown.yml}. The default settings are equivalent to:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{figures:
  dev: ragg::agg_png
  dpi: 96
  dev.args: []
  fig.ext: png
  fig.width: 7.2916667
  fig.height: ~
  fig.retina: 2
  fig.asp: 1.618
  bg: NA
  other.parameters: []
}\if{html}{\out{</div>}}

Most of these parameters are interpreted similarly to knitr chunk
options. \code{other.parameters} is a list of parameters
that will be available to custom graphics output devices such
as HTML widgets.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build-redirects.R
\name{build_redirects}
\alias{build_redirects}
\title{Build redirects}
\usage{
build_redirects(pkg = ".", override = list())
}
\arguments{
\item{pkg}{Path to package.}

\item{override}{An optional named list used to temporarily override
values in \verb{_pkgdown.yml}}
}
\description{
If you change the structure of your documentation (by renaming vignettes or
help topics) you can setup redirects from the old content to the new content.
One or several now-absent pages can be redirected to a new page (or to a new
section of a new page). This works by creating a html page that performs a
"meta refresh", which isn't the best way of doing a redirect but works
everywhere that you might deploy your site.

The syntax is the following, with old paths on the left, and new paths or
URLs on the right.

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{redirects:
  - ["articles/old-vignette-name.html", "articles/new-vignette-name.html"]
  - ["articles/another-old-vignette-name.html", "articles/new-vignette-name.html"]
  - ["articles/yet-another-old-vignette-name.html", "https://pkgdown.r-lib.org/dev"]
}\if{html}{\out{</div>}}

If for some reason you choose to redirect an existing page make sure to
exclude it from the search index, see \code{?build_search}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test.R
\name{test-params}
\alias{test-params}
\title{Test case: params}
\arguments{
\item{...}{ellipsis}
}
\description{
Test case: params
}
\seealso{
Other tests: 
\code{\link{index}},
\code{\link{test-crayon}},
\code{\link{test-dont}},
\code{\link{test-figures}},
\code{\link{test-links}},
\code{\link{test-lists}},
\code{\link{test-long-lines}},
\code{\link{test-output-styles}},
\code{\link{test-sexpr-title}},
\code{\link{test-verbatim}}
}
\concept{tests}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/template.R
\name{template_navbar}
\alias{template_navbar}
\alias{template_reference}
\alias{template_articles}
\title{Generate YAML templates}
\usage{
template_navbar(path = ".")

template_reference(path = ".")

template_articles(path = ".")
}
\arguments{
\item{path}{Path to package root}
}
\description{
Use these function to generate the default YAML that pkgdown uses for
the different parts of \verb{_pkgdown.yml}. This are useful starting
points if you want to customise your site.
}
\examples{
\dontrun{
pkgdown::template_navbar()
}

\dontrun{
pkgdown::template_reference()
}

\dontrun{
pkgdown::template_articles()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test.R
\name{test-dont}
\alias{test-dont}
\title{Test case: don't}
\description{
Test case: don't
}
\examples{
# \dontrun{} --------------------------------------------------------
# always shown; never run

x <- 1
\dontrun{x <- 2}
\dontrun{
  x <- 3
  x <- 4
}
x # should be 1

# \donttest{} -------------------------------------------------------
# only multiline are shown; always run

x <- 1
\donttest{x <- 2}
\donttest{
  x <- 3
  x <- 4
}
x # should be 4

# \testonly{} -----------------------------------------------------
# never shown, never run

x <- 1
\testonly{x <- 2}
\testonly{
  x <- 3
  x <- 4
}
x # should be 1

# \dontshow{} -------------------------------------------------------
# never shown, always run

x <- 1
\dontshow{x <- 2}
\dontshow{
  x <- 3
  x <- 4
}
x # should be 4

# @examplesIf ------------------------------------------------------
# If FALSE, wrapped in if; if TRUE, not seen

x <- 1

\dontshow{if (FALSE) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
x <- 2
\dontshow{\}) # examplesIf}
\dontshow{if (TRUE) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
x <- 3
\dontshow{\}) # examplesIf}
x # should be 3
}
\seealso{
Other tests: 
\code{\link{index}},
\code{\link{test-crayon}},
\code{\link{test-figures}},
\code{\link{test-links}},
\code{\link{test-lists}},
\code{\link{test-long-lines}},
\code{\link{test-output-styles}},
\code{\link{test-params}},
\code{\link{test-sexpr-title}},
\code{\link{test-tables}},
\code{\link{test-verbatim}}
}
\concept{tests}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\name{as_pkgdown}
\alias{as_pkgdown}
\title{Generate pkgdown data structure}
\usage{
as_pkgdown(pkg = ".", override = list())
}
\arguments{
\item{pkg}{Path to package.}

\item{override}{An optional named list used to temporarily override
values in \verb{_pkgdown.yml}}
}
\description{
You will generally not need to use this unless you need a custom site
design and you're writing your own equivalent of \code{\link[=build_site]{build_site()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test.R
\name{test-output-styles}
\alias{test-output-styles}
\title{Test case: output styles}
\description{
Test case: output styles
}
\examples{
# This example illustrates some important output types
# The following output should be wrapped over multiple lines
a <- 1:100
a

cat("This some text!\n")
message("This is a message!")
warning("This is a warning!")

# This is a multi-line block
{
  1 + 2
  2 + 2
}
}
\seealso{
Other tests: 
\code{\link{index}},
\code{\link{test-crayon}},
\code{\link{test-dont}},
\code{\link{test-figures}},
\code{\link{test-links}},
\code{\link{test-lists}},
\code{\link{test-long-lines}},
\code{\link{test-params}},
\code{\link{test-sexpr-title}},
\code{\link{test-verbatim}}
}
\concept{tests}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build-home.R, R/build-home-index.R
\name{build_home}
\alias{build_home}
\alias{build_home_index}
\title{Build home section}
\usage{
build_home(pkg = ".", override = list(), preview = FALSE, quiet = TRUE)

build_home_index(pkg = ".", override = list(), quiet = TRUE)
}
\arguments{
\item{pkg}{Path to package.}

\item{override}{An optional named list used to temporarily override
values in \verb{_pkgdown.yml}}

\item{preview}{If \code{TRUE}, or \code{is.na(preview) && interactive()}, will preview
freshly generated section in browser.}

\item{quiet}{Set to \code{FALSE} to display output of knitr and
pandoc. This is useful when debugging.}
}
\description{
\code{build_home()} function generates pages at the top-level of the site
including:
\itemize{
\item The home page
\item HTML files from any \code{.md} files in \verb{./} or \verb{.github/}.
\item The authors page (from \code{DESCRIPTION})
\item The citation page (from \code{inst/CITATION}, if present).
\item The license page
\item A default 404 page if \verb{.github/404.md} is not found.
}

\code{build_home_index()} rebuilds just the index page; it's useful for rapidly
iterating when experimenting with site styles.
}
\section{Home page}{
The main content of the home page (\code{index.html}) is generated from
\code{pkgdown/index.md}, \code{index.md}, or \code{README.md}, in that order.
Most packages will use \code{README.md} because that's also displayed by GitHub
and CRAN. Use \code{index.md} if you want your package website to look
different to your README, and use \code{pkgdown/index.md} if you don't want that
file to live in your package root directory.

If you use \code{index.Rmd} or \code{README.Rmd} it's your responsibility to knit
the document to create the corresponding \code{.md}. pkgdown does not do this
for you because it only touches files in the \verb{doc/} directory.

Extra markdown files in the base directory (e.g. \code{ROADMAP.md}) or in
\verb{.github/} (e.g. \code{CODE_OF_CONDUCT.md}) are copied by \code{build_home()} to \verb{docs/} and converted to HTML.

The home page also features a sidebar with information extracted from the
package. You can tweak it via the configuration file, to help make the home
page an as informative as possible landing page.
\subsection{Images and figures}{

If you want to include images in your \code{README.md}, they must be stored
somewhere in the package so that they can be displayed on the CRAN website.
The best place to put them is \code{man/figures}. If you are generating figures
with R Markdown, make sure you set up \code{fig.path} as followed:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{knitr::opts_chunk$set(
  fig.path = "man/figures/"
)
}\if{html}{\out{</div>}}

This should usually go in a chunk with \code{include = FALSE}.

\if{html}{\out{<div class="sourceCode markdown">}}\preformatted{```\{r chunk-name, include=FALSE\}`r ''`
knitr::opts_chunk$set(
  fig.path = "man/figures/"
)
```
}\if{html}{\out{</div>}}
}

\subsection{Package logo}{

If you have a package logo, you can include it at the top of your README
in a level-one heading:

\if{html}{\out{<div class="sourceCode markdown">}}\preformatted{# pkgdown <img src="man/figures/logo.png" align="right" />
}\if{html}{\out{</div>}}

\code{\link[=init_site]{init_site()}} will also automatically create a favicon set from your package logo.
}

\subsection{YAML config - title and description}{

By default, the page title and description are extracted automatically from
the \code{Title} and \code{Description} fields \code{DESCRIPTION} (stripping single quotes
off quoted words). CRAN ensures that these fields don't contain phrases
like "R package" because that's obvious on CRAN. To make your package more
findable on search engines, it's good practice to override the \code{title} and
\code{description}, thinking about what people might search for:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{home:
  title: An R package for pool-noodle discovery
  description: >
    Do you love R? Do you love pool-noodles? If so, you might enjoy
    using this package to automatically discover and add pool-noodles
    to your growing collection.
}\if{html}{\out{</div>}}

(Note the use of YAML's \code{>} i.e. "YAML pipes"; this is a convenient way of
writing paragraphs of text.)
}

\subsection{Dev badges}{

pkgdown identifies badges in three ways:
\itemize{
\item Any image-containing links between \verb{<!-- badges: start -->} and
\verb{<!-- badges: end -->}, as e.g. created by \code{usethis::use_readme_md()}
or \code{usethis::use_readme_rmd()}. There should always be an empty line after
the \verb{<!-- badges: end -->} line. If you divide badges into paragraphs,
make sure to add an empty line before the \verb{<!-- badges: end -->} line.
\item Any image-containing links within \verb{<div id="badges"></div>}.
\item Within the first paragraph, if it only contains image-containing links.
}

Identified badges are \strong{removed} from the \emph{main content}.
They are shown or not in the \emph{sidebar} depending on the development mode and
sidebar customization, see the sidebar section.
}
}

\section{Authors}{
By default, pkgdown will display author information in three places:
\itemize{
\item the sidebar,
\item the left part side of the footer,
\item the author page.
}

This documentation describes how to customise the overall author display.
See \code{?build_home} and \code{?build_site} for details about changing the location
of the authors information within the home sidebar and the site footer.
\subsection{Authors ORCID, ROR and bio}{

Author ORCID identification numbers in the \code{DESCRIPTION} are linked using
the ORCID logo,
author ROR identification numbers are linked using the ROR logo:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{Authors@R: c(
    person("Hadley", "Wickham", , "hadley@rstudio.com", role = c("aut", "cre"),
      comment = c(ORCID = "0000-0003-4757-117X")
    ),
    person("Jay", "Hesselberth", role = "aut",
      comment = c(ORCID = "0000-0002-6299-179X")
    ),
   person("Posit Software, PBC", role = c("cph", "fnd"),
          comment = c(ROR = "03wc8by49"))
  )
}\if{html}{\out{</div>}}

If you want to add more details about authors or their involvement with the
package, you can use the comment field, which will be rendered on the
authors page.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{Authors@R: c(
    person("Hadley", "Wickham", , "hadley@rstudio.com", role = c("aut", "cre"),
      comment = c(ORCID = "0000-0003-4757-117X", "Indenter-in-chief")
    ),
    person("Jay", "Hesselberth", role = "aut",
      comment = c(ORCID = "0000-0002-6299-179X")
    )
  )
}\if{html}{\out{</div>}}
}

\subsection{Additional control via YAML}{

You can control additional aspects of the authors display via the \code{authors}
YAML field:
\itemize{
\item display of each author in the footer, sidebar and authors page,
\item which authors (by role) are displayed in the sidebar and footer,
\item text before authors in the footer,
\item text before and after authors in the sidebar,
\item text before and after authors on the authors page.
}

You can modify how each author's name is displayed by adding a subsection
for \code{authors}. Each entry in \code{authors} should be named the author's name
(matching \code{DESCRIPTION}) and can contain \code{href} and/or \code{html} fields:
\itemize{
\item If \code{href} is provided, the author's name will be linked to this URL.
\item If \code{html} is provided, it will be shown instead of the author's name.
This is particularly useful if you want to display the logo of a corporate
sponsor. Use an absolute URL to an image, not a relative link. Use an empty
alternative text rather than no alternative text so a screen-reader would
skip over it.
}

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{authors:
  firstname lastname:
    href: "http://name-website.com"
    html: "<img src='https://website.com/name-picture.png' width=72 alt=''>"
}\if{html}{\out{</div>}}

By default, the "developers" list shown in the sidebar and footer is
populated by the maintainer ("cre"), authors ("aut"), and funder ("fnd")
from the \code{DESCRIPTION}. You could choose other roles for filtering.
With the configuration below:
\itemize{
\item only the maintainer and funder(s) appear in the footer, after the text
"Crafted by",
\item all authors and contributors appear in the sidebar,
\item the authors list on the sidebar is preceded and followed by some text,
\item the authors list on the authors page is preceded and followed by some text.
}

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{authors:
  footer:
    roles: [cre, fnd]
    text: "Crafted by"
  sidebar:
    roles: [aut, ctb]
    before: "So *who* does the work?"
    after: "Thanks all!"
  before: "This package is proudly brought to you by:"
  after: "See the [changelog](news/index.html) for other contributors. :pray:"
}\if{html}{\out{</div>}}

If you want to filter authors based on something else than their roles,
consider using a custom sidebar/footer component
(see \code{?build_home}/\code{?build_site}, respectively).
}
}

\section{Sidebar}{
You can customise the homepage sidebar with the \code{home.sidebar} field.
It's made up of two pieces: \code{structure}, which defines the overall layout,
and \code{components}, which defines what each piece looks like. This organisation
makes it easy to mix and match the pkgdown defaults with your own
customisations.

This is the default structure:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{home:
  sidebar:
    structure: [links, license, community, citation, authors, dev]
}\if{html}{\out{</div>}}

These are drawn from seven built-in components:
\itemize{
\item \code{links}: automated links generated from \code{URL} and \code{BugReports} fields
from \code{DESCRIPTION} plus manual links from the \code{home.links} field:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{home:
  links:
  - text: Link text
    href: https://website.com
  - text: Roadmap
    href: /roadmap.html
}\if{html}{\out{</div>}}
\item \code{license}: Licensing information if \code{LICENSE}/\code{LICENCE} or
\code{LICENSE.md}/\code{LICENCE.md} files are present.
\item \code{community}: links to to \code{.github/CONTRIBUTING.md},
\code{.github/CODE_OF_CONDUCT.md}, etc.
\item \code{citation}: link to package citation information. Uses either
\code{inst/CITATION} or, if absent, information from the \code{DESCRIPTION}.
\item \code{authors}: selected authors from the \code{DESCRIPTION}.
\item \code{dev}: development status badges extracted from \code{README.md}/\code{index.md}.
This is only shown for "development" versions of websites; see
"Development mode" in \code{?build_site} for details.
\item \code{toc}: a table of contents for the README (not shown by default).
}

You can also add your own components, where \code{text} is markdown text:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{home:
  sidebar:
    structure: [authors, custom, toc, dev]
    components:
      custom:
        title: Funding
        text: We are *grateful* for funding!
}\if{html}{\out{</div>}}

Alternatively, you can provide a ready-made sidebar HTML:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{home:
  sidebar:
    html: path-to-sidebar.html
}\if{html}{\out{</div>}}

Or completely remove it:

\if{html}{\out{<div class="sourceCode yaml">}}\preformatted{home:
  sidebar: FALSE
}\if{html}{\out{</div>}}
}

\seealso{
Other site components: 
\code{\link{build_articles}()},
\code{\link{build_llm_docs}()},
\code{\link{build_news}()},
\code{\link{build_reference}()},
\code{\link{build_tutorials}()}
}
\concept{site components}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preview.R
\name{preview_site}
\alias{preview_site}
\title{Open site in browser}
\usage{
preview_site(pkg = ".", path = ".", preview = NA)
}
\arguments{
\item{pkg}{Path to package.}

\item{path}{Path relative to destination}

\item{preview}{If \code{TRUE}, or \code{is.na(preview) && interactive()}, will preview
freshly generated section in browser.}
}
\description{
Open site in browser
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rd.R
\name{rd2html}
\alias{rd2html}
\title{Translate an Rd string to its HTML output}
\usage{
rd2html(x, fragment = TRUE, ...)
}
\arguments{
\item{x}{Rd string. Backslashes must be double-escaped ("\\\\").}

\item{fragment}{logical indicating whether this represents a complete Rd file}

\item{...}{additional arguments for as_html}
}
\description{
Translate an Rd string to its HTML output
}
\examples{
rd2html("a\n\%b\nc")

rd2html("a & b")

rd2html("\\\\strong{\\\\emph{x}}")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean.R
\name{clean_site}
\alias{clean_site}
\title{Clean site}
\usage{
clean_site(pkg = ".")
}
\arguments{
\item{pkg}{Path to package.}
}
\description{
Delete all files in \verb{docs/} (except for \code{CNAME}).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/figure.R
\name{fig_settings}
\alias{fig_settings}
\title{Get current settings for figures}
\usage{
fig_settings()
}
\value{
A list containing the entries from the \code{figures} field in \verb{_pkgdown.yml}
(see \code{\link[=build_reference]{build_reference()}}), with default values added. Computed \code{width} and
\code{height} values (in pixels) are also included.
}
\description{
You will generally not need to use this function unless you are handling
custom plot output.

Packages needing custom parameters should ask users to place them within
the \code{other.parameters} entry under the package name, e.g.

\if{html}{\out{<div class="sourceCode">}}\preformatted{figures:
  other.parameters:
    rgl:
      fig.asp: 1
}\if{html}{\out{</div>}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test.R
\name{test-long-lines}
\alias{test-long-lines}
\title{Test case: long-lines}
\description{
The example results should have the copy button correctly placed when
scrollings
}
\examples{
pkgdown:::ruler()

cat(rep("x ", 100), sep = "")
cat(rep("xy", 100), sep = "")
cat(rep("x ", 100), sep = "")
cat(rep("xy", 100), sep = "")
}
\seealso{
Other tests: 
\code{\link{index}},
\code{\link{test-crayon}},
\code{\link{test-dont}},
\code{\link{test-figures}},
\code{\link{test-links}},
\code{\link{test-lists}},
\code{\link{test-math-examples}},
\code{\link{test-output-styles}},
\code{\link{test-params}},
\code{\link{test-sexpr-title}},
\code{\link{test-tables}},
\code{\link{test-verbatim}}
}
\concept{tests}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build-favicons.R
\name{build_favicons}
\alias{build_favicons}
\alias{build_favicon}
\title{Create favicons from package logo}
\usage{
build_favicons(pkg = ".", overwrite = FALSE)

build_favicon(pkg, overwrite)
}
\arguments{
\item{pkg}{Path to package.}

\item{overwrite}{If \code{TRUE}, re-create favicons from package logo.}
}
\description{
This function auto-detects the location of your package logo (with the name
\code{logo.svg} (recommended format) or \code{logo.png}) and runs it through the
\url{https://realfavicongenerator.net} API to build a complete set of favicons
with different sizes, as needed for modern web usage.
}
\details{
You only need to run the function once. The favicon set will be stored in
\code{pkgdown/favicon} and copied by \code{\link[=init_site]{init_site()}} to the relevant location when
the website is rebuilt.

Once complete, you should add \verb{pkgdown/} to \code{.Rbuildignore } to avoid a NOTE
during package checking.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean.R
\name{clean_site}
\alias{clean_site}
\alias{clean_cache}
\title{Clean site}
\usage{
clean_site(pkg = ".", quiet = FALSE)

clean_cache(pkg = ".", quiet = FALSE)
}
\arguments{
\item{pkg}{Path to package.}

\item{quiet}{If \code{TRUE}, suppresses a message.}
}
\description{
Delete all files in \verb{docs/} (except for \code{CNAME}).

Delete all files in the pkgdown cache directory.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bacon.R
\name{Figure}
\alias{Figure}
\title{Figure}
\description{
Pulled pork is delicious
}
\examples{
plot(1:5)
hist(1:3)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funs.R
\name{a}
\alias{a}
\title{A}
\usage{
a(a, b, c)
}
\arguments{
\item{a}{a letter}

\item{b}{a a number}

\item{c}{a logical}
}
\description{
A
}
\keyword{foo}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funs.R
\name{a}
\alias{a}
\title{Example with HTML dependency}
\usage{
a()
}
\description{
Example with HTML dependency
}
\examples{
a()
}
